--Check if the library was loaded before already w/o chat output
if IsLibSetsAlreadyLoaded(false) then return end

--This file the sets data and info (pre-loaded from the specified API version)
--It should be updated each time the APIversion increases to contain the new/changed data
local lib = LibSets

local tins = table.insert

------------------------------------------------------------------------------------------------------------------------
--> Last updated: API 101033, Ascending Tide, 2022-02-14, Baertram
------------------------------------------------------------------------------------------------------------------------

--SetIds which are blacklisted (old or non-used anymore)
local blacklistedSetIds = {
    [209] = true, --Rüstung des Kodex",
    [264] = true, --Riesenspinne",
    [483] = true, --Template_Drop_Magi",
    [484] = true, --Template_Drop_Stamina",
    [485] = true, --Template_Drop_Tank",
    [486] = true, --Template_Drop_Healer",
    [509] = true, --Template_Drop_Healer P",
    [510] = true, --Template_Drop_Tank P",
    [511] = true, --Template_Drop_Stamina P",
    [512] = true, --Template_Drop_Magi P",
}
lib.blacklistedSetIds = blacklistedSetIds

---------------------------------------------------------------------------------------------------------------------------
--Current APIversion is live or PTS check
local isPTSAPIVersionLive = lib.checkIfPTSAPIVersionIsLive()
---------------------------------------------------------------------------------------------------------------------------
--Future APIversion sets
local setsOfNewerAPIVersion
local zoneIdsOfNewAPIVersionOnly
--APIversion checks
--The setIds of sets which are not on the live server yet, or will be there as the PTS APIversion changes to the live server.
--As long as they are not live yet they will be kept in the library internal tables (see above) BUT will be removed from
--the used game's tables on the live server!
--Each line must be integer key without gaps (for ipairs) = setId, e.g.
--{
--EXAMPLE:
--  536,
--  537,
--}
setsOfNewerAPIVersion = {
    --setIds not yet on live server - BEGIN
    --- Sets of ApiVersion 10103x: xxxxx
    --xxx,
    --yyy,
    --....

    --setIds of APIversion 101041 "Update 41 Scions of Ithelia", 2024-02-02
    --setIds not yet on live server - END
}
lib.setsOfNewerAPIVersion = setsOfNewerAPIVersion
---------------------------------------------------------------------------------------------------------------------------
--Remove wayshrines from the list where zoneIds are only available in newer APIversions
--Each line must be integer key without gaps (for ipairs) = zoneId, e.g.
--{
--  1207,
--  1208,
--}
zoneIdsOfNewAPIVersionOnly = {
    --zoneIds not yet on live server - BEGIN
    --xxx APIVersion 10103x: xxxx

    --zoneIds of APIVersion 101037
    --[1] = xxxx, --zoneName here

    --zoneIds not yet on live server - END
}
lib.zoneIdsOfNewAPIVersionOnly = zoneIdsOfNewAPIVersionOnly
---------------------------------------------------------------------------------------------------------------------------

--The set info from the Excel file within LibSets folder:
--> All information was collected ingame and from different websites by user @Baertram
--The [setId] is the table's key
--setType parameters is one of the following boolean parameter. The one with true is specifying the set's type, see file "LibSets_ConstantsLibraryInternal.lua", section "--Set types"
--wayshrines:       table of wayshrine nodeIds to jump to. You'll be near the set's crafting station/at a wayshrine in the dropZone then. 1st table key=EP, 2nd table key=AD, 3rd table key= DC
--> If a table entry is negative (e.g. -1 or -2, or any value below 0) the jump won't work as no wayshrine is knwon or it is a special one like the Mages's guild island Augvea
--traitsNeeded:     the number of traits you need to research to be able to craft this set (if isCrafted = true)
--dlcId:            the dlcId from table lib.DLCData (key). The set was added with this DLC. 0 if it's the ESO base game
--zoneIds:          table containing the different drop zoneIds of the game where this set can drop
--veteran:          boolean value telling you if you need to do the dungeon in veteran mode to get this set pieces (if the set type is a trial/arena/dungeon where there is normal and veteran mode possible)
-->                 or a table containing the different equipTypes as key, and a boolean value as value to show which of the equipTypes is a veteran dungeon reward (e.g. head and shoulders for monster sets)
--setType:          one of the number constants for the setTypes, see file LibSets_Constants.lua, table LibSets.allowedSetTypes
--multiTrial:       boolean value (only if isTrial=true) telling you if this set can be achieved in multiple trials
--dropMechanic:     a table containing the possible dropMechanics of this set, see file "LibSets_ConstantsLibraryInternal.lua", section "Drop mechanics / cities / etc. for additional drop location
--                  information"
--dropMechanicNames:    table containing the different dropMechanic names (of bosses e.g.). The index of the table reflects the same index in the table "dropMechanic"
--                  The names are stored in different languages. The table's format is: {key = [language] = "String", ...}
-------

--The data below is generated from the Excel document LibSets_SetData.xlsx containing all the set data. The excel map "Sets data" contains the code in the excel's columns
--AR to AU (AV is not needed/used anymore and only still in there for information!)
--The Excel document can be found here: https://github.com/Baertram/LibSets or inside the addon's folder
--**********************************************************************************************************************
lib.setInfo = {
    ---Removed sets (old/inactive ones) --------------------------------------------------------------------------------
    --[[
     [SetId] [SetName EN]
        209 Armor of the Code
        264 Giant Spider
        483	Template_Drop_Magi
        484	Template_Drop_Stamina
        485	Template_Drop_Tank
        486	Template_Drop_Healer
        509	Template_Drop_Healer P
        510	Template_Drop_Tank P
        511	Template_Drop_Stamina P
        512	Template_Drop_Magi P
        500	Bloodlord's Embrace (OLD)
        502	Snow Treaders (OLD)
        504	Malacath's Band of Brutality
    ]]
    ------------------------------------------------------------------------------------------------------------------------
    ------------------------------------------------------------------------------------------------------------------------
    ---Active sets------------------------------------------------------------------------------------------------------
    --[SetId] = {table with setData}, -- SetName EN / SetName DE [Drop zone name 1 EN, Drop zone name 2 EN, Drop zone name 3 EN, ...]
[19] = {wayshrines={185,185,185,}, dlcId=DLC_BASE_GAME	, zoneIds={31,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Vestments of the Warlock / Gewänder des Hexers   [Selene's Web, Reaper's March, Reaper's March]
[20] = {wayshrines={109,109,109,}, dlcId=DLC_BASE_GAME	, zoneIds={103,103,341,103}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Witchman Armor / Hexenwerk   [The Rift, The Rift, The Lion's Den, The Rift]
[21] = {wayshrines={93,93,93,}, dlcId=DLC_BASE_GAME	, zoneIds={101,101,339,101}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Akaviri Dragonguard / Akavirische Drachengarde   [Eastmarch, Eastmarch, Hall of the Dead, Eastmarch]
[22] = {wayshrines={17,17,17,}, dlcId=DLC_BASE_GAME	, zoneIds={19,19,142,19}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Dreamer\'s Mantle / Mantel des Träumers   [Stormhaven, Stormhaven, Bonesnap Ruins, Stormhaven]
[23] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Archer\'s Mind / Schützensinn   [Dragonstar Arena, Craglorn, Craglorn]
[24] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Footman\'s Fortune / Geschick des Fußsoldaten   [Dragonstar Arena, Craglorn, Craglorn]
[25] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Desert Rose / Wüstenrose   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[26] = {wayshrines={131,131,131,}, dlcId=DLC_BASE_GAME	, zoneIds={347,347,557,347}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Prisoner\'s Rags / Lumpen des Gefangenen   [Coldharbour, Coldharbour, Village of the Lost, Coldharbour]
[27] = {wayshrines={93,93,93,}, dlcId=DLC_BASE_GAME	, zoneIds={101,101,339,101}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Fiord\'s Legacy / Fiords Erbe   [Eastmarch, Eastmarch, Hall of the Dead, Eastmarch]
[28] = {wayshrines={191,191,191,}, dlcId=DLC_BASE_GAME	, zoneIds={126,931,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Barkskin / Borkenhaut   [Elden Hollow I, Elden Hollow II, Grahtwood]
[29] = {wayshrines={189,189,189,}, dlcId=DLC_BASE_GAME	, zoneIds={146,933,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Sergeant\'s Mail / Rüstung des Feldwebels   [Wayrest Sewers I, Wayrest Sewers II, Stormhaven]
[30] = {wayshrines={102,102,102,}, dlcId=DLC_BASE_GAME	, zoneIds={58,58,138,58}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Thunderbug\'s Carapace / Donnerkäferpanzer   [Malabal Tor, Malabal Tor, Crimson Cove, Malabal Tor]
[31] = {wayshrines={65,65,65,}, dlcId=DLC_BASE_GAME	, zoneIds={41,41,216,41}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Silks of the Sun / Sonnenseide   [Stonefalls, Stonefalls, Crow's Wood, Stonefalls]
[32] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Healer\'s Habit / Bräuche des Heilers   [Dragonstar Arena, Craglorn, Craglorn]
[33] = {wayshrines={98,98,98,}, dlcId=DLC_BASE_GAME	, zoneIds={283,934,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Viper\'s Sting / Vipernbiss   [Fungal Grotto I, Fungal Grotto II, Stonefalls]
[34] = {wayshrines={28,28,28,}, dlcId=DLC_BASE_GAME	, zoneIds={57,57,306,57}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Night Mother\'s Embrace / Umarmung der Mutter der Nacht   [Deshaan, Deshaan, Forgotten Crypts, Deshaan]
[35] = {wayshrines={193,193,193,}, dlcId=DLC_BASE_GAME	, zoneIds={144,936,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Knightmare / Albtraumritter   [Spindleclutch I, Spindleclutch II, Glenumbra]
[36] = {wayshrines={121,121,121,}, dlcId=DLC_BASE_GAME	, zoneIds={381,381,486,381}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Armor of the Veiled Heritance / Rüstung des Schleiererbes   [Auridon, Auridon, Toothmaul Gully, Auridon]
[37] = {wayshrines={1,177,71,}, traitsNeeded=2, dlcId=DLC_BASE_GAME	, zoneIds={3,381,41}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Death\'s Wind / Todeswind   [Glenumbra, Auridon, Stonefalls]
[38] = {wayshrines={15,169,205,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={19,383,57}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Twilight\'s Embrace / Zwielichtkuss   [Stormhaven, Grahtwood, Deshaan]
[39] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Alessian Order / Alessianischer Orden   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[40] = {wayshrines={216,121,65,}, traitsNeeded=2, dlcId=DLC_BASE_GAME	, zoneIds={3,381,41}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Night\'s Silence / Stille der Nacht   [Glenumbra, Auridon, Stonefalls]
[41] = {wayshrines={82,151,78,}, traitsNeeded=4, dlcId=DLC_BASE_GAME	, zoneIds={20,108,117}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Whitestrake\'s Retribution / Weißplankes Vergeltung   [Rivenspire, Greenshade, Shadowfen]
[43] = {wayshrines={23,164,32,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={19,383,57}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Armor of the Seducer / Rüstung der Verführung   [Stormhaven, Grahtwood, Deshaan]
[44] = {wayshrines={58,101,93,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={104,58,101}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Vampire\'s Kiss / Kuss des Vampirs   [Alik'r Desert, Malabal Tor, Eastmarch]
[46] = {wayshrines={187,187,187,}, dlcId=DLC_BASE_GAME	, zoneIds={64,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Noble Duelist\'s Silks / Seide des edlen Duellanten   [Blessed Crucible, The Rift, The Rift]
[47] = {wayshrines={61,61,61,}, dlcId=DLC_BASE_GAME	, zoneIds={104,104,308,104}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Robes of the Withered Hand / Roben der Verdorrten Hand   [Alik'r Desert, Alik'r Desert, Lost City of the Na-Totambu, Alik'r Desert]
[48] = {wayshrines={13,148,48,}, traitsNeeded=4, dlcId=DLC_BASE_GAME	, zoneIds={20,108,117}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Magnus\' Gift / Magnus' Gabe   [Rivenspire, Greenshade, Shadowfen]
[49] = {wayshrines={65,65,65,}, dlcId=DLC_BASE_GAME	, zoneIds={41,41,216,41}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Shadow of the Red Mountain / Schatten des Roten Berges   [Stonefalls, Stonefalls, Crow's Wood, Stonefalls]
[50] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --The Morag Tong / Morag Tong   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[51] = {wayshrines={34,156,118,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={92,382,103}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Night Mother\'s Gaze / Blick der Mutter der Nacht   [Bangkorai, Reaper's March, The Rift]
[52] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Beckoning Steel / Klingender Stahl   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[53] = {wayshrines={195,195,195,}, dlcId=DLC_BASE_GAME	, zoneIds={449,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --The Ice Furnace / Eisschmiede   [Direfrost Keep, Eastmarch, Eastmarch]
[54] = {wayshrines={7,175,77,}, traitsNeeded=2, dlcId=DLC_BASE_GAME	, zoneIds={3,381,41}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Ashen Grip / Aschengriff   [Glenumbra, Auridon, Stonefalls]
[55] = {wayshrines={193,193,193,}, dlcId=DLC_BASE_GAME	, zoneIds={144,936,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Prayer Shawl / Gebetstuch   [Spindleclutch I, Spindleclutch II, Glenumbra]
[56] = {wayshrines={93,93,93,}, dlcId=DLC_BASE_GAME	, zoneIds={101,101,339,101}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Stendarr\'s Embrace / Stendarrs Umarmung   [Eastmarch, Eastmarch, Hall of the Dead, Eastmarch]
[57] = {wayshrines={214,214,214,}, dlcId=DLC_BASE_GAME	, zoneIds={383,383,124,383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Syrabane\'s Grip / Syrabanns Griff   [Grahtwood, Grahtwood, Root Sunder Ruins, Grahtwood]
[58] = {wayshrines={62,62,62,}, dlcId=DLC_BASE_GAME	, zoneIds={3,3,284,3}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Hide of the Werewolf / Fell des Werwolfs   [Glenumbra, Glenumbra, Bad Man's Hallows, Glenumbra]
[59] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Kyne\'s Kiss / Kynes Kuss   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[60] = {wayshrines={55,55,55,}, dlcId=DLC_BASE_GAME	, zoneIds={20,20,162,20}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Darkstride / Dunkelschritt   [Rivenspire, Rivenspire, Obsidian Scar, Rivenspire]
[61] = {wayshrines={98,98,98,}, dlcId=DLC_BASE_GAME	, zoneIds={283,934,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Dreugh King Slayer / Dreughkönigsschlächter   [Fungal Grotto I, Fungal Grotto II, Stonefalls]
[62] = {wayshrines={48,48,48,}, dlcId=DLC_BASE_GAME	, zoneIds={117,117,134,117}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Hatchling\'s Shell / Schlüpflingspanzer   [Shadowfen, Shadowfen, Sanguine's Demesne, Shadowfen]
[63] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --The Juggernaut / Koloss   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[64] = {wayshrines={143,143,143,}, dlcId=DLC_BASE_GAME	, zoneIds={108,108,137,108}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Shadow Dancer\'s Raiment / Kleidung des Schattentänzers   [Greenshade, Greenshade, Rulanyil's Fall, Greenshade]
[65] = {wayshrines={62,62,62,}, dlcId=DLC_BASE_GAME	, zoneIds={3,3,284,3}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Bloodthorn\'s Touch / Spur eines Blutdorns   [Glenumbra, Glenumbra, Bad Man's Hallows, Glenumbra]
[66] = {wayshrines={48,48,48,}, dlcId=DLC_BASE_GAME	, zoneIds={117,117,134,117}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Robes of the Hist / Roben des Hist   [Shadowfen, Shadowfen, Sanguine's Demesne, Shadowfen]
[67] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Shadow Walker / Schattengänger   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[68] = {wayshrines={131,131,131,}, dlcId=DLC_BASE_GAME	, zoneIds={347,347,557,347}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Stygian / Stygier   [Coldharbour, Coldharbour, Village of the Lost, Coldharbour]
[69] = {wayshrines={214,214,214,}, dlcId=DLC_BASE_GAME	, zoneIds={383,383,124,383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Ranger\'s Gait / Laufstil des Waldläufers   [Grahtwood, Grahtwood, Root Sunder Ruins, Grahtwood]
[70] = {wayshrines={36,36,36,}, dlcId=DLC_BASE_GAME	, zoneIds={92,92,169,92}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Seventh Legion Brute / Rohling der siebten Legion   [Bangkorai, Bangkorai, Razak's Wheel, Bangkorai]
[71] = {wayshrines={185,185,185,}, dlcId=DLC_BASE_GAME	, zoneIds={31,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Durok\'s Bane / Duroks Fluch   [Selene's Web, Reaper's March, Reaper's March]
[72] = {wayshrines={187,187,187,}, dlcId=DLC_BASE_GAME	, zoneIds={64,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Nikulas\' Heavy Armor / Nikulas' schwere Rüstung   [Blessed Crucible, The Rift, The Rift]
[73] = {wayshrines={135,135,135,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={347,347,347}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Oblivion\'s Foe / Erinnerung   [Coldharbour, Coldharbour, Coldharbour]
[74] = {wayshrines={133,133,133,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={347,347,347}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Spectre\'s Eye / Schemenauge   [Coldharbour, Coldharbour, Coldharbour]
[75] = {wayshrines={19,165,24,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={19,383,57}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Torug\'s Pact / Torugs Pakt   [Stormhaven, Grahtwood, Deshaan]
[76] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Robes of Alteration Mastery / Roben der Veränderungsbeherrschung   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[77] = {wayshrines={196,196,196,}, dlcId=DLC_BASE_GAME	, zoneIds={22,104,104}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Crusader / Glaubenskrieger   [Volenfell, Alik'r Desert, Alik'r Desert]
[78] = {wayshrines={9,154,51,}, traitsNeeded=4, dlcId=DLC_BASE_GAME	, zoneIds={20,108,117}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Hist Bark / Histrinde   [Rivenspire, Greenshade, Shadowfen]
[79] = {wayshrines={35,144,111,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={92,382,103}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Willow\'s Path / Weidenpfad   [Bangkorai, Reaper's March, The Rift]
[80] = {wayshrines={39,161,113,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={92,382,103}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Hunding\'s Rage / Hundings Zorn   [Bangkorai, Reaper's March, The Rift]
[81] = {wayshrines={137,103,89,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={104,58,101}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Song of Lamae / Lied der Lamien   [Alik'r Desert, Malabal Tor, Eastmarch]
[82] = {wayshrines={155,105,95,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={104,58,101}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Alessia\'s Bulwark / Alessias Bollwerk   [Alik'r Desert, Malabal Tor, Eastmarch]
[83] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Elf Bane / Elfenfluch   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[84] = {wayshrines={-2,-2,-2,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={208,643}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Orgnum\'s Scales / Orgnums Schuppen   [The Earth Forge, Imperial Sewers]
[85] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Almalexia\'s Mercy / Almalexias Gnade   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[86] = {wayshrines={121,121,121,}, dlcId=DLC_BASE_GAME	, zoneIds={381,381,486,381}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Queen\'s Elegance / Eleganz der Königin   [Auridon, Auridon, Toothmaul Gully, Auridon]
[87] = {wayshrines={-2,-2,-2,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={267,643}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Eyes of Mara / Augen von Mara   [Eyevea, Imperial Sewers]
[88] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Robes of Destruction Mastery / Roben der Zerstörungsbeherrschung   [Dragonstar Arena, Craglorn, Craglorn]
[89] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Sentry / Wachposten   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[90] = {wayshrines={162,162,162,}, dlcId=DLC_BASE_GAME	, zoneIds={382,382,487,382}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Senche\'s Bite / Biss des Senche   [Reaper's March, Reaper's March, The Vile Manse, Reaper's March]
[91] = {wayshrines={184,184,184,}, dlcId=DLC_BASE_GAME	, zoneIds={11,347,347}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Oblivion\'s Edge / Vorteil des Vergessens   [Vaults of Madness, Coldharbour, Coldharbour]
[92] = {wayshrines={-2,-2,-2,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={208,643}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Kagrenac\'s Hope / Kagrenacs Hoffnung   [The Earth Forge, Imperial Sewers]
[93] = {wayshrines={17,17,17,}, dlcId=DLC_BASE_GAME	, zoneIds={19,19,142,19}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Storm Knight\'s Plate / Wut des Sturmritters   [Stormhaven, Stormhaven, Bonesnap Ruins, Stormhaven]
[94] = {wayshrines={131,131,131,}, dlcId=DLC_BASE_GAME	, zoneIds={347,347,557,347}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Meridia\'s Blessed Armor / Meridias gesegnete Rüstung   [Coldharbour, Coldharbour, Village of the Lost, Coldharbour]
[95] = {wayshrines={-2,-2,-2,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={267,643}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Shalidor\'s Curse / Shalidors Fluch   [Eyevea, Imperial Sewers]
[96] = {wayshrines={198,198,198,}, dlcId=DLC_BASE_GAME	, zoneIds={63,930,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Armor of Truth / Rüstung der Wahrheit   [Darkshade Caverns I, Darkshade Caverns II, Deshaan]
[97] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --The Arch-Mage / Erzmagier   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[98] = {wayshrines={55,55,55,}, dlcId=DLC_BASE_GAME	, zoneIds={20,20,162,20}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Necropotence / Nekropotenz   [Rivenspire, Rivenspire, Obsidian Scar, Rivenspire]
[99] = {wayshrines={102,102,102,}, dlcId=DLC_BASE_GAME	, zoneIds={58,58,138,58}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Salvation / Erlösung   [Malabal Tor, Malabal Tor, Crimson Cove, Malabal Tor]
[100] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Hawk\'s Eye / Falkenauge   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[101] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Affliction / Elend (Befallen)   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[102] = {wayshrines={196,196,196,}, dlcId=DLC_BASE_GAME	, zoneIds={22,104,104}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Duneripper\'s Scales / Schuppen des Dünenbrechers   [Volenfell, Alik'r Desert, Alik'r Desert]
[103] = {wayshrines={195,195,195,}, dlcId=DLC_BASE_GAME	, zoneIds={449,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Magicka Furnace / Magickaschmiede   [Direfrost Keep, Eastmarch, Eastmarch]
[104] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Curse Eater / Fluchfresser   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[105] = {wayshrines={121,121,121,}, dlcId=DLC_BASE_GAME	, zoneIds={381,381,486,381}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Twin Sisters / Zwillingsschwestern   [Auridon, Auridon, Toothmaul Gully, Auridon]
[106] = {wayshrines={143,143,143,}, dlcId=DLC_BASE_GAME	, zoneIds={108,108,137,108}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Wilderqueen\'s Arch / Bogen der Wildkönigin   [Greenshade, Greenshade, Rulanyil's Fall, Greenshade]
[107] = {wayshrines={62,62,62,}, dlcId=DLC_BASE_GAME	, zoneIds={3,3,284,3}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Wyrd Tree\'s Blessing / Segen des Wyrdbaums   [Glenumbra, Glenumbra, Bad Man's Hallows, Glenumbra]
[108] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Ravager / Verwüster   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[109] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Light of Cyrodiil / Licht von Cyrodiil   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[110] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={380,935,381}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Sanctuary / Heiligtum   [The Banished Cells I, The Banished Cells II, Auridon]
[111] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Ward of Cyrodiil / Schutz von Cyrodiil   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[112] = {wayshrines={17,17,17,}, dlcId=DLC_BASE_GAME	, zoneIds={19,19,142,19}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Night Terror / Nachtschrecken   [Stormhaven, Stormhaven, Bonesnap Ruins, Stormhaven]
[113] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Crest of Cyrodiil / Wappen von Cyrodiil   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[114] = {wayshrines={162,162,162,}, dlcId=DLC_BASE_GAME	, zoneIds={382,382,487,382}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Soulshine / Seelenschein   [Reaper's March, Reaper's March, The Vile Manse, Reaper's March]
[116] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --The Destruction Suite / Garnitur der Zerstörung   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[117] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Relics of the Physician, Ansur / Relikte des Mediziners Ansur   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[118] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Treasures of the Earthforge / Schätze der Erdenschmiede   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[119] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Relics of the Rebellion / Relikte der Rebellion   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[120] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Arms of Infernace / Waffen Infernals   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[121] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={381,146,126,63,283}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Arms of the Ancestors / Waffen der Ahnen   [The Banished Cells I, Wayrest Sewers I, Elden Hollow I, Darkshade Caverns I, Fungal Grotto I]
[122] = {wayshrines={190,190,190,}, dlcId=DLC_BASE_GAME	, zoneIds={130,932,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Ebon Armory / Ebenerzarsenal   [Crypt of Hearts I, Crypt of Hearts II, Rivenspire]
[123] = {wayshrines={185,185,185,}, dlcId=DLC_BASE_GAME	, zoneIds={31,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hircine\'s Veneer / Hircines Schein   [Selene's Web, Reaper's March, Reaper's March]
[124] = {wayshrines={184,184,184,}, dlcId=DLC_BASE_GAME	, zoneIds={11,347,347}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --The Worm\'s Raiment / Garderobe des Wurms   [Vaults of Madness, Coldharbour, Coldharbour]
[125] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Wrath of the Imperium / Zorn des Kaiserreichs   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[126] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Grace of the Ancients / Anmut der Ahnen   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[127] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Deadly Strike / Tödlicher Stoß   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[128] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Blessing of the Potentates / Segen des Potentaten   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[129] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Vengeance Leech / Saugende Vergeltung   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[130] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Eagle Eye / Adlerauge   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[131] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Bastion of the Heartland / Bastion des Herzlandes   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[132] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Shield of the Valiant / Schild des Tapferen   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[133] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Buffer of the Swift / Dämpfer des Geschwinden   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[134] = {wayshrines={190,190,190,}, dlcId=DLC_BASE_GAME	, zoneIds={130,932,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Shroud of the Lich / Tuch des Lich   [Crypt of Hearts I, Crypt of Hearts II, Rivenspire]
[135] = {wayshrines={109,109,109,}, dlcId=DLC_BASE_GAME	, zoneIds={103,103,341,103}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Draugr\'s Heritage / Erbe des Draugrs   [The Rift, The Rift, The Lion's Den, The Rift]
[136] = {wayshrines={232,232,232,}, dlcId=DLC_BASE_GAME	, zoneIds={639,639,639}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Immortal Warrior / Unsterblicher Krieger   [Sanctum Ophidia, Sanctum Ophidia, Sanctum Ophidia]
[137] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={636,636,636}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Berserking Warrior / Tobender Krieger   [Hel Ra Citadel, Hel Ra Citadel, Hel Ra Citadel]
[138] = {wayshrines={231,231,231,}, dlcId=DLC_BASE_GAME	, zoneIds={638,638,638}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, dropMechanicDropLocationNames={['en']={'Varlariel: Head, chest, shoulders / Two Atronachs: Hands, Waist, Feet','','','','','',},['de']={'Valariel: Kopf. Brust, Schultern / 2 Atronachen: Hände, Hüfte, Füße','','','','','',},}	}, --Defending Warrior / Verteidigender Krieger   [Aetherian Archive, Aetherian Archive, Aetherian Archive]
[139] = {wayshrines={232,232,232,}, dlcId=DLC_BASE_GAME	, zoneIds={639,639,639}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Wise Mage / Weiser Magier   [Sanctum Ophidia, Sanctum Ophidia, Sanctum Ophidia]
[140] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={636,636,636}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Destructive Mage / Zerstörerischer Magier   [Hel Ra Citadel, Hel Ra Citadel, Hel Ra Citadel]
[141] = {wayshrines={231,231,231,}, dlcId=DLC_BASE_GAME	, zoneIds={638,638,638}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, dropMechanicDropLocationNames={['en']={'Varlariel: Head, chest, shoulders / Two Atronachs: Hands, Waist, Feet','','','','','',},['de']={'Valariel: Kopf. Brust, Schultern / 2 Atronachen: Hände, Hüfte, Füße','','','','','',},}	}, --Healing Mage / Heilender Magier   [Aetherian Archive, Aetherian Archive, Aetherian Archive]
[142] = {wayshrines={231,231,231,}, dlcId=DLC_BASE_GAME	, zoneIds={638,638,638}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, dropMechanicDropLocationNames={['en']={'Varlariel: Head, chest, shoulders / Two Atronachs: Hands, Waist, Feet','','','','','',},['de']={'Valariel: Kopf. Brust, Schultern / 2 Atronachen: Hände, Hüfte, Füße','','','','','',},}	}, --Quick Serpent / Flinke Schlange   [Aetherian Archive, Aetherian Archive, Aetherian Archive]
[143] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={636,636,636}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Poisonous Serpent / Giftschlange   [Hel Ra Citadel, Hel Ra Citadel, Hel Ra Citadel]
[144] = {wayshrines={232,232,232,}, dlcId=DLC_BASE_GAME	, zoneIds={639,639,639}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Twice-Fanged Serpent / Doppelzüngige Schlange   [Sanctum Ophidia, Sanctum Ophidia, Sanctum Ophidia]
[145] = {wayshrines={220,220,220,}, dlcId=DLC_BLACKWOOD	, zoneIds={888,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Way of Fire / Weg des Feuers   [Craglorn, Craglorn, Craglorn]
[146] = {wayshrines={220,220,220,}, dlcId=DLC_BLACKWOOD	, zoneIds={888,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Way of Air / Weg der Luft   [Craglorn, Craglorn, Craglorn]
[147] = {wayshrines={220,220,220,}, dlcId=DLC_BLACKWOOD	, zoneIds={888,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Way of Martial Knowledge / Weg der Kampfkunst   [Craglorn, Craglorn, Craglorn]
[148] = {wayshrines={217,217,217,}, traitsNeeded=8, dlcId=DLC_BLACKWOOD	, zoneIds={888,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Way of the Arena / Weg der Arena   [Craglorn, Craglorn, Craglorn]
[155] = {wayshrines={191,191,191,}, dlcId=DLC_BASE_GAME	, zoneIds={126,931,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Undaunted Bastion / Unerschrockenen-Bastion   [Elden Hollow I, Elden Hollow II, Grahtwood]
[156] = {wayshrines={192,192,192,}, dlcId=DLC_BASE_GAME	, zoneIds={148,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Undaunted Infiltrator / Unerschrockener Infiltrator   [Arx Corinium, Shadowfen, Shadowfen]
[157] = {wayshrines={186,186,186,}, dlcId=DLC_BASE_GAME	, zoneIds={38,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Undaunted Unweaver / Unerschrockener Entflechter   [Blackheart Haven, Bangkorai, Bangkorai]
[158] = {wayshrines={197,197,197,}, dlcId=DLC_BASE_GAME	, zoneIds={176,681,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Embershield / Glutschild   [City of Ash I, City of Ash II, Greenshade]
[159] = {wayshrines={197,197,197,}, dlcId=DLC_BASE_GAME	, zoneIds={176,681,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Sunderflame / Trennflamme   [City of Ash I, City of Ash II, Greenshade]
[160] = {wayshrines={197,197,197,}, dlcId=DLC_BASE_GAME	, zoneIds={176,681,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Burning Spellweave / Branntzauberweber   [City of Ash I, City of Ash II, Greenshade]
[161] = {wayshrines={234,234,234,}, traitsNeeded=9, dlcId=DLC_BLACKWOOD	, zoneIds={888,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Twice-Born Star / Doppelstern   [Craglorn, Craglorn, Craglorn]
[162] = {wayshrines={266,266,266,}, dlcId=DLC_BASE_GAME	, zoneIds={934,41,41}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Vila Theran','','','','','',},['de']={'Vila Theran','','','','','',},}	}, --Spawn of Mephala / Mephalas Brut   [Fungal Grotto II, Stonefalls, Stonefalls]
[163] = {wayshrines={267,267,267,}, dlcId=DLC_BASE_GAME	, zoneIds={936,3,3}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Vorenor Winterbourne','','','','','',},['de']={'Vorenor Winterkind','','','','','',},}	}, --Blood Spawn / Blutbrut   [Spindleclutch II, Glenumbra, Glenumbra]
[164] = {wayshrines={236,236,236,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={678,181,181}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Lord Warden Dusk','','','','','',},['de']={'Hochwärter Dämmer','','','','','',},}	}, --Lord Warden / Hochwärter   [Imperial City Prison, Cyrodiil, Cyrodiil]
[165] = {wayshrines={263,263,263,}, dlcId=DLC_BASE_GAME	, zoneIds={933,19,19}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Varaine and Allene Pellingare','','','','','',},['de']={'Varaine und Allene Pellingare','','','','','',},}	}, --Scourge Harvester / Geißelernter   [Wayrest Sewers II, Stormhaven, Stormhaven]
[166] = {wayshrines={264,264,264,}, dlcId=DLC_BASE_GAME	, zoneIds={930,57,57}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Engine Guardian','','','','','',},['de']={'Der Maschinenwächter','','','','','',},}	}, --Engine Guardian / Maschinenwächter   [Darkshade Caverns II, Deshaan, Deshaan]
[167] = {wayshrines={265,265,265,}, dlcId=DLC_BASE_GAME	, zoneIds={931,383,383}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Bogdan the Nightflame','','','','','',},['de']={'Bogdan die Nachtflamme','','','','','',},}	}, --Nightflame / Nachtflamme   [Elden Hollow II, Grahtwood, Grahtwood]
[168] = {wayshrines={269,269,269,}, dlcId=DLC_BASE_GAME	, zoneIds={932,20,20}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Nerien\'eth','','','','','',},['de']={'Nerien\'eth','','','','','',},}	}, --Nerien\'eth / Nerien'eth   [Crypt of Hearts II, Rivenspire, Rivenspire]
[169] = {wayshrines={268,268,268,}, dlcId=DLC_BASE_GAME	, zoneIds={681,108,108}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Valkyn Skoria','','','','','',},['de']={'Valkyn Skoria','','','','','',},}	}, --Valkyn Skoria / Valkyn Skoria   [City of Ash II, Greenshade, Greenshade]
[170] = {wayshrines={262,262,262,}, dlcId=DLC_BASE_GAME	, zoneIds={935,381,381}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'High Kinlord Rilis','','','','','',},['de']={'Sippenhochfürst Rilis','','','','','',},}	}, --Maw of the Infernal / Schlund des Infernalen   [The Banished Cells II, Auridon, Auridon]
[171] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={636,636,636}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Eternal Warrior / Ewiger Krieger   [Hel Ra Citadel, Hel Ra Citadel, Hel Ra Citadel]
[172] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={638,638,638}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, dropMechanicDropLocationNames={['en']={'Varlariel: Head, chest, shoulders / Two Atronachs: Hands, Waist, Feet','','','','','',},['de']={'Valariel: Kopf. Brust, Schultern / 2 Atronachen: Hände, Hüfte, Füße','','','','','',},}	}, --Infallible Mage / Unfehlbare Magierin   [Aetherian Archive, Aetherian Archive, Aetherian Archive]
[173] = {wayshrines={230,230,230,}, dlcId=DLC_BASE_GAME	, zoneIds={639,639,639}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Vicious Serpent / Boshafte Schlange   [Sanctum Ophidia, Sanctum Ophidia, Sanctum Ophidia]
[176] = {wayshrines={-1,-1,-1,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Noble\'s Conquest / Adelssieg   [Imperial City]
[177] = {wayshrines={-1,-1,-1,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Redistributor / Umverteilung   [Imperial City]
[178] = {wayshrines={-1,-1,-1,}, traitsNeeded=9, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Armor Master / Rüstungsmeister   [Imperial City]
[179] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Black Rose / Schwarze Rose   [Imperial Sewers]
[180] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Powerful Assault / Kraftvoller Ansturm   [Imperial Sewers]
[181] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Meritorious Service / Meritorischer Dienst   [Imperial Sewers]
[183] = {wayshrines={247,247,247,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={688,181,181}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Molag Kena','','','','','',},['de']={'Molag Kena','','','','','',},}	}, --Molag Kena / Molag Kena   [White-Gold Tower, Cyrodiil, Cyrodiil]
[184] = {wayshrines={247,247,247,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={688,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Brands of Imperium / Male des Kaiserreichs   [White-Gold Tower, Imperial City]
[185] = {wayshrines={247,247,247,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={688,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Spell Power Cure / Magiekraftheilung   [White-Gold Tower, Imperial City]
[186] = {wayshrines={188,188,188,}, dlcId=DLC_BASE_GAME	, zoneIds={131,58,58}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Jolting Arms / Rüttelnde Rüstung   [Tempest Island, Malabal Tor, Malabal Tor]
[187] = {wayshrines={48,48,48,}, dlcId=DLC_BASE_GAME	, zoneIds={117,117,134,117}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Swamp Raider / Sumpfräuber   [Shadowfen, Shadowfen, Sanguine's Demesne, Shadowfen]
[188] = {wayshrines={188,188,188,}, dlcId=DLC_BASE_GAME	, zoneIds={131,58,58}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Storm Master / Sturmmeister   [Tempest Island, Malabal Tor, Malabal Tor]
[190] = {wayshrines={236,236,236,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={678,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Scathing Mage / Verletzender Magier   [Imperial City Prison, Imperial City]
[193] = {wayshrines={188,188,188,}, dlcId=DLC_BASE_GAME	, zoneIds={131,58,58}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Overwhelming Surge / Überwältigende Woge   [Tempest Island, Malabal Tor, Malabal Tor]
[194] = {wayshrines={189,189,189,}, dlcId=DLC_BASE_GAME	, zoneIds={146,933,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Combat Physician / Feldarzt   [Wayrest Sewers I, Wayrest Sewers II, Stormhaven]
[195] = {wayshrines={236,236,236,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={678,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Sheer Venom / Reingift   [Imperial City Prison, Imperial City]
[196] = {wayshrines={236,236,236,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={678,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Leeching Plate / Auslaugende Rüstung   [Imperial City Prison, Imperial City]
[197] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={380,935,381}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Tormentor / Quälender   [The Banished Cells I, The Banished Cells II, Auridon]
[198] = {wayshrines={247,247,247,}, dlcId=DLC_IMPERIAL_CITY	, zoneIds={688,584}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS}	}, --Essence Thief / Essenzdieb   [White-Gold Tower, Imperial City]
[199] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Shield Breaker / Schildbrecher   [Imperial Sewers]
[200] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Phoenix / Phönix   [Imperial Sewers]
[201] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Reactive Armor / Reaktive Rüstung   [Imperial Sewers]
[204] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_DAILYRANDOMDUNGEONANDICREWARD	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS,LIBSETS_DROP_MECHANIC_MAIL_DAILY_RANDOM_DUNGEON_REWARD}, dropMechanicDropLocationNames={['en']={'','By mail','','','','',},['de']={'','Durch Nachricht','','','','',},}	}, --Endurance / Beständigkeit   [Imperial City]
[205] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_DAILYRANDOMDUNGEONANDICREWARD	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS,LIBSETS_DROP_MECHANIC_MAIL_DAILY_RANDOM_DUNGEON_REWARD}, dropMechanicDropLocationNames={['en']={'','By mail','','','','',},['de']={'','Durch Nachricht','','','','',},}	}, --Willpower / Willenskraft   [Imperial City]
[206] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={584}, veteran=false	, setType=LIBSETS_SETTYPE_DAILYRANDOMDUNGEONANDICREWARD	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_VAULTS,LIBSETS_DROP_MECHANIC_MAIL_DAILY_RANDOM_DUNGEON_REWARD}, dropMechanicDropLocationNames={['en']={'','By mail','','','','',},['de']={'','Durch Nachricht','','','','',},}	}, --Agility / Agilität   [Imperial City]
[207] = {wayshrines={241,241,241,}, traitsNeeded=6, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Law of Julianos / Gesetz von Julianos   [Wrothgar, Wrothgar, Wrothgar]
[208] = {wayshrines={237,237,237,}, traitsNeeded=3, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Trial by Fire / Feuertaufe   [Wrothgar, Wrothgar, Wrothgar]

[210] = {wayshrines={244,244,244,}, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Mark of the Pariah / Zeichen des Ausgestoßenen   [Wrothgar, Wrothgar, Wrothgar, Wrothgar]
[211] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Permafrost / Permafrost   [Maelstrom Arena, Wrothgar, Wrothgar]
[212] = {wayshrines={244,244,244,}, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Briarheart / Dornenherz   [Wrothgar, Wrothgar, Wrothgar, Wrothgar]
[213] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Glorious Defender / Ruhmreicher Verteidiger   [Maelstrom Arena, Wrothgar, Wrothgar]
[214] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Para Bellum / Para Bellum   [Maelstrom Arena, Wrothgar, Wrothgar]
[215] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Elemental Succession / Elementarfolge   [Maelstrom Arena, Wrothgar, Wrothgar]
[216] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Hunt Leader / Jagdleiter   [Maelstrom Arena, Wrothgar, Wrothgar]
[217] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}	}, --Winterborn / Winterkind   [Maelstrom Arena, Wrothgar, Wrothgar]
[218] = {wayshrines={244,244,244,}, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Trinimac\'s Valor / Trinimacs Heldenmut   [Wrothgar, Wrothgar, Wrothgar, Wrothgar]
[219] = {wayshrines={237,237,237,}, traitsNeeded=9, dlcId=DLC_SUMMERSET	, zoneIds={684,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Morkuldin / Morkuldin   [Wrothgar, Wrothgar, Wrothgar]
[224] = {wayshrines={257,257,257,}, traitsNeeded=5, dlcId=DLC_GREYMOOR	, zoneIds={816,816,816}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Tava\'s Favor / Tavas Gunst   [Hew's Bane, Hew's Bane, Hew's Bane]
[225] = {wayshrines={257,257,257,}, traitsNeeded=7, dlcId=DLC_GREYMOOR	, zoneIds={816,816,816}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Clever Alchemist / Schlauer Alchemist   [Hew's Bane, Hew's Bane, Hew's Bane]
[226] = {wayshrines={255,255,255,}, traitsNeeded=9, dlcId=DLC_GREYMOOR	, zoneIds={816,816,816}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Eternal Hunt / Ewige Jagd   [Hew's Bane, Hew's Bane, Hew's Bane]
[227] = {wayshrines={255,255,255,}, dlcId=DLC_GREYMOOR	, zoneIds={816,816,816}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Bahraha\'s Curse / Bahrahas Fluch   [Hew's Bane, Hew's Bane, Hew's Bane]
[228] = {wayshrines={255,255,255,}, dlcId=DLC_GREYMOOR	, zoneIds={816,816,816}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Syvarra\'s Scales / Syvarras Schuppen   [Hew's Bane, Hew's Bane, Hew's Bane]
[229] = {wayshrines={258,258,258,}, dlcId=DLC_THIEVES_GUILD	, zoneIds={725,725,725}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Twilight Remedy / Zwielichtgenesung   [Maw of Lorkhaj, Maw of Lorkhaj, Maw of Lorkhaj]
[230] = {wayshrines={258,258,258,}, dlcId=DLC_THIEVES_GUILD	, zoneIds={725,725,725}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Moondancer / Mondtänzer   [Maw of Lorkhaj, Maw of Lorkhaj, Maw of Lorkhaj]
[231] = {wayshrines={258,258,258,}, dlcId=DLC_THIEVES_GUILD	, zoneIds={725,725,725}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Lunar Bastion / Mondbastion   [Maw of Lorkhaj, Maw of Lorkhaj, Maw of Lorkhaj]
[232] = {wayshrines={258,258,258,}, dlcId=DLC_THIEVES_GUILD	, zoneIds={725,725,725}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Roar of Alkosh / Brüllen von Alkosh   [Maw of Lorkhaj, Maw of Lorkhaj, Maw of Lorkhaj]
[234] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Marksman\'s Crest / Wappen des Meisterschützen   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[235] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Robes of Transmutation / Roben der Transmutation   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[236] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Vicious Death / Grausamer Tod   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[237] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Leki\'s Focus / Lekis Fokus   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[238] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Fasalla\'s Guile / Fasallas List   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[239] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Warrior\'s Fury / Raserei des Kriegers   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[240] = {wayshrines={254,254,254,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Kvatch Gladiator / Gladiator von Kvatch   [Gold Coast, Gold Coast, Gold Coast]
[241] = {wayshrines={251,251,251,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Varen\'s Legacy / Varens Erbe   [Gold Coast, Gold Coast, Gold Coast]
[242] = {wayshrines={254,254,254,}, traitsNeeded=9, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Pelinal\'s Aptitude / Pelinals Talent   [Gold Coast, Gold Coast, Gold Coast]
[243] = {wayshrines={254,254,254,}, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Hide of Morihaus / Haut von Morihaus   [Gold Coast, Gold Coast, Gold Coast]
[244] = {wayshrines={254,254,254,}, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Flanking Strategist / Flankierender Stratege   [Gold Coast, Gold Coast, Gold Coast]
[245] = {wayshrines={254,254,254,}, dlcId=DLC_BASE_GAME	, zoneIds={823,823,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER}, dropMechanicDropLocationNames={['en']={'','','','Dark Brotherhood Sacrament / NPC: Remains-Silent','','',},['de']={'','','','Dunkle Bruderschaft Sakramente / NPC: Bleibt-Schweigsam','','',},}	}, --Sithis\' Touch / Sithis' Berührung   [Gold Coast, Gold Coast, Gold Coast, Gold Coast]
[246] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Galerion\'s Revenge / Galerions Revanche   [Imperial Sewers]
[247] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Vicecanon of Venom / Vizekanoniker des Gifts   [Imperial Sewers]
[248] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Thews of the Harbinger / Muskeln des Vorboten   [Imperial Sewers]
[253] = {wayshrines={-1,-1,-1,}, dlcId=DLC_MORROWIND	, zoneIds={643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY	, dropMechanic={LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_IMPERIAL_CITY_TREASURE_TROVE_SCAMP}	}, --Imperial Physique / Kaiserliche Physis   [Imperial Sewers]
[256] = {wayshrines={260,260,260,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={843,117,117}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Tree-Minder Na-Kesh','','','','','',},['de']={'Baumhirtin Na-Kesh','','','','','',},}	}, --Mighty Chudan / Mächtiger Chudan   [Ruins of Mazzatun, Shadowfen, Shadowfen]
[257] = {wayshrines={261,261,261,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={848,117,117}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Velidreth','','','','','',},['de']={'Velidreth','','','','','',},}	}, --Velidreth / Velidreth   [Cradle of Shadows, Shadowfen, Shadowfen]
[258] = {wayshrines={260,260,260,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={843,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Amber Plasm / Bernsteinplasma   [Ruins of Mazzatun, Shadowfen, Shadowfen]
[259] = {wayshrines={260,260,260,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={843,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Heem-Jas\' Retribution / Heem-Jas' Vergeltung   [Ruins of Mazzatun, Shadowfen, Shadowfen]
[260] = {wayshrines={260,260,260,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={843,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Aspect of Mazzatun / Aspekt von Mazzatun   [Ruins of Mazzatun, Shadowfen, Shadowfen]
[261] = {wayshrines={261,261,261,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={848,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Gossamer / Gespinst   [Cradle of Shadows, Shadowfen, Shadowfen]
[262] = {wayshrines={261,261,261,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={848,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Widowmaker / Witwenmacher   [Cradle of Shadows, Shadowfen, Shadowfen]
[263] = {wayshrines={261,261,261,}, dlcId=DLC_SHADOWS_OF_THE_HIST	, zoneIds={848,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hand of Mephala / Hand von Mephala   [Cradle of Shadows, Shadowfen, Shadowfen]

[265] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={380,381,381}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'High Kinlord Rilis','','','','','',},['de']={'Sippenhochfürst Rilis','','','','','',},}	}, --Shadowrend / Schattenriss   [The Banished Cells I, Auridon, Auridon]
[266] = {wayshrines={98,98,98,}, dlcId=DLC_BASE_GAME	, zoneIds={283,41,41}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Kra\'gh the Dreugh King','','','','','',},['de']={'Kra\'gh der Dreughkönig','','','','','',},}	}, --Kra\'gh / Kra'gh   [Fungal Grotto I, Stonefalls, Stonefalls]
[267] = {wayshrines={193,193,193,}, dlcId=DLC_BASE_GAME	, zoneIds={144,3,3}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Whisperer','','','','','',},['de']={'Die Flüsterin','','','','','',},}	}, --Swarm Mother / Schwarmmutter   [Spindleclutch I, Glenumbra, Glenumbra]
[268] = {wayshrines={198,198,198,}, dlcId=DLC_BASE_GAME	, zoneIds={63,57,57}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Sentinel of Rkugamz','','','','','',},['de']={'Wachposten von Rkugamz','','','','','',},}	}, --Sentinel of Rkugamz / Wachposten von Rkugamz   [Darkshade Caverns I, Deshaan, Deshaan]
[269] = {wayshrines={191,191,191,}, dlcId=DLC_BASE_GAME	, zoneIds={126,383,383}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Canonreeve Oraneth','','','','','',},['de']={'Stiftsvogt Oraneth','','','','','',},}	}, --Chokethorn / Würgedorn   [Elden Hollow I, Grahtwood, Grahtwood]
[270] = {wayshrines={189,189,189,}, dlcId=DLC_BASE_GAME	, zoneIds={146,19,19}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Varaine and Allene Pellingare','','','','','',},['de']={'Varaine und Allene Pellingare','','','','','',},}	}, --Slimecraw / Schleimkropf   [Wayrest Sewers I, Stormhaven, Stormhaven]
[271] = {wayshrines={192,192,192,}, dlcId=DLC_BASE_GAME	, zoneIds={148,117,117}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Sellistrix the Lamia Queen','','','','','',},['de']={'Sellistrix die Lamienkönigin','','','','','',},}	}, --Sellistrix / Sellistrix   [Arx Corinium, Shadowfen, Shadowfen]
[272] = {wayshrines={197,197,197,}, dlcId=DLC_BASE_GAME	, zoneIds={176,108,108}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Razor Master Erthas','','','','','',},['de']={'Klingenmeister Erthas','','','','','',},}	}, --Infernal Guardian / Infernaler Wächter   [City of Ash I, Greenshade, Greenshade]
[273] = {wayshrines={190,190,190,}, dlcId=DLC_BASE_GAME	, zoneIds={130,20,20}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Ilambris-Athor or Ilambris-Zaven','','','','','',},['de']={'Ilambris Zwillinge','','','','','',},}	}, --Ilambris / Ilambris   [Crypt of Hearts I, Rivenspire, Rivenspire]
[274] = {wayshrines={195,195,195,}, dlcId=DLC_BASE_GAME	, zoneIds={449,101,101}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Drodda of Icereach','','','','','',},['de']={'Drodda von Eiskap','','','','','',},}	}, --Iceheart / Eisherz   [Direfrost Keep, Eastmarch, Eastmarch]
[275] = {wayshrines={188,188,188,}, dlcId=DLC_BASE_GAME	, zoneIds={131,58,58}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Stormreeve Neidir','','','','','',},['de']={'Sturmvogt Neidir','','','','','',},}	}, --Stormfist / Sturmfaust   [Tempest Island, Malabal Tor, Malabal Tor]
[276] = {wayshrines={196,196,196,}, dlcId=DLC_BASE_GAME	, zoneIds={22,104,104}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Guardian Council','','','','','',},['de']={'Die Wächter','','','','','',},}	}, --Tremorscale / Bebenschuppe   [Volenfell, Alik'r Desert, Alik'r Desert]
[277] = {wayshrines={186,186,186,}, dlcId=DLC_BASE_GAME	, zoneIds={38,92,92}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Captain Blackheart','','','','','',},['de']={'Kapitän Schwarzherz','','','','','',},}	}, --Pirate Skeleton / Piratenskelett   [Blackheart Haven, Bangkorai, Bangkorai]
[278] = {wayshrines={187,187,187,}, dlcId=DLC_BASE_GAME	, zoneIds={64,103,103}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Lava Queen','','','','','',},['de']={'Die Lavakönigin','','','','','',},}	}, --The Troll King / Trollkönig   [Blessed Crucible, The Rift, The Rift]
[279] = {wayshrines={185,185,185,}, dlcId=DLC_BASE_GAME	, zoneIds={31,382,382}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Selene','','','','','',},['de']={'Selene','','','','','',},}	}, --Selene / Selene   [Selene's Web, Reaper's March, Reaper's March]
[280] = {wayshrines={184,184,184,}, dlcId=DLC_BASE_GAME	, zoneIds={11,347,347}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Mad Architect','','','','','',},['de']={'Der Verrückte Architekt','','','','','',},}	}, --Grothdarr / Grothdarr   [Vaults of Madness, Coldharbour, Coldharbour]
[281] = {wayshrines={138,141,181,}, dlcId=DLC_BASE_GAME	, zoneIds={534,537,535,280,281}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Armor of the Trainee / Rüstung des Auszubildenden   [Stros M'Kai, Khenarthi's Roost, Betnikh, Bleakrock Isle, Bal Foyen]
[282] = {wayshrines={55,55,55,}, dlcId=DLC_BASE_GAME	, zoneIds={20,20,162,20}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Vampire Cloak / Vampirumhang   [Rivenspire, Rivenspire, Obsidian Scar, Rivenspire]
[283] = {wayshrines={61,61,61,}, dlcId=DLC_BASE_GAME	, zoneIds={104,104,308,104}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Sword-Singer / Schwertsänger   [Alik'r Desert, Alik'r Desert, Lost City of the Na-Totambu, Alik'r Desert]
[284] = {wayshrines={61,61,61,}, dlcId=DLC_BASE_GAME	, zoneIds={104,104,308,104}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Order of Diagna / Orden von Diagna   [Alik'r Desert, Alik'r Desert, Lost City of the Na-Totambu, Alik'r Desert]
[285] = {wayshrines={36,36,36,}, dlcId=DLC_BASE_GAME	, zoneIds={92,92,169,92}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Vampire Lord / Vampirfürst   [Bangkorai, Bangkorai, Razak's Wheel, Bangkorai]
[286] = {wayshrines={36,36,36,}, dlcId=DLC_BASE_GAME	, zoneIds={92,92,169,92}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Spriggan\'s Thorns / Dornen des Zweiglings   [Bangkorai, Bangkorai, Razak's Wheel, Bangkorai]
[287] = {wayshrines={214,214,214,}, dlcId=DLC_BASE_GAME	, zoneIds={383,383,124,383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Green Pact / Der Grüne Pakt   [Grahtwood, Grahtwood, Root Sunder Ruins, Grahtwood]
[288] = {wayshrines={143,143,143,}, dlcId=DLC_BASE_GAME	, zoneIds={108,108,137,108}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Beekeeper\'s Gear / Werkzeug des Bienenhüters   [Greenshade, Greenshade, Rulanyil's Fall, Greenshade]
[289] = {wayshrines={102,102,102,}, dlcId=DLC_BASE_GAME	, zoneIds={58,58,138,58}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Spinner\'s Garments / Gewänder des Webers   [Malabal Tor, Malabal Tor, Crimson Cove, Malabal Tor]
[290] = {wayshrines={162,162,162,}, dlcId=DLC_BASE_GAME	, zoneIds={382,382,487,382}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Skooma Smuggler / Skoomaschmuggler   [Reaper's March, Reaper's March, The Vile Manse, Reaper's March]
[291] = {wayshrines={65,65,65,}, dlcId=DLC_BASE_GAME	, zoneIds={41,41,216,41}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Shalk Exoskeleton / Schröterpanzer   [Stonefalls, Stonefalls, Crow's Wood, Stonefalls]
[292] = {wayshrines={28,28,28,}, dlcId=DLC_BASE_GAME	, zoneIds={57,57,306,57}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Mother\'s Sorrow / Muttertränen   [Deshaan, Deshaan, Forgotten Crypts, Deshaan]
[293] = {wayshrines={28,28,28,}, dlcId=DLC_BASE_GAME	, zoneIds={57,57,306,57}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Plague Doctor / Seuchendoktor   [Deshaan, Deshaan, Forgotten Crypts, Deshaan]
[294] = {wayshrines={109,109,109,}, dlcId=DLC_BASE_GAME	, zoneIds={103,103,341,103}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Ysgramor\'s Birthright / Ysgramors Geburtsrecht   [The Rift, The Rift, The Lion's Den, The Rift]
[295] = {wayshrines={194,194,194,}, dlcId=DLC_BASE_GAME	, zoneIds={380,935,381}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Jailbreaker / Ausbrecher   [The Banished Cells I, The Banished Cells II, Auridon]
[296] = {wayshrines={193,193,193,}, dlcId=DLC_BASE_GAME	, zoneIds={144,936,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Spelunker / Höhlenforscher   [Spindleclutch I, Spindleclutch II, Glenumbra]
[297] = {wayshrines={98,98,98,}, dlcId=DLC_BASE_GAME	, zoneIds={283,934,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Spider Cultist Cowl / Spinnenkultistenkutte   [Fungal Grotto I, Fungal Grotto II, Stonefalls]
[298] = {wayshrines={191,191,191,}, dlcId=DLC_BASE_GAME	, zoneIds={126,931,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Light Speaker / Lichtsprecher   [Elden Hollow I, Elden Hollow II, Grahtwood]
[299] = {wayshrines={189,189,189,}, dlcId=DLC_BASE_GAME	, zoneIds={146,933,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Toothrow / Zahnreihe   [Wayrest Sewers I, Wayrest Sewers II, Stormhaven]
[300] = {wayshrines={198,198,198,}, dlcId=DLC_BASE_GAME	, zoneIds={63,930,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Netch\'s Touch / Berührung des Netch   [Darkshade Caverns I, Darkshade Caverns II, Deshaan]
[301] = {wayshrines={198,198,198,}, dlcId=DLC_BASE_GAME	, zoneIds={63,930,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Strength of the Automaton / Stärke des Automaten   [Darkshade Caverns I, Darkshade Caverns II, Deshaan]
[302] = {wayshrines={190,190,190,}, dlcId=DLC_BASE_GAME	, zoneIds={130,932,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Leviathan / Leviathan   [Crypt of Hearts I, Crypt of Hearts II, Rivenspire]
[303] = {wayshrines={192,192,192,}, dlcId=DLC_BASE_GAME	, zoneIds={148,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Lamia\'s Song / Lied der Lamie   [Arx Corinium, Shadowfen, Shadowfen]
[304] = {wayshrines={192,192,192,}, dlcId=DLC_BASE_GAME	, zoneIds={148,117,117}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Medusa / Versteinernder Blick   [Arx Corinium, Shadowfen, Shadowfen]
[305] = {wayshrines={196,196,196,}, dlcId=DLC_BASE_GAME	, zoneIds={22,104,104}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Treasure Hunter / Schatzjäger   [Volenfell, Alik'r Desert, Alik'r Desert]
[307] = {wayshrines={195,195,195,}, dlcId=DLC_BASE_GAME	, zoneIds={449,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Draugr Hulk / Schwerfälliger Draugr   [Direfrost Keep, Eastmarch, Eastmarch]
[308] = {wayshrines={186,186,186,}, dlcId=DLC_BASE_GAME	, zoneIds={38,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Bone Pirate\'s Tatters / Lumpen des Knochenpiraten   [Blackheart Haven, Bangkorai, Bangkorai]
[309] = {wayshrines={186,186,186,}, dlcId=DLC_BASE_GAME	, zoneIds={38,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Knight-errant\'s Mail / Platten des Wanderritters   [Blackheart Haven, Bangkorai, Bangkorai]
[310] = {wayshrines={187,187,187,}, dlcId=DLC_BASE_GAME	, zoneIds={64,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Sword Dancer / Schwerttänzer   [Blessed Crucible, The Rift, The Rift]
[311] = {wayshrines={184,184,184,}, dlcId=DLC_BASE_GAME	, zoneIds={11,347,347}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Rattlecage / Klapperkäfig   [Vaults of Madness, Coldharbour, Coldharbour]
[313] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=528	}, --Titanic Cleave / Titanisches Trennen   [Dragonstar Arena, Craglorn, Craglorn]
[314] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=529	}, --Puncturing Remedy / Durchschlagende Genesung   [Dragonstar Arena, Craglorn, Craglorn]
[315] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=530	}, --Stinging Slashes / Stechende Schnitte   [Dragonstar Arena, Craglorn, Craglorn]
[316] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=0, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=531	}, --Caustic Arrow / Beißender Pfeil   [Dragonstar Arena, Craglorn, Craglorn]
[317] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=532	}, --Destructive Impact / Zerstörerischer Einschlag   [Dragonstar Arena, Craglorn, Craglorn]
[318] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=533	}, --Grand Rejuvenation / Große Verjüngung   [Dragonstar Arena, Craglorn, Craglorn]
[320] = {wayshrines={284,284,284,}, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --War Maiden / Kriegsjungfer   [Vvardenfell, Vvardenfell, Vvardenfell, Vvardenfell]
[321] = {wayshrines={284,284,284,}, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Defiler / Schänder   [Vvardenfell, Vvardenfell, Vvardenfell, Vvardenfell]
[322] = {wayshrines={284,284,284,}, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Warrior-Poet / Kriegerpoet   [Vvardenfell, Vvardenfell, Vvardenfell, Vvardenfell]
[323] = {wayshrines={276,276,276,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Assassin\'s Guile / Assassinenlist   [Vvardenfell, Vvardenfell, Vvardenfell]
[324] = {wayshrines={329,329,329,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Daedric Trickery / Daedrische Gaunerei   [Vvardenfell, Vvardenfell, Vvardenfell]
[325] = {wayshrines={282,282,282,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={849,849,849}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Shacklebreaker / Kettensprenger   [Vvardenfell, Vvardenfell, Vvardenfell]
[326] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-98}, veteran=false	, setType=LIBSETS_SETTYPE_BATTLEGROUND	, dropMechanic={LIBSETS_DROP_MECHANIC_BATTLEGROUND_REWARD}	}, --Vanguard\'s Challenge / Vorhutdisput   [!Battleground]
[327] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-98}, veteran=false	, setType=LIBSETS_SETTYPE_BATTLEGROUND	, dropMechanic={LIBSETS_DROP_MECHANIC_BATTLEGROUND_REWARD}	}, --Coward\'s Gear / Feiglingstracht   [!Battleground]
[328] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-98}, veteran=false	, setType=LIBSETS_SETTYPE_BATTLEGROUND	, dropMechanic={LIBSETS_DROP_MECHANIC_BATTLEGROUND_REWARD}	}, --Knight Slayer / Ritterschlächter   [!Battleground]
[329] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-98}, veteran=false	, setType=LIBSETS_SETTYPE_BATTLEGROUND	, dropMechanic={LIBSETS_DROP_MECHANIC_BATTLEGROUND_REWARD}	}, --Wizard\'s Riposte / Zaubererreplik   [!Battleground]
[330] = {wayshrines={331,331,331,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={975,975,975}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Automated Defense / Automatisierte Verteidigung   [Halls of Fabrication, Halls of Fabrication, Halls of Fabrication]
[331] = {wayshrines={331,331,331,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={975,975,975}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --War Machine / Kriegsmaschine   [Halls of Fabrication, Halls of Fabrication, Halls of Fabrication]
[332] = {wayshrines={331,331,331,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={975,975,975}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Master Architect / Meisterarchitekt   [Halls of Fabrication, Halls of Fabrication, Halls of Fabrication]
[333] = {wayshrines={331,331,331,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={975,975,975}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}	}, --Inventor\'s Guard / Erfindergarde   [Halls of Fabrication, Halls of Fabrication, Halls of Fabrication]
[334] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-98}, veteran=false	, setType=LIBSETS_SETTYPE_BATTLEGROUND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_BATTLEGROUND_REWARD}	}, --Impregnable Armor / Unüberwindliche Rüstung   [!Battleground]
[335] = {wayshrines={332,332,332,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={974,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Draugr\'s Rest / Draugrruhe   [Falkreath Hold, Craglorn, Craglorn]
[336] = {wayshrines={332,332,332,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={974,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Pillar of Nirn / Säulen von Nirn   [Falkreath Hold, Craglorn, Craglorn]
[337] = {wayshrines={332,332,332,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={974,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Ironblood / Eisenblut   [Falkreath Hold, Craglorn, Craglorn]
[338] = {wayshrines={326,326,326,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={973,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Flame Blossom / Flammenblüte   [Bloodroot Forge, Craglorn, Craglorn]
[339] = {wayshrines={326,326,326,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={973,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Blooddrinker / Bluttrinker   [Bloodroot Forge, Craglorn, Craglorn]
[340] = {wayshrines={326,326,326,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={973,888,888}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hagraven\'s Garden / Vettelgarten   [Bloodroot Forge, Craglorn, Craglorn]
[341] = {wayshrines={326,326,326,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={973,888,888}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Earthgore Amalgam','','','','','',},['de']={'Erdbluter-Amalgam','','','','','',},}	}, --Earthgore / Erdbluter   [Bloodroot Forge, Craglorn, Craglorn]
[342] = {wayshrines={332,332,332,}, dlcId=DLC_HORNS_OF_THE_REACH	, zoneIds={974,888,888}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Domihaus the Bloody-Hornedin','','','','','',},['de']={'Domihaus der Blutgehörnte','','','','','',},}	}, --Domihaus / Domihaus   [Falkreath Hold, Craglorn, Craglorn]
[343] = {wayshrines={341,341,341,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1009,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Caluurion\'s Legacy / Caluurions Erbe   [Fang Lair, Bangkorai, Bangkorai]
[344] = {wayshrines={341,341,341,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1009,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Trappings of Invigoration / Stärkungsprunk   [Fang Lair, Bangkorai, Bangkorai]
[345] = {wayshrines={341,341,341,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1009,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Ulfnor\'s Favor / Ulfnors Gunst   [Fang Lair, Bangkorai, Bangkorai]
[346] = {wayshrines={363,363,363,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1010,19,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Jorvuld\'s Guidance / Jorvulds Führung   [Scalecaller Peak, Stormhaven, Stormhaven]
[347] = {wayshrines={363,363,363,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1010,19,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Plague Slinger / Seuchenschleuder   [Scalecaller Peak, Stormhaven, Stormhaven]
[348] = {wayshrines={363,363,363,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1010,19,19}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Curse of Doylemish / Fluch von Doylemish   [Scalecaller Peak, Stormhaven, Stormhaven]
[349] = {wayshrines={341,341,341,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1009,92,92}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Thurvokun','','','','','',},['de']={'Thurvokun','','','','','',},}	}, --Thurvokun / Thurvokun   [Fang Lair, Bangkorai, Bangkorai]
[350] = {wayshrines={363,363,363,}, dlcId=DLC_DRAGON_BONES	, zoneIds={1010,19,19}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Zaan','','','','','',},['de']={'Zaan','','','','','',},}	}, --Zaan / Zaan   [Scalecaller Peak, Stormhaven, Stormhaven]
[351] = {wayshrines={339,339,339,}, traitsNeeded=2, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Innate Axiom / Kernaxiom   [Clockwork City, Clockwork City, Clockwork City]
[352] = {wayshrines={337,337,337,}, traitsNeeded=4, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Fortified Brass / Messingpanzer   [Clockwork City, Clockwork City, Clockwork City]
[353] = {wayshrines={338,338,338,}, traitsNeeded=6, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Mechanical Acuity / Mechanikblick   [Clockwork City, Clockwork City, Clockwork City]
[354] = {wayshrines={338,338,338,}, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Mad Tinkerer / Wahntüftler   [Clockwork City, Clockwork City, Clockwork City]
[355] = {wayshrines={338,338,338,}, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Unfathomable Darkness / Unermessliche Dunkelheit   [Clockwork City, Clockwork City, Clockwork City]
[356] = {wayshrines={338,338,338,}, dlcId=DLC_NECROM	, zoneIds={980,980,980}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Livewire / Stromschlag   [Clockwork City, Clockwork City, Clockwork City]
[357] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Disciplined Slash (Perfected) / Disziplinierter Schnitt (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[358] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Defensive Position (Perfected) / Defensive Position (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[359] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Chaotic Whirlwind (Perfected) / Chaotischer Wirbelwind (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[360] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Piercing Spray (Perfected) / Durchdringende Salve (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[361] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Concentrated Force (Perfected) / Konzentrierte Kraft (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[362] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Timeless Blessing (Perfected) / Zeitloser Segen (vollendet)   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[363] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=357	}, --Disciplined Slash / Disziplinierter Schnitt   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[364] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=358	}, --Defensive Position / Defensive Position   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[365] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=359	}, --Chaotic Whirlwind / Chaotischer Wirbelwind   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[366] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=360	}, --Piercing Spray / Durchdringende Salve   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[367] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=361	}, --Concentrated Force / Konzentrierte Kraft   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[368] = {wayshrines={346,346,346,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={1000,1000,1000}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=362	}, --Timeless Blessing / Zeitloser Segen   [Asylum Sanctorium, Asylum Sanctorium, Asylum Sanctorium]
[369] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=522	}, --Merciless Charge / Gnadenloser Ansturm   [Maelstrom Arena, Wrothgar, Wrothgar]
[370] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=523	}, --Rampaging Slash / Tobender Schnitt   [Maelstrom Arena, Wrothgar, Wrothgar]
[371] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=524	}, --Cruel Flurry / Grausamer Schlaghagel   [Maelstrom Arena, Wrothgar, Wrothgar]
[372] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=525	}, --Thunderous Volley / Donnernde Salve   [Maelstrom Arena, Wrothgar, Wrothgar]
[373] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=526	}, --Crushing Wall / Zermalmende Mauer   [Maelstrom Arena, Wrothgar, Wrothgar]
[374] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=527	}, --Precise Regeneration / Präzise Regeneration   [Maelstrom Arena, Wrothgar, Wrothgar]
[380] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-99}, veteran=false	, setType=LIBSETS_SETTYPE_SPECIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_LEVEL_UP_REWARD}	}, --Prophet\'s / Prophet   [!Level up advisor]
[381] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={-99}, veteran=false	, setType=LIBSETS_SETTYPE_SPECIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_LEVEL_UP_REWARD}	}, --Broken Soul / Zerbrochene Seele   [!Level up advisor]
[382] = {wayshrines={355,355,355,}, dlcId=DLC_NECROM	, zoneIds={1011,1011,1011,1011,1027,1027}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Grace of Gloom / Anmut der Gräue   [Summerset, Summerset, Summerset, Summerset, Artaeum, Artaeum]
[383] = {wayshrines={355,355,355,}, dlcId=DLC_NECROM	, zoneIds={1011,1011,1011,1011,1027,1027}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Gryphon\'s Ferocity / Wildheit des Greifen   [Summerset, Summerset, Summerset, Summerset, Artaeum, Artaeum]
[384] = {wayshrines={355,355,355,}, dlcId=DLC_NECROM	, zoneIds={1011,1011,1011,1011,1027,1027}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Wisdom of Vanus / Weisheit von Vanus   [Summerset, Summerset, Summerset, Summerset, Artaeum, Artaeum]
[385] = {wayshrines={359,359,359,}, traitsNeeded=3, dlcId=DLC_NECROM	, zoneIds={1011,1011,1011}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Adept Rider / Versierter Reiter   [Summerset, Summerset, Summerset]
[386] = {wayshrines={360,360,360,}, traitsNeeded=6, dlcId=DLC_NECROM	, zoneIds={1027,1027,1027}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Sload\'s Semblance / Kreckenantlitz   [Artaeum, Artaeum, Artaeum]
[387] = {wayshrines={354,354,354,}, traitsNeeded=9, dlcId=DLC_NECROM	, zoneIds={1011,1011,1011}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Nocturnal\'s Favor / Nocturnals Gunst   [Summerset, Summerset, Summerset]
[388] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=392	}, --Aegis of Galenwe / Ägis von Galenwe   [Cloudrest, Cloudrest, Cloudrest]
[389] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=393	}, --Arms of Relequen / Waffen von Relequen   [Cloudrest, Cloudrest, Cloudrest]
[390] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=394	}, --Mantle of Siroria / Mantel von Siroria   [Cloudrest, Cloudrest, Cloudrest]
[391] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=395	}, --Vestment of Olorime / Gewandung von Olorime   [Cloudrest, Cloudrest, Cloudrest]
[392] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfect Aegis of Galenwe / Perfekte Ägis von Galenwe   [Cloudrest, Cloudrest, Cloudrest]
[393] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfect Arms of Relequen / Perfekte Waffen von Relequen   [Cloudrest, Cloudrest, Cloudrest]
[394] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfect Mantle of Siroria / Perfekter Mantel von Siroria   [Cloudrest, Cloudrest, Cloudrest]
[395] = {wayshrines={364,364,364,}, dlcId=DLC_SUMMERSET	, zoneIds={1051,1051,1051}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfect Vestment of Olorime / Perfekte Gewandung von Olorime   [Cloudrest, Cloudrest, Cloudrest]
[397] = {wayshrines={370,370,370,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1055,108,108}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Balorgh','','','','','',},['de']={'Balorgh','','','','','',},}	}, --Balorgh / Balorgh   [March of Sacrifices, Greenshade, Greenshade]
[398] = {wayshrines={371,371,371,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1052,382,382}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Vykosa','','','','','',},['de']={'Vykosa','','','','','',},}	}, --Vykosa / Vykosa   [Moon Hunter Keep, Reaper's March, Reaper's March]
[399] = {wayshrines={370,370,370,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1055,108,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hanu\'s Compassion / Hanus Mitgefühl   [March of Sacrifices, Greenshade, Greenshade]
[400] = {wayshrines={370,370,370,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1055,108,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Blood Moon / Blutmond   [March of Sacrifices, Greenshade, Greenshade]
[401] = {wayshrines={370,370,370,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1055,108,108}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Haven of Ursus / Zuflucht von Ursus   [March of Sacrifices, Greenshade, Greenshade]
[402] = {wayshrines={371,371,371,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1052,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Moon Hunter / Mondjäger   [Moon Hunter Keep, Reaper's March, Reaper's March]
[403] = {wayshrines={371,371,371,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1052,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Savage Werewolf / Wilder Werwolf   [Moon Hunter Keep, Reaper's March, Reaper's March]
[404] = {wayshrines={371,371,371,}, dlcId=DLC_WOLFHUNTER	, zoneIds={1052,382,382}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Jailer\'s Tenacity / Tenazität des Kerkermeisters   [Moon Hunter Keep, Reaper's March, Reaper's March]
[405] = {wayshrines={374,374,374,}, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Bright-Throat\'s Boast / Hellhalsstolz   [Murkmire, Murkmire, Murkmire]
[406] = {wayshrines={374,374,374,}, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Dead-Water\'s Guile / Totwassertücke   [Murkmire, Murkmire, Murkmire]
[407] = {wayshrines={374,374,374,}, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Champion of the Hist / Histchampion   [Murkmire, Murkmire, Murkmire]
[408] = {wayshrines={375,375,375,}, traitsNeeded=7, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Grave-Stake Collector / Grabpflocksammler   [Murkmire, Murkmire, Murkmire]
[409] = {wayshrines={379,379,379,}, traitsNeeded=2, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Naga Shaman / Nagaschamane   [Murkmire, Murkmire, Murkmire]
[410] = {wayshrines={379,379,379,}, traitsNeeded=4, dlcId=DLC_ELSWEYR	, zoneIds={726,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Might of the Lost Legion / Macht der Verlorenen Legion   [Murkmire, Murkmire, Murkmire]
[411] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=423	}, --Gallant Charge / Galanter Ansturm   [Blackrose Prison, Murkmire, Murkmire]
[412] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=424	}, --Radial Uppercut / Rundum-Aufwärtsschnitt   [Blackrose Prison, Murkmire, Murkmire]
[413] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=425	}, --Spectral Cloak / Spektraler Mantel   [Blackrose Prison, Murkmire, Murkmire]
[414] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=426	}, --Virulent Shot / Virulenter Schuss   [Blackrose Prison, Murkmire, Murkmire]
[415] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=427	}, --Wild Impulse / Wilder Impuls   [Blackrose Prison, Murkmire, Murkmire]
[416] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=428	}, --Mender\'s Ward / Schutz des Pflegers   [Blackrose Prison, Murkmire, Murkmire]
[417] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Indomitable Fury / Unbeugsamer Zorn   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[418] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Spell Strategist / Zauberstratege   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[419] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Battlefield Acrobat / Schlachtfeldakrobat   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[420] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Soldier of Anguish / Soldat der Pein   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[421] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Steadfast Hero / Standhafter Held   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[422] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BASE_GAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Battalion Defender / Bataillonsverteidiger   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[423] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Gallant Charge / Perfekter galanter Ansturm   [Blackrose Prison, Murkmire, Murkmire]
[424] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Radial Uppercut / Perfekter Rundum-Aufwärtsschnitt   [Blackrose Prison, Murkmire, Murkmire]
[425] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Spectral Cloak / Perfekter spektraler Mantel   [Blackrose Prison, Murkmire, Murkmire]
[426] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Virulent Shot / Perfekter virulenter Schuss   [Blackrose Prison, Murkmire, Murkmire]
[427] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Wild Impulse / Perfekter wilder Impuls   [Blackrose Prison, Murkmire, Murkmire]
[428] = {wayshrines={378,378,378,}, dlcId=DLC_BLACKWOOD	, zoneIds={1082,726,726}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfect Mender\'s Ward / Perfekter Schutz des Pflegers   [Blackrose Prison, Murkmire, Murkmire]
[429] = {wayshrines={389,389,389,}, dlcId=DLC_WRATHSTONE	, zoneIds={1080,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Mighty Glacier / Mächtiger Gletscher   [Frostvault, Eastmarch, Eastmarch]
[430] = {wayshrines={389,389,389,}, dlcId=DLC_WRATHSTONE	, zoneIds={1080,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Tzogvin\'s Warband / Tzogvins Kriegstrupp   [Frostvault, Eastmarch, Eastmarch]
[431] = {wayshrines={389,389,389,}, dlcId=DLC_WRATHSTONE	, zoneIds={1080,101,101}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Icy Conjuror / Eisiger Anrufer   [Frostvault, Eastmarch, Eastmarch]
[432] = {wayshrines={389,389,389,}, dlcId=DLC_WRATHSTONE	, zoneIds={1080,101,101}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Stonekeeper','','','','','',},['de']={'Der Steinwahrer','','','','','',},}	}, --Stonekeeper / Steinwahrer   [Frostvault, Eastmarch, Eastmarch]
[433] = {wayshrines={390,390,390,}, dlcId=DLC_WRATHSTONE	, zoneIds={1081,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Frozen Watcher / Gefrorener Beobachter   [Depths of Malatar, Gold Coast, Gold Coast]
[434] = {wayshrines={390,390,390,}, dlcId=DLC_WRATHSTONE	, zoneIds={1081,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Scavenging Demise / Ausbeutender Niedergang   [Depths of Malatar, Gold Coast, Gold Coast]
[435] = {wayshrines={390,390,390,}, dlcId=DLC_WRATHSTONE	, zoneIds={1081,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Auroran\'s Thunder / Donner des Auroraners   [Depths of Malatar, Gold Coast, Gold Coast]
[436] = {wayshrines={390,390,390,}, dlcId=DLC_WRATHSTONE	, zoneIds={1081,823,823}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Symphony of Blades','','','','','',},['de']={'Die Sinfonie der Klingen','','','','','',},}	}, --Symphony of Blades / Sinfonie der Klingen   [Depths of Malatar, Gold Coast, Gold Coast]
[437] = {wayshrines={386,386,386,}, traitsNeeded=8, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Coldharbour\'s Favorite / Kalthafens Günstling   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[438] = {wayshrines={397,397,397,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Senche-raht\'s Grit / Mut des Senche-raht   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[439] = {wayshrines={382,382,382,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Vastarie\'s Tutelage / Vastaries Vormundschaft   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[440] = {wayshrines={382,382,382,}, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Crafty Alfiq / Listiger Alfiq   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[441] = {wayshrines={382,382,382,}, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Vesture of Darloc Brae / Gewandung von Darloc Brae   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[442] = {wayshrines={382,382,382,}, dlcId=DLC_BASE_GAME	, zoneIds={1086,1086,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Call of the Undertaker / Ruf des Totengräbers   [Northern Elsweyr, Northern Elsweyr, Northern Elsweyr, Northern Elsweyr]
[443] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=448	}, --Eye of Nahviintaas / Auge von Nahviintaas   [Sunspire, Sunspire, Sunspire]
[444] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=449	}, --False God\'s Devotion / Ergebenheit des falschen Gottes   [Sunspire, Sunspire, Sunspire]
[445] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=450	}, --Tooth of Lokkestiiz / Zahn von Lokkestiiz   [Sunspire, Sunspire, Sunspire]
[446] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=451	}, --Claw of Yolnakhriin / Kralle von Yolnahkriin   [Sunspire, Sunspire, Sunspire]
[448] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Eye of Nahviintaas / Perfektioniertes Auge von Nahviintaas   [Sunspire, Sunspire, Sunspire]
[449] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected False God\'s Devotion / Perfektionierte Ergebenheit zum falschen Gott   [Sunspire, Sunspire, Sunspire]
[450] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Tooth of Lokkestiiz / Perfektionierter Zahn von Lokkestiiz   [Sunspire, Sunspire, Sunspire]
[451] = {wayshrines={399,399,399,}, dlcId=DLC_ELSWEYR	, zoneIds={1121,1121,1121}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Claw of Yolnakhriin / Perfektionierte Kralle von Yolnahkriin   [Sunspire, Sunspire, Sunspire]
[452] = {wayshrines={391,391,391,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1122,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hollowfang Thirst / Hohlzahndurst   [Moongrave Fane, Northern Elsweyr, Northern Elsweyr]
[453] = {wayshrines={391,391,391,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1122,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Dro\'Zakar\'s Claws / Dro'Zakars Krallen   [Moongrave Fane, Northern Elsweyr, Northern Elsweyr]
[454] = {wayshrines={391,391,391,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1122,1086,1086}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Renald\'s Resolve / Renalds Entschlossenheit   [Moongrave Fane, Northern Elsweyr, Northern Elsweyr]
[455] = {wayshrines={398,398,398,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1123,383,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Z\'en\'s Redress / Z'ens Wiedergutmachung   [Lair of Maarselok, Grahtwood, Grahtwood]
[456] = {wayshrines={398,398,398,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1123,383,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Azureblight Reaper / Azurfäuleschnitter   [Lair of Maarselok, Grahtwood, Grahtwood]
[457] = {wayshrines={398,398,398,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1123,383,383}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Dragon\'s Defilement / Drachenschändung   [Lair of Maarselok, Grahtwood, Grahtwood]
[458] = {wayshrines={391,391,391,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1122,1086,1086}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Grundwulf','','','','','',},['de']={'Grundwulf','','','','','',},}	}, --Grundwulf / Grundwulf   [Moongrave Fane, Northern Elsweyr, Northern Elsweyr]
[459] = {wayshrines={398,398,398,}, dlcId=DLC_SCALEBREAKER	, zoneIds={1123,383,383}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Maarselok','','','','','',},['de']={'Maarselok','','','','','',},}	}, --Maarselok / Maarselok   [Lair of Maarselok, Grahtwood, Grahtwood]
[465] = {wayshrines={402,402,402,}, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Senchal Defender / Verteidiger von Senchal   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[466] = {wayshrines={402,402,402,}, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Marauder\'s Haste / Marodeurshast   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[467] = {wayshrines={402,402,402,}, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Dragonguard Elite / Drachengardeelite   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[468] = {wayshrines={403,403,403,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Daring Corsair / Kühner Korsar   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[469] = {wayshrines={407,407,407,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Ancient Dragonguard / Alte Drachengarde   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[470] = {wayshrines={406,406,406,}, traitsNeeded=9, dlcId=DLC_BASE_GAME	, zoneIds={1133,1133,1133}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --New Moon Acolyte / Akolythin des Neuen Mondes   [Southern Elsweyr, Southern Elsweyr, Southern Elsweyr]
[471] = {wayshrines={424,424,424,}, dlcId=DLC_HARROWSTORM	, zoneIds={1152,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Hiti\'s Hearth / Hitis Feuerstelle   [Icereach, Wrothgar, Wrothgar]
[472] = {wayshrines={424,424,424,}, dlcId=DLC_HARROWSTORM	, zoneIds={1152,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Titanborn Strength / Titanenkinds Stärke   [Icereach, Wrothgar, Wrothgar]
[473] = {wayshrines={424,424,424,}, dlcId=DLC_HARROWSTORM	, zoneIds={1152,684,684}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Bani\'s Torment / Banis Qual   [Icereach, Wrothgar, Wrothgar]
[474] = {wayshrines={425,425,425,}, dlcId=DLC_HARROWSTORM	, zoneIds={1153,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Draugrkin\'s Grip / Griff der Draugrbrut   [Unhallowed Grave, Bangkorai, Bangkorai]
[475] = {wayshrines={425,425,425,}, dlcId=DLC_HARROWSTORM	, zoneIds={1153,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Aegis Caller / Ägisrufer   [Unhallowed Grave, Bangkorai, Bangkorai]
[476] = {wayshrines={425,425,425,}, dlcId=DLC_HARROWSTORM	, zoneIds={1153,92,92}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Grave Guardian  / Grabwächter   [Unhallowed Grave, Bangkorai, Bangkorai]
[478] = {wayshrines={424,424,424,}, dlcId=DLC_HARROWSTORM	, zoneIds={1152,684,684}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Mother Ciannait','','','','','',},['de']={'Mutter Ciannait','','','','','',},}	}, --Mother Ciannait / Mutter Ciannait   [Icereach, Wrothgar, Wrothgar]
[479] = {wayshrines={425,425,425,}, dlcId=DLC_HARROWSTORM	, zoneIds={1153,92,92}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Kjalnar\'s Nightmare','','','','','',},['de']={'Kjalnar\'s Albtraum','','','','','',},}	}, --Kjalnar\'s Nightmare / Kjalnar\'s Albtraum   [Unhallowed Grave, Bangkorai, Bangkorai]
[480] = {wayshrines={-1,-1,-1,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_VLASTARUS}, dropMechanicDropLocationNames={['en']={'','City Vlastarus (quartermaster)','','','','',},['de']={'','Stadt Vlastarus (Quartiermeister)','','','','',},}	}, --Critical Riposte / Kritischer Konter   [Cyrodiil, Cyrodiil]
[481] = {wayshrines={-1,-1,-1,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_BRUMA}, dropMechanicDropLocationNames={['en']={'','City Bruma (quartermaster)','','','','',},['de']={'','Stadt Bruma (Quartiermeister)','','','','',},}	}, --Unchained Aggressor / Entfesselter Aggressor   [Cyrodiil, Cyrodiil]
[482] = {wayshrines={-1,-1,-1,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CROPSFORD}, dropMechanicDropLocationNames={['en']={'','City Cropsford (quartermaster)','','','','',},['de']={'','Stadt Erntefurt (Quartiermeister)','','','','',},}	}, --Dauntless Combatant / Furchtloser Kombattant   [Cyrodiil, Cyrodiil]




[487] = {wayshrines={418,419,421,}, dlcId=DLC_BASE_GAME	, zoneIds={1160,1160,1186,1160}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Winter\'s Respite / Winterruhe   [Western Skyrim, Western Skyrim, Labyrinthian, Western Skyrim]
[488] = {wayshrines={418,419,421,}, dlcId=DLC_BASE_GAME	, zoneIds={1160,1160,1186,1160}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Venomous Smite / Giftschlag   [Western Skyrim, Western Skyrim, Labyrinthian, Western Skyrim]
[489] = {wayshrines={418,419,421,}, dlcId=DLC_BASE_GAME	, zoneIds={1160,1160,1186,1160}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Eternal Vigor / Ewiger Elan   [Western Skyrim, Western Skyrim, Labyrinthian, Western Skyrim]
[490] = {wayshrines={418,418,418,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={1160,1160,1160}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Stuhn\'s Favor / Stuhns Gunst   [Western Skyrim, Western Skyrim, Western Skyrim]
[491] = {wayshrines={417,417,417,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={1160,1160,1160}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Dragon\'s Appetite / Appetit des Drachen   [Western Skyrim, Western Skyrim, Western Skyrim]
[492] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=493	}, --Kyne\'s Wind / Kynes Wind   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[493] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Kyne\'s Wind / Kynes perfektionierter Wind   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[494] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=495	}, --Vrol\'s Command / Vrols Befehl   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[495] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Vrol\'s Command / Vrols perfektionierter Befehl   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[496] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=497	}, --Roaring Opportunist / Brüllender Opportunist   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[497] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Roaring Opportunist / Perfektionierter brüllender Opportunist   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[498] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=499	}, --Yandir\'s Might / Yandirs Macht   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]
[499] = {wayshrines={434,434,434,}, dlcId=DLC_GREYMOOR	, zoneIds={1196,1196,1196}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Yandir\'s Might / Yandirs perfektionierte Macht   [Kyne's Aegis, Kyne's Aegis, Kyne's Aegis]

[501] = {wayshrines={176,360,179,}, dlcId=DLC_GREYMOOR	, zoneIds={381,1027,534,1020,1027}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_FISHING_HOLE,LIBSETS_DROP_MECHANIC_FISHING_HOLE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}, dropMechanicDropLocationNames={['en']={'','Mystic','Saltwater','','Psijic Portals','',},['de']={'','Mystisch','Salzwasser','','Psijic Portale','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Thrassian_Stranglers',},}, --Thrassian Stranglers / Thrassische Würger   [Auridon, Artaeum, Stros M'Kai, Karnwasten, Artaeum]

[503] = {wayshrines={62,374,107,}, dlcId=DLC_GREYMOOR	, zoneIds={3,726,58,931,108,726}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS}, dropMechanicDropLocationNames={['en']={'','','','Canonreeve Oraneth','','',},['de']={'','','','Stiftsvogt Oraneth','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Ring_of_the_Wild_Hunt',},}, --Ring of the Wild Hunt / Ring der Wilden Jagd   [Glenumbra, Murkmire, Malabal Tor, Elden Hollow II, Greenshade, Murkmire]

[505] = {wayshrines={57,26,186,}, dlcId=DLC_GREYMOOR	, zoneIds={104,57,38,216,888}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS}, dropMechanicDropLocationNames={['en']={'','','Captain Blackheart','','','',},['de']={'','','Kapitän Schwarzherz','','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Torc_of_Tonal_Constancy',},}, --Torc of Tonal Constancy / Torques der Klangkonstanz   [Alik'r Desert, Deshaan, Blackheart Haven, Crow's Wood, Craglorn]
[506] = {wayshrines={430,430,430,}, traitsNeeded=3, dlcId=DLC_MARKARTH	, zoneIds={1161,1161,1161}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Spell Parasite / Magieparasit   [Blackreach: Greymoor Caverns, Blackreach: Greymoor Caverns, Blackreach: Greymoor Caverns]




[513] = {wayshrines={436,436,436,}, dlcId=DLC_STONETHORN	, zoneIds={1201,1160,1160}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Talfyg\'s Treachery / Talfygs Verrat   [Castle Thorn, Western Skyrim, Western Skyrim]
[514] = {wayshrines={436,436,436,}, dlcId=DLC_STONETHORN	, zoneIds={1201,1160,1160}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Unleashed Terror / Entfesselter Schrecken   [Castle Thorn, Western Skyrim, Western Skyrim]
[515] = {wayshrines={436,436,436,}, dlcId=DLC_STONETHORN	, zoneIds={1201,1160,1160}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Crimson Twilight / Scharlachrotes Zwielicht   [Castle Thorn, Western Skyrim, Western Skyrim]
[516] = {wayshrines={433,433,433,}, dlcId=DLC_STONETHORN	, zoneIds={1197,0,0}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Elemental Catalyst / Elementarer Katalysator   [Stone Garden, Tamriel Map, Tamriel Map]
[517] = {wayshrines={433,433,433,}, dlcId=DLC_STONETHORN	, zoneIds={1197,0,0}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Kraglen\'s Howl / Kraglens Heulen   [Stone Garden, Tamriel Map, Tamriel Map]
[518] = {wayshrines={433,433,433,}, dlcId=DLC_STONETHORN	, zoneIds={1197,0,0}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Arkasis\'s Genius / Arkasis' Genie   [Stone Garden, Tamriel Map, Tamriel Map]
[519] = {wayshrines={416,172,109,}, dlcId=DLC_GREYMOOR	, zoneIds={1160,280,103,449,181}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE}, dropMechanicDropLocationNames={['en']={'','All mobs','','Drodda of Icereach','Raelynne Ahham [Underpall Cave]','',},['de']={'','Alle Gegner','','Drodda von Eiskap','Raelynne Ahham [SargtuchHöhle]','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Snow_Treaders',},}, --Snow Treaders / Schneetreter   [Western Skyrim, Bleakrock Isle, The Rift, Direfrost Keep, Cyrodiil]
[520] = {wayshrines={241,144,17,}, dlcId=DLC_GREYMOOR	, zoneIds={684,382,142,584,535}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_MOB_TYPE}, dropMechanicDropLocationNames={['en']={'','','','Patrolling horrors in the districts','Any mob','',},['de']={'','','','Patroullierende Bosse in den Bezirken','Alle Gegner','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Malacath%27s_Band_of_Brutality',},}, --Malacath\'s Band of Brutality / Malacaths Band der Brutalität   [Wrothgar, Reaper's March, Bonesnap Ruins, Imperial City, Betnikh]
[521] = {wayshrines={48,133,190,}, dlcId=DLC_GREYMOOR	, zoneIds={117,557,130,144,643}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE}, dropMechanicDropLocationNames={['en']={'','','Illambris twins','The Whisperer','Wandering bosses','',},['de']={'','','Illambris Zwillinge','Der Flüsterer','Herumwandernde Bosse','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Bloodlord%27s_Embrace',},}, --Bloodlord\'s Embrace / Umarmung des Blutfürsten   [Shadowfen, Village of the Lost, Crypt of Hearts I, Spindleclutch I, Imperial Sewers]
[522] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Merciless Charge / Perfektionierter gnadenloser Ansturm   [Maelstrom Arena, Wrothgar, Wrothgar]
[523] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Rampaging Slash / Perfektionierter tobender Schnitt   [Maelstrom Arena, Wrothgar, Wrothgar]
[524] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Cruel Flurry / Perfektionierter grausamer Schlaghagel   [Maelstrom Arena, Wrothgar, Wrothgar]
[525] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Thunderous Volley / Perfektionierte donnernde Salve   [Maelstrom Arena, Wrothgar, Wrothgar]
[526] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Crushing Wall / Perfektionierte zermalmende Mauer   [Maelstrom Arena, Wrothgar, Wrothgar]
[527] = {wayshrines={249,249,249,}, dlcId=DLC_SUMMERSET	, zoneIds={677,684,684}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Precise Regeneration / Perfektionierte präzise Regeneration   [Maelstrom Arena, Wrothgar, Wrothgar]
[528] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Titanic Cleave / Perfektioniertes titanisches Trennen   [Dragonstar Arena, Craglorn, Craglorn]
[529] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Puncturing Remedy / Perfektionierte durchschlagende Genesung   [Dragonstar Arena, Craglorn, Craglorn]
[530] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Stinging Slashes / Perfektionierte stechende Schnitte   [Dragonstar Arena, Craglorn, Craglorn]
[531] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, isProcSetAllowedInPvP=0, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Caustic Arrow / Perfektionierter beißender Pfeil   [Dragonstar Arena, Craglorn, Craglorn]
[532] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Destructive Impact / Perfektionierter zerstörerischer Einschlag   [Dragonstar Arena, Craglorn, Craglorn]
[533] = {wayshrines={270,270,270,}, dlcId=DLC_CLOCKWORK_CITY	, zoneIds={635,888,888}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Grand Rejuvenation / Pefektionierte Große Verjüngung   [Dragonstar Arena, Craglorn, Craglorn]
[534] = {wayshrines={435,435,435,}, dlcId=DLC_STONETHORN	, zoneIds={1197,0,0}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Arkasis the Mad Alchemist','','','','','',},['de']={'Arkasis der verrückte Alchemist','','','','','',},}	}, --Stone Husk / Steinhülle   [Stone Garden, Tamriel Map, Tamriel Map]
[535] = {wayshrines={436,436,436,}, dlcId=DLC_STONETHORN	, zoneIds={1201,1160,1160}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Lady Thorn','','','','','',},['de']={'Fürstin Dorn','','','','','',},}	}, --Lady Thorn / Fürstin Dorn   [Castle Thorn, Western Skyrim, Western Skyrim]
[536] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Radiant Bastion / Strahlende Bastion   [The Reach, The Reach, The Reach]
[537] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Voidcaller / Leerenrufer   [The Reach, The Reach, The Reach]
[538] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Witch-Knight\'s Defiance / Trotz des Hexenritters   [The Reach, The Reach, The Reach]
[539] = {wayshrines={447,447,447,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Red Eagle\'s Fury / Zorn des Roten Adlers   [The Reach, The Reach, The Reach]
[540] = {wayshrines={442,442,442,}, traitsNeeded=6, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Legacy of Karth / Erbe des Karth   [The Reach, The Reach, The Reach]
[541] = {wayshrines={446,446,446,}, traitsNeeded=9, dlcId=DLC_BASE_GAME	, zoneIds={1208,0,0}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Aetherial Ascension / Ätherischer Aufstieg   [Blackreach: Arkthzand Cavern, Tamriel Map, Tamriel Map]
[542] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Hex Siphon / Fluchsiphon   [The Reach, The Reach, The Reach]
[543] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Pestilent Host / Pestilenter Wirt   [The Reach, The Reach, The Reach]
[544] = {wayshrines={441,441,441,}, dlcId=DLC_BASE_GAME	, zoneIds={1207,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Explosive Rebuke / Explosiver Tadel   [The Reach, The Reach, The Reach]
[557] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=563	}, --Executioner\'s Blade / Henkersklinge   [Vateshran Hollows, The Reach, The Reach]
[558] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=564	}, --Void Bash / LeerenWucht   [Vateshran Hollows, The Reach, The Reach]
[559] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=565	}, --Frenzied Momentum / Tobendes Momentum   [Vateshran Hollows, The Reach, The Reach]
[560] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=566	}, --Point-Blank Snipe / Volltreffer   [Vateshran Hollows, The Reach, The Reach]
[561] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=567	}, --Wrath of Elements / Zorn der Elemente   [Vateshran Hollows, The Reach, The Reach]
[562] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=false	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, perfectedSetId=568	}, --Force Overflow / Machtüberschuss   [Vateshran Hollows, The Reach, The Reach]
[563] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Executioner\'s Blade / Perfektionierte Klinge des Henkers   [Vateshran Hollows, The Reach, The Reach]
[564] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Void Bash / Perfektionierte Leerenwucht   [Vateshran Hollows, The Reach, The Reach]
[565] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Frenzied Momentum / Perfektioniertes tobendes Momentum   [Vateshran Hollows, The Reach, The Reach]
[566] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Point-Blank Snipe / Perfektionierter Volltreffer   [Vateshran Hollows, The Reach, The Reach]
[567] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Wrath of Elements / Perfektionierter Zorn der Elemente   [Vateshran Hollows, The Reach, The Reach]
[568] = {wayshrines={457,457,457,}, dlcId=DLC_GREYMOOR	, zoneIds={1227,1207,1207}, veteran=true	, setType=LIBSETS_SETTYPE_ARENA	, dropMechanic={LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST}, isPerfectedSet=1	}, --Perfected Force Overflow / Perfektionierter Machtüberschuss   [Vateshran Hollows, The Reach, The Reach]
[569] = {wayshrines={437,437,437,}, dlcId=DLC_FLAMES_OF_AMBITION	, zoneIds={1228,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --True-Sworn Fury  / Wahrgeschworenen-Raserei   [Black Drake Villa, Gold Coast, Gold Coast]
[570] = {wayshrines={437,437,437,}, dlcId=DLC_FLAMES_OF_AMBITION	, zoneIds={1228,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Kinras\'s Wrath / Kinra's Zorn   [Black Drake Villa, Gold Coast, Gold Coast]
[571] = {wayshrines={437,437,437,}, dlcId=DLC_FLAMES_OF_AMBITION	, zoneIds={1228,823,823}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Drake\'s Rush / Dracheneile   [Black Drake Villa, Gold Coast, Gold Coast]
[572] = {wayshrines={454,454,454,}, dlcId=DLC_BLACKWOOD	, zoneIds={1229,57,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Unleashed Ritualist / Entfesselter Ritualist   [The Cauldron, Deshaan, Deshaan]
[573] = {wayshrines={454,454,454,}, dlcId=DLC_BLACKWOOD	, zoneIds={1229,57,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Dagon\'s Dominion / Dagons Herrschaft   [The Cauldron, Deshaan, Deshaan]
[574] = {wayshrines={454,454,454,}, dlcId=DLC_BLACKWOOD	, zoneIds={1229,57,57}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Foolkiller\'s Ward / Schutz des Narrentöters   [The Cauldron, Deshaan, Deshaan]
[575] = {wayshrines={240,137,62,}, dlcId=DLC_MARKARTH	, zoneIds={684,104,284,1224,92}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}, dropMechanicDropLocationNames={['en']={'','Coldrock Diggings','','East side of the map. Follow the way, down the stairs on the right, read the "Ancient Text" paper on the table','','',},['de']={'','Kaltstein Ausgrabungen','','Im Osten auf der Map. Weg folgen, rechts die Treppe hinunter, auf dem Tisch den Zettel lesen.','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Ring_of_the_Pale_Order',},}, --Ring of the Pale Order / Ring des Fahlen Ordens   [Wrothgar, Alik'r Desert, Bad Man's Hallows, Nighthollow Keep, Bangkorai]
[576] = {wayshrines={176,169,449,}, dlcId=DLC_MARKARTH	, zoneIds={381,124,1207,380,932}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'','','Safeboxes','High Kinlord Rillis','Nerien\'eth','',},['de']={'','','Wertkasetten','Hochwärter Dämmer','Nerien\'eth','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Pearls_of_Ehlnofey',},}, --Pearls of Ehlnofey / Perlen der Ehlnofey   [Auridon, Root Sunder Ruins, The Reach, The Banished Cells I, Crypt of Hearts II]
[577] = {wayshrines={437,437,437,}, dlcId=DLC_FLAMES_OF_AMBITION	, zoneIds={1228,823,823}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Encrati\'s Behemoth','','','','','',},['de']={'Encrati\'s Behemoth','','','','','',},}	}, --Encrati\'s Behemoth / Encrati\'s Behemoth   [Black Drake Villa, Gold Coast, Gold Coast]
[578] = {wayshrines={454,454,454,}, dlcId=DLC_BLACKWOOD	, zoneIds={1229,57,57}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Baron Zaudrus','','','','','',},['de']={'Baron Zaudrus','','','','','',},}	}, --Baron Zaudrus / Baron Zaudrus   [The Cauldron, Deshaan, Deshaan]
[579] = {wayshrines={458,458,458,}, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Frostbite / Frostbiss   [Blackwood, Blackwood, Blackwood, Blackwood]
[580] = {wayshrines={458,458,458,}, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Deadlands Assassin / Totenländer-Assassine   [Blackwood, Blackwood, Blackwood, Blackwood]
[581] = {wayshrines={458,458,458,}, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Bog Raider / Moorräuber   [Blackwood, Blackwood, Blackwood, Blackwood]
[582] = {wayshrines={484,484,484,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Hist Whisperer / Histflüsterer   [Blackwood, Blackwood, Blackwood]
[583] = {wayshrines={483,483,483,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Heartland Conqueror / Herzlanderoberer   [Blackwood, Blackwood, Blackwood]
[584] = {wayshrines={461,461,461,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={1261,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Diamond\'s Victory / Sieg des Diamanten   [Blackwood, Blackwood, Blackwood]
[585] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=589	}, --Saxhleel Champion / Saxhleel-Champion   [Rockgrove, Rockgrove, Rockgrove]
[586] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=590	}, --Sul-Xan\'s Torment / Sul-Xans Folter   [Rockgrove, Rockgrove, Rockgrove]
[587] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=591	}, --Bahsei\'s Mania / Bahseis Manie   [Rockgrove, Rockgrove, Rockgrove]
[588] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=592	}, --Stone-Talker\'s Oath / Eid des Steinsprechers   [Rockgrove, Rockgrove, Rockgrove]
[589] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Saxhleel Champion / Perfektionierter Saxhleel-Champion   [Rockgrove, Rockgrove, Rockgrove]
[590] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Sul-Xan\'s Torment / Perfected Sul-Xan\'s Torment   [Rockgrove, Rockgrove, Rockgrove]
[591] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Bahsei\'s Mania / Bahseis perfektionierte Manie   [Rockgrove, Rockgrove, Rockgrove]
[592] = {wayshrines={468,468,468,}, dlcId=DLC_ELSWEYR	, zoneIds={1263,1263,1263}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Stone-Talker\'s Oath / Perfektionierter Eid des Steinsprechers   [Rockgrove, Rockgrove, Rockgrove]
[593] = {wayshrines={461,255,30,}, dlcId=DLC_BLACKWOOD	, zoneIds={1261,816,306,117,848}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_REWARD_BY_NPC,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'','Remains-Silent, in Outlaws Refuge or the Thieves Den','','','Velidreth','',},['de']={'','Bleibt-Schweigsam, in Verbrecher Verstecken oder in der Diebeshöhle','','','Velidreth','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Gaze_of_Sithis',},}, --Gaze of Sithis / Gaze of Sithis   [Blackwood, Hew's Bane, Forgotten Crypts, Shadowfen, Cradle of Shadows]
[594] = {wayshrines={464,173,374,}, dlcId=DLC_BLACKWOOD	, zoneIds={1261,281,726,843,117}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_FISHING_HOLE,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_HARVEST_NODES}, dropMechanicDropLocationNames={['en']={'','Daggerfall Covenant mobs','Foul','Tree-Minder Na-Kesh','Alchemy: Pure water and water skins','',},['de']={'','Dolchsturz-Bündnis Gegner','Schmutzwasser','Baum-Betreuer Na-Kesh','Alchemie: Reines Wasser und Wasserschläuche','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Harpooner%27s_Wading_Kilt',},}, --Harpooner\'s Wading Kilt / Harpooner\'s Wading Kilt   [Blackwood, Bal Foyen, Murkmire, Ruins of Mazzatun, Shadowfen]
[596] = {wayshrines={461,229,270,}, dlcId=DLC_BLACKWOOD	, zoneIds={1261,888,888,584,181}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_OBLIVION_PORTAL_FINAL_CHEST,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES,LIBSETS_DROP_MECHANIC_ARENA_STAGE_CHEST,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY}, dropMechanicDropLocationNames={['en']={'','','','Patrolling horrors in the districts','','',},['de']={'','','','Patroullierende Bosse in den Bezirken','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Death_Dealer%27s_Fete',},}, --Death Dealer\'s Fete / Death Dealer\'s Fete   [Blackwood, Craglorn, Craglorn, Imperial City, Cyrodiil]
[597] = {wayshrines={451,184,225,}, dlcId=DLC_BLACKWOOD	, zoneIds={1160,11,888,936,1260}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, isProcSetAllowedInPvP=1, dropMechanic={LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON}, dropMechanicDropLocationNames={['en']={'','Mad Architect','Hircine\'s Haunt','Vorenor Winterbourne','','',},['de']={'','Verrückte Architekt','Hircines Schlupfwinkel','Vorenor Wintergeboren','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Shapeshifter%27s_Chain',},}, --Shapeshifter\'s Chain / Shapeshifter\'s Chain   [Western Skyrim, Vaults of Madness, Craglorn, Spindleclutch II, The Silent Halls]
[598] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BLACKWOOD	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Zoal the Ever-Wakeful [Elvengarden district, north-west]','','','','','',},['de']={'Zoal der Immerwache [Elfengarten Bezirk, Nord-Westen]','','','','','',},}	}, --Zoal the Ever-Wakeful / Zoal der Immerwache   [Imperial City, Imperial Sewers]
[599] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BLACKWOOD	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Immolator Charr [Temple district, south]','','','','','',},['de']={'Entflammer Charr [Tempelbezirk, Süden]','','','','','',},}	}, --Immolator Charr / Entflammer Charr   [Imperial City, Imperial Sewers]
[600] = {wayshrines={-1,-1,-1,}, dlcId=DLC_BLACKWOOD	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Glorgoloch the Destroyer [Arena district, north-east]','','','','','',},['de']={'Glorgoloch der Zerstörer [Arenabezirk, Nord-Osten]','','','','','',},}	}, --Glorgoloch the Destroyer / Glorgoloch der Zerstörer   [Imperial City, Imperial Sewers]
[602] = {wayshrines={469,469,469,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1268,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Crimson Oath\'s Rive / Spalt des Karmesinschwurs   [The Dread Cellar, Blackwood, Blackwood]
[603] = {wayshrines={469,469,469,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1268,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Scorion\'s Feast  / Festmahl des Skorions   [The Dread Cellar, Blackwood, Blackwood]
[604] = {wayshrines={469,469,469,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1268,1261,1261}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Rush of Agony / Auftreibende Agonie   [The Dread Cellar, Blackwood, Blackwood]
[605] = {wayshrines={470,470,470,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1267,3,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Silver Rose Vigil / Silberrosen-Wacht   [Red Petal Bastion, Glenumbra, Glenumbra]
[606] = {wayshrines={470,470,470,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1267,3,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Thunder Caller / Donnerrufer   [Red Petal Bastion, Glenumbra, Glenumbra]
[607] = {wayshrines={470,470,470,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1267,3,3}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Grisly Gourmet / Grausiger Gourmet   [Red Petal Bastion, Glenumbra, Glenumbra]
[608] = {wayshrines={470,470,470,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1267,3,3}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Prior Thierric Sarazen','','','','','',},['de']={'Prior Thierric Sarazen','','','','','',},}	}, --Prior Thierric / Prior Thierric   [Red Petal Bastion, Glenumbra, Glenumbra]
[609] = {wayshrines={469,469,469,}, dlcId=DLC_WAKING_FLAME	, zoneIds={1268,1261,1261}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Magma Incarnate','','','','','',},['de']={'Magmaverkörperung','','','','','',},}	}, --Magma Incarnate / Magmaverkörperung   [The Dread Cellar, Blackwood, Blackwood]
[610] = {wayshrines={478,478,478,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1286,1286,1286}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Wretched Vitality / Elendsvitalität   [The Deadlands, The Deadlands, The Deadlands]
[611] = {wayshrines={494,494,494,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={1286,1286,1286}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Deadlands Demolisher / Totenländer-Demolierer   [The Deadlands, The Deadlands, The Deadlands]
[612] = {wayshrines={489,489,489,}, traitsNeeded=5, dlcId=DLC_WRATHSTONE	, zoneIds={1283,1283,1283}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Iron Flask / Eisenfläschchen   [The Shambles, The Shambles, The Shambles]
[613] = {wayshrines={493,493,493,}, dlcId=DLC_BASE_GAME	, zoneIds={1286,1283,1283}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Eye of the Grasp / Griffblick   [The Deadlands, The Shambles, The Shambles]
[614] = {wayshrines={493,493,493,}, dlcId=DLC_BASE_GAME	, zoneIds={1286,1283,1283}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Hexos' Ward / Hexos-Abschirmung   [The Deadlands, The Shambles, The Shambles]
[615] = {wayshrines={493,493,493,}, dlcId=DLC_BASE_GAME	, zoneIds={1286,1283,1283}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST}	}, --Kynmarcher's Cruelty / Grausamkeit der Kynandantin   [The Deadlands, The Shambles, The Shambles]
[616] = {wayshrines={-1,-1,-1,}, dlcId=DLC_WAKING_FLAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Dark Convergence / Dunkle Konvergenz   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[617] = {wayshrines={-1,-1,-1,}, dlcId=DLC_WAKING_FLAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Plaguebreak / Seuchenbruch   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[618] = {wayshrines={-1,-1,-1,}, dlcId=DLC_WAKING_FLAME	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Hrothgar\'s Chill / Hrothgars Kühle   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[619] = {wayshrines={497,497,497,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1301,1011,1011}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Maligalig\'s Maelstrom / Maligaligs Mahlstrom   [Coral Aerie, Summerset, Summerset]
[620] = {wayshrines={497,497,497,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1301,1011,1011}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Gryphon\'s Reprisal / Vergeltung des Greifen   [Coral Aerie, Summerset, Summerset]
[621] = {wayshrines={497,497,497,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1301,1011,1011}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Glacial Guardian / Gletscherwächter   [Coral Aerie, Summerset, Summerset]
[622] = {wayshrines={498,498,498,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1302,20,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Turning Tide / Gezeitenwechsel   [Shipwright's Regret, Rivenspire, Rivenspire]
[623] = {wayshrines={498,498,498,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1302,20,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Storm-Cursed\'s Revenge / Rache des Sturmverfluchten   [Shipwright's Regret, Rivenspire, Rivenspire]
[624] = {wayshrines={498,498,498,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1302,20,20}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Spriggan\’s Vigor / Elan des Zweiglings   [Shipwright's Regret, Rivenspire, Rivenspire]
[625] = {wayshrines={130,491,493,}, dlcId=DLC_DEADLANDS	, zoneIds={347,1286,1283,380,935,1282}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER}, dropMechanicDropLocationNames={['en']={'','Deadrats','Den of the Unmaker','Any mob','Any mob','',},['de']={'','Todesratten','Bau des Entschaffers','Alle Gegner','Alle Gegner','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Markyn_Ring_of_Majesty',},}, --Markyn Ring of Majesty / Markyn-Ring der Erhabenheit   [Coldharbour, The Deadlands, The Shambles, The Banished Cells I, The Banished Cells II, Fargrave]
[626] = {wayshrines={479,253,253,}, dlcId=DLC_DEADLANDS	, zoneIds={1286,823,823,1228,974}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_DUNGEON_CHEST}, dropMechanicDropLocationNames={['en']={'','Limenauruus [Tribunes Folley]','','Any mob','','',},['de']={'','Limenauruus [Die Tribunentorheit]','','Alle Gegner','','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Belharza%27s_Band',},}, --Belharza's Band / Belharzas Band   [The Deadlands, Gold Coast, Gold Coast, Black Drake Villa, Falkreath Hold]
[627] = {wayshrines={479,458,75,}, dlcId=DLC_DEADLANDS	, zoneIds={1286,1261,41,0,0}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_MOB_TYPE}, dropMechanicDropLocationNames={['en']={'The Abomination Cradle','Safeboxes','Shivering Shrine','Clanfear','Dremora','',},['de']={'The Abomination Cradle','Wertkasetten','Schauderschrein','Klanban','Dremora','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Spaulder_of_Ruin',},}, --Spaulder of Ruin / Schulterbänder des Untergangs   [The Deadlands, Blackwood, Stonefalls, Tamriel Map, Tamriel Map]
[629] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Rallying Cry / Schlachtruf   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[630] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Hew and Sunder  / Kappspalter   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[631] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Enervating Aura  / Enervierende Aura   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[632] = {wayshrines={497,497,497,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1301,1011,1011}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Varallion','','','','','',},['de']={'Varallion','','','','','',},}	}, --Kargaeda / Kargaeda   [Coral Aerie, Summerset, Summerset]
[633] = {wayshrines={498,498,498,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={1302,20,20}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Nazaray ','','','','','',},['de']={'Nazaray ','','','','','',},}	}, --Nazaray  / Nazaray    [Shipwright's Regret, Rivenspire, Rivenspire]
[634] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Nunatak [Memorial district, north]','','','','','',},['de']={'Nunatak [Gedenkbezirk, Norden]','','','','','',},}	}, --Nunatak / Nunatak   [Imperial City, Imperial Sewers]
[635] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Lady Malygda [Aboretum, south-east]','','','','','',},['de']={'Fürstin Malydga [Arboretum, Süd-Osten]','','','','','',},}	}, --Lady Malydga / Fürstin Malygda   [Imperial City, Imperial Sewers]
[636] = {wayshrines={-1,-1,-1,}, dlcId=DLC_ASCENDING_TIDE	, zoneIds={584,643}, veteran=false	, setType=LIBSETS_SETTYPE_IMPERIALCITY_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME,LIBSETS_DROP_MECHANIC_TELVAR_EQUIPMENT_LOCKBOX_MERCHANT}, dropMechanicDropLocationNames={['en']={'Baron Thirsk [Nobles district, south-west]','','','','','',},['de']={'Baron Thirsk [Adelsbezirk, Süd-Westen]','','','','','',},}	}, --Baron Thirsk / Baron Thirsk   [Imperial City, Imperial Sewers]
[640] = {wayshrines={503,503,503,}, traitsNeeded=3, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Order\'s Wrath / Zorn des Ordens   [High Isle, High Isle, High Isle]
[641] = {wayshrines={506,506,506,}, traitsNeeded=5, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Serpent\'s Disdain / Verachtung der Schlange   [High Isle, High Isle, High Isle]
[642] = {wayshrines={511,511,511,}, traitsNeeded=7, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Druid\'s Braid / Druidenflechte   [High Isle, High Isle, High Isle]
[643] = {wayshrines={512,512,512,}, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Blessing of High Isle / Segen von Hochinsel   [High Isle, High Isle, High Isle, High Isle, High Isle]
[644] = {wayshrines={512,512,512,}, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Steadfast\'s Mettle / Mut der Standhaften   [High Isle, High Isle, High Isle, High Isle, High Isle]
[645] = {wayshrines={512,512,512,}, dlcId=DLC_BASE_GAME	, zoneIds={1318,1318,1318,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Systres\' Scowl / Blick der Systren   [High Isle, High Isle, High Isle, High Isle, High Isle]
[646] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=653	}, --Whorl of the Depths / Wirbel der Tiefen   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[647] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=652	}, --Coral Riptide / Korallenbrandung   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[648] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=651	}, --Pearlescent Ward / Perlenglanz-Schutz   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[649] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=650	}, --Pillager\'s Profit / Profit des Brandschatzers   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[650] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Pillager\'s Profit / Perfektionierter Profit des Brandschatzers   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[651] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Pearlescent Ward / Perfektionierter Perlenglanz-Schutz   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[652] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Coral Riptide / Perfektionierte Korallenbrandung   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[653] = {wayshrines={505,505,505,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1344,1344,1344}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, --Perfected Whorl of the Depths / Perfektionierter Wirbel der Tiefen   [Dreadsail Reef, Dreadsail Reef, Dreadsail Reef]
[654] = {wayshrines={479,156,479,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1286,487,1286,1286,678}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_PUBLIC_DUNGEON_CHEST   ,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_HARVEST_NODES,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'Watcher','Vile Manse','Watchlings','Clothing','Watcher \'The Gravelight Sentry\'','',},['de']={'Beobachter','Düsteres Herrenhaus','Wächter','Schneider','Beobachter \'Grablichtwache\'','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Mora%27s_Whispers',},}, --Mora\'s Whispers / Moras Flüstern   [The Deadlands, The Vile Manse, The Deadlands, The Deadlands, Imperial City Prison]
[655] = {wayshrines={383,402,-1,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1086,1133,1121,1123,1122}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE}, dropMechanicDropLocationNames={['en']={'Safeboxes','Psijic portal','Dragons','Maarselok','Big Khajiit','',},['de']={'Wertkasetten','Psijic Portal','Drachen','Maarselok','Große Khajiit','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Dov-rha_Sabatons',},}, --Dov-rha Sabatons / Dov-rha-Panzerschuhe   [Northern Elsweyr, Southern Elsweyr, Sunspire, Lair of Maarselok, Moongrave Fane]
[656] = {wayshrines={354,519,43,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1011,1318,104,0,1011}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}, dropMechanicDropLocationNames={['en']={'Lesser Sea Adders','Mornard Falls','','Jewelry crafting writ','Shell Trove','',},['de']={'Kleine Seeotter','Hadolid Matrone','','Schmuck Handwerksschrieb','Muschel Fund','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Dov-rha_Sabatons',},}, --Lefthander\'s Aegis Belt / Ägisriemen der Linkshänder   [Summerset, High Isle, Alik'r Desert, Tamriel Map, Summerset]
[657] = {wayshrines={511,117,502,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1318,103,1318,0,57,383}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_DAIL_QUEST_REWARD_COFFER,LIBSETS_DROP_MECHANIC_DAIL_QUEST_REWARD_COFFER}, dropMechanicDropLocationNames={['en']={'Giant Clam','Frozen Ruins','Serpent Bog','Jewelry survey','Mages guild','Mages guild',},['de']={'Riesenmuschel','Gefrorene Ruine','Schlangensumpf','Schmuck Fundbericht','Magiergilde','Magiergilde',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Sea-Serpent%27s_Coil',},}, --Sea-Serpent\'s Coil / Wicklung der Seeschlange   [High Isle, The Rift, High Isle, Tamriel Map, Deshaan, Grahtwood]
[658] = {wayshrines={518,18,100,}, dlcId=DLC_HIGH_ISLE	, zoneIds={1318,19,58,3,726}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_HARVEST_NODES,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}, dropMechanicDropLocationNames={['en']={'Volcanic Vent','Dreugh Waters','Any monster','Blacksmithing','Safeboxes & Thieves Troves','',},['de']={'Vulkanischer Schlot','Dreugh Gewässer','Jedes Monster','Schmied','Wertkasetten & Diebesgut','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Oakensoul_Ring',},}, --Oakensoul Ring / Eichenseelenring   [High Isle, Stormhaven, Malabal Tor, Glenumbra, Murkmire]
[660] = {wayshrines={520,520,520,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1360,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Deeproot Zeal / Tiefwurz-Eifer   [Earthen Root Enclave, High Isle, High Isle]
[661] = {wayshrines={520,520,520,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1360,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Stone\'s Accord / Steinabkommen   [Earthen Root Enclave, High Isle, High Isle]
[662] = {wayshrines={520,520,520,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1360,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Rage of the Ursauk  / Zorns des Ursauk   [Earthen Root Enclave, High Isle, High Isle]
[663] = {wayshrines={521,521,521,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1361,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Pangrit Denmother / Pangrit-Baumutter   [Graven Deep, High Isle, High Isle]
[664] = {wayshrines={521,521,521,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1361,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Grave Inevitability / Unvermeidlichkeit des Grabs   [Graven Deep, High Isle, High Isle]
[665] = {wayshrines={521,521,521,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1361,1318,1318}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Phylactery\'s Grasp / Phylakteriumsgriff   [Graven Deep, High Isle, High Isle]
[666] = {wayshrines={520,520,520,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1360,1318,1318}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Archdruid Devyric','','','','','',},['de']={'Erzdruide Devyric','','','','','',},}	}, --Archdruid Devyric / Erzdruide Devyric   [Earthen Root Enclave, High Isle, High Isle]
[667] = {wayshrines={521,521,521,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={1361,1318,1318}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Euphotic Gatekeeper','','','','','',},['de']={'Euphotischer Torhüter','','','','','',},}	}, --Euphotic Gatekeeper / Euphotischer Torhüter   [Graven Deep, High Isle, High Isle]
[668] = {wayshrines={-1,-1,-1,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Langour of Peryite / Peryites Entkräftung   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[669] = {wayshrines={-1,-1,-1,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Nocturnal\'s Ploy  / Nocturnals List   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[670] = {wayshrines={-1,-1,-1,}, dlcId=DLC_LOST_DEPTHS	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Mara\’s Balm / Maras Trost   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[671] = {wayshrines={524,524,524,}, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Back-Alley Gourmand / Hinterhof-Gourmet   [Galen, Galen, Galen, Galen]
[672] = {wayshrines={524,524,524,}, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Phoenix Moth Theurge / Phönixmotten-Theurg   [Galen, Galen, Galen, Galen]
[673] = {wayshrines={524,524,524,}, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT}	}, --Bastion of Draoife / Bastion des Draoife   [Galen, Galen, Galen, Galen]
[674] = {wayshrines={529,505,529,}, dlcId=DLC_FIRESONG	, zoneIds={1318,1318,1318,1318,1334,162}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_HARVEST_NODES,LIBSETS_DROP_MECHANIC_MOB_TYPE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON}, dropMechanicDropLocationNames={['en']={'Safeboxes & Thieves Troves','Faun Falls','Wood','Hunter','','All bosses',},['de']={'Wertkasetten & Diebesgut','Faunfälle','Holz','Jäger','','Alle Bosse',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Mythic_Items#Faun.27s_Lark_Cladding',},}, --Faun\'s Lark Cladding / Faunstreich   [High Isle, High Isle, High Isle, High Isle, Whalefall, Obsidian Scar]
[675] = {wayshrines={176,527,527,}, dlcId=DLC_FIRESONG	, zoneIds={486,1383,1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES,LIBSETS_DROP_MECHANIC_HARVEST_NODES}, dropMechanicDropLocationNames={['en']={'Group boss','Valley of the Watcher','Daily delve quest coffer','Volcanic vent','Clothing','',},['de']={'Gruppen Boss','Tal des Wächters','Tägliche Verlies Quest Belehnung','Vulkanische Ventile','Schneider','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Mythic_Items#Stormweaver.27s_Cavort',},}, --Stormweaver\'s Cavort / Sturmwebertollen   [Toothmaul Gully, Galen, Galen, Galen, Galen]
[676] = {wayshrines={529,350,529,}, dlcId=DLC_FIRESONG	, zoneIds={1383,1011,1383,1302,935}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_FISHING_HOLE,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'Inside Blacksmithing Crate from Blacksmithing Writ Dailies','Indric Frolic','Any','Captain Numirril','High Kinlord Rilis','',},['de']={'In Schmiede-Truhen von Schmied Handwerksschrieben','Indriktollerei','Alle','Kapitän Numirril','Sippenhochfürst Rilis','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Mythic_Items#Syrabane.27s_Ward',},}, --Syrabane\'s Ward / Syrabanns Schutz   [Galen, Summerset, Galen, Shipwright's Regret, The Banished Cells II]
[677] = {wayshrines={528,528,528,}, traitsNeeded=3, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Chimera\'s Rebuke / Zurechtweisung der Schimäre   [Galen, Galen, Galen]
[678] = {wayshrines={530,530,530,}, traitsNeeded=3, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Old Growth Brewer / Altwuchs-Brauer   [Galen, Galen, Galen]
[679] = {wayshrines={527,527,527,}, traitsNeeded=3, dlcId=DLC_MORROWIND	, zoneIds={1383,1383,1383}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, --Claw of the Forest Wraith / Kralle des Waldgespensts   [Galen, Galen, Galen]
[680] = {wayshrines={531,531,531,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1389,41,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Ritemaster\'s Bond / Bindung des Ritenmeisters   [Bal Sunnar, Stonefalls, Stonefalls]
[681] = {wayshrines={531,531,531,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1389,41,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Nix-Hound\'s Howl / Nixhund-Heulen   [Bal Sunnar, Stonefalls, Stonefalls]
[682] = {wayshrines={531,531,531,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1389,41,41}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Telvanni Enforcer / Telvanni-Vollstrecker   [Bal Sunnar, Stonefalls, Stonefalls]
[683] = {wayshrines={531,531,531,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1389,41,41}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Roksa the Warped','','','','','',},['de']={'Roksa die Verzerrte','','','','','',},}	}, --Roksa the Warped / Roksa die Verkrümmte   [Bal Sunnar, Stonefalls, Stonefalls]
[684] = {wayshrines={532,532,532,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1390,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Runecarver\’s Blaze / Lodern des Runenschnitzers   [Scrivener's Hall, The Rift, The Rift]
[685] = {wayshrines={532,532,532,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1390,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Apocryphal Inspiration / Apocryphische Inspiration   [Scrivener's Hall, The Rift, The Rift]
[686] = {wayshrines={532,532,532,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1390,103,103}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Abyssal Brace / Abgründiges Wappnen   [Scrivener's Hall, The Rift, The Rift]
[687] = {wayshrines={532,532,532,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1390,103,103}, veteran={[EQUIP_TYPE_HEAD]=true, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_MONSTER, undauntedChestId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Ozezan the Inferno','','','','','',},['de']={'Ozezan das Inferno','','','','','',},}	}, --Ozezan the Inferno / Ozezan das Inferno   [Scrivener's Hall, The Rift, The Rift]
[688] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Snake in the Stars / Schlange inmitten der Sterne   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[689] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Shell Splitter / Panzerspalten   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[690] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Judgement of Akatosh  / Urteil von Akatosh   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[691] = {wayshrines={536,545,538,}, dlcId=DLC_NECROM	, zoneIds={1414,1413,1414,1414,1389}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'','Uzur the All-Seeing [The Disquiet Study]','Corlys the chainmaker [Clamorclap Bowl]','','Endboss','',},['de']={'','Uzur der Allessehende [Beunruhigenden Studien]','Corlys der Kenntenschmied [Zeterklatsch Senke]','','Endgegner','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Cryptcanon_Vestments',},}, -- Cryptcanon Vestments / Gewandung des Kryptenvogts   [Telvanni Peninsula, Apocrypha, Telvanni Peninsula, Telvanni Peninsula, Bal Sunnar]
[692] = {wayshrines={549,280,282,}, dlcId=DLC_NECROM	, zoneIds={1415,849,919,1301,1361}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS,LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}, dropMechanicDropLocationNames={['en']={'','Nchuleft delve Boss','Forgotten Waste, Group boss','Sload, Last secret boss','Side bosses','',},['de']={'','Nchuleft Verlies Boss','Vergessenes Ödland, Gruppen Boss','Sload, letzter geheimer Boss','Nebenbosse','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Esoteric_Environment_Greaves',},}, -- Esoteric Environment Greaves / Beinschienen der esoterischen Umgebung   [Gorne, Vvardenfell, Forgotten Wastes, Coral Aerie, Graven Deep]
[694] = {wayshrines={538,543,546,}, dlcId=DLC_NECROM	, zoneIds={1414,1413,1416,849,1390}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_DAILY_QUEST_REWARD_COFFER,LIBSETS_DROP_MECHANIC_DUNGEON_CHEST}, dropMechanicDropLocationNames={['en']={'Cammonaruhn','Runemaster\'s Acropolis','The Underweave','Daily worldboss','Trophy room','',},['de']={'Cammonaruhn','Akropolis des Runenmeisters','Das Untergewölbe','Täglicher Weltboss','Tröphäenraum','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Velothi_Ur-Mage%27s_Amulet',},}, -- Velothi Ur-Mage\'s Amulet /  Velothi Ur-Mage\'s Amulet   [Telvanni Peninsula, Apocrypha, The Underweave, Vvardenfell, Scrivener's Hall]
[695] = {wayshrines={542,542,542,}, traitsNeeded=5, dlcId=DLC_NECROM	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, -- Shattered Fate /  Shattered Fate   [Apocrypha, Apocrypha, Apocrypha]
[696] = {wayshrines={535,535,535,}, traitsNeeded=3, dlcId=DLC_NECROM	, zoneIds={1414,1414,1414}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, -- Telvanni Efficiency /  Telvanni Efficiency   [Telvanni Peninsula, Telvanni Peninsula, Telvanni Peninsula]
[697] = {wayshrines={546,546,546,}, traitsNeeded=7, dlcId=DLC_NECROM	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CRAFTED	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED}	}, -- Seeker Synthesis /  Seeker Synthesis   [Apocrypha, Apocrypha, Apocrypha]
[698] = {wayshrines={541,540,535,}, dlcId=DLC_NECROM	, zoneIds={1413,1413,1414,1416,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_CRAFTED,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS}	}, -- Vivec\'s Duality /  Vivec\'s Duality   [Apocrypha, Apocrypha, Telvanni Peninsula, The Underweave, Apocrypha, Apocrypha]
[699] = {wayshrines={541,540,535,}, dlcId=DLC_NECROM	, zoneIds={1413,1413,1414,1416,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS}	}, -- Camonna Tong /  Camonna Tong   [Apocrypha, Apocrypha, Telvanni Peninsula, The Underweave, Apocrypha, Apocrypha]
[700] = {wayshrines={541,540,535,}, dlcId=DLC_NECROM	, zoneIds={1413,1413,1414,1416,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_OVERLAND	, dropMechanic={LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_OVERLAND_CHEST,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_PUBLIC_DUNGEON,LIBSETS_DROP_MECHANIC_OVERLAND_LOOT,LIBSETS_DROP_MECHANIC_OVERLAND_WORLDBOSS}	}, -- Adamant Lurker /  Adamant Lurker   [Apocrypha, Apocrypha, Telvanni Peninsula, The Underweave, Apocrypha, Apocrypha]
[701] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=708	}, -- Peace and Serenity /  Peace and Serenity   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[702] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=707	}, -- Ansuul\'s Torment /  Ansuul\'s Torment   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[703] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=706	}, -- Test of Resolve /  Test of Resolve   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[704] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=false	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, perfectedSetId=705	}, -- Transformative Hope /  Transformative Hope   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[705] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, -- Perfected Transformative Hope /  Perfected Transformative Hope   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[706] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, -- Perfected Test of Resolve /  Perfected Test of Resolve   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[707] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, -- Perfected Ansuul\'s Torment /  Perfected Ansuul\'s Torment   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[708] = {wayshrines={534,534,534,}, dlcId=DLC_SCRIBES_OF_FATE	, zoneIds={1427,1427,1427}, veteran=true	, setType=LIBSETS_SETTYPE_TRIAL	, dropMechanic={LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS,LIBSETS_DROP_MECHANIC_TRIAL_BOSS}, isPerfectedSet=1	}, -- Perfected Peace and Serenity /  Perfected Peace and Serenity   [Sanity's Edge, Sanity's Edge, Sanity's Edge]
[711] = {wayshrines={-1,-1,-1,}, dlcId=NO_DLC_UPDATE39	, zoneIds={181,181}, veteran={[EQUIP_TYPE_HEAD]=false, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_CYRODIIL_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT}	}, --Jerall Mountains Warchief / Jerall Mountains Warchief   [Cyrodiil, Cyrodiil]
[712] = {wayshrines={-1,-1,-1,}, dlcId=NO_DLC_UPDATE39	, zoneIds={181,181}, veteran={[EQUIP_TYPE_HEAD]=false, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_CYRODIIL_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT}	}, --Nibenay Bay Battlereeve / Nibenay Bay Battlereeve   [Cyrodiil, Cyrodiil]
[713] = {wayshrines={-1,-1,-1,}, dlcId=NO_DLC_UPDATE39	, zoneIds={181,181}, veteran={[EQUIP_TYPE_HEAD]=false, [EQUIP_TYPE_SHOULDERS]=false}	, setType=LIBSETS_SETTYPE_CYRODIIL_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT}	}, --Colovian Highlands General / Colovian Highlands General   [Cyrodiil, Cyrodiil]
[722] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=117	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Arcanist','','','','','',},['de']={'Arkanist','','','','','',},}	}, --Reawakened Hierophant / Reawakened Hierophant   [Apocrypha, Apocrypha, Apocrypha]
[723] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=1	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Dragonknight','','','','','',},['de']={'Drachenritter','','','','','',},}	}, --Basalt-Blooded Warrior / Basalt-Blooded Warrior   [Apocrypha, Apocrypha, Apocrypha]
[724] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=5	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Necromancer','','','','','',},['de']={'Nekromant','','','','','',},}	}, --Nobility in Decay / Adel im Verfall   [Apocrypha, Apocrypha, Apocrypha]
[726] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=3	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Nightblade','','','','','',},['de']={'Nachtklinge','','','','','',},}	}, --Soulcleaver / Seelenspalter   [Apocrypha, Apocrypha, Apocrypha]
[727] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=2	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Sorcerer','','','','','',},['de']={'Zauberer','','','','','',},}	}, --Monolith of Storms / Monolith der Stürme   [Apocrypha, Apocrypha, Apocrypha]
[728] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=6	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Templar','','','','','',},['de']={'Templer','','','','','',},}	}, --Wrathsun / Wutsonne   [Apocrypha, Apocrypha, Apocrypha]
[729] = {wayshrines={543,543,543,}, dlcId=NO_DLC_SECRET_OF_THE_TELVANNI	, zoneIds={1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_CLASS, classId=4	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Warden','','','','','',},['de']={'Hüter','','','','','',},}	}, --Gardener of Seasons / Gärtner der Jahreszeiten   [Apocrypha, Apocrypha, Apocrypha]
[735] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1471}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Blind Path Induction / Blind Path Induction   [Bedlam Veil]
[736] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1471}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Tarnished Nightmare / Tarnished Nightmare   [Bedlam Veil]
[737] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1471}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Reflected Fury / Reflected Fury   [Bedlam Veil]
[730] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1470}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Cinders of Anthelmir / Cinders of Anthelmir   [Oathsworn Pit]
[731] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1470}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Sluthrug\'s Hunger / Sluthrug\'s Hunger   [Oathsworn Pit]
[732] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1470}, veteran=false	, setType=LIBSETS_SETTYPE_DUNGEON	, dropMechanic={LIBSETS_DROP_MECHANIC_GROUP_DUNGEON_BOSS}	}, --Black-Grove Grounding / Black-Grove Grounding   [Oathsworn Pit]
[738] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1471}, veteran=false	, setType=LIBSETS_SETTYPE_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'The Blind','','','','','',},['de']={'Der Blinde','','','','','',},}	}, --The Blind / Der Blinde   [Bedlam Veil]
[734] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1470}, veteran=false	, setType=LIBSETS_SETTYPE_MONSTER	, dropMechanic={LIBSETS_DROP_MECHANIC_MONSTER_NAME}, dropMechanicDropLocationNames={['en']={'Anthelmirs Konstrukt','','','','','',},['de']={'Anthelmir\'s Construct','','','','','',},}	}, --Anthelmir\'s Construct / Anthelmirs Konstrukt   [Oathsworn Pit]
[693] = {wayshrines={546,546,546,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={1413,1413,1413,1413,1413}, veteran=false	, setType=LIBSETS_SETTYPE_MYTHIC	, dropMechanic={LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE,LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE,LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE,LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE,LIBSETS_DROP_MECHANIC_ENDLESS_ARCHIVE}, dropMechanicDropLocationNames={['en']={'Infinite Archive Chest','Infinite Archive Chest','Infinite Archive Chest','Infinite Archive Chest','Infinite Archive Chest','',},['de']={'Endloses Archiv Truhe','Endloses Archiv Truhe','Endloses Archiv Truhe','Endloses Archiv Truhe','Endloses Archiv Truhe','',},}	, addTooltip={['en']='https://en.uesp.net/wiki/Online:Torc_of_the_Last_Ayleid_King',},}, --Torc of the Last Ayleid King / Torc of the Last Ayleid King   [Apocrypha, Apocrypha, Apocrypha, Apocrypha, Apocrypha]
[754] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Oakfather\'s Retribution / Oakfather\'s Retribution   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[755] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_AP_ELITE_GEAR_LOCKBOX_MERCHANT,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Blunted Blades / Blunted Blades   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
[756] = {wayshrines={-1,-1,-1,}, dlcId=DLC_SCIONS_OF_ITHELIA	, zoneIds={181,181,181,181,181,181}, veteran=false	, setType=LIBSETS_SETTYPE_CYRODIIL	, dropMechanic={LIBSETS_DROP_MECHANIC_MAIL_PVP_REWARDS_FOR_THE_WORTHY,LIBSETS_DROP_MECHANIC_CYRODIIL_BOARD_MISSIONS,LIBSETS_DROP_MECHANIC_CITY_CYRODIIL_CHEYDINHAL_CHORROL_WEYNON_PRIORY,LIBSETS_DROP_MECHANIC_OVERLAND_BOSS_DELVE,LIBSETS_DROP_MECHANIC_DOLMEN_HARROWSTORM_MAGICAL_ANOMALIES}	}, --Baan Dar\'s Blessing / Baan Dar\'s Blessing   [Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil, Cyrodiil]
} --lib.setInfo
--**********************************************************************************************************************

---------------------------------------------------------------------------------------------------------------------------
--This table contains special set items, which got no unique ESO setId. A "fake" unique setId is needed to be manually created, starting at: 9999001!
--The "fake" setId will be only 1 for the whole set. The data table should contain information about the set like wayshrines, zoneIds dlcId and setType value too, like the normal sets in table LibSets.setInfo (see above).
--> This table LibSets.noSetIdSets is basically designed to make the addon compatible with set itemIds for items not having a setId (yet) but being related to a set.
--> Examples could be the master weapons
---> Be sure to provide the values which normaly the ESO API function "GetItemLinkSetInfo" returns (numBonuses, maxEquipped,) inside this table as well!
--The itemIds possible to be used are stored in the table lib.setDataPreloaded[LIBSETS_TABLEKEY_SETNAMES_NO_SETID] below, using the setId as key, a table as value, with the contents = [itemId]=true (just like the table lib.setDataPreloaded[LIBSETS_TABLEKEY_SETITEMIDS] (see below)
--The setNames for the specialSets are stored in the table lib.setDataPreloaded.setNamesNoSetId below using the names given from set information pages for ESO
--**********************************************************************************************************************
lib.noSetIdSets = {
    --Example set 1
    --[9999001] = {wayshrines={-1,-1,-1,}, dlcId=0	, zoneIds={635}, veteran=true, setType=LIBSETS_SETTYPE_ARENA, numBonuses=1, maxEquipped=1},	 --Example set name 1 / Example Set Name 1   [Example set's drop location name]
}
--**********************************************************************************************************************
------------------------------------------------------------------------------------------------------------------------
--One itemId to be used to build itemLink etc. for each setId in the game.
--Preloaded itemIds of all set items
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------


--****************************************************************************************************************************************************************************************
--****************************************************************************************************************************************************************************************
--****************************************************************************************************************************************************************************************
--****************************************************************************************************************************************************************************************
--****************************************************************************************************************************************************************************************
lib.setDataPreloaded = {
    [LIBSETS_TABLEKEY_LASTCHECKEDAPIVERSION] = lib.lastSetsPreloadedCheckAPIVersion,
    ------------------------------------------------------------------------------------------------------------------------
    --......................................................................................................................
    --The list of setItemIdsNoSetId below is predefined and does not change unless the table LibSets.noSetIdSets does not change (see above)
    -->You need to manually add the entries in here matching the new defined nonESOsetId from table LiBSets.noSetIdSets as key and a table of [itemId]=true, lines as value
    [LIBSETS_TABLEKEY_SETITEMIDS_NO_SETID] = {
        --[99990001]={[itemId1]=true,[itemId2]=true},
    },
    ------------------------------------------------------------------------------------------------------------------------
    --......................................................................................................................
    --The list of setNamesNoSetId below is predefined and does not change unless the table LibSets.noSetIdSets does not change (see above)
    -->You need to manually add the entries in here matching the new defined nonESOsetId from table LiBSets.noSetIdSets as key and a table of [language]="Set name in this language String", lines as value
    [LIBSETS_TABLEKEY_SETNAMES_NO_SETID] = {
        --[[
        [9999001] = {
            ["de"] = "Set Name 1",
            ["en"] = "Set name 1",
            ["fr"] = "",
            ["ru"] = "",
            ["jp"] = "",
        },
        ]]
    },
    ------------------------------------------------------------------------------------------------------------------------

    --......................................................................................................................
    --Below you'll find a table with the "compressed" itemids of each set's items.
    -->"Compressed" means there won't be all itemIds explicitly provided if some itemIds follow in a range for the same setId.
    --> If the itemId is a string it's compressed: "109568,58" means starting at itemId 109568, and the next 58 itemIds belong to the same setId
    --> If the itemId is a number it's an explict itemId of the setId
    --The table's key is the [setId] and the value is a subTable containing the compressed itemIds where the key is a non-gap index and the value is the itemId number or
    --itemId_compressed String
    --

    --[[ to generate the following list of ["setItemIds"]:
         --Mandatory 1--
         1* Uncomment the line ## LibSets_Debug.lua in the LibSets.txt file
         2* /reloadui with the latest version
         3* Use /script LibSets.DebugScanAllSetData() to scan all the itemIds for sets and set items.
            This will create a SavedVariables table ["setItemIds"] containing the setIds and the set's itemIds entries (uncompressed)
            and a SavedVariables table ["setItemIds_Compressed"] which is shown below. The table "setItemIds" will be deleted automatically if you do not
            set the booean parameter keepUncompressedSetItemIds as you call the function LibSets.DebugScanAllSetData(keepUncompressedSetItemIds)!
            Attention: This can take about 1-5 minutes and the client will be slightly laggy during this scan period! Watch the chat for the itemId packages scanned.
         4* Copy the entire ["setItemIds_Compressed"] table from the LibSets.lua saved variables
         5* use a lua minifier (e.g. https://mothereff.in/lua-minifier) to remove spaces etc.
            IMPORTANT: Remove the , at the end of the copied table contents and remove the [" and "] around setItemids, in order to let the lua minfier recognize the variable setItemIds properly!
         6* The result should be the compressed table below then starting right of [LIBSETS_TABLEKEY_SETITEMIDS]= -> Remove the "setItemIds =" at the beginning and just copy the { [setId]={[1]=[itemId]=...},....} table containing all setIds and their compressed itemIds, until the closing }

           IMPORTANT:
           Make sure to update the variable "lib.lastSetsPreloadedCheckAPIVersion" in file LibSets_ConstantsLibraryInternal.lua after updating the itemIds table, so one can see when the itemIds were updated last.
    ]]
    [LIBSETS_TABLEKEY_SETITEMIDS]={[19]={[16]="109568,58",[1]=22762,[2]=85693,[3]="29513,3",[4]=109627,[5]=85688,[6]="56891,3",[7]="85633,54",[8]="85689,3",[9]="85694,57",[10]="85761,20",[11]=109628,[12]=22200,[13]=55386,[14]="109459,108",[15]=22205},[20]={[1]=10978,[2]=101597,[3]=101602,[4]=101609,[5]="29517,3",[6]=10973,[7]="101515,81",[8]=10985,[9]=10992,[10]=101616,[11]=101658,[12]="43040,1",[13]="101598,3",[14]="101610,5",[15]="56874,1",[16]=101664,[17]="101603,5",[18]=34842,[19]=34847,[20]="101666,7",[21]=44905,[22]=44909,[23]="101659,3",[24]=55387,[25]="101617,14",[26]=101663,[27]=101665,[28]="101632,25",[29]=101674,[30]=101675,[31]=101676},[21]={[1]=7664,[2]="84542,1",[3]=7697,[4]="84544,19",[5]="84565,6",[6]="84573,65",[7]=7674,[8]="84640,44",[9]=7700,[10]=7724,[11]=7727,[12]=10062,[13]=29521,[14]=29522,[15]=29523,[16]=29524,[17]=34620,[18]=34622,[19]=43035,[20]=43037,[21]=55388,[22]=56895,[23]=56896,[24]=56897,[25]=56898,[26]=56899,[27]=61087,[28]=84527,[29]=84540,[30]=84541,[31]=84564,[32]="84528,11",[33]="84523,3",[34]=84572,[35]=84639},[22]={[1]=2503,[2]=2520,[3]="97954,53",[4]=2552,[5]=6874,[6]=35023,[7]=98008,[8]="98011,28",[9]="98041,5",[10]=98009,[11]=98040,[12]=98010,[13]=98047,[14]="98048,60",[15]=27197,[16]=98109,[17]=4607,[18]="98110,13",[19]=7342,[20]=35027,[21]=42962,[22]=42964},[23]={[1]="55391,1",[2]="87498,38",[3]=87537,[4]="43761,2",[5]=87539,[6]=87538,[7]="87540,11",[8]="87552,39",[9]="87600,11",[10]="87620,19"},[24]={[1]=43764,[2]="43765,1",[3]="55393,1",[4]="55950,2",[5]=55957,[6]="86934,94",[7]=55958,[8]=87088,[9]="87036,3",[10]="87040,7",[11]="87056,27"},[25]={[1]=65167,[2]=65168,[3]="55395,1",[4]=65169,[5]=65170,[6]=65171,[7]=65172,[8]=65173,[9]=65174,[10]="43767,2",[11]="65073,1",[12]="90227,5",[13]="90233,125",[14]=65175,[15]="65097,12",[16]="65132,12",[17]=65178,[18]="90225,1",[19]=65176,[20]=65177,[21]=65185,[22]=65186,[23]=90359,[24]=90360,[25]="90362,5",[26]=90361,[27]=65184,[28]=65179,[29]=65180,[30]=65181,[31]=65182,[32]="90368,26",[33]=65183},[26]={[1]=43770,[2]=43771,[3]=101882,[4]="42981,1",[5]=43772,[6]=101883,[7]=101884,[8]=101989,[9]=101990,[10]=102000,[11]=102002,[12]=102003,[13]=102011,[14]=102014,[15]=102015,[16]=102016,[17]="15730,1",[18]="55397,1",[19]="101880,1",[20]="15742,3",[21]=102017,[22]=102041,[23]=15769,[24]="101991,8",[25]="102004,6",[26]=15777,[27]=55946,[28]=102001,[29]=15728,[30]=42985,[31]="102027,13",[32]=15739,[33]=34530,[34]="102012,1",[35]="101885,2",[36]="101888,100",[37]=102049,[38]=102026,[39]="102018,7",[40]="102042,6"},[27]={[1]=7661,[2]="43773,2",[3]=7680,[4]=10058,[5]=100333,[6]=100349,[7]=100350,[8]=100351,[9]=100352,[10]="100211,121",[11]=7705,[12]="100353,19",[13]=7728,[14]="100334,14",[15]=10066,[16]=29989,[17]=34617,[18]=34621,[19]="55399,1",[20]=43036,[21]=43038},[28]={[1]=16214,[2]=108649,[3]=108650,[4]=108652,[5]="43776,2",[6]="55401,1",[7]=15767,[8]="92447,54",[9]=92502,[10]="92503,105",[11]=22124,[12]=61003,[13]=108651,[14]="108549,69",[15]="108620,28",[16]=28886,[17]="108653,57",[18]=108619},[29]={[1]="43779,2",[2]=22108,[3]=22119,[4]=26166,[5]=16228,[6]="89623,53",[7]="89709,75",[8]="108731,88",[9]=34374,[10]=61005,[11]=89677,[12]=89692,[13]=89703,[14]=89707,[15]=89708,[16]="55403,1",[17]=28121,[18]="89693,9",[19]="89704,2",[20]=29460,[21]="89678,13",[22]=108820,[23]="108821,71"},[30]={[1]=34752,[2]="34755,2",[3]=99973,[4]="43782,2",[5]="55405,1",[6]=54864,[7]=99865,[8]="99838,1",[9]=10885,[10]="99866,106",[11]=15525,[12]=15538,[13]=26137,[14]="99840,24",[15]="99974,25"},[31]={[1]=7390,[2]="34980,2",[3]=7411,[4]=1530,[5]="84705,14",[6]=29936,[7]=43017,[8]=44869,[9]="43786,1",[10]="55407,1",[11]=84720,[12]=84723,[13]="56902,1",[14]=84730,[15]="84721,1",[16]="84724,5",[17]=84805,[18]=56901,[19]=84847,[20]=84848,[21]="84731,4",[22]=7428,[23]="84748,56",[24]="84806,40",[25]=43785,[26]=56900,[27]="84736,9",[28]="84746,1",[29]="84849,25"},[32]={[1]="43788,2",[2]="55953,1",[3]="55959,1",[4]="87116,36",[5]=87277,[6]="55409,1",[7]="87153,1",[8]=87216,[9]="87155,58",[10]="87223,11",[11]="87244,28"},[33]={[1]=28115,[2]=29096,[3]=43792,[4]=108199,[5]=108200,[6]=108201,[7]=108202,[8]=108203,[9]=108204,[10]=108205,[11]=108206,[12]=108207,[13]=108208,[14]=108209,[15]=108210,[16]=108211,[17]=108212,[18]=108213,[19]=108219,[20]=108220,[21]=108221,[22]=108222,[23]=108223,[24]=108224,[25]=108225,[26]=108226,[27]=108227,[28]=108228,[29]=108229,[30]=108230,[31]=108231,[32]=108232,[33]=108233,[34]=108234,[35]=108235,[36]=108236,[37]=108237,[38]=108238,[39]=108239,[40]=108240,[41]=108241,[42]=108242,[43]=108243,[44]=108244,[45]=108245,[46]=108246,[47]=108247,[48]=108248,[49]=108249,[50]=108250,[51]=108251,[52]=108252,[53]=108253,[54]=108259,[55]=108260,[56]=108261,[57]=108262,[58]=108263,[59]=108264,[60]=108265,[61]=108266,[62]=108267,[63]=108268,[64]=108269,[65]=108270,[66]=108271,[67]="55412,1",[68]=108272,[69]=108273,[70]=43791,[71]=43793,[72]="56876,3",[73]=81168,[74]=81176,[75]=108311,[76]=108312,[77]=108313,[78]=108319,[79]=108320,[80]=108321,[81]=108322,[82]=108323,[83]=108324,[84]=108325,[85]=108326,[86]=108327,[87]=108328,[88]=108329,[89]=108330,[90]=108331,[91]=108332,[92]=108333,[93]=108339,[94]=108340,[95]=108341,[96]=108342,[97]=108343,[98]=108344,[99]=108345,[100]=108346,[101]="80640,3",[102]="80655,1",[103]="80658,5",[104]="80665,35",[105]=10961,[106]="80549,90",[107]=32175,[108]="80644,10",[109]="108185,8",[110]="108288,3",[111]=29088,[112]=32168,[113]=80657,[114]=80664,[115]=108279,[116]=108280,[117]=108281,[118]=108282,[119]=108283,[120]=108284,[121]=108285,[122]=108286,[123]=108287,[124]=108292,[125]=108293,[126]=108299,[127]=108300,[128]=108301,[129]=108302,[130]=108303,[131]=108304,[132]=108305,[133]=108306,[134]=108307,[135]=108308,[136]=108309,[137]=108310},[34]={[1]="96970,14",[2]=4289,[3]=4317,[4]="75134,40",[5]=4315,[6]="75063,62",[7]="75128,4",[8]=7335,[9]=10365,[10]=96988,[11]="34575,1",[12]="43794,2",[13]="55414,1",[14]="75176,24",[15]=96989,[16]="75202,22",[17]="96985,1",[18]=7326,[19]=7330,[20]=43024,[21]=75126,[22]=75127,[23]=75133,[24]=75175,[25]=75201,[26]=96987,[27]=43023,[28]="96990,33",[29]="97024,3"},[35]={[1]=43799,[2]=80406,[3]=108053,[4]=108055,[5]=108080,[6]=108081,[7]=108082,[8]=108107,[9]="29065,1",[10]=108108,[11]=108109,[12]=108110,[13]=108111,[14]=108117,[15]=108118,[16]=108119,[17]=108120,[18]=108121,[19]=108122,[20]=108123,[21]=108124,[22]=108125,[23]=108126,[24]=108127,[25]=108128,[26]="32181,1",[27]="43797,1",[28]="55416,1",[29]="56880,2",[30]="80397,7",[31]="80408,15",[32]="80424,4",[33]=108129,[34]="80429,2",[35]="80432,2",[36]="80435,8",[37]="80459,89",[38]=108138,[39]="80444,4",[40]="80449,9",[41]="108003,8",[42]=108130,[43]=80405,[44]=80407,[45]=81167,[46]=81175,[47]="108037,15",[48]=108054,[49]=108131,[50]=108151,[51]=108158,[52]="108017,14",[53]=108159,[54]=108160,[55]=108161,[56]=108162,[57]=108163,[58]=108164,[59]=29075,[60]=108137,[61]=108139,[62]=108140,[63]=108141,[64]=108142,[65]=108143,[66]=108147,[67]=108148,[68]=108149,[69]="108056,15",[70]=108144,[71]=108145,[72]=108146,[73]="108077,2",[74]=108150,[75]=108157,[76]="108083,8",[77]="108097,9"},[36]={[1]="7647,1",[2]=7599,[3]=7608,[4]="42988,1",[5]="43800,2",[6]=1373,[7]=7626,[8]=34437,[9]=42991,[10]=94330,[11]="94254,75",[12]="94332,50",[13]=94331,[14]="94384,7",[15]=94392,[16]="55418,1",[17]="94393,16",[18]=94383,[19]=94410,[20]="94411,4"},[37]={[1]=43803,[2]=43804,[3]=54629,[4]=54630,[5]=54631,[6]=54632,[7]=56103,[8]=56104,[9]=56105,[10]=56106,[11]=56107,[12]=56108,[13]=56109,[14]=56110,[15]=56111,[16]=56112,[17]=56113,[18]=56114,[19]=56115,[20]=56116,[21]=56117,[22]=56118,[23]=56119,[24]=56120,[25]=56121,[26]=56122,[27]=56123,[28]=56124,[29]=56125,[30]=56126,[31]=56127,[32]=56128,[33]=56129,[34]=56130,[35]=56131,[36]=56132,[37]=56133,[38]=56134,[39]=56135,[40]=56136,[41]=56137,[42]=137515,[43]=137557,[44]=137599,[45]=137641,[46]=137683,[47]=137725,[48]=137767,[49]=137809,[50]=139676,[51]=139718,[52]=139766,[53]=139808,[54]="43805,1",[55]=140036,[56]=140126,[57]=140168,[58]=139898,[59]="46153,352",[60]="46527,4",[61]=139856,[62]=139946,[63]="46507,18",[64]=139988,[65]=140078},[38]={[1]=46879,[2]=46880,[3]=46881,[4]=46882,[5]=137516,[6]=137558,[7]=137684,[8]=137726,[9]=137768,[10]=139719,[11]=139767,[12]=139809,[13]=139857,[14]=139899,[15]=139947,[16]="43807,3",[17]=139989,[18]=140037,[19]=46684,[20]=46707,[21]="46532,59",[22]="46592,81",[23]=137600,[24]=137810,[25]="46689,7",[26]=139677,[27]=140127,[28]=140079,[29]="46712,166",[30]=137642,[31]="46890,18",[32]="56138,34",[33]=140169,[34]="46883,5",[35]="46910,4"},[39]={[1]=88099,[2]=88100,[3]=88101,[4]=88102,[5]=88111,[6]=88112,[7]=88113,[8]=88114,[9]=88123,[10]=88124,[11]=88125,[12]=88126,[13]=88127,[14]=88128,[15]=88129,[16]=88130,[17]="43811,3",[18]="43823,3",[19]="43835,3",[20]=88131,[21]="55420,2",[22]="66111,41",[23]=88134,[24]=88132,[25]=88141,[26]="87985,78",[27]=88142,[28]=88143,[29]=88144,[30]=88145,[31]=88146,[32]="88064,34",[33]=88133,[34]="88103,7",[35]=88135,[36]=88136,[37]=88137,[38]=88138,[39]="88115,7",[40]=88139,[41]=88140},[40]={[1]=43818,[2]=56179,[3]=56180,[4]=56181,[5]=56182,[6]=56183,[7]="43815,1",[8]=56173,[9]=56174,[10]=56175,[11]=56176,[12]=56177,[13]=56178,[14]=56184,[15]=56185,[16]=56186,[17]=56187,[18]=56188,[19]=56189,[20]=56190,[21]=56192,[22]=56193,[23]=56195,[24]=56196,[25]=56197,[26]=56198,[27]=56199,[28]=56200,[29]=137517,[30]=140080,[31]=56191,[32]=137769,[33]=137811,[34]=139678,[35]=139720,[36]=139768,[37]=139810,[38]=139858,[39]=139900,[40]=139948,[41]=139990,[42]=140038,[43]=140128,[44]=140170,[45]="46915,188",[46]=56194,[47]=56201,[48]=56205,[49]="47272,25",[50]=56202,[51]=56203,[52]=56204,[53]=56206,[54]=56207,[55]=137559,[56]=137601,[57]=137643,[58]=137685,[59]=137727,[60]="47104,166"},[41]={[1]=47403,[2]=47404,[3]=47405,[4]=47406,[5]=56208,[6]=56209,[7]=56210,[8]=56211,[9]=56212,[10]=56213,[11]=56214,[12]=56215,[13]=56216,[14]=56217,[15]=56218,[16]=56219,[17]=56220,[18]=56221,[19]=56222,[20]=56223,[21]=56225,[22]=56226,[23]=56227,[24]=56228,[25]=56229,[26]=56230,[27]=56231,[28]=56232,[29]="43819,3",[30]="47298,104",[31]=56233,[32]=139949,[33]="47407,208",[34]=56224,[35]=56234,[36]=139679,[37]=139721,[38]=139769,[39]=139811,[40]=139991,[41]="47616,38",[42]=137812,[43]=139859,[44]=139901,[45]=140039,[46]=140081,[47]=140129,[48]=140171,[49]=56235,[50]=56238,[51]=56239,[52]=56236,[53]="47656,18",[54]=56237,[55]=56240,[56]=56241,[57]=56242,[58]=137518,[59]=137560,[60]=137602,[61]=137644,[62]=137686,[63]="47676,4",[64]=137728,[65]=137770},[43]={[1]=56243,[2]=56245,[3]=56246,[4]=56247,[5]=56248,[6]=56249,[7]=56251,[8]=56252,[9]=56253,[10]=56254,[11]=56255,[12]=137519,[13]=137561,[14]=137603,[15]=137645,[16]=137729,[17]=137813,[18]=139680,[19]=139770,[20]=56250,[21]="43827,3",[22]=137687,[23]=137771,[24]=139812,[25]=140130,[26]=140172,[27]=140082,[28]=139722,[29]="47681,356",[30]="48039,18",[31]=56244,[32]=139860,[33]=139902,[34]=139950,[35]=139992,[36]="48059,4",[37]="56256,21",[38]=140040},[44]={[1]=48439,[2]=48440,[3]=48442,[4]=56278,[5]=56279,[6]=56280,[7]=56281,[8]=56282,[9]=56283,[10]=56284,[11]=56285,[12]=56286,[13]=56287,[14]=56288,[15]=56289,[16]=56290,[17]=56291,[18]=56293,[19]=56294,[20]=56295,[21]=56296,[22]=56297,[23]=56298,[24]=56299,[25]=56300,[26]=56301,[27]=56302,[28]=56303,[29]=56304,[30]="43831,3",[31]=137646,[32]=139681,[33]=139723,[34]=139771,[35]=139813,[36]=139861,[37]=139903,[38]=139993,[39]=140173,[40]=137562,[41]=137604,[42]=140131,[43]="48064,63",[44]=139951,[45]=140041,[46]=140083,[47]=56305,[48]=56292,[49]=56306,[50]=56310,[51]="48422,16",[52]=56307,[53]=56308,[54]=56309,[55]=56311,[56]=56312,[57]=137520,[58]=137688,[59]=137730,[60]=137772,[61]="48443,3",[62]=137814,[63]="48128,292"},[46]={[16]="109832,169",[1]=22166,[2]=23663,[3]=23670,[4]=55423,[5]=91213,[6]=34381,[7]=91281,[8]="91180,32",[9]=22161,[10]=43842,[11]="43839,1",[12]="91214,66",[13]="91282,3",[14]=91286,[15]="91287,62"},[47]={[1]=42950,[2]=42954,[3]=42955,[4]=86217,[5]="86188,25",[6]=34394,[7]=42953,[8]=86214,[9]=86339,[10]="43843,3",[11]="56904,1",[12]=86340,[13]="86215,1",[14]=86342,[15]=86341,[16]=7514,[17]=7524,[18]=7547,[19]=34396,[20]=86343,[21]="86218,53",[22]=55424,[23]=86344,[24]="86272,66",[25]=26518,[26]="86346,11",[27]=86345},[48]={[1]=56313,[2]=56314,[3]=56315,[4]=56316,[5]=56317,[6]=56318,[7]=56320,[8]=56321,[9]=56323,[10]=56324,[11]=56325,[12]=56326,[13]=56327,[14]=56328,[15]=56329,[16]=56330,[17]=56331,[18]=56332,[19]=56333,[20]=56334,[21]=56335,[22]=56336,[23]=56337,[24]=56338,[25]=56339,[26]=56340,[27]="43847,3",[28]=56341,[29]=137521,[30]=140084,[31]=137815,[32]=139682,[33]=139724,[34]=139772,[35]=139814,[36]=139862,[37]=140042,[38]=139994,[39]=140132,[40]=140174,[41]="48447,192",[42]=56319,[43]=139904,[44]=139952,[45]="48805,18",[46]=56322,[47]=56342,[48]=56343,[49]=56344,[50]=56345,[51]=56346,[52]=56347,[53]=137563,[54]=137605,[55]=137647,[56]=137689,[57]=137731,[58]="48825,4",[59]=137773,[60]="48640,163"},[49]={[1]=7385,[2]=7410,[3]=29939,[4]="43020,1",[5]=7416,[6]="84896,56",[7]="84954,21",[8]=7408,[9]=84953,[10]="43851,3",[11]="84980,3",[12]=84977,[13]=84978,[14]=70,[15]=34978,[16]=43018,[17]=55425,[18]=56906,[19]=56907,[20]=84976,[21]="84985,6",[22]="84992,65",[23]=84979,[24]=84984},[50]={[1]="43855,3",[2]=55426,[3]=57488,[4]="93357,82",[5]="93440,78"},[51]={[1]=43859,[2]=43860,[3]=43861,[4]=56348,[5]=56349,[6]=56350,[7]=56351,[8]=56352,[9]=56353,[10]=56354,[11]=56356,[12]=56357,[13]=56359,[14]=56360,[15]=56361,[16]=56362,[17]=56363,[18]=56364,[19]=56365,[20]=56366,[21]=56367,[22]=56368,[23]=56369,[24]=56370,[25]=56371,[26]=56372,[27]=56373,[28]=56374,[29]=56375,[30]=139953,[31]=56358,[32]=139725,[33]=139773,[34]=139815,[35]=139995,[36]=140043,[37]=140085,[38]=56355,[39]=137816,[40]=139863,[41]="48830,321",[42]=139905,[43]=140133,[44]=140175,[45]=56376,[46]=56377,[47]=56378,[48]=56379,[49]=56380,[50]=137522,[51]="49188,24",[52]=137564,[53]=137606,[54]=137648,[55]=137690,[56]=137732,[57]=137774,[58]=139683,[59]="49152,34",[60]="56381,1"},[52]={[1]=55427,[2]=65187,[3]=65188,[4]=65189,[5]=65190,[6]=65191,[7]=65192,[8]=65193,[9]=65194,[10]=65195,[11]=65196,[12]=65197,[13]=65198,[14]=65199,[15]="43863,3",[16]="65224,4",[17]=65200,[18]="88713,62",[19]=65202,[20]=88776,[21]=88777,[22]=88778,[23]=88779,[24]=88780,[25]=65201,[26]=65213,[27]=65214,[28]=65215,[29]=65216,[30]=65217,[31]=65218,[32]=65219,[33]=65220,[34]=65221,[35]=65222,[36]=65223,[37]="88781,50",[38]=65203,[39]=65204,[40]=65205,[41]=65206,[42]=65207,[43]=65208,[44]=65209,[45]=65210,[46]=65211,[47]=65212,[48]="88832,42"},[53]={[1]="56908,1",[2]=33159,[3]=33170,[4]=86520,[5]="43867,3",[6]="86379,27",[7]="86419,73",[8]=55428,[9]=86407,[10]="86501,18",[11]="109407,11",[12]=33272,[13]=86418,[14]="109277,125",[15]=109403,[16]=109404,[17]=109405,[18]=109406,[19]=109419,[20]=109420,[21]=109421,[22]=109422,[23]=109423,[24]=109424,[25]=109425,[26]=109426,[27]=109427,[28]=109428,[29]=109429,[30]=109430,[31]=109431,[32]=109432,[33]=109433,[34]=109434,[35]=109437,[36]=109438,[37]=109436,[38]="86408,9",[39]=109435},[54]={[1]=49503,[2]=49504,[3]=49505,[4]=56383,[5]=56384,[6]=56385,[7]=56386,[8]=56387,[9]=56388,[10]=56389,[11]=56390,[12]=56391,[13]=56392,[14]=56393,[15]=56394,[16]=56395,[17]=56396,[18]=56397,[19]=56398,[20]=56399,[21]=56400,[22]=56401,[23]=56402,[24]=56403,[25]=56404,[26]=56405,[27]=56406,[28]=56407,[29]=56408,[30]=56409,[31]=56410,[32]="49213,289",[33]=56411,[34]="43871,2",[35]=137607,[36]=137649,[37]="49506,63",[38]=56416,[39]=137565,[40]=56412,[41]=56413,[42]=56414,[43]=56415,[44]=56417,[45]=137691,[46]="49571,24",[47]=137523,[48]=137733,[49]=137775,[50]=137817,[51]=139684,[52]=139726,[53]=139774,[54]=139816,[55]=139864,[56]=139906,[57]=139954,[58]=139996,[59]=140044,[60]=140086,[61]=140134,[62]=140176},[55]={[1]=10067,[2]=28116,[3]=28884,[4]=29072,[5]=29076,[6]=29087,[7]=90451,[8]=90512,[9]=90516,[10]=90527,[11]=90580,[12]=107851,[13]=107852,[14]=107853,[15]=107854,[16]=107855,[17]=107856,[18]=107857,[19]=107858,[20]=107859,[21]=107860,[22]=107861,[23]=107862,[24]=107863,[25]=107864,[26]=107865,[27]=107866,[28]=107867,[29]="90416,26",[30]="90467,1",[31]="90581,4",[32]=107880,[33]="43877,2",[34]=107887,[35]="90528,51",[36]=107888,[37]=107889,[38]=107890,[39]=107891,[40]=107893,[41]="90443,7",[42]=107892,[43]="90452,14",[44]="90469,42",[45]="90513,2",[46]=107894,[47]="43875,1",[48]=107897,[49]=107895,[50]=107896,[51]=107898,[52]=107899,[53]=107900,[54]=107903,[55]=107904,[56]="90517,9",[57]=107901,[58]=107902,[59]=107886,[60]=107868,[61]=107869,[62]=107870,[63]=107871,[64]=107872,[65]=107873,[66]=107874,[67]=107875,[68]=107876,[69]=107877,[70]=107878,[71]=107879,[72]=107881,[73]=107882,[74]=107883,[75]=107884,[76]="107812,38",[77]=107885,[78]=107905,[79]=107906,[80]=107907,[81]=107908,[82]=107909,[83]=107910,[84]=107911,[85]=107912,[86]=107913,[87]=107914,[88]=107915,[89]=107916,[90]=107917,[91]=107918,[92]=107919,[93]=107920,[94]=107921,[95]=107922,[96]=107923,[97]=107924,[98]=107925,[99]=107926,[100]=107927,[101]=107928,[102]=107929,[103]=107930,[104]=107931,[105]=107932,[106]=107933,[107]=107934,[108]=107935,[109]=107936,[110]=107937,[111]=107938,[112]=107939,[113]=107940,[114]=107941,[115]=107942,[116]=107943,[117]=107944,[118]=107945,[119]=107946,[120]=107947,[121]=107948,[122]=107949,[123]=107950,[124]=107951,[125]=107952,[126]=107953,[127]=107954,[128]=107955,[129]=107956,[130]=107957,[131]=107958,[132]=107959,[133]=107960,[134]=107961,[135]=107962,[136]=107963,[137]=107964,[138]=107965,[139]=107966,[140]=107967,[141]=107968,[142]=107969,[143]=107970,[144]=107971,[145]=107972,[146]=107973,[147]=107974,[148]=107975,[149]=107976,[150]=107977,[151]=107978,[152]=107979,[153]=107980,[154]=107981},[56]={[1]="43880,4",[2]=7668,[3]="100020,63",[4]=43034,[5]=100084,[6]="34618,1",[7]=43039,[8]=7709,[9]=100155,[10]="100085,10",[11]="100156,33",[12]=100154,[13]="100096,57"},[57]={[1]="43885,4",[2]="56910,3",[3]="85095,3",[4]=1788,[5]="85106,127",[6]=10273,[7]=34662,[8]=34667,[9]="85237,6",[10]=42994,[11]=42996,[12]=85235,[13]="85078,15",[14]=85100,[15]=85234,[16]=85101,[17]=85102,[18]=85103,[19]=85104,[20]=85105,[21]=85244,[22]="85245,2",[23]=1672,[24]=1715,[25]=1730,[26]=85094,[27]=85099,[28]=85236},[58]={[1]=1108,[2]=29944,[3]=34296,[4]=34298,[5]=59712,[6]=86008,[7]=86009,[8]=86010,[9]=86080,[10]=86121,[11]=86130,[12]=86131,[13]="43890,4",[14]="86006,1",[15]=86132,[16]=86133,[17]=1129,[18]=1160,[19]="86011,4",[20]=1088,[21]="86016,63",[22]=7359,[23]="86081,39",[24]="86122,7",[25]="86135,16",[26]=86134,[27]=86152,[28]=42956,[29]=42958,[30]="86153,14",[31]="86173,1"},[59]={[1]="93545,1",[2]="93548,4",[3]="93554,6",[4]="93568,34",[5]=43895,[6]="93539,1",[7]=43896,[8]="65801,41",[9]="93542,1",[10]=93563,[11]="93607,5",[12]=93561,[13]="93614,26",[14]="93645,7",[15]="93604,1",[16]="93666,16",[17]=93684,[18]="93654,10",[19]=43897,[20]="93687,8",[21]=43898,[22]=93562,[23]=43899,[24]=93642,[25]="93696,4",[26]=93565},[60]={[1]="7295,1",[2]=74824,[3]="7299,1",[4]="42968,1",[5]="43900,2",[6]="43903,1",[7]="56914,1",[8]=74826,[9]=85887,[10]=85888,[11]="74731,20",[12]=26579,[13]=34890,[14]=74825,[15]="74752,71",[16]="74834,1",[17]=34888,[18]="74827,6",[19]="74836,56",[20]="85824,62",[21]="85889,48",[22]=7311,[23]="85946,19"},[61]={[1]=29067,[2]=29086,[3]=29095,[4]=32164,[5]=32176,[6]=43905,[7]=43906,[8]=43907,[9]=43908,[10]=43909,[11]=80947,[12]=80948,[13]=80949,[14]=80950,[15]=108455,[16]=108464,[17]=108508,[18]=108509,[19]=108510,[20]="56883,3",[21]=108511,[22]="80860,35",[23]=108512,[24]=139,[25]=81170,[26]=81178,[27]=108513,[28]="80896,50",[29]=108523,[30]=108525,[31]=108526,[32]=108527,[33]=108528,[34]="80951,60",[35]=108514,[36]=108515,[37]="108465,42",[38]=108516,[39]=108517,[40]=108518,[41]=108519,[42]=108520,[43]=108521,[44]="108456,7",[45]=108522,[46]="108367,87",[47]=108524},[62]={[1]=99178,[2]=99189,[3]=99206,[4]=99207,[5]=99208,[6]="43910,4",[7]=99209,[8]=99210,[9]="99064,7",[10]=7471,[11]=10314,[12]=22750,[13]=99088,[14]=99092,[15]=99095,[16]=99097,[17]=99110,[18]=43028,[19]=43031,[20]=99096,[21]="99072,15",[22]="99089,2",[23]=7440,[24]="99093,1",[25]=43033,[26]=10346,[27]="99098,11",[28]=34933,[29]=7462,[30]="99111,66",[31]="99179,9",[32]="99190,15",[33]="99211,14"},[63]={[1]=43915,[2]=43916,[3]=43917,[4]=43918,[5]="65759,32",[6]=43919,[7]="65792,8",[8]="88895,159",[9]=89055,[10]=89056},[64]={[1]="10882,1",[2]="43923,1",[3]=10889,[4]="74893,2",[5]="74896,1",[6]="56887,1",[7]=10896,[8]=10898,[9]=10901,[10]=43921,[11]="10878,1",[12]="43000,1",[13]=74898,[14]="74902,97",[15]=74899,[16]=34707,[17]=34708,[18]=43922,[19]="75002,5",[20]=74900,[21]=43920,[22]=74901,[23]=75000,[24]="75008,54",[25]=75001},[65]={[1]=471,[2]="43925,2",[3]="96396,8",[4]="96408,54",[5]=34299,[6]="96463,1",[7]="96466,4",[8]="42959,2",[9]="29942,1",[10]="96504,2",[11]=96465,[12]=96502,[13]="96472,29",[14]=96503,[15]="96508,3",[16]=1092,[17]=1131,[18]=1133,[19]=43928,[20]=59945,[21]="96554,11",[22]=43929,[23]=96405,[24]=96406,[25]=96407,[26]=96471,[27]=96507,[28]=96553,[29]="96512,40"},[66]={[1]=34932,[2]=34934,[3]=34936,[4]=34937,[5]=43930,[6]=43931,[7]=43932,[8]=43933,[9]=43934,[10]=90667,[11]=98799,[12]=98800,[13]=98801,[14]=98802,[15]=98803,[16]=98804,[17]=98805,[18]=98806,[19]=98807,[20]=98808,[21]=98809,[22]=98810,[23]=98811,[24]=98812,[25]=98813,[26]=98814,[27]=98815,[28]=98816,[29]=98817,[30]=98818,[31]=98819,[32]=98820,[33]=98821,[34]=98822,[35]=98823,[36]=98824,[37]=98825,[38]=98826,[39]=98827,[40]="90607,16",[41]=98828,[42]=98848,[43]=98849,[44]=98850,[45]=98851,[46]=98852,[47]=98853,[48]=98854,[49]=98855,[50]="90624,42",[51]=7445,[52]=10283,[53]=22753,[54]=22757,[55]=98856,[56]=98857,[57]=98858,[58]=98859,[59]=98860,[60]=98829,[61]=98833,[62]=98834,[63]=98835,[64]=98830,[65]=98831,[66]="98691,107",[67]=98832,[68]=98836,[69]=98837,[70]=98838,[71]=98839,[72]=98840,[73]=98841,[74]=98842,[75]=98843,[76]=98844,[77]=98845,[78]=98846,[79]="90668,108",[80]=98847},[67]={[1]="43936,3",[2]=43935,[3]=92831,[4]="92811,19",[5]="92832,95",[6]="92928,44"},[68]={[1]=34528,[2]=102115,[3]="15772,1",[4]=34529,[5]="43940,4",[6]=34527,[7]=34531,[8]=102113,[9]=102114,[10]="15726,1",[11]="56916,2",[12]=102228,[13]="102071,41",[14]=102229,[15]=10861,[16]=15738,[17]=15740,[18]=15746,[19]="102231,1",[20]=15766,[21]=15770,[22]="102116,27",[23]=102230,[24]="102144,83"},[69]={[1]="98155,18",[2]=1662,[3]=1664,[4]="34665,1",[5]="98145,7",[6]=42992,[7]=42995,[8]=98175,[9]="98289,1",[10]=98176,[11]=98288,[12]="98177,110",[13]=98291,[14]="98292,11",[15]=1691,[16]=1727,[17]=1729,[18]=98153,[19]=98154,[20]=98174,[21]="98304,2",[22]=1713,[23]=43945,[24]=43946,[25]=43947,[26]=43948,[27]=43949},[70]={[1]=15600,[2]=15606,[3]="42976,1",[4]=100834,[5]="43950,1",[6]="100769,14",[7]="100787,44",[8]="100832,1",[9]=15613,[10]=42979,[11]="15616,1",[12]="43952,2",[13]="100836,11",[14]=100848,[15]="100849,4",[16]="100855,5",[17]=15621,[18]=15624,[19]=100785,[20]=100786,[21]=100861,[22]="100862,1",[23]=100784,[24]=100835,[25]="100864,66",[26]=34486,[27]=100854},[71]={[1]=22182,[2]="43955,4",[3]=22188,[4]=23662,[5]=34383,[6]=109671,[7]=109672,[8]=109673,[9]=109674,[10]=109675,[11]=109676,[12]=109677,[13]=109678,[14]=109679,[15]=109680,[16]=109681,[17]=109682,[18]=109683,[19]=109684,[20]=109685,[21]=109686,[22]=109687,[23]=109688,[24]=109689,[25]=109690,[26]=109691,[27]=109692,[28]=109693,[29]=109694,[30]=109695,[31]=109696,[32]=109697,[33]=109698,[34]=109699,[35]=109700,[36]=109701,[37]=109702,[38]=109703,[39]=109704,[40]=109705,[41]=109706,[42]=109707,[43]=109708,[44]=109709,[45]=109710,[46]=109711,[47]=109712,[48]=109713,[49]=109714,[50]=109715,[51]=109716,[52]=109717,[53]=109718,[54]=109719,[55]=109720,[56]=109721,[57]=109722,[58]=109723,[59]=109724,[60]=109725,[61]=109726,[62]=109727,[63]=109728,[64]=109729,[65]=109730,[66]=109731,[67]=109732,[68]=109733,[69]=109734,[70]=109735,[71]=109736,[72]=109737,[73]=109738,[74]="109650,20",[75]=109739,[76]="88167,161",[77]=109740,[78]=109741,[79]=109742,[80]=109743,[81]=109744,[82]=109745,[83]=109746,[84]=109747,[85]=109748,[86]=109749,[87]=109750,[88]=109751,[89]=109752,[90]=109753,[91]=109754,[92]=109755,[93]=109756,[94]=109757,[95]=109758,[96]=109759,[97]=109760,[98]=109761,[99]=109762,[100]=109763,[101]=109764,[102]=109765,[103]=109766,[104]=109767,[105]=109768,[106]=109769,[107]=109770,[108]=109771,[109]=109772,[110]=109773,[111]=109774,[112]=109775,[113]=109776,[114]=109777,[115]=109778,[116]=109779,[117]=109780,[118]=109781,[119]=109782,[120]=109783,[121]=109784,[122]=109785,[123]=109786,[124]=109787,[125]=109788,[126]=109789,[127]=109790,[128]=109791,[129]=109792,[130]=109793,[131]=109794,[132]=109795,[133]=109796,[134]=109797,[135]=109798,[136]=109799,[137]=109800,[138]=109801,[139]=109802,[140]=109803,[141]=109804,[142]=109805,[143]=109806,[144]=109807,[145]=109808,[146]=109809,[147]=109810,[148]=109811},[72]={[1]="43960,4",[2]="56919,2",[3]=110168,[4]=110169,[5]=110167,[6]="85269,113",[7]="85391,19",[8]=22162,[9]="110023,56",[10]="110170,14",[11]=22184,[12]="110080,86"},[73]={[1]=43968,[2]=43969,[3]=49596,[4]=56418,[5]=56419,[6]=56420,[7]=56421,[8]=56423,[9]=56424,[10]=56425,[11]=56426,[12]=56427,[13]=56428,[14]=56429,[15]=56430,[16]=56431,[17]=56432,[18]=56433,[19]=56434,[20]=56436,[21]=56437,[22]=56438,[23]=56439,[24]=56440,[25]=56441,[26]=56442,[27]=56443,[28]=56444,[29]=56445,[30]="43965,2",[31]="49597,2",[32]=137650,[33]=139685,[34]=56435,[35]=137608,[36]=137818,[37]=139727,[38]=139775,[39]=139817,[40]=139865,[41]=139907,[42]=61022,[43]=137566,[44]=140087,[45]=140135,[46]=139997,[47]=140045,[48]="49600,63",[49]=139955,[50]=56422,[51]=56446,[52]=56447,[53]=56448,[54]=56449,[55]=56450,[56]=56451,[57]=56452,[58]=61020,[59]=61021,[60]=137692,[61]=137734,[62]="49954,22",[63]=137524,[64]=137776,[65]="49664,288",[66]=140177},[74]={[1]=56453,[2]=56454,[3]=56455,[4]=56456,[5]=56457,[6]=56458,[7]=56459,[8]=56460,[9]=56461,[10]=56462,[11]=56463,[12]=56464,[13]=56465,[14]=56466,[15]=56467,[16]=56468,[17]=56469,[18]=56470,[19]=56471,[20]=56472,[21]=56473,[22]=56474,[23]=56475,[24]=56476,[25]="43971,5",[26]="49977,137",[27]="50118,1",[28]=56477,[29]=56479,[30]="50115,1",[31]=137525,[32]=140088,[33]="50121,54",[34]=56480,[35]=139686,[36]=139728,[37]=139776,[38]=139818,[39]=139866,[40]=139908,[41]="50176,157",[42]=50120,[43]=56478,[44]=137609,[45]=139956,[46]=139998,[47]=140136,[48]=140178,[49]="50335,17",[50]=50117,[51]=56481,[52]=56482,[53]=56483,[54]=56484,[55]=56485,[56]=56486,[57]=56487,[58]=137567,[59]=137651,[60]=137693,[61]=137735,[62]="50354,3",[63]=137777,[64]=137819,[65]=140046},[75]={[1]=50633,[2]=50634,[3]=50635,[4]=56502,[5]=56503,[6]=56504,[7]=56505,[8]=56506,[9]=56507,[10]=56508,[11]=56509,[12]=56510,[13]=56511,[14]=56512,[15]=56513,[16]=56514,[17]=56515,[18]=56516,[19]=56517,[20]=56518,[21]=56519,[22]="43977,2",[23]="50358,274",[24]=56520,[25]="50636,51",[26]=56521,[27]=137820,[28]=139729,[29]=139777,[30]=139819,[31]=139867,[32]=139957,[33]=139999,[34]=140047,[35]=140089,[36]=140137,[37]=140179,[38]="50688,51",[39]=56522,[40]=68348,[41]=68349,[42]=137526,[43]=137568,[44]=137610,[45]=137652,[46]=137694,[47]=137736,[48]=137778,[49]=139687,[50]="56488,13",[51]=139909},[76]={[1]="43983,5",[2]="91371,20",[3]="91392,148"},[77]={[1]="93060,6",[2]="93068,76",[3]=33155,[4]="43993,1",[5]=33289,[6]="92993,65",[7]=33163,[8]="93147,7",[9]=93059,[10]=93145,[11]=93146,[12]=109101,[13]=109102,[14]=109103,[15]=109104,[16]=109106,[17]=109107,[18]=109108,[19]=109109,[20]=109110,[21]=109111,[22]=109112,[23]=109113,[24]=109114,[25]=109115,[26]=109116,[27]=109117,[28]=109118,[29]=109119,[30]=109142,[31]=109143,[32]=109153,[33]=109154,[34]=109177,[35]=109178,[36]=109179,[37]=109180,[38]=109181,[39]=109182,[40]=109183,[41]=109184,[42]=109185,[43]="65581,18",[44]=93067,[45]=109105,[46]=109144,[47]=109145,[48]=109146,[49]=109147,[50]=109148,[51]=109149,[52]=109150,[53]=109151,[54]=109152,[55]="43989,3",[56]="65622,12",[57]="65657,9",[58]="109095,5",[59]="109120,21",[60]="109187,69",[61]=109186,[62]="109155,21"},[78]={[1]=56523,[2]=56524,[3]=56525,[4]=56526,[5]=56527,[6]=56528,[7]=56529,[8]=56530,[9]=56531,[10]=56532,[11]=56533,[12]=56534,[13]=56535,[14]=56536,[15]=56537,[16]=56538,[17]=56539,[18]=56540,[19]=56541,[20]=56542,[21]=56543,[22]=56544,[23]=56545,[24]=56546,[25]=56547,[26]=56548,[27]=56549,[28]=56550,[29]=56551,[30]=56552,[31]=56553,[32]=56557,[33]=137527,[34]=137569,[35]="50740,356",[36]=56554,[37]="51098,16",[38]="51116,4",[39]=56555,[40]=56556,[41]=137611,[42]=137653,[43]=137695,[44]=137737,[45]=137779,[46]=137821,[47]=139688,[48]=139730,[49]=139778,[50]=139820,[51]=139868,[52]="43995,5",[53]=139910,[54]=139958,[55]=140000,[56]=140048,[57]=140090,[58]=140138,[59]=140180},[79]={[1]=44001,[2]=44002,[3]=44003,[4]=44004,[5]=44005,[6]=44006,[7]=54191,[8]=56559,[9]=56560,[10]=56561,[11]=56562,[12]=56563,[13]=56564,[14]=56565,[15]=56566,[16]=56567,[17]=56568,[18]=56569,[19]=56570,[20]=56571,[21]=56572,[22]=56573,[23]=56574,[24]=56575,[25]=56576,[26]=56577,[27]=56579,[28]=56580,[29]=56581,[30]=56582,[31]=56583,[32]=139689,[33]=140181,[34]="51121,78",[35]="51200,144",[36]=56558,[37]=56578,[38]=56584,[39]=137528,[40]=137570,[41]=139779,[42]=139821,[43]=139869,[44]=139911,[45]=139959,[46]=140001,[47]=140049,[48]=140091,[49]=140139,[50]=56588,[51]=56589,[52]=56590,[53]="51346,131",[54]=56585,[55]=56586,[56]=137612,[57]="51479,16",[58]=56587,[59]=56591,[60]=56592,[61]=137654,[62]=137696,[63]=137738,[64]=137780,[65]=137822,[66]="51497,4",[67]=139731},[80]={[1]=51687,[2]=51689,[3]=51690,[4]=51691,[5]=51692,[6]=54192,[7]="51693,18",[8]=56624,[9]=56625,[10]=56626,[11]=56627,[12]=137529,[13]="44007,2",[14]="44011,1",[15]="51639,47",[16]=137571,[17]="51502,62",[18]="51566,71",[19]=137613,[20]="51878,4",[21]=137655,[22]="54193,1",[23]="56593,28",[24]="56622,1",[25]="68362,4",[26]=137697,[27]=139690,[28]=139960,[29]=140002,[30]=140050,[31]=140092,[32]=140140,[33]=140182,[34]="51712,164",[35]=137739,[36]=137781,[37]=137823,[38]=139732,[39]=139780,[40]=139822,[41]=139870,[42]=139912},[81]={[64]="52224,14",[1]=44018,[2]=52209,[3]=52210,[4]=56628,[5]=56629,[6]=56630,[7]=56631,[8]=56632,[9]=56633,[10]=56634,[11]=56635,[12]=56636,[13]=56637,[14]=56638,[15]=56639,[16]=56640,[17]=56641,[18]=56642,[19]=56643,[20]=56644,[21]=56645,[22]=56646,[23]=56647,[24]=56648,[25]=56649,[26]=56650,[27]=56651,[28]="44013,3",[29]="51883,321",[30]=56652,[31]="52205,3",[32]=56653,[33]=56656,[34]=137740,[35]=137782,[36]=137824,[37]=139691,[38]=139733,[39]=139781,[40]=139823,[41]=139871,[42]=139961,[43]=140003,[44]=140051,[45]=140093,[46]=140141,[47]=140183,[48]="52211,12",[49]=56655,[50]=56658,[51]=56659,[52]=56654,[53]="52240,23",[54]=56657,[55]=56660,[56]=56661,[57]=56662,[58]=137530,[59]=137572,[60]=137614,[61]=137656,[62]=137698,[63]=139913},[82]={[1]=56663,[2]=56664,[3]=56665,[4]=56666,[5]=56667,[6]=56668,[7]=56669,[8]=56670,[9]=56671,[10]=56672,[11]=56673,[12]=56674,[13]=56675,[14]=56676,[15]=56682,[16]=56683,[17]=56684,[18]=56686,[19]=56687,[20]=56688,[21]=56689,[22]="44019,5",[23]=56677,[24]=56678,[25]=56679,[26]=56680,[27]=56681,[28]=56685,[29]=56690,[30]=56691,[31]="52264,356",[32]="52622,17",[33]="52641,3",[34]=56692,[35]=56693,[36]=56694,[37]=56695,[38]=56696,[39]=56697,[40]=137531,[41]=137573,[42]=137615,[43]=137657,[44]=137699,[45]=137741,[46]=137783,[47]=137825,[48]=139692,[49]=139734,[50]=139782,[51]=139824,[52]=139872,[53]=139914,[54]=139962,[55]=140004,[56]=140052,[57]=140094,[58]=140142,[59]=140184},[83]={[1]="65539,25",[2]=89849,[3]=89850,[4]=89851,[5]=89852,[6]=89853,[7]=89854,[8]=89885,[9]=89886,[10]=89887,[11]="44025,5",[12]="65565,15",[13]=89888,[14]="89805,43",[15]=89889,[16]=89895,[17]=89896,[18]=89897,[19]=89898,[20]=89899,[21]=89900,[22]="89856,28",[23]="89901,65",[24]=89855,[25]=89890,[26]=89891,[27]=89892,[28]=89893,[29]=89894},[84]={[1]=44481,[2]=44482,[3]=44483,[4]=44484,[5]=52676,[6]=52735,[7]=52736,[8]="44032,2",[9]=52740,[10]=61015,[11]=61016,[12]=61017,[13]=61018,[14]=137616,[15]=137658,[16]=137700,[17]=137742,[18]=137784,[19]=140143,[20]="52645,30",[21]="52677,57",[22]="52737,1",[23]="52741,260",[24]="53003,22",[25]="56698,34",[26]=44031,[27]=44036,[28]=52739,[29]=137532,[30]=137574,[31]=137826,[32]=139693,[33]=139735,[34]=139783,[35]=139825,[36]=139873,[37]=139915,[38]=139963,[39]=140005,[40]=140053,[41]=140095,[42]=140185},[85]={[1]=91141,[2]=91142,[3]=91143,[4]=91144,[5]=91145,[6]=91146,[7]="44037,5",[8]="66063,47",[9]=91151,[10]=91153,[11]=91152,[12]=91156,[13]=91157,[14]=91158,[15]="90989,146",[16]=91154,[17]="91147,3",[18]=91155,[19]="91136,4"},[86]={[1]=7628,[2]=7635,[3]=7652,[4]=42986,[5]=42990,[6]="29952,1",[7]=80814,[8]=80840,[9]=80844,[10]=80851,[11]=114286,[12]=56889,[13]=81169,[14]=81177,[15]="34440,1",[16]="44043,5",[17]="80701,112",[18]="80845,5",[19]="80852,7",[20]=7598,[21]=7624,[22]="80815,24",[23]="80841,2",[24]="94246,7"},[87]={[1]=56733,[2]=56734,[3]=56735,[4]=56736,[5]=56737,[6]=56739,[7]=56740,[8]=56741,[9]=56742,[10]=56743,[11]=56744,[12]=56745,[13]=56746,[14]=56747,[15]=56748,[16]=56749,[17]=56750,[18]=56751,[19]=56752,[20]=56753,[21]=56754,[22]=56755,[23]=56756,[24]=56757,[25]=56758,[26]=56759,[27]=56760,[28]="44049,5",[29]=56761,[30]=140054,[31]=56738,[32]=137827,[33]=139694,[34]=139736,[35]=139784,[36]=139826,[37]=139874,[38]=139916,[39]=139964,[40]=140006,[41]=140186,[42]="53026,221",[43]=140096,[44]=140144,[45]=56762,[46]=56765,[47]=56766,[48]=56763,[49]="53384,16",[50]=56764,[51]=56767,[52]=137533,[53]=137575,[54]=137617,[55]=137659,[56]=137701,[57]=137743,[58]="53402,4",[59]=137785,[60]="53248,134"},[88]={[1]="44055,5",[2]=87463,[3]="87307,97",[4]="87414,11",[5]="87435,20",[6]="87459,2"},[89]={[1]=93925,[2]=93926,[3]=93927,[4]=93928,[5]=93929,[6]=93930,[7]=93931,[8]=93932,[9]=93933,[10]=93934,[11]=93935,[12]=93936,[13]=93937,[14]=93938,[15]=93939,[16]="44061,1",[17]="44063,3",[18]=93940,[19]=93942,[20]=93941,[21]=93950,[22]=93951,[23]=93983,[24]="66021,26",[25]=93984,[26]=93985,[27]=93986,[28]=93949,[29]=93943,[30]="93987,77",[31]=93944,[32]=93945,[33]=93947,[34]="93903,21",[35]=93946,[36]="66048,14",[37]=93948,[38]="93952,30"},[90]={[1]=10944,[2]=34798,[3]=34801,[4]="44067,5",[5]=81134,[6]=81137,[7]=81151,[8]=81154,[9]="81135,1",[10]="81138,12",[11]=81157,[12]="81012,121",[13]=121254,[14]=114459,[15]="81152,1",[16]="81158,5",[17]=10239,[18]="81155,1",[19]=43010,[20]=43013,[21]=54360,[22]=54363,[23]=56890,[24]=81171,[25]=81179,[26]="114453,5"},[91]={[1]=44727,[2]="110951,127",[3]=44731,[4]=111079,[5]="44073,5",[6]=54247,[7]=44756,[8]=44758,[9]="111080,23",[10]=44748,[11]="111104,8"},[92]={[1]=56768,[2]=56769,[3]=56770,[4]=56771,[5]=56772,[6]=56773,[7]=56774,[8]=56775,[9]=56776,[10]=56777,[11]=56778,[12]=56779,[13]=56780,[14]=56781,[15]=56782,[16]=56783,[17]=56784,[18]=56786,[19]=56787,[20]=56788,[21]=56789,[22]=56790,[23]=56791,[24]=56792,[25]=56793,[26]=56794,[27]=56795,[28]="44079,5",[29]=56797,[30]=139917,[31]=56796,[32]=56785,[33]=139785,[34]=139827,[35]=139965,[36]=140007,[37]=140055,[38]="53407,352",[39]=137786,[40]=137828,[41]=139875,[42]=140097,[43]=140145,[44]=140187,[45]=139737,[46]=56798,[47]=56801,[48]="53765,16",[49]=56799,[50]=56800,[51]=56802,[52]=137534,[53]=137576,[54]=137618,[55]=137660,[56]=137702,[57]=137744,[58]="53783,4",[59]=139695,[60]="53760,3"},[93]={[1]="2514,1",[2]=2521,[3]=2527,[4]=2539,[5]=2544,[6]=42963,[7]=42966,[8]="44085,6",[9]=42967,[10]="56922,1",[11]=86698,[12]="86588,29",[13]="86620,77",[14]=2474,[15]="86581,6",[16]=2519,[17]=35024,[18]="86561,19",[19]="86618,1",[20]="86699,23"},[94]={[1]="102266,2",[2]=15765,[3]=34532,[4]="15734,3",[5]="102253,6",[6]=15732,[7]=102261,[8]="102270,22",[9]=15741,[10]="42983,1",[11]="44092,6",[12]="102373,1",[13]="102294,77",[14]=102293,[15]="102375,1",[16]=102372,[17]=42980,[18]=102260,[19]=102262,[20]=102263,[21]=102264,[22]=102265,[23]=102269,[24]=15768,[25]=15774,[26]=15776,[27]="102377,22",[28]=54729,[29]="102400,14",[30]=54730},[95]={[1]=44099,[2]=44100,[3]=44101,[4]=44102,[5]=44104,[6]=54083,[7]=61014,[8]=137535,[9]=137577,[10]="53788,294",[11]=137619,[12]=137829,[13]="54084,60",[14]=137787,[15]=137703,[16]=139738,[17]="54146,22",[18]=139876,[19]=40259,[20]=137661,[21]=137745,[22]=139696,[23]=139786,[24]=139828,[25]="56803,28",[26]=140188,[27]=139918,[28]=139966,[29]=140008,[30]="56832,5",[31]=140056,[32]=140098,[33]=140146},[96]={[1]=7690,[2]=28119,[3]=30584,[4]=89161,[5]=89162,[6]=89163,[7]=89164,[8]=89165,[9]=89166,[10]=109015,[11]=109045,[12]=109046,[13]=109047,[14]=109048,[15]=109049,[16]=109050,[17]=109051,[18]=109052,[19]=109053,[20]=109054,[21]="44105,5",[22]=109055,[23]=109056,[24]="89077,10",[25]=109057,[26]=109068,[27]=109069,[28]=109070,[29]=109071,[30]=29449,[31]=109074,[32]="89088,72",[33]=109072,[34]=109073,[35]=16209,[36]=109058,[37]=109059,[38]=109060,[39]=109061,[40]="89167,71",[41]=109062,[42]=109063,[43]=109065,[44]="109016,28",[45]=109064,[46]=109066,[47]="108913,101",[48]=109067},[97]={[1]=64938,[2]=64939,[3]=64940,[4]=64941,[5]=64942,[6]=64943,[7]=64944,[8]="44111,6",[9]="64919,18",[10]="64951,15",[11]="91562,12",[12]="91575,15",[13]=64945,[14]=64950,[15]=91727,[16]=91728,[17]=91729,[18]=91730,[19]=91731,[20]=64946,[21]=64947,[22]=64948,[23]=64949,[24]="91591,56",[25]="91648,78"},[98]={[1]="44118,6",[2]=86871,[3]=86874,[4]="86748,1",[5]="42972,1",[6]=86875,[7]=86908,[8]=86876,[9]="86743,4",[10]=86914,[11]=86873,[12]=86855,[13]=86857,[14]=86870,[15]="86750,33",[16]=86872,[17]=86918,[18]=86929,[19]=7292,[20]=7298,[21]=34889,[22]=34887,[23]="86784,70",[24]=86856,[25]=7313,[26]="86858,11",[27]=86930,[28]=7315,[29]="86877,30",[30]="86909,4",[31]="86915,2",[32]=7302,[33]="86919,9",[34]="86932,1"},[99]={[1]=15577,[2]=15580,[3]=15583,[4]=34754,[5]=43008,[6]=85502,[7]=85503,[8]=85597,[9]=85598,[10]=85599,[11]="43006,1",[12]="44125,6",[13]="85604,8",[14]=15527,[15]=15548,[16]="85451,50",[17]=15540,[18]=85600,[19]=85601,[20]=85602,[21]=85615,[22]="85504,92",[23]=85603},[100]={[1]="44134,4",[2]=44132,[3]="93182,1",[4]=44133,[5]=93284,[6]=93285,[7]=93290,[8]="93175,4",[9]="93184,1",[10]=93199,[11]="93201,3",[12]="93208,36",[13]="93187,9",[14]="93250,15",[15]=93206,[16]="93246,2",[17]="93268,10",[18]="93280,3",[19]="93286,3",[20]="93291,13",[21]="93306,1",[22]="93309,7",[23]="93320,4",[24]="93326,1",[25]="93329,7"},[101]={[1]=89330,[2]=89331,[3]=89332,[4]=89333,[5]=89334,[6]=89335,[7]=89358,[8]=89359,[9]=89360,[10]=89361,[11]=89362,[12]=89363,[13]=89364,[14]=89365,[15]="44139,6",[16]="64747,12",[17]="64782,12",[18]="64817,12",[19]="64852,2",[20]="89259,63",[21]=89366,[22]="89323,6",[23]=89369,[24]=89367,[25]=89368,[26]=89400,[27]=89401,[28]="89336,7",[29]=89402,[30]=89404,[31]=89405,[32]="89344,13",[33]=89403,[34]=89370,[35]="89371,21",[36]=89393,[37]=89394,[38]=89395,[39]=89396,[40]=89397,[41]="89406,14",[42]=89398,[43]=89399},[102]={[1]="44146,5",[2]=33273,[3]="105592,128",[4]=33277,[5]="105722,2",[6]="105726,1",[7]=34379,[8]=44152,[9]=105721,[10]=105725,[11]="105728,25"},[103]={[1]=33160,[2]="44153,6",[3]="106884,3",[4]=33275,[5]="106889,113",[6]=106888,[7]=107003,[8]=33287,[9]=34378,[10]="107004,3",[11]=107015,[12]="107008,6",[13]="107016,37"},[104]={[1]=90944,[2]=90946,[3]="44160,6",[4]=90945,[5]="55872,6",[6]=90947,[7]=90949,[8]=90948,[9]="90798,81",[10]="90880,63",[11]="90951,16",[12]=90950},[105]={[1]=34438,[2]=34439,[3]="92265,28",[4]=29542,[5]="92295,1",[6]=42987,[7]=44168,[8]=44169,[9]="55879,6",[10]=44171,[11]=92294,[12]="92300,94",[13]=92298,[14]=92395,[15]=3158,[16]=7588,[17]=7593,[18]=7605,[19]=44167,[20]=44170,[21]="92396,19",[22]=34442,[23]=44173,[24]=92297,[25]=44172,[26]="92416,10",[27]=92299},[106]={[1]=10847,[2]=42998,[3]=43002,[4]=44178,[5]=97423,[6]=97427,[7]=97429,[8]=97497,[9]="44174,3",[10]=97526,[11]=97557,[12]=97565,[13]="97408,1",[14]=97503,[15]=97517,[16]=97537,[17]=97545,[18]="97442,19",[19]="97463,6",[20]=34709,[21]=97417,[22]="97424,1",[23]=44179,[24]=97426,[25]="97483,6",[26]=97428,[27]=34711,[28]=32063,[29]="97504,5",[30]="97523,2",[31]="97527,2",[32]="97543,1",[33]=97477,[34]="97546,3",[35]="97563,1",[36]=44180,[37]=97437,[38]="97566,3"},[107]={[1]=34297,[2]=96887,[3]=96917,[4]="44181,6",[5]=317,[6]=96860,[7]=96918,[8]="29946,1",[9]="34300,1",[10]=96919,[11]=96921,[12]=96920,[13]="96769,90",[14]="96888,28",[15]=852,[16]="96861,25",[17]=1116,[18]=1143,[19]=42957,[20]=96922,[21]="96924,6",[22]=96923},[108]={[1]=44188,[2]=44189,[3]=44190,[4]="55886,6",[5]=44192,[6]=44191,[7]=44193,[8]=44194,[9]="89441,158",[10]="89600,3"},[109]={[32]="91904,18",[1]=44195,[2]="55893,6",[3]=44196,[4]=44197,[5]="91753,59",[6]=44199,[7]=45279,[8]=45724,[9]=45725,[10]="91872,1",[11]=45730,[12]="91814,56",[13]=45733,[14]=45794,[15]=91871,[16]=91874,[17]=44198,[18]=45748,[19]=45749,[20]=45753,[21]=45758,[22]=45759,[23]=45760,[24]=91813,[25]="91875,28",[26]=44200,[27]=44201,[28]=45738,[29]=45739,[30]=45744,[31]=45746},[110]={[1]="44202,6",[2]=28113,[3]=29069,[4]=102557,[5]=102570,[6]=102571,[7]=102572,[8]=102573,[9]=102574,[10]=102575,[11]=102576,[12]=102578,[13]=7717,[14]=29085,[15]=102577,[16]="102435,1",[17]=28109,[18]="102438,118",[19]="102579,25",[20]="102558,11",[21]=32178,[22]=102437},[111]={[1]="44210,4",[2]="45736,1",[3]=93870,[4]="45742,1",[5]=93871,[6]=93873,[7]=93874,[8]=93875,[9]="45751,1",[10]=93876,[11]=44209,[12]=44215,[13]=93854,[14]=93856,[15]="45756,1",[16]=93862,[17]="55900,6",[18]=93864,[19]=93865,[20]=93877,[21]=93878,[22]="93857,4",[23]=45726,[24]=45728,[25]=45734,[26]=45745,[27]=93855,[28]="93795,58",[29]=93863,[30]=93872,[31]="93721,66",[32]="93881,1",[33]=93879,[34]=93880,[35]="93788,6",[36]="93866,3"},[112]={[1]="2516,1",[2]="35025,1",[3]="97772,1",[4]=2538,[5]="44216,6",[6]=2542,[7]=6872,[8]=97774,[9]=2501,[10]=35022,[11]=42965,[12]="97775,16",[13]="97792,141"},[113]={[1]=45735,[2]="45754,1",[3]=45750,[4]=88423,[5]="44223,6",[6]="88349,69",[7]="88419,3",[8]="88500,1",[9]="88503,2",[10]=45727,[11]=45747,[12]=45793,[13]=88425,[14]=88479,[15]="55907,6",[16]=88483,[17]="88426,52",[18]=88424,[19]=88484,[20]="45740,1",[21]="88480,2",[22]="88488,3",[23]="45731,1",[24]=88499,[25]="88485,1",[26]=88487,[27]=88492,[28]="88494,4",[29]=88506,[30]=88507,[31]=88493,[32]="88508,2",[33]=88502},[114]={[1]="10939,1",[2]="44230,6",[3]="101291,12",[4]=10914,[5]=10922,[6]=34800,[7]=34802,[8]=43011,[9]="101283,6",[10]=101209,[11]="101142,66",[12]=101282,[13]=101290,[14]="101210,2",[15]=43015,[16]=54361,[17]=54365,[18]="101214,67",[19]=101213},[116]={[1]="54258,1",[2]=54257,[3]=55369,[4]=55378,[5]=55366,[6]="55376,1",[7]=55375},[117]={[2]=55380,[1]=55379},[118]={[2]=55370,[1]=55365},[119]={[1]=55367,[2]=55371,[3]=55372},[120]={[1]=54267,[2]=54268,[3]=55363,[4]=55364,[5]=55373},[121]={[2]=55374,[1]=55368},[122]={[1]=54272,[2]=73142,[3]=73143,[4]=73144,[5]=73145,[6]=73146,[7]="77750,11",[8]=73147,[9]=73148,[10]="73132,9",[11]=73149,[12]=73150,[13]=73151,[14]=73152,[15]=73153,[16]=73154,[17]=106159,[18]=106189,[19]=106190,[20]=106257,[21]=106265,[22]="54273,5",[23]=106266,[24]="73480,16",[25]="54989,1",[26]="77604,7",[27]="77659,7",[28]="77714,11",[29]=106267,[30]="73497,21",[31]="77786,11",[32]=106270,[33]="77824,9",[34]=16047,[35]=16145,[36]=34376,[37]=106268,[38]=106280,[39]=106282,[40]="77822,1",[41]=106283,[42]=106284,[43]=106285,[44]=106286,[45]=106272,[46]="106147,11",[47]=106269,[48]=106273,[49]=106274,[50]="106287,21",[51]=106271,[52]=106275,[53]=106276,[54]=106277,[55]=106278,[56]="106160,28",[57]="106191,65",[58]=106279,[59]=106281,[60]="106258,6"},[123]={[1]=54252,[2]=73118,[3]=73119,[4]=73120,[5]=105074,[6]=105076,[7]=105185,[8]="54279,5",[9]=105186,[10]=105187,[11]=105188,[12]=105189,[13]=105190,[14]=105191,[15]=105192,[16]=105193,[17]=105194,[18]=105195,[19]="77798,5",[20]=23666,[21]=23668,[22]="77729,8",[23]="77805,4",[24]="77834,2",[25]="105077,107",[26]=77804,[27]="77837,8",[28]="105037,36",[29]=105075,[30]="73109,8",[31]="73121,10",[32]=105197,[33]=105198,[34]="73441,38",[35]="77612,7",[36]="77667,7",[37]="77726,2",[38]="77762,11",[39]=105196},[124]={[1]=44733,[2]=44759,[3]=54251,[4]=73097,[5]=73098,[6]=110774,[7]=110777,[8]=110778,[9]=110779,[10]=110780,[11]=110781,[12]=110782,[13]=110783,[14]=110784,[15]=110785,[16]=110786,[17]=110787,[18]=110788,[19]=110789,[20]=110790,[21]=110791,[22]=110792,[23]=110793,[24]=110794,[25]=110795,[26]=110805,[27]=110806,[28]=110807,[29]=110808,[30]=110809,[31]=110810,[32]=110811,[33]=110812,[34]=110813,[35]="54285,1",[36]="54399,4",[37]="110760,13",[38]=44729,[39]=44726,[40]="110775,1",[41]=44730,[42]="77846,11",[43]=44762,[44]=110799,[45]="73402,38",[46]=110800,[47]="77810,11",[48]=110801,[49]=110798,[50]=73099,[51]=73100,[52]=73101,[53]=73102,[54]=73103,[55]="110817,112",[56]=34384,[57]=73104,[58]=73105,[59]=110796,[60]=110797,[61]="77650,8",[62]=110802,[63]=110816,[64]="73083,13",[65]=110815,[66]=110803,[67]="73106,2",[68]="77705,8",[69]="77774,11",[70]="77738,11",[71]=110804,[72]=110814},[125]={[16]=59930,[1]=54293,[2]="54287,1",[3]=54292,[4]="54405,1",[5]="54440,2",[6]="89987,1",[7]="89989,1",[8]="89991,101",[9]="55863,2",[10]="54290,1",[11]="55845,2",[12]=54439,[13]=59924,[14]=59936,[15]="55854,2"},[126]={[1]=54443,[2]=55848,[3]=59925,[4]=59931,[5]=59937,[6]=90119,[7]=90135,[8]=90161,[9]=90170,[10]="55383,2",[11]=90199,[12]=90201,[13]=90200,[14]=54295,[15]=54407,[16]=55857,[17]="90106,5",[18]="90112,6",[19]="90120,14",[20]="90136,24",[21]="90162,7",[22]="90202,9",[23]=55866,[24]="90171,27"},[127]={[1]=54296,[2]=54298,[3]="54408,1",[4]="147374,53",[5]=54444,[6]=54445,[7]=55849,[8]=55858,[9]=55867,[10]=59926,[11]=59932,[12]=59938,[13]="87868,103",[14]=87866,[15]=147373,[16]="147365,7",[17]=87867,[18]="147512,1"},[128]={[1]="54410,2",[2]=87837,[3]=87838,[4]=87843,[5]="54446,2",[6]=87850,[7]="55850,1",[8]=87836,[9]=54301,[10]=87818,[11]=87819,[12]="55859,1",[13]=87820,[14]="55868,1",[15]=87831,[16]=87851,[17]=54300,[18]=54302,[19]=59927,[20]="87747,60",[21]=59933,[22]="87808,9",[23]=87852,[24]="87821,9",[25]="87832,3",[26]=59939,[27]="87839,3",[28]="87844,5"},[129]={[1]="55870,1",[2]=54303,[3]=59928,[4]=59934,[5]=92192,[6]=92193,[7]=92196,[8]=92204,[9]=92205,[10]=92209,[11]="54304,1",[12]="54413,2",[13]="54449,2",[14]="55852,1",[15]=92210,[16]="55861,1",[17]=92222,[18]=92211,[19]="92194,1",[20]="92206,2",[21]=59940,[22]="92197,6",[23]="92215,6",[24]=92212,[25]=92213,[26]="92146,13",[27]=92214,[28]="92160,31",[29]=92223,[30]="92224,27"},[130]={[1]=54437,[2]="92137,7",[3]=183204,[4]=59935,[5]="183206,2",[6]="183213,3",[7]="54328,1",[8]="54473,1",[9]="59727,2",[10]="183217,6",[11]=183225,[12]="183210,2",[13]=54438,[14]=59929,[15]=59941,[16]=92135,[17]=92136,[18]=183205,[19]=183209,[20]="183226,5",[21]=183224,[22]="183232,3"},[131]={[1]="54321,6",[2]="54430,6",[3]="54466,6",[4]="88531,44",[5]="88576,116"},[132]={[1]="54307,6",[2]="54416,6",[3]="54452,6",[4]="92629,42",[5]="92672,118"},[133]={[1]=91949,[2]=91950,[3]=91951,[4]=91952,[5]=92055,[6]=92056,[7]=92057,[8]=92058,[9]=92059,[10]="54315,5",[11]="54423,6",[12]=92060,[13]="54459,6",[14]=92091,[15]=92061,[16]=92095,[17]="91944,2",[18]=92096,[19]=92097,[20]="91953,101",[21]="92062,28",[22]=54314,[23]="92098,15",[24]="91947,1",[25]=92092,[26]=92093,[27]=92094},[134]={[1]=16144,[2]=23723,[3]="105932,3",[4]=54343,[5]=23716,[6]=44255,[7]=44271,[8]="105937,6",[9]=44262,[10]=54344,[11]=54987,[12]=54992,[13]="105774,48",[14]="105900,30",[15]=61023,[16]=105823,[17]=105892,[18]=105899,[19]="105824,67",[20]="105893,5",[21]=105931,[22]=105936},[135]={[1]=10976,[2]="44265,2",[3]=10972,[4]=10981,[5]=10990,[6]=11004,[7]=34843,[8]=34846,[9]=43042,[10]=43044,[11]=44260,[12]=44261,[13]=44270,[14]="101697,149",[15]=44907,[16]=101847,[17]="44247,4",[18]="101853,2",[19]=101848,[20]=101850,[21]=44263,[22]=101849,[23]=101852,[24]=101856,[25]="101857,1",[26]=101851},[136]={[1]="54874,6",[2]="55778,6",[3]="113926,93",[4]="114028,11",[5]="114048,19"},[137]={[1]="54883,1",[2]="113371,7",[3]=113379,[4]=113388,[5]=113387,[6]="113385,1",[7]="113389,10",[8]=113506,[9]="55785,1",[10]="55787,1",[11]="113405,2",[12]="113473,6",[13]=54881,[14]=54882,[15]=113505,[16]="113445,14",[17]=113499,[18]=113507,[19]=113508,[20]=113539,[21]="113408,31",[22]="113493,5",[23]=113533,[24]="113509,3"},[138]={[1]="54885,3",[2]="59097,1",[3]="112816,60",[4]="55789,3",[5]=112877,[6]=112879,[7]=112878,[8]="112881,14",[9]="112896,13",[10]="112918,11",[11]=112880,[12]="112938,19"},[139]={[8]="113681,20",[1]="54889,6",[2]=113650,[3]="55794,5",[4]=55793,[5]="113553,96",[6]="113660,3",[7]="113664,7"},[140]={[1]=54897,[2]="55800,5",[3]=54896,[4]=113010,[5]="54898,3",[6]=113011,[7]=113013,[8]="112998,11",[9]="113014,81",[10]="113105,11",[11]="113126,17",[12]=113012,[13]=113144,[14]=113145,[15]=113146},[141]={[8]="112571,20",[1]="54902,3",[2]="55806,1",[3]="55808,1",[4]="112443,58",[5]="112502,3",[6]="112506,34",[7]="112550,11"},[142]={[1]=112763,[2]=112765,[3]="54906,6",[4]=112762,[5]=112764,[6]=112766,[7]="55810,6",[8]=112767,[9]="112634,5",[10]="112640,87",[11]=112803,[12]="112736,11",[13]="112756,5",[14]=112768,[15]="112769,6"},[143]={[1]="54913,1",[2]="54915,1",[3]="55817,3",[4]="113189,93",[5]="113291,11",[6]="113311,19"},[144]={[1]="55821,3",[2]="113797,2",[3]="113866,19",[4]="113912,1",[5]=54920,[6]="54917,2",[7]="113744,52",[8]="113800,37",[9]="113846,11"},[145]={[8]="95393,38",[1]=81312,[2]=95390,[3]=95392,[4]="54921,6",[5]=81310,[6]=95391,[7]="95270,119"},[146]={[1]="54928,6",[2]=81311,[3]=95647,[4]="95643,3",[5]=81314,[6]="95648,95",[7]="95744,60"},[147]={[1]="54935,6",[2]=81313,[3]=81315,[4]="95452,35",[5]="95488,133"},[148]={[1]=56069,[2]=56070,[3]=56071,[4]=56072,[5]=56073,[6]=56074,[7]=56075,[8]=56076,[9]=56077,[10]=56078,[11]=56079,[12]=56080,[13]=56081,[14]=56082,[15]=56083,[16]=56084,[17]=56085,[18]=56086,[19]=56087,[20]=56088,[21]=56089,[22]=56090,[23]=56092,[24]=56093,[25]=56094,[26]=56095,[27]=56096,[28]=56097,[29]=137704,[30]=139739,[31]=139787,[32]=139967,[33]=140009,[34]=140057,[35]=140099,[36]=140147,[37]=137788,[38]=139829,[39]=139877,[40]="54787,35",[41]=137830,[42]=139919,[43]=140189,[44]="54942,34",[45]="55017,243",[46]=56068,[47]=56091,[48]=56098,[49]=56099,[50]=56100,[51]=56101,[52]=56102,[53]=137536,[54]=137578,[55]=137620,[56]=137662,[57]=137746,[58]=139697},[155]={[1]="59156,1",[2]=16213,[3]=16221,[4]=22128,[5]=57666,[6]=57667,[7]=58686,[8]=58687,[9]="59241,1",[10]=58706,[11]=58707,[12]=58708,[13]=58709,[14]=59221,[15]="59337,6",[16]="59358,6",[17]=29458,[18]=29459,[19]=29462,[20]=58642,[21]=58665,[22]=58666,[23]=58667,[24]=58685,[25]=58688,[26]=59155,[27]=59158,[28]=59176,[29]=59200,[30]="58640,1",[31]=103678,[32]="58643,3",[33]=103679,[34]="58661,3",[35]="59159,2",[36]="58682,2",[37]="59177,5",[38]="59197,2",[39]="59134,6",[40]=103680,[41]=103681,[42]="59201,2",[43]=59239,[44]=59240,[45]=103682,[46]="59243,2",[47]=103697,[48]=103683,[49]="103554,123",[50]=103684,[51]=103696,[52]="103685,10",[53]=103698,[54]="103706,9",[55]="57668,4",[56]=103699,[57]=103700,[58]=103701,[59]="58703,2",[60]=103702,[61]="59316,6",[62]=103703,[63]=103704,[64]="58724,6",[65]="59218,2",[66]="59222,2",[67]="59253,6",[68]=103705,[69]="59274,6",[70]="59295,6"},[156]={[1]=23731,[2]="106561,6",[3]="106676,5",[4]=28978,[5]="106569,5",[6]=58647,[7]="58648,5",[8]=58674,[9]="58668,5",[10]=106546,[11]="106610,64",[12]=106568,[13]=106584,[14]=106585,[15]=106586,[16]=106587,[17]=106588,[18]=106575,[19]="57673,6",[20]="58689,6",[21]="58710,6",[22]=106582,[23]=106583,[24]="58731,6",[25]="106547,13",[26]=106603,[27]="106520,25",[28]="106589,13",[29]=106604,[30]=106605,[31]=106606,[32]=106607,[33]="106576,5",[34]=106608,[35]=106609,[36]=106675},[157]={[1]=107339,[2]=107340,[3]=107341,[4]=107342,[5]=107344,[6]=107345,[7]=107346,[8]=107347,[9]=107348,[10]=107349,[11]=107350,[12]=107357,[13]=107358,[14]=107359,[15]=107360,[16]=107361,[17]="57680,6",[18]="58675,6",[19]="58696,2",[20]="58699,3",[21]=107362,[22]="58654,6",[23]="58717,6",[24]="58738,6",[25]=107379,[26]=22171,[27]="107385,19",[28]="107406,12",[29]="107420,6",[30]=22157,[31]="107351,5",[32]=22187,[33]=107343,[34]="107364,13",[35]=22189,[36]=107363,[37]=107378,[38]=107380,[39]=107381,[40]=107382,[41]=107383,[42]=107384,[43]=107405,[44]="107257,6",[45]=107419,[46]="107264,74"},[158]={[1]=58535,[2]="58556,6",[3]=58536,[4]=104917,[5]=104921,[6]=104934,[7]="58132,6",[8]=104938,[9]=104939,[10]=104940,[11]="58577,6",[12]=104937,[13]="58598,6",[14]=104936,[15]="58624,1",[16]=5832,[17]=23721,[18]=55769,[19]=58537,[20]=58540,[21]="58619,4",[22]=104904,[23]=104920,[24]=104935,[25]=58538,[26]="104855,17",[27]="104877,14",[28]=23734,[29]="104874,1",[30]=55468,[31]="104899,4",[32]="104913,3",[33]="104905,7",[34]="104941,13",[35]="104962,17",[36]=58539,[37]="104922,11",[38]="104987,29",[39]="104892,6",[40]=58541,[41]=104873,[42]=104876,[43]=104955,[44]=104956,[45]=104957,[46]=104958,[47]=104959,[48]=104960,[49]=104961,[50]=104980,[51]=104981,[52]=104982,[53]=104983,[54]=104984,[55]=104985,[56]="104918,1",[57]=104986},[159]={[1]=58632,[2]="55464,1",[3]="58139,6",[4]="58542,6",[5]="58563,6",[6]=5831,[7]=104755,[8]="58584,6",[9]="58605,6",[10]="58626,5",[11]="104673,11",[12]="104685,4",[13]="104690,1",[14]=23715,[15]=34377,[16]="104692,11",[17]=28979,[18]=55467,[19]=104777,[20]="104704,50",[21]="104756,20",[22]=55469,[23]="104778,56"},[160]={[1]=104629,[2]=104630,[3]=104631,[4]=104632,[5]=104634,[6]=104635,[7]="58148,2",[8]=104633,[9]=58146,[10]=58147,[11]=104606,[12]="58151,1",[13]=104610,[14]="58549,6",[15]=104615,[16]=104618,[17]="58570,6",[18]=104650,[19]=23710,[20]=23714,[21]="58591,6",[22]="58612,6",[23]=55463,[24]="58633,6",[25]="104489,116",[26]=104651,[27]="104607,2",[28]="104482,4",[29]=104660,[30]="104487,1",[31]="104611,3",[32]=55466,[33]="104616,1",[34]="104619,9",[35]="104636,13"},[161]={[1]=137537,[2]=137579,[3]=137621,[4]=137663,[5]=137705,[6]=137747,[7]=137789,[8]=137831,[9]=139698,[10]=139740,[11]=139788,[12]=139920,[13]=139968,[14]=140010,[15]=140148,[16]=140190,[17]="58153,214",[18]=139830,[19]=139878,[20]=140058,[21]=140100,[22]="58368,134"},[162]={[1]="59380,11",[2]="59392,23",[3]=76530,[4]="76466,1",[5]="76526,3",[6]=76531,[7]="76468,3"},[163]={[1]="59416,35",[2]=76549,[3]="76484,5",[4]="76544,3",[5]=76548},[164]={[1]="59452,3",[2]="59456,31",[3]=76536,[4]="76472,3",[5]="76532,3",[6]=76537,[7]="76476,1"},[165]={[1]=76554,[2]="59488,31",[3]=76555,[4]="59520,3",[5]="76490,5",[6]="76550,3"},[166]={[1]=76455,[2]=76514,[3]=76515,[4]=76516,[5]=76517,[6]=76519,[7]="59524,35",[8]="76456,3",[9]=76454,[10]=76518},[167]={[1]=76460,[2]=76461,[3]=76462,[4]=76463,[5]=76464,[6]=76465,[7]=76520,[8]=76523,[9]=76524,[10]=76525,[11]="59560,23",[12]=76521,[13]=76522,[14]="59584,11"},[168]={[1]=76449,[2]=76450,[3]=76451,[4]=76452,[5]=76453,[6]=76508,[7]="59596,35",[8]=76448,[9]=76509,[10]=76510,[11]=76511,[12]=76512,[13]=76513},[169]={[1]=76506,[2]="59632,15",[3]=76507,[4]="59648,19",[5]="76442,5",[6]="76502,3"},[170]={[1]=76437,[2]=76438,[3]=76439,[4]="59668,35",[5]=76441,[6]=76440,[7]=76500,[8]=76501,[9]="76496,3",[10]=76436},[171]={[1]=59741,[2]="59804,6",[3]=59742,[4]=79967,[5]=59740,[6]="59822,6",[7]="59844,6",[8]="80035,10",[9]=59738,[10]="80053,14",[11]="59888,6",[12]="80075,10",[13]=59739,[14]="59866,6",[15]=79968,[16]=79984,[17]=79985,[18]="79934,32",[19]=79986,[20]=79987,[21]=79988,[22]=79989,[23]=79990,[24]=80006,[25]=80009,[26]="59743,1",[27]="79991,14",[28]=80072,[29]=80073,[30]=80074,[31]="59760,6",[32]="79969,14",[33]=81172,[34]="59782,6",[35]=80007,[36]=80008,[37]=80010,[38]=80011,[39]=80012,[40]="80046,6",[41]=80030,[42]=80032,[43]=80033,[44]="80013,14",[45]=80034,[46]=80031,[47]=80028,[48]=80029,[49]="80068,3",[50]=81164},[172]={[1]=59796,[2]=59797,[3]=59798,[4]=59799,[5]=59800,[6]=59801,[7]=59802,[8]=59803,[9]=80338,[10]=80339,[11]=80340,[12]="59902,1",[13]=80341,[14]=80342,[15]=80343,[16]=80344,[17]=80345,[18]="59858,7",[19]=80360,[20]="59880,7",[21]="59836,7",[22]="59752,7",[23]=80361,[24]=80362,[25]="59774,7",[26]=80363,[27]="59904,9",[28]=80364,[29]=80384,[30]=80385,[31]=80386,[32]=80388,[33]=80389,[34]=80390,[35]=80391,[36]=80392,[37]=80393,[38]=81166,[39]=81174,[40]="80240,13",[41]="80254,7",[42]="80262,53",[43]=59818,[44]=80365,[45]=80367,[46]="80238,1",[47]=80366,[48]="80316,5",[49]=59821,[50]=80382,[51]=80383,[52]="80322,1",[53]="80368,13",[54]=59819,[55]=59820,[56]="80394,2",[57]=80387,[58]="80324,13",[59]="80346,13"},[173]={[1]=59874,[2]=59875,[3]="59829,6",[4]=59876,[5]=59877,[6]=59878,[7]="59851,6",[8]=59879,[9]=59897,[10]=80120,[11]="59745,6",[12]="59767,6",[13]="59789,6",[14]="59811,6",[15]="59895,1",[16]="59898,3",[17]=80119,[18]=80121,[19]="80086,32",[20]=80122,[21]=80123,[22]=80124,[23]=80229,[24]="80126,1",[25]=80230,[26]=80231,[27]=81165,[28]=81173,[29]="80128,96",[30]=59873,[31]="80232,5",[32]=80125,[33]=80225,[34]=80226,[35]=80227,[36]=80228},[176]={[1]=137538,[2]=137580,[3]=137622,[4]=137664,[5]=137706,[6]=137748,[7]=137790,[8]=137832,[9]=139699,[10]=139741,[11]=139789,[12]=139831,[13]="59946,349",[14]=139879,[15]=139921,[16]=139969,[17]=140011,[18]=140059,[19]=140101,[20]=140149,[21]=140191},[177]={[1]=137623,[2]=137665,[3]=137707,[4]=137749,[5]=137791,[6]=137833,[7]=139700,[8]=139742,[9]=139790,[10]=139832,[11]=139880,[12]=137539,[13]=137581,[14]=140012,[15]=140060,[16]=140102,[17]="60296,119",[18]=139922,[19]=139970,[20]=140150,[21]=140192,[22]="60416,229"},[178]={[1]=137540,[2]=137582,[3]=137624,[4]=137666,[5]=137708,[6]=137750,[7]=137792,[8]=139701,[9]=139743,[10]=139791,[11]=139833,[12]=137834,[13]=139881,[14]=139923,[15]=139971,[16]=140193,[17]="60646,281",[18]=140013,[19]=140061,[20]=140103,[21]=140151,[22]="60928,67"},[179]={[1]=69478,[2]=69479,[3]=116584,[4]=116585,[5]=116586,[6]=116587,[7]=116588,[8]=116604,[9]=116605,[10]=116606,[11]=116607,[12]="68432,6",[13]="68454,6",[14]="68476,3",[15]="68480,2",[16]=116627,[17]="68498,6",[18]="69473,4",[19]="116523,36",[20]="116560,6",[21]="116567,9",[22]="116577,5",[23]="116589,14",[24]="116608,17",[25]="116633,51",[26]=116583,[27]="116519,1",[28]=116626,[29]=116628,[30]=116629,[31]=116630,[32]=116631,[33]=116632},[180]={[1]=68535,[2]="68557,6",[3]=68536,[4]=68537,[5]=68538,[6]=117197,[7]="68540,1",[8]="68579,6",[9]=117219,[10]=117220,[11]=117221,[12]=117222,[13]=117223,[14]=117224,[15]=117225,[16]="117103,76",[17]=68539,[18]=117244,[19]=117245,[20]=117247,[21]="117180,1",[22]=117246,[23]=117242,[24]=69480,[25]="117182,14",[26]=117097,[27]="117226,14",[28]=117098,[29]="117204,14",[30]=117241,[31]=117243,[32]=117248,[33]=117637,[34]=117638,[35]="68601,6",[36]="69481,5",[37]="117087,9",[38]="117099,3",[39]="117198,5"},[181]={[32]=116854,[1]=116793,[2]=116794,[3]=116809,[4]=116810,[5]=116811,[6]=116812,[7]=116813,[8]=116814,[9]=116815,[10]="68637,7",[11]="68659,5",[12]="68665,1",[13]="68681,7",[14]="68615,7",[15]="69487,7",[16]=116816,[17]=116849,[18]=116847,[19]="116705,30",[20]=116848,[21]=117633,[22]="116736,56",[23]=117634,[24]=116853,[25]=116850,[26]="116795,13",[27]="116817,29",[28]=116851,[29]="116894,1",[30]=116852,[31]="116855,37"},[183]={[1]="68107,35",[2]=76542,[3]=76543,[4]="76478,1",[5]="76480,3",[6]="76538,3"},[184]={[1]=64868,[2]=64869,[3]=76561,[4]=76646,[5]="76644,1",[6]="76648,2",[7]="64870,1",[8]="64865,2",[9]=76732,[10]=76733,[11]=76734,[12]=112319,[13]="64760,6",[14]="76556,4",[15]="76735,3",[16]="112243,75",[17]="112323,52",[18]="64830,6",[19]="112320,2",[20]=112376,[21]="64795,6",[22]=112379,[23]=112377,[24]=69393,[25]=69396,[26]=76562,[27]=76647,[28]=112378,[29]=112380,[30]=112381,[31]=112382,[32]=112383,[33]="112384,11",[34]=112396,[35]=112397,[36]=112398,[37]=112399,[38]=112400,[39]=112401,[40]=112402,[41]=112403,[42]=112404},[185]={[1]=111870,[2]=111871,[3]=111950,[4]=111951,[5]=111952,[6]=111953,[7]=111954,[8]=111955,[9]="66272,7",[10]=111956,[11]=111957,[12]=111991,[13]=111992,[14]=111993,[15]=111994,[16]="66167,7",[17]="66202,7",[18]="69394,1",[19]="76622,7",[20]="66237,7",[21]="76710,7",[22]=111995,[23]=111998,[24]="76798,1",[25]=111996,[26]=112033,[27]="76800,5",[28]=112038,[29]="111872,77",[30]=112039,[31]=114285,[32]=112031,[33]=111997,[34]=112026,[35]="111958,32",[36]="111999,26",[37]=112027,[38]=112028,[39]=112029,[40]=112030,[41]="112034,3",[42]=112032},[186]={[1]="65040,2",[2]=76667,[3]=76668,[4]=76669,[5]=76670,[6]=76671,[7]="65043,3",[8]=33274,[9]="76754,6",[10]=33284,[11]=76666,[12]=110586,[13]=110678,[14]=110679,[15]="65110,6",[16]=110680,[17]="65145,6",[18]="76578,6",[19]="65075,6",[20]=110682,[21]="110578,7",[22]=33167,[23]=76672,[24]=110681,[25]=110719,[26]=110720,[27]="110587,4",[28]=110735,[29]="110592,85",[30]=110717,[31]=110684,[32]=110713,[33]="110685,27",[34]="110736,3",[35]=110683,[36]=110714,[37]=110715,[38]=110716,[39]="110721,13",[40]=110718},[187]={[1]=22747,[2]=34935,[3]=99000,[4]="67295,6",[5]=99001,[6]=99002,[7]="43029,1",[8]=99003,[9]=99035,[10]=10318,[11]=98900,[12]=98903,[13]=98935,[14]=98997,[15]=98998,[16]="67330,6",[17]=98999,[18]="67400,6",[19]=7476,[20]=43032,[21]="98882,5",[22]="98888,1",[23]="98895,4",[24]=99041,[25]=99042,[26]=99043,[27]="67365,6",[28]="76637,6",[29]="76725,6",[30]="76813,6",[31]="98890,4",[32]="98901,1",[33]=10327,[34]="98916,18",[35]=10324,[36]=99039,[37]="98936,60",[38]="99036,2",[39]=99040,[40]="98904,4",[41]="98909,6",[42]="99004,30"},[188]={[1]="65650,6",[2]="65685,6",[3]=33305,[4]="65720,6",[5]=33276,[6]="76703,6",[7]="110408,5",[8]="110418,1",[9]=34380,[10]=76796,[11]=110450,[12]=110452,[13]=110453,[14]=76615,[15]=76797,[16]="65615,6",[17]=110451,[18]=110454,[19]="76616,5",[20]="110415,2",[21]=110455,[22]="76791,4",[23]="110396,11",[24]="110422,27",[25]="110457,27",[26]=110414,[27]="110489,2",[28]="110492,2",[29]="110495,6",[30]="110520,6",[31]=110456,[32]=110485,[33]="110420,1",[34]=110486,[35]="110502,17",[36]="110527,30",[37]=110487,[38]=110488},[190]={[1]=76602,[2]=76783,[3]=111336,[4]=111337,[5]=111338,[6]=111339,[7]=111340,[8]=111341,[9]=111342,[10]=111343,[11]=111344,[12]=111345,[13]=111346,[14]=111347,[15]=111348,[16]=111349,[17]="76603,1",[18]=111350,[19]=111378,[20]=111379,[21]=111380,[22]=111381,[23]="67567,7",[24]="67602,2",[25]="67605,4",[26]="67637,2",[27]="67640,1",[28]=111414,[29]="67642,2",[30]="67672,7",[31]="69400,1",[32]="76600,1",[33]="76605,2",[34]="76688,7",[35]="76776,6",[36]="111315,20",[37]=69397,[38]="111360,17",[39]=111384,[40]=111423,[41]=111449,[42]=111450,[43]=111451,[44]=111452,[45]="111351,8",[46]=111453,[47]=111454,[48]=111455,[49]=111385,[50]=111413,[51]=111415,[52]="111382,1",[53]=111416,[54]="111386,26",[55]=111417,[56]=111418,[57]=111419,[58]=111420,[59]=111421,[60]="111424,23",[61]=111422,[62]=111448,[63]="111456,28"},[193]={[1]=65922,[2]=65927,[3]="65887,7",[4]=65925,[5]="110205,7",[6]=65923,[7]=65924,[8]="110214,1",[9]=65926,[10]="65992,7",[11]=76680,[12]="76592,7",[13]="110218,61",[14]="76768,7",[15]=110309,[16]=110311,[17]=110310,[18]="110216,1",[19]=110308,[20]=65929,[21]=110304,[22]=110305,[23]="110288,15",[24]="110280,7",[25]=110306,[26]=110307,[27]=33285,[28]=65928,[29]=65958,[30]=65959,[31]=65960,[32]=65961,[33]=65962,[34]=65963,[35]="110312,23",[36]=65964,[37]=110213,[38]=110341,[39]=76681,[40]=76682,[41]=76683,[42]=76684,[43]=76685,[44]="110336,4",[45]=33176,[46]="110342,32",[47]=33188,[48]=65957,[49]=76686,[50]=76687},[194]={[1]=29010,[2]=67952,[3]=67953,[4]=103771,[5]=103787,[6]=103788,[7]=103789,[8]=103790,[9]=103791,[10]=103792,[11]=103793,[12]=103794,[13]=103795,[14]=103796,[15]=103797,[16]=103798,[17]=103799,[18]="29464,1",[19]=103875,[20]=103876,[21]="67847,7",[22]=76570,[23]=103877,[24]="67882,7",[25]="67917,7",[26]="67955,4",[27]="76563,4",[28]="76568,1",[29]="76651,7",[30]="76739,7",[31]="103736,20",[32]="103757,7",[33]="103765,5",[34]="103772,14",[35]="103800,74",[36]="103878,4",[37]="103883,2",[38]=16219,[39]=28110,[40]=67954,[41]="103887,18",[42]=103886},[195]={[1]=111512,[2]="67015,6",[3]=111560,[4]="76696,6",[5]="111519,39",[6]=111559,[7]="111506,5",[8]=111561,[9]=111594,[10]="67085,6",[11]="69398,1",[12]="76785,5",[13]=111599,[14]=111600,[15]=111601,[16]=111602,[17]=111603,[18]="67050,6",[19]="67120,6",[20]=111598,[21]=111604,[22]="76608,6",[23]=111562,[24]=111564,[25]=111595,[26]=111596,[27]="111566,27",[28]=111563,[29]=111597,[30]=111605,[31]=111606,[32]=111632,[33]="111607,8",[34]=111633,[35]=111634,[36]=111635,[37]=111638,[38]=111639,[39]=111664,[40]=111665,[41]=111666,[42]=111667,[43]="111616,12",[44]=76784,[45]="111636,1",[46]=111513,[47]=111514,[48]=111515,[49]=111516,[50]=111517,[51]=111518,[52]=111565,[53]=111629,[54]="111640,23",[55]=111630,[56]=111631},[196]={[1]=111787,[2]=111833,[3]="66440,6",[4]=76679,[5]="69402,1",[6]="66475,6",[7]="66545,6",[8]=111834,[9]="66510,6",[10]=111835,[11]="76585,6",[12]="76673,5",[13]="76761,6",[14]="111788,5",[15]="111829,3",[16]="111840,9",[17]=111836,[18]="111688,98",[19]=111837,[20]=114284,[21]="111794,27",[22]=111838,[23]=111839,[24]="111822,3",[25]="111826,2"},[197]={[1]="102808,26",[2]=28123,[3]=32179,[4]="66757,4",[5]="102836,16",[6]=29083,[7]=66720,[8]="102853,3",[9]=28112,[10]=66721,[11]="102858,5",[12]=102857,[13]="66790,6",[14]="66829,2",[15]=102864,[16]=102887,[17]="66825,2",[18]=102888,[19]=102889,[20]=102890,[21]=102891,[22]=102892,[23]=102886,[24]=76809,[25]="76630,6",[26]=102875,[27]=76810,[28]="76806,2",[29]="102865,9",[30]="102876,9",[31]=66724,[32]=66828,[33]=76811,[34]=66722,[35]=34370,[36]=61002,[37]=66723,[38]=76812,[39]=102919,[40]=102920,[41]=102921,[42]=102922,[43]=102923,[44]=102924,[45]=102925,[46]="102893,18",[47]=102926,[48]=102927,[49]="102912,5",[50]=66725,[51]=66726,[52]=66755,[53]=66756,[54]=76718,[55]=76719,[56]=76720,[57]=76721,[58]=76722,[59]=76723,[60]=76724,[61]="102928,41",[62]=102835,[63]=102918},[198]={[1]="65440,6",[2]=112075,[3]=112076,[4]=112077,[5]=112078,[6]=112079,[7]=112080,[8]=112081,[9]=112144,[10]=112155,[11]=112156,[12]=112157,[13]=112158,[14]=112159,[15]="65335,6",[16]="76571,6",[17]="76747,6",[18]="112061,13",[19]=112160,[20]="65370,6",[21]="65405,6",[22]=112185,[23]="76659,6",[24]=112161,[25]=112183,[26]="112082,45",[27]=112220,[28]=112221,[29]=69392,[30]="112128,15",[31]=112222,[32]=112218,[33]=112184,[34]=112186,[35]=112187,[36]=112188,[37]="112145,9",[38]="112190,27",[39]=112189,[40]="112162,20",[41]=112219},[199]={[1]=69501,[2]="68755,6",[3]="68777,6",[4]=117607,[5]=117608,[6]="68711,6",[7]="69495,5",[8]=117609,[9]="68733,6",[10]=117612,[11]="117451,52",[12]="117641,1",[13]=117610,[14]=117611,[15]="117504,102"},[200]={[1]="68791,7",[2]=68864,[3]="68813,7",[4]=116968,[5]=116969,[6]="68835,7",[7]=116967,[8]=116966,[9]="68857,6",[10]=116963,[11]=116964,[12]=116965,[13]=116985,[14]="69502,7",[15]="116896,22",[16]="116927,13",[17]="116944,2",[18]=116991,[19]="116919,7",[20]="116941,2",[21]=116992,[22]=117635,[23]=117636,[24]=116986,[25]=116987,[26]="116947,1",[27]=116988,[28]="116949,13",[29]="116971,13",[30]="116993,72",[31]=116970,[32]=116989,[33]=116990},[201]={[1]=117382,[2]=117383,[3]=117384,[4]="68872,6",[5]=117385,[6]=117386,[7]=117387,[8]=117388,[9]=117639,[10]="68894,6",[11]="68918,4",[12]="117269,32",[13]="117302,4",[14]="117307,14",[15]="117322,6",[16]="69510,6",[17]="117329,52",[18]="117389,41",[19]="68938,6",[20]=117640,[21]="68916,1"},[204]={[1]="69061,58",[2]=69147,[3]="69120,26",[4]=55963,[5]="69148,22"},[205]={[1]=64488,[2]="69171,76",[3]="69248,32"},[206]={[2]="69376,14",[1]="69281,94"},[207]={[1]=71747,[2]=137667,[3]=137709,[4]=137751,[5]=137793,[6]=137835,[7]=139744,[8]=139792,[9]=139834,[10]=139882,[11]=139924,[12]="69577,54",[13]=137541,[14]=137583,[15]=137625,[16]=139702,[17]=139972,[18]=140014,[19]=140062,[20]=140104,[21]=140152,[22]=140194,[23]="69632,294"},[208]={[1]=137542,[2]=137584,[3]=137626,[4]=137668,[5]=137710,[6]=137752,[7]=137794,[8]=139703,[9]=139745,[10]=139793,[11]=140063,[12]=139925,[13]=139973,[14]=140015,[15]=140153,[16]=140195,[17]="69927,216",[18]=137836,[19]=139835,[20]=139883,[21]=140105,[22]="70144,132"},[209]={[1]=137543,[2]=137585,[3]=137627,[4]=137669,[5]=137711,[6]=137753,[7]=137795,[8]=137837,[9]=139704,[10]=139746,[11]=139794,[12]=139836,[13]=139884,[14]=139926,[15]=139974,[16]=140016,[17]=140064,[18]=140106,[19]=140154,[20]=140196},[210]={[1]=68633,[2]=68654,[3]=68655,[4]=68656,[5]=68657,[6]=68658,[7]=68674,[8]=68675,[9]=68676,[10]=68677,[11]=68678,[12]=68679,[13]=68680,[14]=70984,[15]=70985,[16]=71297,[17]=79386,[18]=79387,[19]=79388,[20]=129638,[21]=129647,[22]=129656,[23]=129674,[24]=129792,[25]="68608,6",[26]="68630,2",[27]="68634,2",[28]="70999,1",[29]="79389,47",[30]=70997,[31]=71261,[32]=71270,[33]="68652,1",[34]=71279,[35]=71306,[36]="129799,3",[37]=71002,[38]="129810,3",[39]="129639,7",[40]=71288,[41]=79385,[42]="129657,16",[43]="129675,10",[44]=129793,[45]=129794,[46]="129648,7",[47]=129795,[48]=129796,[49]="129775,16",[50]=129797,[51]=129798},[211]={[1]="68784,6",[2]=71265,[3]="68850,6",[4]="68806,6",[5]=71301,[6]=71310,[7]=83413,[8]=131955,[9]="68828,6",[10]=131964,[11]=83416,[12]=71292,[13]="131907,38",[14]="131947,7",[15]=71274,[16]=71283,[17]="131965,45",[18]=131946,[19]="131956,7"},[212]={[1]=71304,[2]=129816,[3]="68447,4",[4]="68469,6",[5]=129815,[6]=71012,[7]="68513,6",[8]=68452,[9]="68491,6",[10]=71286,[11]=71295,[12]="79333,26",[13]="129850,3",[14]=68453,[15]="129817,25",[16]=70990,[17]=71015,[18]=71017,[19]=71259,[20]=71268,[21]=71277,[22]="79360,24",[23]="129698,5",[24]="129704,41"},[213]={[1]="68718,2",[2]="68722,2",[3]="68740,6",[4]="68762,6",[5]="131701,4",[6]=68696,[7]="131707,67",[8]="68698,4",[9]=68697,[10]=131674,[11]=131675,[12]=131676,[13]=131677,[14]=131678,[15]=131688,[16]="131680,7",[17]=131694,[18]=131696,[19]=131697,[20]=131698,[21]=131699,[22]=131700,[23]=131706,[24]=131679,[25]=71263,[26]=71272,[27]=71281,[28]=71290,[29]=83391,[30]=83395,[31]=131776,[32]=68721,[33]=71308,[34]=131673,[35]="131689,4",[36]=71299,[37]=131695,[38]=131775},[214]={[1]="68623,1",[2]=71307,[3]="68625,4",[4]="68645,6",[5]="68667,3",[6]=131648,[7]="68671,2",[8]=131647,[9]=83411,[10]="68689,6",[11]="83405,1",[12]=71262,[13]=71271,[14]=71280,[15]="131650,9",[16]=71289,[17]=71298,[18]="131556,27",[19]=131649,[20]="131584,62"},[215]={[1]=71309,[2]="68705,5",[3]="68725,7",[4]=83383,[5]=83388,[6]=83390,[7]=68704,[8]=71291,[9]="68747,7",[10]="68769,7",[11]=68703,[12]=71264,[13]=71273,[14]="131790,49",[15]=71282,[16]="131840,53",[17]=71300},[216]={[1]=132075,[2]="68821,6",[3]=132076,[4]="68843,6",[5]=132078,[6]="68801,4",[7]=132077,[8]="68865,4",[9]=132079,[10]="132024,6",[11]="132033,4",[12]="132040,12",[13]=83399,[14]="132053,1",[15]=132032,[16]=132038,[17]=68799,[18]="132055,2",[19]="132060,14",[20]=132080,[21]=68800,[22]="132103,24",[23]=68870,[24]=68871,[25]=71266,[26]=71275,[27]=71284,[28]=71293,[29]=71302,[30]=71311,[31]=132031,[32]=132039,[33]=132058,[34]=132059,[35]=132081,[36]="132082,13",[37]="132096,5",[38]=132102},[217]={[1]="68527,5",[2]=71260,[3]="68593,7",[4]=71269,[5]=71278,[6]="68533,1",[7]=71287,[8]="83419,1",[9]=71296,[10]=71305,[11]=83424,[12]=83426,[13]=131447,[14]=131456,[15]="68571,7",[16]=131531,[17]="68549,7",[18]=131532,[19]=131525,[20]=131526,[21]="131439,7",[22]="131448,7",[23]=131527,[24]=131528,[25]="131457,67",[26]="131533,9",[27]=131529,[28]=131530},[218]={[1]=68439,[2]=71303,[3]=79452,[4]=79459,[5]=79486,[6]=129611,[7]=129612,[8]=129613,[9]=129614,[10]=129615,[11]=129616,[12]=129617,[13]=129618,[14]=129619,[15]=129620,[16]=129621,[17]=129622,[18]=129623,[19]=129624,[20]=129625,[21]=129855,[22]=129856,[23]=129857,[24]=129858,[25]=129859,[26]=129860,[27]=129861,[28]=129862,[29]=129863,[30]="68440,6",[31]="70987,1",[32]="79437,12",[33]="79453,5",[34]="79460,7",[35]="79468,9",[36]=129864,[37]="68461,7",[38]="68483,7",[39]="68505,7",[40]="79478,7",[41]=129870,[42]="79487,1",[43]=129865,[44]=71004,[45]=71258,[46]=71267,[47]=71276,[48]=71285,[49]=71294,[50]=79450,[51]=79451,[52]=129866,[53]="129578,32",[54]=129867,[55]=129868,[56]=129869,[57]=129871,[58]=129872,[59]=129873,[60]=129874,[61]=129875,[62]=129876,[63]=129877,[64]=129878,[65]=129879,[66]=129880,[67]=129881,[68]=129882,[69]=129883,[70]=129884,[71]=129885,[72]=129886,[73]=129895,[74]=129896,[75]=129897,[76]=129898},[219]={[1]=137712,[2]=137754,[3]=137796,[4]=137838,[5]=139747,[6]=139795,[7]=139837,[8]=139885,[9]=139927,[10]=139975,[11]=140017,[12]=137544,[13]=137586,[14]=137628,[15]=140107,[16]=140155,[17]="70627,28",[18]=137670,[19]=139705,[20]=140065,[21]=140197,[22]="70656,320"},[224]={[1]=137545,[2]=137587,[3]=137629,[4]=137671,[5]=137713,[6]=137755,[7]=137797,[8]=137839,[9]=139706,[10]=139748,[11]=139796,[12]=139838,[13]=139886,[14]=139928,[15]=139976,[16]=140018,[17]=140066,[18]="71791,349",[19]=140108,[20]=140156,[21]=140198},[225]={[1]=137672,[2]=137714,[3]=137756,[4]=137798,[5]=137840,[6]=139749,[7]=139797,[8]=139839,[9]=139887,[10]=139929,[11]=139977,[12]=137546,[13]=137588,[14]=137630,[15]=140109,[16]=140157,[17]="72141,50",[18]=139707,[19]=140019,[20]=140067,[21]=140199,[22]="72192,298"},[226]={[1]=137547,[2]=137589,[3]=137631,[4]=137673,[5]=137715,[6]=137757,[7]=137841,[8]=139708,[9]=139750,[10]=139798,[11]=140068,[12]=139930,[13]=139978,[14]=140020,[15]=140158,[16]=140200,[17]="72491,212",[18]=137799,[19]=139840,[20]=139888,[21]=140110,[22]="72704,136"},[227]={[1]="72841,71",[2]="73168,38",[3]="114460,19",[4]="114481,40",[5]="114523,19",[6]="114544,19",[7]="114565,19",[8]="114586,19",[9]="114607,19",[10]="114628,1",[11]="199739,74"},[228]={[16]="114688,124",[1]=114679,[2]=114680,[3]=114681,[4]=114682,[5]=114683,[6]="72913,71",[7]=114684,[8]=114685,[9]=199889,[10]="73207,8",[11]="73216,29",[12]=114686,[13]="114651,27",[14]="199815,73",[15]=114687},[229]={[1]=73035,[2]=73036,[3]=73288,[4]=73289,[5]=73290,[6]=77642,[7]=77643,[8]=77697,[9]=77898,[10]=79735,[11]="73011,18",[12]=79740,[13]=79741,[14]=79742,[15]=79743,[16]="73030,3",[17]=79923,[18]=79736,[19]=79737,[20]=79738,[21]=79739,[22]="73285,2",[23]=79922,[24]=73034,[25]=73291,[26]="73292,5",[27]="77698,6",[28]="77899,6",[29]="77942,11",[30]=79924,[31]="73298,25",[32]="77644,5",[33]="77894,3",[34]="114877,11",[35]=79925,[36]=79926,[37]=79927,[38]=79928,[39]=79929,[40]=79930,[41]=79931,[42]=79932,[43]=79933,[44]=114840,[45]="77990,11",[46]=114836},[230]={[1]=73246,[2]=73247,[3]=77627,[4]=77628,[5]=77629,[6]=77871,[7]=77872,[8]=77873,[9]=77874,[10]=79718,[11]=79719,[12]=79720,[13]=79721,[14]=79898,[15]=79899,[16]="72985,8",[17]=79900,[18]=79901,[19]=79902,[20]=79903,[21]="72996,14",[22]="73248,1",[23]="73250,16",[24]="73267,6",[25]="73274,3",[26]="73278,4",[27]="73283,1",[28]="77630,4",[29]="77682,7",[30]="77875,6",[31]="77918,11",[32]="79722,4",[33]="72994,1",[34]=77870,[35]=79904,[36]=79905,[37]=79906,[38]=79907,[39]=79908,[40]=79909,[41]="77966,11",[42]=114834,[43]=114838,[44]="114853,11"},[231]={[1]=79716,[2]=79717,[3]=114837,[4]=114841,[5]=114842,[6]=114843,[7]=114844,[8]=114845,[9]=114846,[10]="73060,22",[11]="73363,38",[12]="77620,6",[13]=114847,[14]=114849,[15]="77906,11",[16]="79710,5",[17]=114833,[18]="77858,11",[19]="77954,11",[20]=114848,[21]=114850,[22]="77675,6",[23]=114851,[24]=114852,[25]="79886,11"},[232]={[32]=79727,[1]="77635,2",[2]="79728,6",[3]=77690,[4]=77691,[5]=77693,[6]=77694,[7]=77695,[8]=77932,[9]=77933,[10]=77934,[11]=77935,[12]="73037,22",[13]="73324,19",[14]="73345,17",[15]="77638,3",[16]="77882,11",[17]="77930,1",[18]=77936,[19]="77978,11",[20]=77941,[21]="114865,11",[22]=73344,[23]=77692,[24]=77696,[25]=77937,[26]=77938,[27]=114835,[28]=114839,[29]="79910,11",[30]=77939,[31]=77940},[234]={[1]=199500,[2]="73873,61",[3]="79744,6",[4]=199501,[5]=199502,[6]="74059,6",[7]=199503,[8]=199505,[9]=79759,[10]="199506,12",[11]=199499,[12]="199426,72",[13]=199504},[235]={[1]=79860,[2]=79861,[3]=79862,[4]=79863,[5]="74222,17",[6]="79845,8",[7]="79854,5",[8]="199519,96",[9]=79864,[10]="74240,54",[11]=79865},[236]={[1]=79842,[2]=79843,[3]=79844,[4]=199616,[5]=199617,[6]=199618,[7]=199619,[8]=199620,[9]=199621,[10]=199622,[11]=199623,[12]=199624,[13]=199625,[14]=199626,[15]=199627,[16]=199628,[17]=199629,[18]=199630,[19]=199631,[20]=199632,[21]=199633,[22]=199634,[23]=199635,[24]=199636,[25]=199637,[26]=199638,[27]=199639,[28]=199640,[29]=199641,[30]=199642,[31]=199643,[32]="74210,11",[33]="79824,17",[34]=199644,[35]="74149,42",[36]=199645,[37]=74192,[38]=74193,[39]=74194,[40]=74195,[41]=74196,[42]=74197,[43]=74198,[44]=74199,[45]=74200,[46]=74201,[47]=74202,[48]=74203,[49]=74204,[50]=74205,[51]=74206,[52]=74207,[53]=74208,[54]=74209,[55]=199646,[56]=199647,[57]=199648,[58]=199649,[59]=199650,[60]=199651,[61]=199652,[62]=199653,[63]=199654,[64]=199658,[65]=199659,[66]=199660,[67]=199661,[68]="199662,17",[69]=199655,[70]=199656,[71]=199657,[72]="199680,32"},[237]={[1]=199333,[2]=199334,[3]=199335,[4]=199375,[5]=199376,[6]=199377,[7]=199378,[8]=199379,[9]=199380,[10]=199381,[11]=199382,[12]=199383,[13]=199384,[14]=199385,[15]=199386,[16]=199387,[17]=199388,[18]=199389,[19]=199390,[20]=199391,[21]=199392,[22]=199393,[23]=199394,[24]=199395,[25]=199396,[26]=199397,[27]=199398,[28]="73935,48",[29]=199399,[30]=199400,[31]=199415,[32]=199416,[33]=199417,[34]=199418,[35]=199419,[36]="73984,12",[37]=199420,[38]=199421,[39]=199422,[40]=199423,[41]=199424,[42]=199425,[43]=199413,[44]=199401,[45]=199402,[46]=199403,[47]=199404,[48]=199405,[49]="79764,19",[50]=199406,[51]=199407,[52]=199408,[53]=199409,[54]=199411,[55]="199336,38",[56]=199410,[57]=199412,[58]="74066,6",[59]=199414},[238]={[1]=199194,[2]=199195,[3]=199196,[4]=199197,[5]=199198,[6]=199199,[7]=199200,[8]=199201,[9]=199202,[10]=199203,[11]=199204,[12]=199205,[13]=199207,[14]=199186,[15]=199187,[16]=199188,[17]=199208,[18]=199209,[19]=199210,[20]="79784,19",[21]=199189,[22]=199190,[23]=199191,[24]=199192,[25]=199193,[26]=199206,[27]="74002,56",[28]="74073,6",[29]=199211,[30]="73997,4",[31]=199214,[32]=199212,[33]=199213,[34]=199228,[35]=199229,[36]=199230,[37]=199231,[38]=199232,[39]=199233,[40]=199234,[41]=199235,[42]=199236,[43]=199237,[44]=199238,[45]="199147,20",[46]=199239,[47]=199227,[48]=199215,[49]=199216,[50]=199217,[51]=199218,[52]=199219,[53]=199220,[54]="199168,17",[55]=199221,[56]=199222,[57]=199223,[58]=199224,[59]=199225,[60]=199226},[239]={[1]=199265,[2]=199266,[3]=199267,[4]=199268,[5]=199269,[6]=199270,[7]=199271,[8]=199272,[9]=199273,[10]=199274,[11]=199275,[12]=199276,[13]=199277,[14]=199278,[15]=199279,[16]=199280,[17]=199281,[18]=199282,[19]=199283,[20]=199284,[21]=199285,[22]=199286,[23]=199287,[24]=199288,[25]=199289,[26]=199290,[27]=199291,[28]=199292,[29]=199293,[30]=199294,[31]=199295,[32]=199296,[33]=199297,[34]=199298,[35]="74080,68",[36]="79804,19",[37]=199300,[38]=199264,[39]=199299,[40]=199301,[41]=199302,[42]=199303,[43]=199304,[44]=199305,[45]=199306,[46]=199307,[47]=199308,[48]=199309,[49]=199310,[50]=199311,[51]=199312,[52]=199313,[53]=199314,[54]=199315,[55]=199316,[56]=199325,[57]=199326,[58]=199327,[59]=199328,[60]=199329,[61]=199330,[62]=199331,[63]=199332,[64]="199240,23",[65]=199317,[66]=199318,[67]=199319,[68]=199320,[69]=199321,[70]=199322,[71]=199323,[72]=199324},[240]={[1]=137548,[2]=137590,[3]=137632,[4]=137674,[5]=137716,[6]=137758,[7]=137800,[8]=137842,[9]=139709,[10]=139751,[11]=139799,[12]=139841,[13]=139889,[14]=139931,[15]=139979,[16]=140021,[17]=140069,[18]="75386,349",[19]=140111,[20]=140159,[21]=140201},[241]={[1]=137717,[2]=137759,[3]=137801,[4]=137843,[5]=139752,[6]=139800,[7]=139842,[8]=139890,[9]=139932,[10]=139980,[11]=140022,[12]=137549,[13]=137591,[14]=137633,[15]=140112,[16]=140160,[17]="75736,39",[18]=137675,[19]=139710,[20]=140070,[21]=140202,[22]="75776,309"},[242]={[1]=137550,[2]=137592,[3]=137634,[4]=137676,[5]=137718,[6]=137760,[7]=137802,[8]=137844,[9]=139711,[10]=139753,[11]=139801,[12]=139933,[13]=139981,[14]=140023,[15]=140161,[16]=140203,[17]="76086,201",[18]=139843,[19]=139891,[20]=140071,[21]=140113,[22]="76288,147"},[243]={[4]="77056,19",[1]=77516,[2]=77517,[3]="76916,139"},[244]={[2]="77518,1",[1]="77076,159"},[245]={[1]="77236,75",[2]="77312,203",[3]="77520,5"},[246]={[1]="78048,12",[2]="78068,4",[3]="78074,1",[4]="78223,12",[5]="78243,4",[6]="78249,1",[7]="78258,12",[8]="78278,4",[9]="78319,1",[10]=78065,[11]=78080,[12]="78284,1",[13]="78293,10",[14]=78313,[15]="78097,1",[16]="78112,1",[17]="78102,8",[18]=78314,[19]="78117,13",[20]=78305,[21]="78138,4",[22]=78315,[23]="78083,12",[24]="78173,4",[25]="78179,1",[26]="78188,12",[27]=78316,[28]="78144,1",[29]="78153,12",[30]=78317,[31]=78304,[32]=78888,[33]="78208,7",[34]=78889,[35]=78890,[36]=78891,[37]=78892,[38]=78893},[247]={[1]=78585,[2]=78896,[3]=78897,[4]=78584,[5]="78328,7",[6]="78336,4",[7]="78356,19",[8]="78377,1",[9]="78384,1",[10]="78391,19",[11]="78426,19",[12]="78461,19",[13]="78496,19",[14]="78531,19",[15]="78566,17",[16]="78601,6",[17]="78894,1",[18]="78898,1"},[248]={[1]=78867,[2]=78868,[3]=78870,[4]=78871,[5]=78872,[6]=78865,[7]=78900,[8]="78713,19",[9]=78866,[10]=78869,[11]="78783,19",[12]="78608,19",[13]="78643,19",[14]="78678,19",[15]=78903,[16]="78748,19",[17]="78818,19",[18]=78632,[19]=78640,[20]=78665,[21]=78673,[22]=78864,[23]="78853,10",[24]=78901,[25]=78902,[26]=78904,[27]=78905},[253]={[4]="79235,12",[1]="78906,9",[2]=79220,[3]="78930,261"},[256]={[1]="82176,47"},[257]={[1]="82128,47"},[258]={[2]="82432,148",[1]="82411,20"},[259]={[2]="82688,75",[1]="82602,85"},[260]={[1]="82229,161"},[261]={[1]="82966,169"},[262]={[2]="83200,118",[1]="83157,42"},[263]={[2]="82944,1",[1]="82784,159"},[264]={[1]=94452,[2]=94456,[3]=94724,[4]=94728,[5]=94996,[6]=95000},[265]={[1]="94460,3",[2]="94464,3",[3]="94596,7",[4]="94732,7",[5]="94868,7",[6]="95004,7",[7]="95140,7"},[266]={[1]="94468,7",[2]="94604,7",[3]="94740,7",[4]="94876,7",[5]="95012,7",[6]="95148,7"},[267]={[1]="94476,7",[2]="94612,7",[3]="94748,7",[4]="94884,7",[5]="95020,7",[6]="95156,7"},[268]={[1]="94484,7",[2]="94620,7",[3]="94756,7",[4]="94892,7",[5]="95028,7",[6]="95164,3",[7]="95168,3"},[269]={[1]="94492,7",[2]="94628,7",[3]="94764,7",[4]="94900,7",[5]="95036,3",[6]="95040,3",[7]="95172,7"},[270]={[1]="94500,7",[2]="94636,7",[3]="94772,7",[4]="94908,3",[5]="94912,3",[6]="95044,7",[7]="95180,7"},[271]={[1]="94508,7",[2]="94644,7",[3]="94780,3",[4]="94784,3",[5]="94916,7",[6]="95052,7",[7]="95188,7"},[272]={[1]="94516,7",[2]="94652,3",[3]="94656,3",[4]="94788,7",[5]="94924,7",[6]="95060,7",[7]="95196,7"},[273]={[1]="94524,3",[2]="94528,3",[3]="94660,7",[4]="94796,7",[5]="94932,7",[6]="95068,7",[7]="95204,7"},[274]={[1]="94532,7",[2]="94668,7",[3]="94804,7",[4]="94940,7",[5]="95076,7",[6]="95212,7"},[275]={[1]="94540,7",[2]="94676,7",[3]="94812,7",[4]="94948,7",[5]="95084,7",[6]="95220,7"},[276]={[1]="94548,7",[2]="94684,7",[3]="94820,7",[4]="94956,7",[5]="95092,7",[6]="95228,3",[7]="95232,3"},[277]={[1]="94556,7",[2]="94692,7",[3]="94828,7",[4]="94964,7",[5]="95100,3",[6]="95104,3",[7]="95236,7"},[278]={[1]="94564,7",[2]="94700,7",[3]="94836,7",[4]="94972,3",[5]="94976,3",[6]="95108,7",[7]="95244,7"},[279]={[1]="94572,7",[2]="94708,7",[3]="94844,3",[4]="94848,3",[5]="94980,7",[6]="95116,7",[7]="95252,7"},[280]={[1]="94580,7",[2]="94716,3",[3]="94720,3",[4]="94852,7",[5]="94988,7",[6]="95124,7",[7]="95260,7"},[281]={[1]=7369,[2]=15567,[3]=95965,[4]=95973,[5]=96170,[6]=96245,[7]=96347,[8]=36581,[9]=96244,[10]=96351,[11]="44459,1",[12]=7392,[13]=96243,[14]="95866,13",[15]="36574,1",[16]="43576,6",[17]="95862,2",[18]="95970,2",[19]="95974,15",[20]=96383,[21]="96010,1",[22]="96024,5",[23]="96031,1",[24]="96046,4",[25]=1119,[26]="96052,1",[27]="96201,1",[28]=7368,[29]="96246,8",[30]="96343,3",[31]=1115,[32]=7358,[33]="96241,1",[34]="96159,10",[35]="96171,8",[36]="96236,1",[37]="96352,10",[38]=36573,[39]=43574,[40]="96238,2",[41]="96348,2",[42]=44458,[43]=95828},[282]={[1]=98635,[2]="7317,1",[3]="98522,1",[4]=98636,[5]="34891,1",[6]="42970,1",[7]="98509,12",[8]=7294,[9]=7301,[10]="98524,35",[11]="98637,33",[12]=7314,[13]="98560,74"},[283]={[1]=7508,[2]="99288,2",[3]=7515,[4]=99429,[5]="99298,3",[6]="99303,1",[7]=99438,[8]="99308,2",[9]="99317,3",[10]=99430,[11]=99428,[12]=7545,[13]=99418,[14]=99419,[15]=54633,[16]="99322,5",[17]=10086,[18]=99283,[19]="99328,8",[20]=34393,[21]=42952,[22]=99284,[23]=99302,[24]="99338,6",[25]="99348,2",[26]="99358,6",[27]="99368,2",[28]="99378,6",[29]="99388,2",[30]="99398,1",[31]=99321,[32]="99401,3",[33]="99408,2",[34]=7556,[35]="99420,4",[36]="99439,5",[37]=99400},[284]={[1]=42951,[2]=99472,[3]=99473,[4]="7529,1",[5]="7568,1",[6]="99465,6",[7]=7546,[8]="99474,109",[9]=10087,[10]=34392,[11]=34395,[12]="99584,42",[13]=7520,[14]=7525,[15]=34397},[285]={[16]=100535,[1]=15599,[2]=15603,[3]="100536,29",[4]=15608,[5]=15619,[6]="100533,1",[7]=15658,[8]=15660,[9]=34487,[10]="100397,133",[11]=42978,[12]=100396,[13]=100531,[14]=100532,[15]="34483,1"},[286]={[1]=15604,[2]=26164,[3]=100602,[4]=100615,[5]=100617,[6]=15594,[7]="42974,1",[8]=15610,[9]=100612,[10]="100587,14",[11]=15620,[12]=15623,[13]=15625,[14]=100616,[15]="100603,4",[16]="100608,3",[17]=34485,[18]="100613,1",[19]=34482,[20]="100618,130"},[287]={[1]=1674,[2]=1697,[3]=1799,[4]=4290,[5]=7347,[6]=10272,[7]=34663,[8]=34664,[9]="97590,124",[10]=97715,[11]="97716,13",[12]=42997,[13]=97730,[14]="97731,20",[15]=42993},[288]={[1]=10880,[2]=10904,[3]=10912,[4]=42999,[5]=43003,[6]=98445,[7]="10848,1",[8]=98452,[9]=98454,[10]=98456,[11]=98464,[12]="98327,72",[13]=10893,[14]=10900,[15]=98453,[16]="98400,1",[17]=34710,[18]=34712,[19]="98402,42",[20]="98446,5",[21]="98457,6",[22]=98455,[23]="98465,23"},[289]={[1]="99778,23",[2]=15524,[3]=15526,[4]="99803,13",[5]=15543,[6]=15578,[7]="43004,1",[8]=34753,[9]=43009,[10]=54863,[11]=99664,[12]="99647,15",[13]="99666,100",[14]=99665,[15]=99777,[16]=99767,[17]=54865,[18]=99827,[19]="99768,8",[20]=99663,[21]=99802},[290]={[1]=10921,[2]=10927,[3]=10934,[4]=27215,[5]=34797,[6]=34799,[7]=43012,[8]=43014,[9]=101101,[10]=101102,[11]=101103,[12]=101120,[13]="100951,149",[14]="101104,15"},[291]={[1]=6900,[2]=7382,[3]=7384,[4]=7418,[5]="7387,1",[6]=34977,[7]=34979,[8]=40300,[9]="96733,15",[10]=43016,[11]="96587,32",[12]=96726,[13]="96621,104",[14]=43019,[15]=96728,[16]=96727,[17]=96732,[18]="96729,1",[19]=96620,[20]=96731},[292]={[16]="97406,1",[1]=4308,[2]=4318,[3]=4325,[4]=4326,[5]=43027,[6]="97217,36",[7]=97254,[8]=97297,[9]=7325,[10]="97255,24",[11]=34577,[12]="97298,106",[13]=43022,[14]=43025,[15]="97280,16"},[293]={[1]=4305,[2]=4319,[3]=4323,[4]="97036,2",[5]=4827,[6]="34572,2",[7]="97039,157",[8]=43026,[9]=97035},[294]={[1]=10983,[2]="101352,4",[3]=10150,[4]=10989,[5]=11002,[6]=11006,[7]="101358,11",[8]=11008,[9]=43043,[10]="101371,2",[11]=43045,[12]=101351,[13]=101357,[14]=101376,[15]="34844,1",[16]="101324,26",[17]=101370,[18]="101377,116",[19]=101374,[20]=101375},[295]={[1]=32167,[2]="29073,1",[3]=32165,[4]=29071,[5]="102626,29",[6]="102656,131"},[296]={[1]=29097,[2]=32166,[3]="103142,9",[4]=32163,[5]=32191,[6]="102990,150",[7]=34369,[8]=54245,[9]=61001,[10]=103141},[297]={[1]=32177,[2]=103310,[3]=103315,[4]=103316,[5]=103317,[6]=103318,[7]=103319,[8]="103207,20",[9]=34371,[10]=103182,[11]=103320,[12]="103271,7",[13]=103331,[14]=103335,[15]="103250,7",[16]="103172,1",[17]="103183,1",[18]="103187,7",[19]="103203,2",[20]="103229,7",[21]="103245,2",[22]=103339,[23]="103266,2",[24]="103287,2",[25]=29070,[26]="103313,1",[27]="103329,1",[28]=28122,[29]=29084,[30]="103308,1",[31]=61000,[32]=103334,[33]="103292,3",[34]=103336,[35]=103337,[36]=103338,[37]=103340,[38]=103341,[39]="103296,3"},[298]={[1]=16212,[2]=22125,[3]=22127,[4]=103407,[5]=103504,[6]=103505,[7]=103506,[8]=103507,[9]=103508,[10]=103509,[11]="103376,9",[12]=103510,[13]=103511,[14]=103523,[15]=103525,[16]="103363,1",[17]="103408,10",[18]=103526,[19]="103397,9",[20]=103502,[21]=15546,[22]=26168,[23]=103460,[24]=103481,[25]=29467,[26]=103439,[27]="103420,3",[28]="103424,3",[29]="103441,7",[30]="103462,7",[31]=34373,[32]=103527,[33]=103528,[34]=103529,[35]=103530,[36]=103531,[37]=103532,[38]="103483,7"},[299]={[1]=16202,[2]=16230,[3]=29451,[4]=104038,[5]="103927,8",[6]="104039,49",[7]=7666,[8]=22118,[9]="103936,101"},[300]={[16]="104269,5",[1]=16215,[2]=22112,[3]=22116,[4]=29461,[5]=29466,[6]=104255,[7]=104268,[8]=104275,[9]=15679,[10]="104109,82",[11]=61004,[12]="104192,62",[13]="104256,11",[14]="104276,2",[15]=16211},[301]={[8]="104448,13",[1]=15545,[2]=22121,[3]=28114,[4]=34372,[5]=5921,[6]=134673,[7]="104300,147"},[302]={[8]="106132,2",[1]=16042,[2]=23727,[3]=54988,[4]=54991,[5]=23713,[6]="105965,18",[7]="105984,142"},[303]={[8]="106496,2",[1]=16046,[2]="106329,84",[3]=23726,[4]=106414,[5]=16146,[6]=34375,[7]="106415,80"},[304]={[4]="106752,111",[1]="16044,1",[2]=16149,[3]="106702,49"},[305]={[1]=33179,[2]=33288,[3]="105401,70",[4]=33153,[5]="105472,98"},[307]={[1]=33283,[2]="107075,161",[3]=33286},[308]={[1]=22176,[2]=107598,[3]="107448,71",[4]=22156,[5]="107520,77",[6]="107599,11",[7]=34382,[8]="107612,1",[9]="107616,1"},[309]={[4]="107776,15",[1]=22196,[2]=23669,[3]="107630,145"},[310]={[1]=105327,[2]=105362,[3]=105363,[4]=105367,[5]="105219,4",[6]=105355,[7]=22169,[8]=105347,[9]="105234,9",[10]=22194,[11]="105253,19",[12]=105375,[13]="105274,9",[14]="105294,9",[15]=105227,[16]=105247,[17]=105354,[18]=105374,[19]="105314,9",[20]=105287,[21]=105307,[22]="105334,9",[23]="105356,5",[24]="105376,4"},[311]={[1]=54516,[2]="44760,1",[3]="54249,1",[4]=54246,[5]=111292,[6]="111133,154",[7]="111289,2",[8]=44728,[9]="111293,1",[10]=44732,[11]=111288},[313]={[1]=55934,[2]=55964,[3]=55970,[4]=55976,[5]=55982,[6]=55988,[7]=55994,[8]=133786,[9]=133787,[10]=133788,[11]=133820,[12]=133821,[13]=133822,[14]=133837,[15]=133838,[16]=133839,[17]=133854,[18]="133923,1",[19]=133855,[20]=133856,[21]="133872,1",[22]=133871,[23]=133888,[24]="133803,2",[25]=133889,[26]=133890,[27]=133922},[314]={[1]="133796,2",[2]="133811,1",[3]="133814,1",[4]=55983,[5]=55989,[6]="133778,3",[7]=55995,[8]=133795,[9]=133813,[10]=133845,[11]=133862,[12]=133865,[13]=133879,[14]=133896,[15]=133883,[16]=55965,[17]=55971,[18]=133794,[19]="133828,4",[20]=55977,[21]="133846,3",[22]=133917,[23]=133930,[24]="133863,1",[25]="133880,2",[26]=55935,[27]=133866,[28]="133914,2"},[315]={[1]=55996,[2]=133884,[3]=133885,[4]=55990,[5]="133799,3",[6]=94206,[7]=133886,[8]="133782,3",[9]="133816,3",[10]=133918,[11]=55978,[12]=55966,[13]=55972,[14]="133833,3",[15]=55984,[16]="133850,3",[17]="133867,3",[18]="133919,2",[19]=133887},[316]={[1]=55937,[2]=55967,[3]=55973,[4]=55979,[5]=55985,[6]=55991,[7]=55997,[8]=133789,[9]=133806,[10]=133823,[11]=133840,[12]=133857,[13]=133874,[14]=133891,[15]=133925},[317]={[1]=55938,[2]=55968,[3]=55980,[4]=55992,[5]=55986,[6]=55998,[7]="133807,2",[8]=55974,[9]="57458,1",[10]=94207,[11]="57448,6",[12]="57455,2",[13]="133876,1",[14]="133841,2",[15]="133892,2",[16]=94208,[17]="133825,1",[18]=133824,[19]="133790,2",[20]=133875,[21]="133858,2",[22]=133909,[23]=133926,[24]=133927,[25]=133928},[318]={[1]=55939,[2]=55969,[3]=55975,[4]=55981,[5]=55987,[6]=55993,[7]=55999,[8]=133793,[9]=133810,[10]=133827,[11]=133844,[12]=133861,[13]=133878,[14]=133895,[15]=133929},[320]={[8]="126051,1",[1]=126076,[2]=126080,[3]="122792,87",[4]=126067,[5]=126045,[6]=126049,[7]="122880,81"},[321]={[8]="123136,8",[1]=126053,[2]=126057,[3]=126069,[4]=126079,[5]=126041,[6]=126044,[7]="122983,152"},[322]={[1]=126043,[2]=126054,[3]=126058,[4]=126059,[5]="122610,13",[6]="122624,147"},[323]={[1]=137551,[2]=137593,[3]=137635,[4]=137677,[5]=137719,[6]=137761,[7]=139712,[8]=139754,[9]=139802,[10]=139982,[11]=140024,[12]=137803,[13]=137845,[14]=139844,[15]=139892,[16]=139934,[17]="121551,304",[18]=140072,[19]=140114,[20]=140162,[21]=140204,[22]="121856,44"},[324]={[1]=137552,[2]=137594,[3]=137636,[4]=137678,[5]=137720,[6]=137762,[7]=137804,[8]=137846,[9]=139713,[10]=139755,[11]=139803,[12]="121901,349",[13]=139845,[14]=139893,[15]=139935,[16]=139983,[17]=140025,[18]=140073,[19]=140115,[20]=140163,[21]=140205},[325]={[1]=137637,[2]=137679,[3]=137721,[4]=137763,[5]=137805,[6]=137847,[7]=139756,[8]=139804,[9]=139846,[10]=139894,[11]=140164,[12]=137553,[13]=137595,[14]=140026,[15]=140074,[16]=140116,[17]="122251,116",[18]=139714,[19]=139936,[20]=139984,[21]=140206,[22]="122368,232"},[326]={[1]="123166,161"},[327]={[2]="123392,117",[1]="123348,43"},[328]={[2]="123648,51",[1]="123530,117"},[329]={[1]="123721,169"},[330]={[4]="124074,19",[1]="123912,93",[2]="124014,11",[3]="124034,19"},[331]={[1]="124094,65",[2]="124160,27",[3]="124196,11",[4]="124216,19",[5]="124256,19"},[332]={[1]="124276,97",[2]="124383,11",[3]="124404,11",[4]="124416,8",[5]="124446,20"},[333]={[4]="124637,20",[1]="124467,97",[2]="124574,11",[3]="124595,20"},[334]={[2]="125952,22",[1]="125689,262"},[335]={[2]="127488,13",[1]="127332,155"},[336]={[1]="127523,161"},[337]={[2]="127232,79",[1]="127150,81"},[338]={[2]="128000,104",[1]="127935,64"},[339]={[2]="128256,31",[1]="128126,129"},[340]={[1]="127753,161"},[341]={[2]="127744,8",[1]="127705,38"},[342]={[2]="128320,35",[1]="128308,11"},[343]={[1]="128554,169"},[344]={[2]="128768,138",[1]="128745,22"},[345]={[1]="128372,137",[2]="128512,21",[3]=128511},[346]={[1]="129109,170",[2]="129280,16",[3]="129298,1"},[347]={[1]="129300,161"},[348]={[2]="129024,64",[1]="128927,96"},[349]={[1]="129482,47"},[350]={[2]="129536,41",[1]="129530,5"},[351]={[1]=137554,[2]=137596,[3]=137638,[4]=137680,[5]=137722,[6]=137764,[7]=137806,[8]=137848,[9]=139715,[10]=139757,[11]=139805,[12]=139937,[13]=139985,[14]=140027,[15]=140075,[16]=140207,[17]="130370,189",[18]=139847,[19]=139895,[20]=140117,[21]=140165,[22]="130560,159"},[352]={[1]=137555,[2]=137597,[3]=137639,[4]=137681,[5]=137723,[6]=137765,[7]=137807,[8]="130720,349",[9]=137849,[10]=139716,[11]=139758,[12]=139806,[13]=139848,[14]=139896,[15]=139938,[16]=139986,[17]=140028,[18]=140076,[19]=140118,[20]=140166,[21]=140208},[353]={[1]=137556,[2]=137724,[3]=137766,[4]=137808,[5]=137850,[6]=139807,[7]=139849,[8]=139897,[9]=139939,[10]=139987,[11]=140029,[12]=137598,[13]=137640,[14]=139717,[15]=140167,[16]=140209,[17]="131070,1",[18]=137682,[19]=140077,[20]="131072,347",[21]=139759,[22]=140119},[354]={[8]="133598,1",[1]=133227,[2]=133232,[3]=133233,[4]=133234,[5]=133238,[6]="132848,15",[7]="132864,153"},[355]={[8]="133230,1",[1]=133600,[2]=133601,[3]="133039,80",[4]=133235,[5]=133226,[6]=133228,[7]="133120,80"},[356]={[1]=133236,[2]=133237,[3]=133239,[4]="132666,161",[5]="133596,1",[6]=133229,[7]=133240},[357]={[1]=133253,[2]="133251,1",[3]="133302,2",[4]=133270,[5]="133337,1",[6]="133285,2",[7]=133319,[8]=133336,[9]=133353,[10]=133387,[11]="133268,1",[12]="133320,1",[13]="133354,1",[14]="133388,1"},[358]={[1]="133259,4",[2]=133280,[3]=133327,[4]="133243,3",[5]="133276,3",[6]=133310,[7]="133293,4",[8]=133361,[9]="133328,3",[10]="133344,4",[11]="133379,3",[12]="133312,2",[13]=133311,[14]=133395},[359]={[1]=133281,[2]=133282,[3]="133248,2",[4]=133315,[5]=133316,[6]=133332,[7]=133333,[8]=133349,[9]="133266,1",[10]="133317,1",[11]="133351,1",[12]="133283,1",[13]=133247,[14]="133385,1",[15]="133264,1",[16]="133298,3",[17]=133350,[18]=133383,[19]="133334,1",[20]=133384},[360]={[8]=133390,[1]=133254,[2]=133271,[3]=133288,[4]=133305,[5]=133322,[6]=133339,[7]=133356},[361]={[1]=133257,[2]=133323,[3]=133340,[4]=133357,[5]="133255,1",[6]=133391,[7]="133272,2",[8]="133289,2",[9]="133307,1",[10]="133324,1",[11]="133341,1",[12]="133358,1",[13]="133392,1",[14]=133306},[362]={[8]=133394,[1]=133258,[2]=133275,[3]=133292,[4]=133309,[5]=133326,[6]=133343,[7]=133360},[363]={[1]=133455,[2]="133421,2",[3]=133438,[4]="133404,2",[5]=133457,[6]=133506,[7]=133440,[8]=133456,[9]="133472,2",[10]="133489,2",[11]=133439,[12]="133507,1"},[364]={[1]=133416,[2]="133429,4",[3]="133446,4",[4]=133463,[5]="133498,3",[6]="133396,3",[7]=133532,[8]="133412,3",[9]="133464,3",[10]=133497,[11]=133514,[12]="133480,4"},[365]={[1]=133403,[2]=133434,[3]="133400,1",[4]=133402,[5]="133417,3",[6]="133453,1",[7]=133451,[8]="133436,1",[9]=133452,[10]=133468,[11]=133469,[12]=133502,[13]=133503,[14]="133470,1",[15]=133486,[16]=133435,[17]=133485,[18]="133487,1",[19]="133504,1"},[366]={[1]=133407,[2]=133424,[3]=133441,[4]=133458,[5]=133475,[6]=133492,[7]=133509},[367]={[1]=133444,[2]=133459,[3]=133460,[4]=133461,[5]=133510,[6]="133408,1",[7]="133442,1",[8]="133476,2",[9]="133493,2",[10]="133425,2",[11]=133410,[12]="133511,1"},[368]={[1]=133411,[2]=133428,[3]=133445,[4]=133462,[5]=133479,[6]=133496,[7]=133513},[369]={[1]=79699,[2]="133633,2",[3]="133667,2",[4]="133735,2",[5]="71118,17",[6]=79700,[7]=79701,[8]=133650,[9]=133651,[10]="79876,2",[11]=133652,[12]=133702,[13]=133701,[14]=133703,[15]=133719,[16]=133720,[17]=133718,[18]="133684,2",[19]="133769,2"},[370]={[1]="71106,10",[2]="71148,2",[3]="71661,1",[4]="71663,1",[5]="79704,1",[6]="79866,3",[7]="79879,1",[8]="133641,4",[9]="133658,4",[10]="133675,4",[11]="133692,4",[12]=133628,[13]="133709,4",[14]="133726,4",[15]=71151,[16]=133625,[17]=133627,[18]=71117,[19]="133760,4",[20]=79697,[21]=79698,[22]=133626,[23]=133743,[24]=133777},[371]={[32]=133733,[1]=71104,[2]=71138,[3]=79696,[4]=79702,[5]=79881,[6]="71139,2",[7]=71105,[8]=79875,[9]=133629,[10]=133630,[11]="133680,3",[12]=133631,[13]=133632,[14]="71136,1",[15]="133646,1",[16]=133648,[17]=133664,[18]=133665,[19]="133714,3",[20]=133649,[21]=133663,[22]="133765,3",[23]=133666,[24]=133697,[25]=133698,[26]=133700,[27]=133734,[28]="71100,3",[29]=133699,[30]=133731,[31]=133732},[372]={[1]=71142,[2]=71143,[3]=71144,[4]=79703,[5]=79874,[6]=133636,[7]=133653,[8]=133670,[9]=133687,[10]=133704,[11]=133721,[12]=133738,[13]="71145,2",[14]=133755,[15]=133772},[373]={[1]=71169,[2]=79871,[3]=133690,[4]=133722,[5]=133723,[6]=133724,[7]="71152,15",[8]="79706,2",[9]="79872,1",[10]="133637,2",[11]="133654,2",[12]="133671,2",[13]="133688,1",[14]=71168,[15]="133705,2",[16]="133739,2",[17]="133773,2"},[374]={[1]=79709,[2]=79870,[3]=133640,[4]=133657,[5]=133674,[6]=133691,[7]=133708,[8]="71170,5",[9]=133725,[10]=133742,[11]=133776},[380]={[1]="134799,2",[2]="134956,2",[3]="134962,5",[4]="134976,1",[5]=134975,[6]="135131,2"},[381]={[2]=134960,[1]=134955},[382]={[1]=135204,[2]="134699,3",[3]="134704,1",[4]="135162,5",[5]=135215,[6]=134692,[7]="135168,35",[8]="135205,5",[9]="135211,3",[10]="135216,1",[11]="135218,105"},[383]={[1]=135485,[2]="134715,2",[3]="134719,1",[4]=135484,[5]=135471,[6]=135483,[7]="135344,79",[8]=135487,[9]=134703,[10]=135486,[11]="135472,10",[12]="135424,46",[13]=135488,[14]="135489,16"},[384]={[1]=134722,[2]=152150,[3]="134713,1",[4]=134718,[5]="135526,153",[6]=134696,[7]=134707,[8]=134711,[9]="135680,15"},[385]={[1]=139851,[2]=139940,[3]=139941,[4]=140030,[5]=140031,[6]=140120,[7]="135717,349",[8]="138713,7",[9]="140210,1",[10]=139850,[11]="139760,1",[12]=140121},[386]={[1]=138721,[2]=138722,[3]=138723,[4]=138724,[5]=138725,[6]=138726,[7]=138727,[8]=138728,[9]=139762,[10]=139763,[11]=139943,[12]=140212,[13]="136067,124",[14]=140213,[15]=139852,[16]="140032,1",[17]=139853,[18]="140122,1",[19]=139942,[20]="136192,224"},[387]={[1]=138729,[2]=138730,[3]=138731,[4]=138732,[5]=138733,[6]=138734,[7]=138735,[8]=138736,[9]=139764,[10]=139765,[11]=140125,[12]=140214,[13]="136417,286",[14]=140215,[15]=140034,[16]="139854,1",[17]=140035,[18]="139944,1",[19]=140124,[20]="136704,62"},[388]={[1]=136895,[2]=136896,[3]=136897,[4]=136898,[5]=136899,[6]=136900,[7]=136901,[8]=136902,[9]="136767,64",[10]=136906,[11]=136907,[12]="136832,28",[13]=136908,[14]=136903,[15]="136889,5",[16]=136904,[17]=136905,[18]="136869,11"},[389]={[1]=137077,[2]=137078,[3]=137079,[4]=137080,[5]=137081,[6]=137082,[7]=137083,[8]=137084,[9]="136949,10",[10]=137088,[11]=137089,[12]="136960,82",[13]=137090,[14]=137085,[15]="137071,5",[16]=137086,[17]=137087,[18]="137051,11"},[390]={[1]="137131,84",[2]="137216,12",[3]="137259,20",[4]=137230,[5]="137238,11"},[391]={[1]="137322,97",[2]="137429,11",[3]="137450,20"},[392]={[1]="137964,19",[2]="137984,73",[3]="138066,11",[4]="138086,19",[5]="138126,12"},[393]={[4]="138308,12",[1]="138146,93",[2]="138248,11",[3]="138268,19"},[394]={[4]="138498,12",[1]="138328,97",[2]="138435,11",[3]="138456,20"},[395]={[4]="138689,12",[1]="138519,97",[2]="138626,11",[3]="138647,20"},[397]={[2]="141632,37",[1]="141622,9"},[398]={[2]="141696,21",[1]="141670,25"},[399]={[2]="140800,63",[1]="140694,105"},[400]={[1]=141028,[2]=141029,[3]=141030,[4]=141031,[5]="140885,8",[6]="140899,14",[7]=141032,[8]="140919,8",[9]=141033,[10]=141013,[11]="140928,25",[12]="140959,14",[13]=141027,[14]=141040,[15]="141019,2",[16]="141042,4",[17]=141039,[18]="141022,4",[19]="140999,13",[20]=141041,[21]="140979,14"},[401]={[2]="140544,129",[1]="140512,31"},[402]={[2]="141312,106",[1]="141249,62"},[403]={[2]="141568,33",[1]="141440,127"},[404]={[1]="141067,161"},[405]={[2]=145542,[1]="142600,169"},[406]={[2]=145543,[1]="142418,161"},[407]={[1]=145544,[2]="142236,99",[3]="142336,61"},[408]={[2]="142848,312",[1]="142791,56"},[409]={[2]="143360,170",[1]="143161,198"},[410]={[2]="143872,28",[1]="143531,340"},[411]={[1]="145011,3",[2]=145031,[3]=145112,[4]=145129,[5]="145027,3",[6]=145146,[7]=145163,[8]="145044,4",[9]="145061,4",[10]="145079,3",[11]="145095,4",[12]="145113,3",[13]="145130,3",[14]=145078,[15]="145147,3"},[412]={[16]="145156,1",[1]="145019,2",[2]=145070,[3]=145087,[4]="145036,2",[5]=145072,[6]=145053,[7]="145054,1",[8]=145121,[9]=145138,[10]=145155,[11]="145088,1",[12]=145071,[13]="145104,2",[14]="145122,1",[15]="145139,1"},[413]={[1]="145015,3",[2]="145032,3",[3]="145049,3",[4]="145066,3",[5]="145083,3",[6]="145100,3",[7]="145117,3",[8]="145134,3",[9]="145152,2",[10]=145151},[414]={[1]=145022,[2]=145039,[3]=145056,[4]=145073,[5]=145090,[6]=145107,[7]=145124,[8]=145141,[9]=145158},[415]={[1]="145040,2",[2]=145023,[3]="145108,2",[4]="145143,1",[5]="145160,1",[6]=145024,[7]=145025,[8]=145059,[9]=145074,[10]=145075,[11]=145076,[12]=145125,[13]="145091,2",[14]="145126,1",[15]=145142,[16]="145057,1",[17]=145159},[416]={[1]=145026,[2]=145043,[3]=145060,[4]=145077,[5]=145094,[6]=145111,[7]=145128,[8]=145145,[9]=145162},[417]={[1]="143901,169"},[418]={[2]="144128,133",[1]="144092,35"},[419]={[2]="144384,60",[1]="144283,100"},[420]={[1]="144465,161"},[421]={[1]="144647,161"},[422]={[2]="144896,94",[1]="144829,66"},[423]={[16]="145300,3",[1]="145164,2",[2]=145184,[3]=145282,[4]=145299,[5]="145180,3",[6]=145167,[7]=145265,[8]="145197,4",[9]=145316,[10]="145214,1",[11]="145216,2",[12]="145231,4",[13]="145248,4",[14]="145266,3",[15]="145283,3"},[424]={[16]=145240,[1]="145189,1",[2]=145257,[3]=145274,[4]="145172,1",[5]=145291,[6]=145308,[7]="145223,2",[8]="145241,1",[9]="145275,1",[10]=145174,[11]="145309,1",[12]="145206,2",[13]="145258,1",[14]="145292,1",[15]=145191},[425]={[1]="145168,3",[2]="145185,3",[3]="145202,3",[4]="145219,3",[5]="145236,3",[6]="145253,3",[7]="145270,3",[8]="145287,3",[9]="145304,3"},[426]={[1]=145175,[2]=145192,[3]=145209,[4]=145226,[5]=145243,[6]=145260,[7]=145277,[8]=145294,[9]=145311},[427]={[1]="145176,1",[2]=145178,[3]="145210,2",[4]=145195,[5]=145261,[6]="145245,1",[7]=145244,[8]=145278,[9]=145279,[10]="145193,1",[11]="145227,2",[12]="145313,1",[13]=145280,[14]=145312,[15]="145262,1",[16]="145296,1",[17]=145295},[428]={[1]=145179,[2]=145196,[3]=145213,[4]=145230,[5]=145247,[6]=145264,[7]=145281,[8]=145298,[9]=145315},[429]={[2]="146176,62",[1]="146077,98"},[430]={[1]="146259,161"},[431]={[1]="146441,169"},[432]={[1]="146632,47"},[433]={[2]="146688,153",[1]="146680,7"},[434]={[2]="146944,79",[1]="146862,81"},[435]={[2]="147200,13",[1]="147044,155"},[436]={[2]="147264,18",[1]="147235,28"},[437]={[2]="147968,349",[1]="147948,19"},[438]={[2]="148480,207",[1]="148318,161"},[439]={[2]="148992,65",[1]="148688,303"},[440]={[1]=150391,[2]=150392,[3]=150394,[4]="149240,7",[5]=150403,[6]="149248,161",[7]=150419},[441]={[1]=150399,[2]="149431,72",[3]="149504,88",[4]="150385,2",[5]="150389,1",[6]="150411,2",[7]="150415,3"},[442]={[1]=150388,[2]=150393,[3]=150395,[4]=150396,[5]=150397,[6]="149058,161",[7]=150384,[8]=150398,[9]=150400,[10]=150401,[11]=150414},[443]={[2]="150272,44",[1]="150168,103"},[444]={[1]=150139,[2]="149977,38",[3]="150016,109"},[445]={[1]="149795,141"},[446]={[1]="149613,141"},[448]={[1]="151369,148",[2]="151539,12",[3]="151552,7"},[449]={[1]="151178,117",[2]="151296,30",[3]="151348,20"},[450]={[1]="150996,43",[2]="151040,97",[3]="151158,19"},[451]={[2]="150976,19",[1]="150814,141"},[452]={[2]="152576,152",[1]="152538,37"},[453]={[2]="152832,78",[1]="152729,102"},[454]={[1]="152356,181"},[455]={[1]="153093,190"},[456]={[2]="153344,121",[1]="153284,59"},[457]={[2]="153088,4",[1]="152911,176"},[458]={[1]="152260,47"},[459]={[2]="152320,35",[1]="152308,11"},[465]={[1]=153836,[2]=153837,[3]="154836,43",[4]=153840,[5]="154880,137"},[466]={[8]="155136,75",[1]="153833,1",[2]="153838,1",[3]="155021,91",[4]="155113,1",[5]="155115,2",[6]="155118,1",[7]="155120,15"},[467]={[8]="155392,4",[1]=153835,[2]="153828,4",[3]="155215,148",[4]="155364,4",[5]="155369,1",[6]=155371,[7]="155372,19"},[468]={[2]="155648,125",[1]="155404,243"},[469]={[1]="155778,369"},[470]={[2]="156160,361",[1]="156152,7"},[471]={[2]="156928,111",[1]="156849,78"},[472]={[2]="157184,40",[1]="157043,140"},[473]={[1]="157228,181"},[474]={[2]="157696,29",[1]="157535,160"},[475]={[1]="157729,181"},[476]={[2]="157952,143",[1]="157914,37"},[478]={[1]="158160,35",[2]="158199,8",[3]="158208,2"},[479]={[2]="158259,11",[1]="158222,35"},[480]={[1]="158316,369"},[481]={[2]="158720,339",[1]="158690,29"},[482]={[2]="159232,201",[1]="159064,167"},[487]={[1]="160753,1",[2]=159699,[3]="160627,122",[4]=159701,[5]="159729,1",[6]=160750,[7]="160751,1",[8]=159726,[9]="160755,12",[10]=159702,[11]=159714,[12]=159715,[13]=159717,[14]=159711,[15]=159712,[16]="160768,49",[17]=159713},[488]={[1]=160977,[2]=160978,[3]="159718,2",[4]=160980,[5]=160987,[6]=160998,[7]=160999,[8]=161000,[9]="159697,1",[10]=161001,[11]=161002,[12]="159722,2",[13]=159727,[14]=159693,[15]=159700,[16]=160979,[17]="160821,151",[18]="160974,2",[19]="160981,5",[20]="160988,9",[21]=159707,[22]=160973},[489]={[1]="159703,3",[2]="159708,2",[3]=159721,[4]=159728,[5]="159694,2",[6]="161006,1",[7]=159725,[8]="161009,14",[9]=161008,[10]="161024,163"},[490]={[2]="161280,310",[1]="161221,58"},[491]={[2]="161792,172",[1]="161595,196"},[492]={[1]=163701,[2]=163702,[3]=163703,[4]=163736,[5]=163737,[6]=163742,[7]="162374,130",[8]=163743,[9]=163749,[10]=163748,[11]=163761,[12]=163762,[13]=163767,[14]=163768,[15]=163754,[16]=163755,[17]="163608,1",[18]=163700},[493]={[1]=163552,[2]="162917,130",[3]="163539,1",[4]=163551,[5]=163553,[6]=163543,[7]=163542,[8]="163561,2",[9]="163571,2",[10]="163580,2",[11]=163541,[12]=163693,[13]="163590,2",[14]=163692,[15]="163600,2",[16]="163672,11",[17]=163538,[18]=163691},[494]={[2]="162304,66",[1]="162229,74"},[495]={[1]=163544,[2]=163554,[3]=163564,[4]=163574,[5]=163583,[6]=163593,[7]=163625,[8]=163626,[9]=163627,[10]=163628,[11]=163629,[12]=163633,[13]=163634,[14]=163635,[15]=163636,[16]=163684,[17]="162772,43",[18]=163528,[19]=163630,[20]=163631,[21]=163632,[22]="162816,97"},[496]={[1]=163606,[2]=163607,[3]=163752,[4]=163759,[5]=163760,[6]=163765,[7]="161965,82",[8]="163696,3",[9]="163731,1",[10]="163740,1",[11]="163746,1",[12]=163766,[13]="162048,48"},[497]={[1]=163532,[2]=163533,[3]=163534,[4]=163535,[5]=163536,[6]=163537,[7]=163548,[8]=163549,[9]=163550,[10]=163558,[11]=163559,[12]=163560,[13]=163568,[14]=163569,[15]=163570,[16]="162508,51",[17]=163578,[18]=163660,[19]=163661,[20]=163663,[21]="162560,79",[22]=163662,[23]=163598,[24]=163579,[25]="163664,7",[26]=163587,[27]=163588,[28]=163589,[29]="163688,2",[30]=163597,[31]=163599},[498]={[1]=163695,[2]=163729,[3]=163730,[4]=163738,[5]=163739,[6]="162100,125",[7]=163744,[8]=163745,[9]=163756,[10]="163603,1",[11]=163757,[12]=163694,[13]="163763,1",[14]=163750,[15]=163751},[499]={[1]=163685,[2]=163686,[3]="162761,1",[4]=163531,[5]="163545,2",[6]="162763,5",[7]="162643,117",[8]=163687,[9]="163529,1",[10]="163555,2",[11]="163565,2",[12]="163575,2",[13]="163584,2",[14]="163594,2",[15]="163648,11"},[501]={[1]=164291},[503]={[1]=163052},[505]={[1]=163451},[506]={[2]="163328,98",[1]="163057,270"},[513]={[2]="164352,131",[1]="164293,58"},[514]={[2]="164608,60",[1]="164487,120"},[515]={[1]="164672,181"},[516]={[2]="164864,183",[1]="164857,6"},[517]={[2]="165120,112",[1]="165051,68"},[518]={[2]="165376,41",[1]="165236,139"},[519]={[1]=165879},[520]={[1]=165880},[521]={[1]=165899},[522]={[1]=166213,[2]="166211,1",[3]="166297,1",[4]=166228,[5]="166194,2",[6]=166229,[7]="166245,2",[8]=166230,[9]=166279,[10]="166262,2",[11]="166315,2",[12]=166296,[13]="166280,1"},[523]={[1]="166186,2",[2]=166189,[3]="166202,4",[4]="166219,4",[5]="166236,4",[6]="166253,4",[7]=166270,[8]=166287,[9]=166304,[10]="166288,3",[11]="166307,3",[12]=166271,[13]="166272,2",[14]=166323},[524]={[1]="166190,3",[2]="166209,1",[3]="166226,1",[4]=166207,[5]="166260,1",[6]="166294,1",[7]=166224,[8]=166225,[9]="166241,3",[10]="166277,1",[11]=166258,[12]=166259,[13]="166292,1",[14]=166208,[15]=166275,[16]="166311,3",[17]=166276},[525]={[8]=166318,[1]=166197,[2]=166214,[3]=166231,[4]=166248,[5]=166265,[6]=166282,[7]=166299},[526]={[1]=166200,[2]="166215,2",[3]=166283,[4]="166198,1",[5]=166300,[6]="166232,2",[7]="166250,1",[8]="166267,1",[9]="166301,1",[10]="166284,1",[11]="166319,2",[12]=166249,[13]=166266},[527]={[8]=166322,[1]=166201,[2]=166218,[3]=166235,[4]=166252,[5]=166269,[6]=166286,[7]=166303},[528]={[1]=166091,[2]=166108,[3]=166125,[4]="166074,2",[5]=166142,[6]=166143,[7]="166109,1",[8]="166057,2",[9]="166092,1",[10]="166177,2",[11]=166144,[12]="166126,1",[13]=166159,[14]="166160,1"},[529]={[1]=166086,[2]="166049,3",[3]=166116,[4]=166167,[5]=166185,[6]="166065,4",[7]="166082,3",[8]="166099,4",[9]="166117,3",[10]="166150,4",[11]="166134,3",[12]=166133,[13]="166169,3"},[530]={[1]=166105,[2]=166155,[3]="166070,3",[4]=166121,[5]=166088,[6]=166138,[7]="166106,1",[8]="166140,1",[9]=166139,[10]=166055,[11]=166104,[12]=166156,[13]=166173,[14]=166056,[15]=166087,[16]="166157,1",[17]=166122,[18]="166053,1",[19]="166089,1",[20]=166176,[21]="166123,1",[22]="166174,1"},[531]={[8]=166180,[1]=166060,[2]=166077,[3]=166094,[4]=166111,[5]=166128,[6]=166145,[7]=166162},[532]={[1]="166061,2",[2]=166080,[3]="166129,2",[4]="166164,1",[5]=166095,[6]=166097,[7]=166112,[8]=166114,[9]=166163,[10]="166078,1",[11]=166096,[12]="166146,2",[13]=166113,[14]="166181,2"},[533]={[8]=166184,[1]=166064,[2]=166081,[3]=166098,[4]=166115,[5]=166132,[6]=166149,[7]=166166},[534]={[1]="167035,4",[2]="167040,30",[3]="167072,11"},[535]={[2]="167148,11",[1]="167111,35"},[536]={[1]=167396,[2]=167398,[3]=167402,[4]=167404,[5]="167430,161",[6]=167407,[7]=167426},[537]={[1]="167595,77",[2]=167417,[3]=167419,[4]=167422,[5]=167674,[6]="167676,1",[7]="167680,84",[8]=167393,[9]=167427,[10]=167673,[11]=167675,[12]=167678,[13]=167679},[538]={[1]="167910,1",[2]=167392,[3]="167913,8",[4]=167390,[5]=167397,[6]="167410,3",[7]=167400,[8]=167416,[9]=167425,[10]="167926,1",[11]=167904,[12]=167909,[13]=167912,[14]=167922,[15]="167768,133",[16]=167923,[17]=167929,[18]=167903,[19]=167925,[20]=167928,[21]=167924,[22]="167905,3",[23]=167902},[539]={[1]="167999,369"},[540]={[2]="168448,294",[1]="168373,74"},[541]={[2]="168960,156",[1]="168747,212"},[542]={[2]="169216,71",[1]="169118,97"},[543]={[1]="169291,161"},[544]={[2]="169472,145",[1]="169456,15"},[557]={[1]=169883,[2]=169914,[3]="169880,1",[4]=169882,[5]="169897,3",[6]="169933,1",[7]=169931,[8]="169916,1",[9]=169932,[10]=169948,[11]=169949,[12]=169982,[13]=169983,[14]="169950,1",[15]=169966,[16]=169915,[17]=169965,[18]="169967,1",[19]="169984,1"},[558]={[1]="169909,4",[2]=169896,[3]="169892,3",[4]="169876,3",[5]=169977,[6]=169994,[7]="169926,4",[8]="169944,3",[9]=169943,[10]="169960,4",[11]="169978,3"},[559]={[1]=169935,[2]="169901,2",[3]=169918,[4]="169884,2",[5]=169937,[6]=169986,[7]=169920,[8]=169936,[9]="169952,2",[10]="169969,2",[11]=169919,[12]="169987,1"},[560]={[1]=169887,[2]=169904,[3]=169921,[4]=169938,[5]=169955,[6]=169972,[7]=169989},[561]={[1]=169924,[2]=169939,[3]=169940,[4]=169941,[5]=169990,[6]="169888,1",[7]="169922,1",[8]="169956,2",[9]="169973,2",[10]="169905,2",[11]=169890,[12]="169991,1"},[562]={[1]=169891,[2]=169908,[3]=169925,[4]=169942,[5]=169959,[6]=169976,[7]=169993},[563]={[1]=170034,[2]=170067,[3]=170068,[4]="169999,3",[5]=170083,[6]=170084,[7]=170085,[8]=170099,[9]="170016,1",[10]=170100,[11]="170018,1",[12]=170033,[13]=170086,[14]="170050,3",[15]=170101,[16]=170102,[17]="170035,1",[18]="170069,1",[19]=170116,[20]=170117,[21]=170118,[22]=170119},[564]={[1]=169998,[2]=170079,[3]=170095,[4]=170096,[5]=170111,[6]=170112,[7]="170028,4",[8]="169995,2",[9]="170011,4",[10]=170113,[11]=171547,[12]="170062,4",[13]="170080,2",[14]="170097,1",[15]="170114,1",[16]=170128,[17]="170045,2",[18]="170048,1",[19]=171548},[565]={[1]=170087,[2]=170088,[3]=170103,[4]="170003,1",[5]=170005,[6]=170071,[7]="170020,1",[8]=170104,[9]="170054,2",[10]=170022,[11]=170089,[12]=170105,[13]="170037,2",[14]="170072,1",[15]=170120,[16]=170121,[17]=170122},[566]={[8]=170123,[1]=170006,[2]=170023,[3]=170040,[4]=170057,[5]=170074,[6]=170090,[7]=170106},[567]={[1]=170009,[2]=170075,[3]=170091,[4]=170092,[5]="170007,1",[6]=170107,[7]="170024,2",[8]=170108,[9]="170059,1",[10]="170076,1",[11]=170058,[12]="170041,2",[13]=170093,[14]=170109,[15]=170124,[16]=170125,[17]=170126},[568]={[8]=170127,[1]=170010,[2]=170027,[3]=170044,[4]=170061,[5]=170078,[6]=170094,[7]=170110},[569]={[2]="170240,160",[1]="170231,8"},[570]={[2]="170496,69",[1]="170404,91"},[571]={[1]="170569,161"},[572]={[2]="170752,151",[1]="170734,17"},[573]={[2]="171008,60",[1]="170907,100"},[574]={[1]="171072,161"},[575]={[1]=171436},[576]={[1]=171437},[577]={[1]="171602,35",[2]="171639,8",[3]="171648,2"},[578]={[2]="171695,11",[1]="171658,35"},[579]={[1]=174840,[2]=174848,[3]=174858,[4]=174860,[5]=174861,[6]=174862,[7]=174871,[8]=174873,[9]=174874,[10]=174863,[11]="171948,83",[12]="172032,85"},[580]={[8]="174875,1",[1]="172121,161",[2]="174865,2",[3]=174839,[4]=174841,[5]=174846,[6]=174872,[7]="174869,1"},[581]={[1]=174845,[2]=174849,[3]=174850,[4]=174852,[5]="172286,1",[6]=174853,[7]=174856,[8]="172288,159",[9]="174842,1",[10]=174855},[582]={[2]="172544,280",[1]="172455,88"},[583]={[2]="173056,142",[1]="172829,226"},[584]={[2]="173568,4",[1]="173203,364"},[585]={[1]="173857,141"},[586]={[1]=174581,[2]=174582,[3]=174591,[4]=174592,[5]=174601,[6]=174602,[7]=174611,[8]="173728,95",[9]="174631,1",[10]="174641,1",[11]=174612,[12]=174621,[13]=174622,[14]="173824,29",[15]="174651,1"},[587]={[16]=174399,[1]=174400,[2]=174409,[3]=174410,[4]=174419,[5]=174420,[6]=174429,[7]=174438,[8]=174439,[9]=174457,[10]=174466,[11]=174467,[12]=174476,[13]=174448,[14]=174449,[15]="173591,133"},[588]={[1]=174284,[2]=174285,[3]=174294,[4]=174295,[5]=175038,[6]="174002,77",[7]=174265,[8]=174266,[9]=174274,[10]=174275,[11]="174080,44",[12]="174225,1",[13]="174235,1",[14]=175039,[15]="174130,1",[16]=176056,[17]="174245,1",[18]=175040,[19]=175041,[20]=175042,[21]="174255,1",[22]=175043,[23]=175044},[589]={[1]=175046,[2]="174878,159",[3]=175048},[590]={[1]="174485,95",[2]="174583,7",[3]="174593,7",[4]="174653,7",[5]=175045,[6]="174603,7",[7]="174613,7",[8]="174623,7",[9]=175049,[10]="174633,7",[11]="174643,7"},[591]={[1]=174834,[2]=175047,[3]=175050,[4]=174833,[5]="174304,31",[6]="174336,62",[7]="174401,7",[8]="174411,7",[9]="174421,7",[10]="174430,7",[11]="174440,7",[12]="174450,6",[13]="174458,7",[14]="174468,7",[15]="174477,7"},[592]={[1]="174125,1",[2]="174132,92",[3]="174227,7",[4]="174237,7",[5]="174247,7",[6]="174257,7",[7]="174267,6",[8]="174276,7",[9]="174286,7",[10]="174296,7",[11]="174821,11"},[593]={[1]=175525},[594]={[1]=175524},[596]={[1]=175527},[597]={[1]=175528},[598]={[2]="175233,11",[1]="175196,35"},[599]={[1]="175252,35",[2]="175289,6",[3]="175296,4"},[600]={[2]="175345,11",[1]="175308,35"},[602]={[2]="177408,145",[1]="177392,15"},[603]={[2]="177664,62",[1]="177557,106"},[604]={[1]="177730,161"},[605]={[2]="177920,136",[1]="177895,24"},[606]={[2]="178176,53",[1]="178060,115"},[607]={[1]="178233,161"},[608]={[2]="178608,11",[1]="178571,35"},[609]={[2]="178664,11",[1]="178627,35"},[610]={[1]="178806,369"},[611]={[2]="179200,349",[1]="179180,19"},[612]={[2]="179712,211",[1]="179554,157"},[613]={[8]="179968,125",[1]=182826,[2]=182830,[3]=182832,[4]=182841,[5]=182842,[6]=182845,[7]="179924,43"},[614]={[1]=182833,[2]=182843,[3]="180097,126",[4]=182829,[5]="180224,34",[6]="182838,2"},[615]={[8]=182827,[1]=182831,[2]=182834,[3]=182835,[4]=182836,[5]=182837,[6]=182846,[7]="180262,161"},[616]={[2]="180480,116",[1]="180427,52"},[617]={[2]="180736,25",[1]="180600,135"},[618]={[1]="180765,161"},[619]={[2]="180992,107",[1]="180930,61"},[620]={[2]="181248,16",[1]="181103,144"},[621]={[1]="181268,161"},[622]={[2]="181760,105",[1]="181704,55"},[623]={[2]="182016,22",[1]="181869,146"},[624]={[1]="182042,161"},[625]={[1]=182208},[626]={[1]=182209},[627]={[1]=181695},[629]={[2]="183296,109",[1]="183236,59"},[630]={[2]="183552,18",[1]="183409,142"},[631]={[1]="183574,161"},[632]={[1]="183739,4",[2]="183744,30",[3]="183776,11"},[633]={[1]="183795,12",[2]="183808,22",[3]="183832,11"},[634]={[1]="183903,32",[2]="183936,2",[3]="183940,11"},[635]={[1]="183959,35",[2]="183996,3",[3]="184000,7"},[636]={[2]="184052,11",[1]="184015,35"},[640]={[2]="184832,308",[1]="184771,60"},[641]={[2]="185344,176",[1]="185151,192"},[642]={[2]="185856,44",[1]="185531,324"},[643]={[1]=188110,[2]=188112,[3]=188118,[4]=188119,[5]=188120,[6]="185907,169",[7]=188109,[8]="188125,1",[9]=188124,[10]=188129,[11]=188132},[644]={[1]=188127,[2]=188133,[3]="186080,31",[4]=188121,[5]="186112,129",[6]="188106,1",[7]=188104,[8]=188111,[9]="188115,2"},[645]={[1]=188103,[2]=188105,[3]=188108,[4]=188113,[5]=188114,[6]="186245,122",[7]=188122,[8]=188131,[9]="186368,38",[10]=188123,[11]=188128},[646]={[16]=187218,[1]=187219,[2]=187228,[3]=187229,[4]=187238,[5]=187239,[6]=187248,[7]=187257,[8]=187258,[9]=187276,[10]=187285,[11]=187286,[12]=187295,[13]=187267,[14]=187268,[15]="186410,133"},[647]={[16]="187470,1",[1]=187410,[2]=187411,[3]=187420,[4]=187421,[5]=187430,[6]=187431,[7]=187440,[8]="186547,76",[9]=187441,[10]="186624,7",[11]="186632,1",[12]="186634,38",[13]="187400,1",[14]="187450,1",[15]="187460,1"},[648]={[1]="186676,141"},[649]={[1]=187103,[2]=187104,[3]=187113,[4]=187114,[5]=187879,[6]="186821,58",[7]=187084,[8]=187085,[9]=187093,[10]=187094,[11]="186880,63",[12]="187044,1",[13]="187054,1",[14]=187880,[15]="186949,1",[16]=188140,[17]="187064,1",[18]=187881,[19]=187882,[20]=187883,[21]="187074,1",[22]=187884,[23]=187885},[650]={[1]="186944,1",[2]="186951,92",[3]="187046,7",[4]="187056,7",[5]="187066,7",[6]="187076,7",[7]="187086,6",[8]="187095,7",[9]="187105,7",[10]="187115,7",[11]="187640,7",[12]="187648,3"},[651]={[1]=187757,[2]="187480,159",[3]=187756},[652]={[1]="187304,87",[2]="187392,7",[3]="187402,7",[4]="187412,7",[5]="187422,7",[6]="187432,7",[7]="187442,7",[8]="187452,7",[9]="187462,7",[10]="187472,7",[11]="187754,1"},[653]={[1]=187653,[2]=187752,[3]=187753,[4]=187652,[5]="187123,12",[6]="187136,81",[7]="187220,7",[8]="187230,7",[9]="187240,7",[10]="187249,7",[11]="187259,7",[12]="187269,6",[13]="187277,7",[14]="187287,7",[15]="187296,7"},[654]={[1]=187654},[655]={[1]=187655},[656]={[1]=187656},[657]={[1]=187657},[658]={[1]=187658},[660]={[2]="188416,90",[1]="188345,70"},[661]={[2]="188672,7",[1]="188510,161"},[662]={[1]="188683,161"},[663]={[2]="188928,81",[1]="188848,79"},[664]={[1]="189013,169"},[665]={[1]="189186,161"},[666]={[1]="189351,24",[2]="189376,10",[3]="189388,11"},[667]={[1]="189407,32",[2]="189440,2",[3]="189444,11"},[668]={[1]="189506,169"},[669]={[2]="189696,144",[1]="189679,16"},[670]={[2]="189952,53",[1]="189844,107"},[671]={[1]=191209,[2]=191217,[3]=191219,[4]=191220,[5]="190367,96",[6]=191222,[7]=191226,[8]="190464,72",[9]=191224},[672]={[1]=191212,[2]=191213,[3]=191225,[4]="190540,161",[5]=191210,[6]=191218,[7]=191223},[673]={[1]=191221,[2]=191227,[3]="190705,14",[4]="191214,2",[5]=191211,[6]="190720,146"},[674]={[1]=190886},[675]={[1]=190887},[676]={[1]=190888},[677]={[2]="191488,113",[1]="191232,255"},[678]={[1]="191612,369"},[679]={[2]="192000,361",[1]="191992,7"},[680]={[2]="192768,17",[1]="192616,151"},[681]={[1]="192789,161"},[682]={[2]="193024,91",[1]="192954,69"},[683]={[1]="193119,32",[2]="193152,2",[3]="193156,11"},[684]={[2]="193280,64",[1]="193175,104"},[685]={[1]="193348,161"},[686]={[2]="193536,138",[1]="193513,22"},[687]={[2]="193715,11",[1]="193678,35"},[688]={[1]="193850,169"},[689]={[2]="194048,136",[1]="194023,24"},[690]={[2]="194304,45",[1]="194188,115"},[691]={[1]=194509},[692]={[1]=194510},[693]={[1]=194511},[694]={[1]=194512},[695]={[1]="194562,369"},[696]={[2]="195072,239",[1]="194942,129"},[697]={[2]="195584,107",[1]="195322,261"},[698]={[1]=197793,[2]=197794,[3]=197803,[4]=197804,[5]=197808,[6]="195698,141",[7]=197810,[8]=197815,[9]="195840,27",[10]=197812,[11]=197813},[699]={[1]=197796,[2]=197800,[3]=197801,[4]=197805,[5]="195871,161",[6]=197795,[7]=197807,[8]=197814,[9]=197817},[700]={[8]="197797,2",[1]=197806,[2]=197809,[3]=197811,[4]=197816,[5]=197802,[6]="196036,59",[7]="196096,101"},[701]={[16]=197108,[1]="196215,133",[2]=197031,[3]=197032,[4]=197041,[5]=197042,[6]=197051,[7]=197052,[8]=197061,[9]=197070,[10]=197071,[11]=197080,[12]=197081,[13]=197089,[14]=197098,[15]=197099},[702]={[1]=197233,[2]=197234,[3]=197243,[4]="196352,92",[5]=197223,[6]=197224,[7]=197244,[8]="196445,1",[9]="196447,30",[10]="197213,1",[11]="197253,1",[12]="197263,1",[13]="197273,1",[14]="197283,1"},[703]={[2]="196608,14",[1]="196481,126"},[704]={[1]=196898,[2]=196906,[3]="196762,1",[4]=196888,[5]=196897,[6]="196626,130",[7]=196916,[8]="196857,1",[9]=196907,[10]=196926,[11]=196927,[12]="196867,1",[13]="196877,1",[14]=196887,[15]=196917},[705]={[1]="196757,1",[2]="196764,92",[3]="196859,4",[4]="196864,2",[5]="196869,7",[6]="196879,7",[7]="196889,7",[8]="196899,6",[9]="196908,7",[10]="196918,7",[11]="196928,7",[12]="197453,11"},[706]={[1]="197293,82",[2]="197376,76",[3]="197471,1"},[707]={[1]="197117,2",[2]="197120,92",[3]="197215,7",[4]="197225,7",[5]="197235,7",[6]="197245,7",[7]="197255,7",[8]="197265,7",[9]="197275,7",[10]="197285,7",[11]="197469,1"},[708]={[1]="196936,94",[2]="197090,7",[3]=197465,[4]="197100,7",[5]="197109,7",[6]="197072,7",[7]="197043,7",[8]=197466,[9]=197468,[10]="197033,7",[11]="197053,7",[12]=197467,[13]="197062,7",[14]="197082,6"},[711]={[1]="198677,35",[2]="198714,5",[3]="198720,5"},[712]={[1]="198762,21",[2]="198784,13",[3]="198799,11"},[713]={[1]="198818,29",[2]="198848,5",[3]="198855,11"},[722]={[1]="199966,169",[2]="200154,6",[3]="200174,6",[4]="200194,6",[5]="200214,6",[6]="200234,6",[7]="200254,6",[8]="200274,6",[9]="200294,6",[10]="200319,6",[11]="200339,6",[12]="200359,6",[13]="200379,6",[14]="200399,6",[15]="200419,6",[16]="200439,6",[17]="200459,6"},[723]={[1]="200139,14",[2]="200161,12",[3]="200181,10",[4]="200192,1",[5]="200201,12",[6]="200221,12",[7]="200241,12",[8]="200261,12",[9]="200281,12",[10]="200472,175"},[724]={[1]=200816,[2]=200817,[3]="200406,12",[4]="200446,12",[5]=200818,[6]="200426,12",[7]=200819,[8]="200304,14",[9]="200326,12",[10]="200346,12",[11]="200366,12",[12]="200386,12",[13]=200824,[14]=200820,[15]=200821,[16]="200649,54",[17]="200704,111",[18]=200822,[19]=200823},[726]={[2]="200935,175",[1]="200826,105"},[727]={[1]="201112,103",[2]="201216,1",[3]="201221,175"},[728]={[2]="201507,175",[1]="201398,105"},[729]={[1]="201684,43",[2]="201728,61",[3]="201793,175"},[730]={[2]="201984,156",[1]="201971,12"},[731]={[2]="202240,65",[1]="202144,95"},[732]={[1]="202309,161"},[734]={[1]="202475,20",[2]="202496,14",[3]="202512,11"},[735]={[1]="202531,169"},[736]={[2]="202752,113",[1]="202704,47"},[737]={[2]="203008,22",[1]="202869,138"},[738]={[1]="203034,35",[2]="203072,10",[3]=203071},[754]={[2]="204032,38",[1]="203901,130"},[755]={[1]="204074,161"},[756]={[2]="204288,112",[1]="204239,48"}},
    --LIBSETS_TABLEKEY_SETITEMIDS

    ------------------------------------------------------------------------------------------------------------------------
    --......................................................................................................................
    --Below you'll find the list of preLoaded setNames in different languages.
    --The table's key is the [setId] and the value is a subtable containing the language as key (e.g. ["en"]) and the "SetName" as String value

    --[[ to generate the following list of ["setNames"]:
         --Mandatory 1--
         1* Uncomment the line ## LibSets_Debug.lua in the LibSets.txt file
         2* /reloadui with the latest version
         3* Use /script LibSets.DebugGetAllSetNames() to scan the setIds for their names, using the current clientsLanguage
         -> This will write the setnames to the SavedVariables table ["setNames"] with the current clientLanguage as key
         --Optional steps for other languages--
         4* Change the language ingame using /script SetCVar("language.2", "en") e.g. (or fr, de, ru, es, pl, it, pt, jp, etc.)
         5* repeat the steps 3 to 5 for other languages (if needed)
         --Mandatory 2--
         6* copy the entire ["setNames"] array from the LibSets.lua saved variables
         7* Use a lua minifier to shrink the code, e.g. https://mothereff.in/lua-minifier
         8* Put the shrinked lua table contents below in the table "setNames"

         Attention:
         If you need to add new languages later on you can "combine" the exisitng table below with new entries via the
         file "LibSets_Debug.lua", function debugBuildMixedSetNames.
         Check the section "-- MIXING NEW SET NAMES INTO THE PRELOADED DATA" in file "LibSets_Debug.lua" for a description how to combine the table below with new/changed entries of languages
     ]]
    --The preloaded set names
    [LIBSETS_TABLEKEY_SETNAMES]={[19]={["zh"]="法士装束",["es"]="Vestimentas del mago oscuro",["fr"]="Les Vêtements du sorcier",["de"]="Gewänder des Hexers",["ru"]="Одеяние чародея",["en"]="Vestments of the Warlock"},[20]={["zh"]="巫师护甲",["es"]="Armadura del brujo",["fr"]="Armure d'homme-médecine",["de"]="Hexenwerk",["ru"]="Доспех колдуна",["en"]="Witchman Armor"},[21]={["zh"]="阿卡维尔神龙卫",["es"]="Guardia del Dragón akaviri",["fr"]="La Garde du dragon akaviroise",["de"]="Akavirische Drachengarde",["ru"]="Акавирская Драконья стража",["en"]="Akaviri Dragonguard"},[22]={["zh"]="空想家斗篷",["es"]="Manto del soñador",["fr"]="Manteau du rêveur",["de"]="Mantel des Träumers",["ru"]="Накидка сновидца",["en"]="Dreamer's Mantle"},[23]={["zh"]="弓箭手思维",["es"]="Mente de arquero",["fr"]="L’Esprit de l'archer",["de"]="Schützensinn",["ru"]="Разум лучника",["en"]="Archer's Mind"},[24]={["zh"]="步兵之幸",["es"]="Fortuna del lacayo",["fr"]="La Chance du valet",["de"]="Geschick des Fußsoldaten",["ru"]="Удача пехотинца",["en"]="Footman's Fortune"},[25]={["zh"]="沙漠玫瑰",["es"]="Rosa del desierto",["fr"]="La Rose du désert",["de"]="Wüstenrose",["ru"]="Пустынная роза",["en"]="Desert Rose"},[26]={["zh"]="褴褛囚衣",["es"]="Harapos del prisionero",["fr"]="Les Haillons de prisonnier",["de"]="Lumpen des Gefangenen",["ru"]="Лохмотья узника",["en"]="Prisoner's Rags"},[27]={["zh"]="菲奥德的遗产",["es"]="Legado del fiordo",["fr"]="L’Héritage de Fiord",["de"]="Fiords Erbe",["ru"]="Наследие Фьорда",["en"]="Fiord's Legacy"},[28]={["zh"]="树肤",["es"]="Piel robliza",["fr"]="La Peau d'écorce",["de"]="Borkenhaut",["ru"]="Древесный покров",["en"]="Barkskin"},[29]={["zh"]="中士的锁甲",["es"]="Cota de sargento",["fr"]="La Cotte de mailles du sergent",["de"]="Rüstung des Feldwebels",["ru"]="Доспехи сержанта",["en"]="Sergeant's Mail"},[30]={["zh"]="雷光虫的甲壳",["es"]="Caparazón de bicho de trueno",["fr"]="Carapace de foudroptère",["de"]="Donnerkäferpanzer",["ru"]="Панцирь грозового жука",["en"]="Thunderbug's Carapace"},[31]={["zh"]="太阳之丝",["es"]="Seda del sol",["fr"]="Les Atours du soleil",["de"]="Sonnenseide",["ru"]="Шелка солнца",["en"]="Silks of the Sun"},[32]={["zh"]="治疗者的习惯",["es"]="Hábito del sanador",["fr"]="Le Froc du guérisseur",["de"]="Bräuche des Heilers",["ru"]="Мастерство целителя",["en"]="Healer's Habit"},[33]={["zh"]="毒蛇噬咬",["es"]="Picadura de víbora",["fr"]="La Morsure de la vipère",["de"]="Vipernbiss",["ru"]="Жало гадюки",["en"]="Viper's Sting"},[34]={["zh"]="夜母之拥",["es"]="Abrazo de la Madre Noche",["fr"]="Étreinte de la Mère de la nuit",["de"]="Umarmung der Mutter der Nacht",["ru"]="Объятия Матери Ночи",["en"]="Night Mother's Embrace"},[35]={["zh"]="骑士梦魇",["es"]="Caballero del delirio",["fr"]="Le Chevalier-cauchemar",["de"]="Albtraumritter",["ru"]="Кошмарный рыцарь",["en"]="Knightmare"},[36]={["zh"]="蒙面遗族护甲",["es"]="Armadura de la Herencia Velada",["fr"]="Armure de l'Héritage voilé",["de"]="Rüstung des Schleiererbes",["ru"]="Доспех Скрытого Наследия",["en"]="Armor of the Veiled Heritance"},[37]={["zh"]="死亡之风",["es"]="Viento de la muerte",["fr"]="Le Vent mortel",["de"]="Todeswind",["ru"]="Ветер смерти",["en"]="Death's Wind"},[38]={["zh"]="影暮之拥",["es"]="Abrazo del crepúsculo",["fr"]="L’Étreinte du crépuscule",["de"]="Zwielichtkuss",["ru"]="Объятия сумрака",["en"]="Twilight's Embrace"},[39]={["zh"]="阿莱西亚教团",["es"]="Orden de Alessia",["fr"]="L’Ordre d'Alessia",["de"]="Alessianischer Orden",["ru"]="Алессианский орден",["en"]="Alessian Order"},[40]={["zh"]="夜之寂静",["es"]="Silencio de la noche",["fr"]="Le Silence de la nuit",["de"]="Stille der Nacht",["ru"]="Молчание ночи",["en"]="Night's Silence"},[41]={["zh"]="怀特斯垂克的报复",["es"]="Castigo de Descarga Blanca",["fr"]="La Rétribution de Blancserpent",["de"]="Weißplankes Vergeltung",["ru"]="Возмездие Вайтстрейка",["en"]="Whitestrake's Retribution"},[43]={["zh"]="诱魔铠甲",["es"]="Armadura de la seductora",["fr"]="L’Armure de la séductrice",["de"]="Rüstung der Verführung",["ru"]="Доспех соблазнителя",["en"]="Armor of the Seducer"},[44]={["zh"]="吸血鬼之吻",["es"]="Beso vampírico",["fr"]="Le Baiser du vampire",["de"]="Kuss des Vampirs",["ru"]="Поцелуй вампира",["en"]="Vampire's Kiss"},[46]={["zh"]="高贵决斗者丝绸",["es"]="Atavíos del noble duelista",["fr"]="Atours du noble duelliste",["de"]="Seide des edlen Duellanten",["ru"]="Шелка благородного дуэлянта",["en"]="Noble Duelist's Silks"},[47]={["zh"]="枯手长袍",["es"]="Túnica de la Mano Marchita",["fr"]="Robes de la Main de Gloire",["de"]="Roben der Verdorrten Hand",["ru"]="Одежды Иссохшей Длани",["en"]="Robes of the Withered Hand"},[48]={["zh"]="马格努斯的赠礼",["es"]="Don de Magnus",["fr"]="Le Présent de Magnus",["de"]="Magnus' Gabe",["ru"]="Дар Магнуса",["en"]="Magnus' Gift"},[49]={["zh"]="红山之影",["es"]="Sombra de la Montaña Roja",["fr"]="L’Ombre du Mont Écarlate",["de"]="Schatten des Roten Berges",["ru"]="Тень Красной горы",["en"]="Shadow of the Red Mountain"},[50]={["zh"]="莫拉格帮",["es"]="El Morag Tong",["fr"]="La Morag Tong",["de"]="Morag Tong",["ru"]="Мораг Тонг",["en"]="The Morag Tong"},[51]={["zh"]="夜母的凝视",["es"]="Mirada de la Madre Noche",["fr"]="Le Regard de la Mère de la nuit",["de"]="Blick der Mutter der Nacht",["ru"]="Взгляд Матери Ночи",["en"]="Night Mother's Gaze"},[52]={["zh"]="召集之钢",["es"]="Llamada del acero",["fr"]="L’Appel de l'acier",["de"]="Klingender Stahl",["ru"]="Манящая сталь",["en"]="Beckoning Steel"},[53]={["zh"]="寒冰熔炉",["es"]="Fragua gélida",["fr"]="Le Fourneau de glace",["de"]="Eisschmiede",["ru"]="Ледяное горнило",["en"]="The Ice Furnace"},[54]={["zh"]="灰烬之握",["es"]="Yugo de ceniza",["fr"]="La Poigne de cendres",["de"]="Aschengriff",["ru"]="Пепельная хватка",["en"]="Ashen Grip"},[55]={["zh"]="祈祷披肩",["es"]="Mantón de oración",["fr"]="Châle de prière",["de"]="Gebetstuch",["ru"]="Молитвенное покрывало",["en"]="Prayer Shawl"},[56]={["zh"]="思坦达之拥",["es"]="Abrazo de Stendarr",["fr"]="L’Étreinte de Stendarr",["de"]="Stendarrs Umarmung",["ru"]="Объятия Стендарра",["en"]="Stendarr's Embrace"},[57]={["zh"]="塞拉班之握",["es"]="Agarre de Syrabane",["fr"]="L’Emprise de Syrabane",["de"]="Syrabanns Griff",["ru"]="Хватка Сирабейна",["en"]="Syrabane's Grip"},[58]={["zh"]="狼人之皮",["es"]="Piel del hombre lobo",["fr"]="La Peau de loup-garou",["de"]="Fell des Werwolfs",["ru"]="Шкура вервольфа",["en"]="Hide of the Werewolf"},[59]={["zh"]="凯恩之吻",["es"]="Beso de Kyne",["fr"]="Le Baiser de Kyne",["de"]="Kynes Kuss",["ru"]="Поцелуй Кин",["en"]="Kyne's Kiss"},[60]={["zh"]="黑暗阔步",["es"]="Paso sombrío",["fr"]="Le Sentier obscur",["de"]="Dunkelschritt",["ru"]="Поступь тьмы",["en"]="Darkstride"},[61]={["zh"]="蟹灵王杀手",["es"]="Asesino del dreugh rey",["fr"]="Le Tueur du roi dreugh",["de"]="Dreughkönigsschlächter",["ru"]="Победитель короля дреугов",["en"]="Dreugh King Slayer"},[62]={["zh"]="幼体之壳",["es"]="Cascarón del retoño",["fr"]="Coquille du rejeton",["de"]="Schlüpflingspanzer",["ru"]="Скорлупа детеныша",["en"]="Hatchling's Shell"},[63]={["zh"]="主宰",["es"]="Mastodonte",["fr"]="Le Mastodonte",["de"]="Koloss",["ru"]="Джаггернаут",["en"]="The Juggernaut"},[64]={["zh"]="暗影舞者装束",["es"]="Vestimenta de bailarín de las sombras",["fr"]="La Parure du bateleur d'ombre",["de"]="Kleidung des Schattentänzers",["ru"]="Одежды теневого танцора",["en"]="Shadow Dancer's Raiment"},[65]={["zh"]="血棘之触",["es"]="Toque de la Espina Sangrienta",["fr"]="Toucher de Sangrépine",["de"]="Spur eines Blutdorns",["ru"]="Касание Кровавого Шипа",["en"]="Bloodthorn's Touch"},[66]={["zh"]="希思特长袍",["es"]="Túnica del hist",["fr"]="Robes de l'Hist",["de"]="Roben des Hist",["ru"]="Облачение хиста",["en"]="Robes of the Hist"},[67]={["zh"]="暗影行者",["es"]="Caminante de las sombras",["fr"]="La Marcheuse d'ombres",["de"]="Schattengänger",["ru"]="Теневой ходок",["en"]="Shadow Walker"},[68]={["zh"]="幽冥",["es"]="Estigio",["fr"]="Le Stygien",["de"]="Stygier",["ru"]="Зловещий",["en"]="Stygian"},[69]={["zh"]="游侠的身姿",["es"]="Andares del montaraz",["fr"]="La Parure de l'éclaireur",["de"]="Laufstil des Waldläufers",["ru"]="Поступь следопыта",["en"]="Ranger's Gait"},[70]={["zh"]="第七军团之勇",["es"]="Bruto de la Séptima Legión",["fr"]="Brute de la septième Légion",["de"]="Rohling der Siebten Legion",["ru"]="Громила Седьмого легиона",["en"]="Seventh Legion Brute"},[71]={["zh"]="杜洛克的灾祸",["es"]="Perdición de Durok",["fr"]="Le Fléau de Durok",["de"]="Duroks Fluch",["ru"]="Погибель Дарока",["en"]="Durok's Bane"},[72]={["zh"]="尼古拉斯的重甲",["es"]="Armadura pesada de Nikulas",["fr"]="L’Armure lourde de Nikulas",["de"]="Nikulas' schwere Rüstung",["ru"]="Тяжелые доспехи Никуласа",["en"]="Nikulas' Heavy Armor"},[73]={["zh"]="湮灭之敌",["es"]="Enemigo de Oblivion",["fr"]="L’Adversaire d'Oblivion",["de"]="Erinnerung",["ru"]="Враг Обливиона",["en"]="Oblivion's Foe"},[74]={["zh"]="幽鬼之眼",["es"]="Ojo de espectro",["fr"]="L’Œil du spectre",["de"]="Schemenauge",["ru"]="Глаз призрака",["en"]="Spectre's Eye"},[75]={["zh"]="托拉格的契约",["es"]="Pacto de Torug",["fr"]="Le Pacte de Torug",["de"]="Torugs Pakt",["ru"]="Пакт Торуга",["en"]="Torug's Pact"},[76]={["zh"]="变化精通长袍",["es"]="Túnica de maestría en alteración",["fr"]="Les Robes de maîtrise de transformation",["de"]="Roben der Veränderungsbeherrschung",["ru"]="Облачение мастерства изменения",["en"]="Robes of Alteration Mastery"},[77]={["zh"]="远征者",["es"]="Cruzado",["fr"]="Le Croisé",["de"]="Glaubenskrieger",["ru"]="Крестоносец",["en"]="Crusader"},[78]={["zh"]="希思特树皮",["es"]="Corteza del hist",["fr"]="L’Écorce d'Hist",["de"]="Histrinde",["ru"]="Кора хиста",["en"]="Hist Bark"},[79]={["zh"]="柳树小径",["es"]="Sendero del sauce",["fr"]="Le Sentier des saules",["de"]="Weidenpfad",["ru"]="Ивовая тропа",["en"]="Willow's Path"},[80]={["zh"]="亨丁之怒",["es"]="Rabia de Hunding",["fr"]="La Rage de Hunding",["de"]="Hundings Zorn",["ru"]="Ярость Хандинга",["en"]="Hunding's Rage"},[81]={["zh"]="拉玛耶之歌",["es"]="Canción de Lamae",["fr"]="Le Chant de Lamae",["de"]="Lamaes Lied",["ru"]="Песнь Лами",["en"]="Song of Lamae"},[82]={["zh"]="阿莱西亚的壁垒",["es"]="Baluarte de Alessia",["fr"]="Le Rempart d'Alessia",["de"]="Alessias Bollwerk",["ru"]="Бастион Алессии",["en"]="Alessia's Bulwark"},[83]={["zh"]="精灵之灾",["es"]="Perdición de los elfos",["fr"]="Le Fléau des Elfes",["de"]="Elfenfluch",["ru"]="Бич эльфов",["en"]="Elf Bane"},[84]={["zh"]="奥格努姆之鳞",["es"]="Escamas de Orgnum",["fr"]="Les Écailles d'Orgnum",["de"]="Orgnums Schuppen",["ru"]="Чешуя Оргнума",["en"]="Orgnum's Scales"},[85]={["zh"]="阿玛莱西亚的慈悲",["es"]="Piedad de Almalexia",["fr"]="La Clémence d'Almalexia",["de"]="Almalexias Gnade",["ru"]="Милосердие Алмалексии",["en"]="Almalexia's Mercy"},[86]={["zh"]="女王之优雅",["es"]="Elegancia de la reina",["fr"]="Élégance de la reine",["de"]="Eleganz der Königin",["ru"]="Элегантность королевы",["en"]="Queen's Elegance"},[87]={["zh"]="玛拉之眼",["es"]="Ojos de Mara",["fr"]="Les Yeux de Mara",["de"]="Augen von Mara",["ru"]="Глаза Мары",["en"]="Eyes of Mara"},[88]={["zh"]="毁灭精通长袍",["es"]="Túnica de maestría en destrucción",["fr"]="Les Robes de maîtrise de la destruction",["de"]="Roben der Zerstörungsbeherrschung",["ru"]="Облачение мастерства разрушения",["en"]="Robes of Destruction Mastery"},[89]={["zh"]="哨兵",["es"]="Centinela",["fr"]="La Sentinelle",["de"]="Wachposten",["ru"]="Дозорный",["en"]="Sentry"},[90]={["zh"]="森切噬咬",["es"]="Mordisco del senche",["fr"]="Morsure de senche",["de"]="Biss des Senche",["ru"]="Укус сенча",["en"]="Senche's Bite"},[91]={["zh"]="湮灭之锋",["es"]="Linde de Oblivion",["fr"]="Le Tranchant d'Oblivion",["de"]="Vorteil des Vergessens",["ru"]="Лезвие Обливиона",["en"]="Oblivion's Edge"},[92]={["zh"]="格瑞纳克的希望",["es"]="Esperanza de Kagrenac",["fr"]="L’Espoir de Kagrenac",["de"]="Kagrenacs Hoffnung",["ru"]="Надежда Кагренака",["en"]="Kagrenac's Hope"},[93]={["zh"]="风暴骑士铠甲",["es"]="Peto del Caballero de la Tormenta",["fr"]="L’Armure de plate du chevalier-tempête",["de"]="Wut des Sturmritters",["ru"]="Панцирь грозового рыцаря",["en"]="Storm Knight's Plate"},[94]={["zh"]="美瑞蒂亚的受佑护甲",["es"]="Armadura bendita de Meridia",["fr"]="Armure bénie de Méridia",["de"]="Meridias gesegnete Rüstung",["ru"]="Благословенный доспех Меридии",["en"]="Meridia's Blessed Armor"},[95]={["zh"]="沙利多的诅咒",["es"]="Maldición de Shalidor",["fr"]="La Malédiction de Shalidor",["de"]="Shalidors Fluch",["ru"]="Проклятие Шалидора",["en"]="Shalidor's Curse"},[96]={["zh"]="真理之甲",["es"]="Armadura de la verdad",["fr"]="L’Armure de vérité",["de"]="Rüstung der Wahrheit",["ru"]="Доспехи истины",["en"]="Armor of Truth"},[97]={["zh"]="大法师",["es"]="El archimago",["fr"]="L’Archimage",["de"]="Erzmagier",["ru"]="Архимаг",["en"]="The Arch-Mage"},[98]={["zh"]="死灵全能",["es"]="Necropotencia",["fr"]="Nécropotence",["de"]="Nekropotenz",["ru"]="Сила смерти",["en"]="Necropotence"},[99]={["zh"]="救赎",["es"]="Salvación",["fr"]="Le Salut",["de"]="Erlösung",["ru"]="Спасение",["en"]="Salvation"},[100]={["zh"]="鹰睛石",["es"]="Ojo de halcón",["fr"]="L’Œil de faucon",["de"]="Falkenauge",["ru"]="Ястребиный глаз",["en"]="Hawk's Eye"},[101]={["zh"]="痛苦",["es"]="Aflicción",["fr"]="Affliction",["de"]="Befall",["ru"]="Недуг",["en"]="Affliction"},[102]={["zh"]="沙丘开膛者之鳞",["es"]="Escamas de destripador de dunas",["fr"]="Écailles de l'éventreur des dunes",["de"]="Schuppen des Dünenbrechers",["ru"]="Чешуя дюнороя",["en"]="Duneripper's Scales"},[103]={["zh"]="魔力熔炉",["es"]="Fragua de magia",["fr"]="Le Fourneau de magie",["de"]="Magickaschmiede",["ru"]="Магическое горнило",["en"]="Magicka Furnace"},[104]={["zh"]="噬咒者",["es"]="Devorador de maldiciones",["fr"]="Le Mangeur de malédiction",["de"]="Fluchfresser",["ru"]="Поглотитель проклятий",["en"]="Curse Eater"},[105]={["zh"]="姐妹花",["es"]="Gemelas",["fr"]="Les Sœurs jumelles",["de"]="Zwillingsschwestern",["ru"]="Сестры-близнецы",["en"]="Twin Sisters"},[106]={["zh"]="荒野女王弓手",["es"]="Arco de la Reina Silvestre",["fr"]="Arche de la Reine-nature",["de"]="Bogen der Wildkönigin",["ru"]="Ветви Лесной королевы",["en"]="Wilderqueen's Arch"},[107]={["zh"]="命运之树的祝福",["es"]="Bendición del árbol del wyrd",["fr"]="Bénédiction du Wyrd",["de"]="Segen des Wyrdbaums",["ru"]="Благословение Вещего древа",["en"]="Wyrd Tree's Blessing"},[108]={["zh"]="破坏者",["es"]="Devastador",["fr"]="L’Ensemble du ravageur",["de"]="Verwüster",["ru"]="Разоритель",["en"]="Ravager"},[109]={["zh"]="西罗帝尔之光",["es"]="Luz de Cyrodiil",["fr"]="La Lumière de Cyrodiil",["de"]="Licht von Cyrodiil",["ru"]="Свет Сиродила",["en"]="Light of Cyrodiil"},[110]={["zh"]="圣所",["es"]="Santuario",["fr"]="Le Sanctuaire",["de"]="Heiligtum",["ru"]="Святилище",["en"]="Sanctuary"},[111]={["zh"]="西罗帝尔结界",["es"]="Custodia de Cyrodiil",["fr"]="La Défense de Cyrodiil",["de"]="Schutz von Cyrodiil",["ru"]="Оберег Сиродила",["en"]="Ward of Cyrodiil"},[112]={["zh"]="午夜惊魂",["es"]="Terror nocturno",["fr"]="Terreur nocturne",["de"]="Nachtschrecken",["ru"]="Ночной Кошмар",["en"]="Night Terror"},[113]={["zh"]="西罗帝尔饰章",["es"]="Blasón de Cyrodiil",["fr"]="Les Armoiries de Cyrodiil",["de"]="Wappen von Cyrodiil",["ru"]="Герб Сиродила",["en"]="Crest of Cyrodiil"},[114]={["zh"]="灵魂之光",["es"]="El alma luminosa",["fr"]="L’Âme lumineuse",["de"]="Seelenschein",["ru"]="Сияние души",["en"]="Soulshine"},[116]={["zh"]="毁灭套装",["es"]="Séquito de destrucción",["fr"]="La suite de destruction",["de"]="Garnitur der Zerstörung",["ru"]="Набор разрушения",["en"]="The Destruction Suite"},[117]={["zh"]="医生安枢瑞的圣遗物",["es"]="Reliquias del físico Ansur",["fr"]="Les reliques du docteur Ansur",["de"]="Relikte des Mediziners Ansur",["ru"]="Реликвии целителя Ансура",["en"]="Relics of the Physician, Ansur"},[118]={["zh"]="大地熔炉的宝藏",["es"]="Tesoros de Forja de la Tierra",["fr"]="Les Trésors de la Forgeterre",["de"]="Schätze der Erdenschmiede",["ru"]="Сокровища Земной кузницы",["en"]="Treasures of the Earthforge"},[119]={["zh"]="反叛圣遗物",["es"]="Reliquias de la rebelión",["fr"]="Les Reliques de la rébellion",["de"]="Relikte der Rebellion",["ru"]="Реликвии восстания",["en"]="Relics of the Rebellion"},[120]={["zh"]="因弗内斯的武装",["es"]="Armas de Infernace",["fr"]="Les Armes d'Infernace",["de"]="Waffen Infernals",["ru"]="Снаряжение раскаленного горнила",["en"]="Arms of Infernace"},[121]={["zh"]="祖先武装",["es"]="Armas de los ancestros",["fr"]="Les Armes des ancêtres",["de"]="Waffen der Ahnen",["ru"]="Оружие предков",["en"]="Arms of the Ancestors"},[122]={["zh"]="黑檀铠甲",["es"]="Armadura de ébano",["fr"]="L’Armure d'ébène",["de"]="Ebenerzarsenal",["ru"]="Эбонитовый арсенал",["en"]="Ebon Armory"},[123]={["zh"]="海尔辛的虚饰",["es"]="Manto de Hircine",["fr"]="La Meute d'Hircine",["de"]="Hircines Schein",["ru"]="Маскировка Хирсина",["en"]="Hircine's Veneer"},[124]={["zh"]="黑虫的装束",["es"]="Vestimenta del gusano",["fr"]="La Tenue du Ver",["de"]="Garderobe des Wurms",["ru"]="Одежды Червя",["en"]="The Worm's Raiment"},[125]={["zh"]="帝国之怒",["es"]="Cólera del Imperio",["fr"]="La Fureur de l'Empire",["de"]="Zorn des Kaiserreichs",["ru"]="Гнев Империи",["en"]="Wrath of the Imperium"},[126]={["zh"]="古人的恩宠",["es"]="Gracia de los ancestros",["fr"]="La Grâce des anciens",["de"]="Anmut der Ahnen",["ru"]="Милость древних",["en"]="Grace of the Ancients"},[127]={["zh"]="致命打击",["es"]="Golpe mortal",["fr"]="La Frappe mortelle",["de"]="Tödlicher Stoß",["ru"]="Смертоносный удар",["en"]="Deadly Strike"},[128]={["zh"]="首席谋士的祝福",["es"]="Bendición de los potentados",["fr"]="La Bénédiction des monarques",["de"]="Segen des Potentaten",["ru"]="Благословение потентатов",["en"]="Blessing of the Potentates"},[129]={["zh"]="复仇榨取",["es"]="Retribución vengativa",["fr"]="Rétribution",["de"]="Saugende Vergeltung",["ru"]="Мстительное поглощение",["en"]="Vengeance Leech"},[130]={["zh"]="鹰眼",["es"]="Ojo del águila",["fr"]="L’Œil d'aigle",["de"]="Adlerauge",["ru"]="Орлиный взор",["en"]="Eagle Eye"},[131]={["zh"]="哈特兰棱堡",["es"]="Bastión de las Tierras Centrales",["fr"]="Le Bastion du continent",["de"]="Bastion des Herzlandes",["ru"]="Оплот Среднеземья",["en"]="Bastion of the Heartland"},[132]={["zh"]="勇士之盾",["es"]="Escudo del valiente",["fr"]="Le Bouclier du vaillant",["de"]="Schild des Tapferen",["ru"]="Щит храбреца",["en"]="Shield of the Valiant"},[133]={["zh"]="迅捷缓冲",["es"]="Parapeto del ágil",["fr"]="Le Boutoir de rapidité",["de"]="Dämpfer des Geschwinden",["ru"]="Награда за резвость",["en"]="Buffer of the Swift"},[134]={["zh"]="巫妖的裹尸布",["es"]="Sudario del liche",["fr"]="Le Suaire de la liche",["de"]="Tuch des Lichs",["ru"]="Покров лича",["en"]="Shroud of the Lich"},[135]={["zh"]="尸鬼的遗产",["es"]="Herencia del draugr",["fr"]="L’Héritage du Draugr",["de"]="Erbe des Draugrs",["ru"]="Наследие драугров",["en"]="Draugr's Heritage"},[136]={["zh"]="不朽战士",["es"]="Guerrero inmortal",["fr"]="Le Guerrier immortel",["de"]="Unsterblicher Krieger",["ru"]="Бессмертный воин",["en"]="Immortal Warrior"},[137]={["zh"]="狂怒战士",["es"]="Guerrero frenético",["fr"]="Le Guerrier furieux",["de"]="Tobender Krieger",["ru"]="Яростный воин",["en"]="Berserking Warrior"},[138]={["zh"]="防御战士",["es"]="Guerrero defensor",["fr"]="Le Guerrier défenseur",["de"]="Verteidigender Krieger",["ru"]="Защищающийся воин",["en"]="Defending Warrior"},[139]={["zh"]="博学法师",["es"]="Mago sabio",["fr"]="Le Mage avisé",["de"]="Weiser Magier",["ru"]="Мудрый маг",["en"]="Wise Mage"},[140]={["zh"]="毁灭法师",["es"]="Mago destructor",["fr"]="Le Mage destructeur",["de"]="Zerstörerischer Magier",["ru"]="Маг-разрушитель",["en"]="Destructive Mage"},[141]={["zh"]="治疗法师",["es"]="Mago sanador",["fr"]="Le Mage guérisseur",["de"]="Heilender Magier",["ru"]="Маг-целитель",["en"]="Healing Mage"},[142]={["zh"]="迅捷蝮蛇",["es"]="Serpiente veloz",["fr"]="Le Serpent rapide",["de"]="Flinke Schlange",["ru"]="Стремительный змей",["en"]="Quick Serpent"},[143]={["zh"]="致命毒蛇",["es"]="Serpiente venenosa",["fr"]="Le Serpent venimeux",["de"]="Giftschlange",["ru"]="Ядовитый змей",["en"]="Poisonous Serpent"},[144]={["zh"]="双牙蛇",["es"]="Serpiente de colmillos dobles",["fr"]="Le Serpent à deux crocs",["de"]="Doppelzüngige Schlange",["ru"]="Дважды клыкастый змей",["en"]="Twice-Fanged Serpent"},[145]={["zh"]="火之道",["es"]="Senda del fuego",["fr"]="La Voie du feu",["de"]="Weg des Feuers",["ru"]="Путь испепеления",["en"]="Way of Fire"},[146]={["zh"]="气之道",["es"]="Senda del aire",["fr"]="La Voie de l'air",["de"]="Weg der Luft",["ru"]="Путь воздуха",["en"]="Way of Air"},[147]={["zh"]="武学之道",["es"]="Senda del conocimiento marcial",["fr"]="La Voie de la connaissance martiale",["de"]="Weg der Kampfkunst",["ru"]="Путь боевого искусства",["en"]="Way of Martial Knowledge"},[148]={["zh"]="竞技场之道",["es"]="Senda de la Arena",["fr"]="La Voie de l'arène",["de"]="Weg der Arena",["ru"]="Путь Арены",["en"]="Way of the Arena"},[155]={["zh"]="无畏棱堡",["es"]="Baluarte de los Intrépidos",["fr"]="Bastion indomptable",["de"]="Unerschrockenen-Bastion",["ru"]="Оплот Неустрашимых",["en"]="Undaunted Bastion"},[156]={["zh"]="无畏渗透者",["es"]="Infiltrado intrépido",["fr"]="Infiltrateur indomptable",["de"]="Unerschrockener Infiltrator",["ru"]="Лазутчик Неустрашимых",["en"]="Undaunted Infiltrator"},[157]={["zh"]="无畏拆解者",["es"]="Desenmarañador intrépido",["fr"]="Détrameur indomptable",["de"]="Unerschrockener Entflechter",["ru"]="Развоплотитель Неустрашимых",["en"]="Undaunted Unweaver"},[158]={["zh"]="余烬之盾",["es"]="Ascuas protectoras",["fr"]="Bouclier de braise",["de"]="Glutschild",["ru"]="Тлеющий покров",["en"]="Embershield"},[159]={["zh"]="割裂火焰",["es"]="Llama divisora",["fr"]="Scindeflamme",["de"]="Trennflamme",["ru"]="Языки пламени",["en"]="Sunderflame"},[160]={["zh"]="燃烧法术编织",["es"]="Tejido arcano abrasador",["fr"]="Le Tramesort ardent",["de"]="Branntzauberweber",["ru"]="Плетение огненных чар",["en"]="Burning Spellweave"},[161]={["zh"]="再生之星",["es"]="Estrella renacida",["fr"]="Étoile gémellaire",["de"]="Doppelstern",["ru"]="Дважды рожденная звезда",["en"]="Twice-Born Star"},[162]={["zh"]="梅法拉的后裔",["es"]="Engendro de Mephala",["fr"]="L’Engeance de Méphala",["de"]="Mephalas Brut",["ru"]="Порождение Мефалы",["en"]="Spawn of Mephala"},[163]={["zh"]="血裔",["es"]="Engendro de Sangre",["fr"]="L’Engeance de sang",["de"]="Die Blutbrut",["ru"]="Порождение Крови",["en"]="Bloodspawn"},[164]={["zh"]="典狱长",["es"]="Lord guardián",["fr"]="Le Seigneur gardien",["de"]="Hochwärter",["ru"]="Лорд-надзиратель",["en"]="Lord Warden"},[165]={["zh"]="灾厄收割者",["es"]="Segadora azotadora",["fr"]="Moissonneur calamiteux",["de"]="Auspeitscherin",["ru"]="Жнец-палач",["en"]="Scourge Harvester"},[166]={["zh"]="引擎守护者",["es"]="Guardián del motor",["fr"]="Le Gardien du moteur",["de"]="Maschinenwächter",["ru"]="Механический страж",["en"]="Engine Guardian"},[167]={["zh"]="夜焰",["es"]="Llama Nocturna",["fr"]="La Nocteflamme",["de"]="Nachtflamme",["ru"]="Пламя Ночи",["en"]="Nightflame"},[168]={["zh"]="奈恩斯",["es"]="Nerien'eth",["fr"]="Nérien'eth",["de"]="Nerien'eth",["ru"]="Нериен'эт",["en"]="Nerien'eth"},[169]={["zh"]="瓦尔金·斯科里亚",["es"]="Valkyn Skoria",["fr"]="Valkyn Skoria",["de"]="Valkyn Skoria",["ru"]="Валкин Скория",["en"]="Valkyn Skoria"},[170]={["zh"]="炼狱巨口",["es"]="Fauces del Infernal",["fr"]="Gueule de l'Infernal",["de"]="Schlund des Infernalen",["ru"]="Огненная Пасть",["en"]="Maw of the Infernal"},[171]={["zh"]="永恒战士",["es"]="Guerrero eterno",["fr"]="L’Éternel Guerrier",["de"]="Ewiger Krieger",["ru"]="Вечный воин",["en"]="Eternal Warrior"},[172]={["zh"]="无懈可击法师",["es"]="Mago infalible",["fr"]="L’Infaillible Mage",["de"]="Unfehlbare Magierin",["ru"]="Безупречный маг",["en"]="Infallible Mage"},[173]={["zh"]="恶毒蟒蛇",["es"]="Serpiente despiadada",["fr"]="Le Cruel Serpent",["de"]="Boshafte Schlange",["ru"]="Беспощадный змей",["en"]="Vicious Serpent"},[176]={["zh"]="贵族的征服",["es"]="Conquista del noble",["fr"]="Le Butin du noble",["de"]="Adelssieg",["ru"]="Знатный завоеватель",["en"]="Noble's Conquest"},[177]={["zh"]="再分配者",["es"]="Redistribuidor",["fr"]="Redistributeur",["de"]="Umverteilung",["ru"]="Перераспределение",["en"]="Redistributor"},[178]={["zh"]="护甲大师",["es"]="Maestro armero",["fr"]="Maître armurier",["de"]="Rüstungsmeister",["ru"]="Мастер-бронник",["en"]="Armor Master"},[179]={["zh"]="黑玫瑰",["es"]="Rosa negra",["fr"]="La Rose noire",["de"]="Schwarze Rose",["ru"]="Черная роза",["en"]="Black Rose"},[180]={["zh"]="强力突袭",["es"]="Asalto poderoso",["fr"]="L’Assaut puissant",["de"]="Kraftvoller Ansturm",["ru"]="Мощный натиск",["en"]="Powerful Assault"},[181]={["zh"]="功勋",["es"]="Servicio meritorio",["fr"]="Le Service émérite",["de"]="Meritorischer Dienst",["ru"]="Достойная служба",["en"]="Meritorious Service"},[183]={["zh"]="莫拉格·科娜",["es"]="Molag Kena",["fr"]="Molag Kena",["de"]="Molag Kena",["ru"]="Молаг Кена",["en"]="Molag Kena"},[184]={["zh"]="帝国烙印",["es"]="Marcas del Imperio",["fr"]="Fers d'Imperium",["de"]="Male des Kaiserreichs",["ru"]="Клейма Империи",["en"]="Brands of Imperium"},[185]={["zh"]="法力治愈",["es"]="Potencia curativa",["fr"]="Puissance curative",["de"]="Magiekraftheilung",["ru"]="Сила исцеления",["en"]="Spell Power Cure"},[186]={["zh"]="震击装备",["es"]="Armas de conmoción",["fr"]="Les Armes de la décharge",["de"]="Rüttelnde Rüstung",["ru"]="Оружие разряда",["en"]="Jolting Arms"},[187]={["zh"]="沼泽掠夺者",["es"]="Saqueador del pantano",["fr"]="Pilleur du marais",["de"]="Sumpfräuber",["ru"]="Болотный налетчик",["en"]="Swamp Raider"},[188]={["zh"]="风暴领主",["es"]="Maestro de la tormenta",["fr"]="La Maîtrise de la tempête",["de"]="Sturmmeister",["ru"]="Повелитель бури",["en"]="Storm Master"},[190]={["zh"]="无情法师",["es"]="Mago mordaz",["fr"]="Le Mage brûlant",["de"]="Verletzender Magier",["ru"]="Язвительный маг",["en"]="Scathing Mage"},[193]={["zh"]="急剧涌动",["es"]="Impulso abrumador",["fr"]="L’Élan de Suprématie",["de"]="Überwältigende Woge",["ru"]="Сокрушительный вихрь",["en"]="Overwhelming Surge"},[194]={["zh"]="战斗医师",["es"]="Médico de combate",["fr"]="Le Médecin de terrain",["de"]="Feldarzt",["ru"]="Полевой врач",["en"]="Combat Physician"},[195]={["zh"]="猛毒",["es"]="Veneno absoluto",["fr"]="Le Venin absolu",["de"]="Reingift",["ru"]="Чистый яд",["en"]="Sheer Venom"},[196]={["zh"]="吸血板甲",["es"]="Placa de sanguijuela",["fr"]="La Plaque sangsue",["de"]="Auslaugende Rüstung",["ru"]="Панцирь высасывания",["en"]="Leeching Plate"},[197]={["zh"]="苔魔",["es"]="Atormentador",["fr"]="Le Tortionnaire",["de"]="Quälender",["ru"]="Истязатель",["en"]="Tormentor"},[198]={["zh"]="精华盗贼",["es"]="Ladrón de esencia",["fr"]="Voleur d'essence",["de"]="Essenzdieb",["ru"]="Похититель сущности",["en"]="Essence Thief"},[199]={["zh"]="碎盾者",["es"]="Rompe escudos",["fr"]="Brise-bouclier",["de"]="Schildbrecher",["ru"]="Сокрушитель щитов",["en"]="Shield Breaker"},[200]={["zh"]="凤凰",["es"]="Fénix",["fr"]="Le Phénix",["de"]="Phönix",["ru"]="Феникс",["en"]="Phoenix"},[201]={["zh"]="反应护甲",["es"]="Armadura reactiva",["fr"]="Armure réactive",["de"]="Reaktive Rüstung",["ru"]="Отражающая броня",["en"]="Reactive Armor"},[204]={["zh"]="耐久",["es"]="Resistencia",["fr"]="Endurance",["de"]="Beständigkeit",["ru"]="Выносливость",["en"]="Endurance"},[205]={["zh"]="意志",["es"]="Voluntad",["fr"]="Volonté",["de"]="Willenskraft",["ru"]="Сила воли",["en"]="Willpower"},[206]={["zh"]="敏捷",["es"]="Agilidad",["fr"]="Agilité",["de"]="Agilität",["ru"]="Ловкость",["en"]="Agility"},[207]={["zh"]="尤里安诺斯的法律",["es"]="Ley de Julianos",["fr"]="Loi de Julianos",["de"]="Gesetz von Julianos",["ru"]="Закон Джулианоса",["en"]="Law of Julianos"},[208]={["zh"]="烈焰试炼",["es"]="Prueba de fuego",["fr"]="Épreuve du feu",["de"]="Feuertaufe",["ru"]="Испытание огнем",["en"]="Trial by Fire"},[210]={["zh"]="帕里亚标记",["es"]="Marca del Paria",["fr"]="La Marque du Paria",["de"]="Zeichen des Ausgestoßenen",["ru"]="Клеймо отверженного",["en"]="Mark of the Pariah"},[211]={["zh"]="永霜",["es"]="Permafrost",["fr"]="Le Permafrost",["de"]="Permafrost",["ru"]="Вечная мерзлота",["en"]="Permafrost"},[212]={["zh"]="荆棘之心",["es"]="Corazón de espino",["fr"]="Roncecœur",["de"]="Dornenherz",["ru"]="Вересковое сердце",["en"]="Briarheart"},[213]={["zh"]="光荣防御者",["es"]="Protector glorioso",["fr"]="Défense glorieuse",["de"]="Ruhmreicher Verteidiger",["ru"]="Прославленный защитник",["en"]="Glorious Defender"},[214]={["zh"]="帕拉贝伦",["es"]="Para Bellum",["fr"]="Para Bellum",["de"]="Para Bellum",["ru"]="Готовность к войне",["en"]="Para Bellum"},[215]={["zh"]="元素继承",["es"]="Sucesión elemental",["fr"]="Succession élémentaire",["de"]="Elementarfolge",["ru"]="Преемственность стихий",["en"]="Elemental Succession"},[216]={["zh"]="领猎者",["es"]="Líder de la caza",["fr"]="Le Chef de la Chasse",["de"]="Jagdleiter",["ru"]="Возглавляющий охоту",["en"]="Hunt Leader"},[217]={["zh"]="冬裔",["es"]="Sangre Invernal",["fr"]="Nédhiver",["de"]="Winterkind",["ru"]="Зиморожденные",["en"]="Winterborn"},[218]={["zh"]="崔尼马克的英勇",["es"]="Valor de Trinimac",["fr"]="La Valeur de Trinimac",["de"]="Trinimacs Heldenmut",["ru"]="Доблесть Тринимака",["en"]="Trinimac's Valor"},[219]={["zh"]="莫库尔丁",["es"]="Morkuldin",["fr"]="Morkuldin",["de"]="Morkuldin",["ru"]="Моркулдин",["en"]="Morkuldin"},[224]={["zh"]="塔瓦的恩惠",["es"]="Favor de Tava",["fr"]="La Faveur de Tava",["de"]="Tavas Gunst",["ru"]="Покровительство Тавы",["en"]="Tava's Favor"},[225]={["zh"]="聪慧炼金术士",["es"]="Alquimista astuto",["fr"]="L’Alchimiste astucieux",["de"]="Schlauer Alchemist",["ru"]="Умный алхимик",["en"]="Clever Alchemist"},[226]={["zh"]="永恒狩猎",["es"]="Caza Eterna",["fr"]="La Chasse éternelle",["de"]="Ewige Jagd",["ru"]="Вечная охота",["en"]="Eternal Hunt"},[227]={["zh"]="巴哈拉的诅咒",["es"]="Maldición de Bahraha",["fr"]="La Malédiction de Bahraha",["de"]="Bahrahas Fluch",["ru"]="Проклятие Барахи",["en"]="Bahraha's Curse"},[228]={["zh"]="希瓦拉之鳞",["es"]="Escamas de Syvarra",["fr"]="Les Écailles de Syvarra",["de"]="Syvarras Schuppen",["ru"]="Чешуя Сиварры",["en"]="Syvarra's Scales"},[229]={["zh"]="影暮疗法",["es"]="Remedio crepuscular",["fr"]="Le Remède du crépuscule",["de"]="Zwielichtgenesung",["ru"]="Сумеречное лекарство",["en"]="Twilight Remedy"},[230]={["zh"]="月舞者",["es"]="Danzador lunar",["fr"]="Danselune",["de"]="Mondtänzer",["ru"]="Лунный танцор",["en"]="Moondancer"},[231]={["zh"]="月之棱堡",["es"]="Bastión lunar",["fr"]="Bastion lunaire",["de"]="Mondbastion",["ru"]="Лунный бастион",["en"]="Lunar Bastion"},[232]={["zh"]="艾尔克许的咆哮",["es"]="Rugido de Alkosh",["fr"]="Le Rugissement d'Alkosh",["de"]="Brüllen von Alkosh",["ru"]="Рев Алкоша",["en"]="Roar of Alkosh"},[234]={["zh"]="神射手饰章",["es"]="Emblema del tirador",["fr"]="Emblème du tireur d'élite",["de"]="Wappen des Meisterschützen",["ru"]="Герб меткого стрелка",["en"]="Marksman's Crest"},[235]={["zh"]="蜕变长袍",["es"]="Túnica de transmutación",["fr"]="Robes de transmutation",["de"]="Roben der Transmutation",["ru"]="Одеяние превращения",["en"]="Robes of Transmutation"},[236]={["zh"]="邪恶死亡",["es"]="Muerte cruel",["fr"]="Mort cruelle",["de"]="Grausamer Tod",["ru"]="Ужасная смерть",["en"]="Vicious Death"},[237]={["zh"]="勒奇的专注",["es"]="Enfoque de Leki",["fr"]="Focalisation de Léki",["de"]="Lekis Fokus",["ru"]="Взгляд Леки",["en"]="Leki's Focus"},[238]={["zh"]="法萨拉的狡诈",["es"]="Perfidia de Fasalla",["fr"]="Perfidie de Fasalla",["de"]="Fasallas List",["ru"]="Коварство Фасаллы",["en"]="Fasalla's Guile"},[239]={["zh"]="战士之怒",["es"]="Furia del guerrero",["fr"]="Furie du Guerrier",["de"]="Raserei des Kriegers",["ru"]="Ярость воина",["en"]="Warrior's Fury"},[240]={["zh"]="科瓦奇角斗士",["es"]="Gladiador de Kvatch",["fr"]="Le gladiateur de Kvatch",["de"]="Gladiator von Kvatch",["ru"]="Гладиатор Кватча",["en"]="Kvatch Gladiator"},[241]={["zh"]="瓦仁的遗产",["es"]="Legado de Varen",["fr"]="L’Héritage de Varen",["de"]="Varens Erbe",["ru"]="Наследие Варена",["en"]="Varen's Legacy"},[242]={["zh"]="佩里纳尔之怒",["es"]="Ira de Pelinal",["fr"]="Le Courroux de Pélinal",["de"]="Pelinals Zorn",["ru"]="Ярость Пелинала",["en"]="Pelinal's Wrath"},[243]={["zh"]="莫瑞豪斯之皮",["es"]="Piel de Morihaus",["fr"]="La Peau de Morihaus",["de"]="Haut von Morihaus",["ru"]="Шкура Морихауса",["en"]="Hide of Morihaus"},[244]={["zh"]="侧翼战略家",["es"]="Estratega de flanqueo",["fr"]="Le Stratège du débordement",["de"]="Flankierender Stratege",["ru"]="Мастер фланговой атаки",["en"]="Flanking Strategist"},[245]={["zh"]="西帝斯之触",["es"]="Toque de Sithis",["fr"]="La Caresse de Sithis",["de"]="Sithis' Berührung",["ru"]="Прикосновение Ситиса",["en"]="Sithis' Touch"},[246]={["zh"]="加里昂的复仇",["es"]="Venganza de Galerion",["fr"]="La vengeance de Galérion",["de"]="Galerions Revanche",["ru"]="Возмездие Галериона",["en"]="Galerion's Revenge"},[247]={["zh"]="猛毒精装",["es"]="Vicecanónigo del veneno",["fr"]="Le vice-chanoine du venin",["de"]="Vizekanoniker des Gifts",["ru"]="Каноник-отравитель",["en"]="Vicecanon of Venom"},[248]={["zh"]="先驱的活力",["es"]="Fuerza del heraldo",["fr"]="Les muscles du héraut",["de"]="Muskeln des Vorboten",["ru"]="Сила Предвестника",["en"]="Thews of the Harbinger"},[253]={["zh"]="帝都形体",["es"]="Constitución imperial",["fr"]="Le physique impérial",["de"]="Kaiserliche Physis",["ru"]="Имперское телосложение",["en"]="Imperial Physique"},[256]={["zh"]="强大的楚丹",["es"]="Chudan la Imponente",["fr"]="Le Gros Chudan",["de"]="Mächtiger Chudan",["ru"]="Могучая Чудан",["en"]="Mighty Chudan"},[257]={["zh"]="薇利德雷斯",["es"]="Velidreth",["fr"]="Velidreth",["de"]="Velidreth",["ru"]="Велидрет",["en"]="Velidreth"},[258]={["zh"]="琥珀浆体",["es"]="Plasma ámbar",["fr"]="Plasme ambré",["de"]="Bernsteinplasma",["ru"]="Янтарная плазма",["en"]="Amber Plasm"},[259]={["zh"]="希姆·贾斯的惩罚",["es"]="Castigo de Heem-Jas",["fr"]="Châtiment d'Heem-Jas",["de"]="Heem-Jas' Vergeltung",["ru"]="Возмездие Хим-Джаса",["en"]="Heem-Jas' Retribution"},[260]={["zh"]="马扎顿之相",["es"]="Aspecto de Mazzatun",["fr"]="Aspect de Mazzatun",["de"]="Aspekt von Mazzatun",["ru"]="Воплощение Маззатуна",["en"]="Aspect of Mazzatun"},[261]={["zh"]="蛛丝",["es"]="Diáfano",["fr"]="Diaphane",["de"]="Gespinst",["ru"]="Паутинный",["en"]="Gossamer"},[262]={["zh"]="寡妇制造者",["es"]="Creador de viudas",["fr"]="Deuil",["de"]="Witwenmacher",["ru"]="Вдоводел",["en"]="Widowmaker"},[263]={["zh"]="梅法拉之手",["es"]="Mano de Mephala",["fr"]="Main de Méphala",["de"]="Hand von Mephala",["ru"]="Рука Мефалы",["en"]="Hand of Mephala"},[265]={["zh"]="影裂",["es"]="Desgarradora de Sombras",["fr"]="Taillombre",["de"]="Schattenriss",["ru"]="Теневик",["en"]="Shadowrend"},[266]={["zh"]="克拉格",["es"]="Kra'gh",["fr"]="Kra'gh",["de"]="Kra'gh",["ru"]="Кра'гх",["en"]="Kra'gh"},[267]={["zh"]="蛛群之母",["es"]="Madre del Enjambre",["fr"]="La Mère de la nuée",["de"]="Schwarmmutter",["ru"]="Мать стаи",["en"]="Swarm Mother"},[268]={["zh"]="库加姆的哨兵",["es"]="Centinela de Rkugamz",["fr"]="La sentinelle de Rkugamz",["de"]="Wachposten von Rkugamz",["ru"]="Страж Ркугамза",["en"]="Sentinel of Rkugamz"},[269]={["zh"]="窒息荆棘",["es"]="Espina Ahogadora",["fr"]="La Ronce étouffeuse",["de"]="Würgedorn",["ru"]="Удушай",["en"]="Chokethorn"},[270]={["zh"]="黏爪",["es"]="Zarpa Embarrada",["fr"]="Rampefange",["de"]="Schleimkropf",["ru"]="Слизебрюх",["en"]="Slimecraw"},[271]={["zh"]="塞利斯特利克斯",["es"]="Sellistrix",["fr"]="Sellistrix",["de"]="Sellistrix",["ru"]="Селлистрикс",["en"]="Sellistrix"},[272]={["zh"]="炼狱守护者",["es"]="Guardián Infernal",["fr"]="Gardien infernal",["de"]="Infernaler Wächter",["ru"]="Огненный страж",["en"]="Infernal Guardian"},[273]={["zh"]="伊拉姆布里斯",["es"]="Ilambris",["fr"]="Ilambris",["de"]="Ilambris",["ru"]="Иламбрисы",["en"]="Ilambris"},[274]={["zh"]="寒冰之心",["es"]="Corazón Gélido",["fr"]="Cœur-de-glace",["de"]="Eisherz",["ru"]="Ледяное Сердце",["en"]="Iceheart"},[275]={["zh"]="暴风拳",["es"]="Puño de Tormenta",["fr"]="Poigne-tempête",["de"]="Sturmfaust",["ru"]="Штормовой Кулак",["en"]="Stormfist"},[276]={["zh"]="战栗之鳞",["es"]="Escama Trémula",["fr"]="Tremblécaille",["de"]="Bebenschuppe",["ru"]="Чешуйчатый Сотрясатель",["en"]="Tremorscale"},[277]={["zh"]="海盗骷髅",["es"]="Pirata esqueleto",["fr"]="Pirate squelettique",["de"]="Skelettpirat",["ru"]="Скелет-пират",["en"]="Pirate Skeleton"},[278]={["zh"]="巨魔王",["es"]="Rey trol",["fr"]="Le roi des Trolls",["de"]="Trollkönig",["ru"]="Король троллей",["en"]="The Troll King"},[279]={["zh"]="夕月",["es"]="Selene",["fr"]="Sélène",["de"]="Selene",["ru"]="Селена",["en"]="Selene"},[280]={["zh"]="格罗斯达尔",["es"]="Grothdarr",["fr"]="Grothdarr",["de"]="Grothdarr",["ru"]="Гротдарр",["en"]="Grothdarr"},[281]={["zh"]="初学者护甲",["es"]="Armadura del novicio",["fr"]="Armure du débutant",["de"]="Rüstung des Auszubildenden",["ru"]="Броня ученика",["en"]="Armor of the Trainee"},[282]={["zh"]="吸血鬼斗篷",["es"]="Capa de vampiro",["fr"]="Cape du vampire",["de"]="Vampirumhang",["ru"]="Вампирская скрытность",["en"]="Vampire Cloak"},[283]={["zh"]="剑咏者",["es"]="Cantor de espada",["fr"]="Chante-épée",["de"]="Schwertsänger",["ru"]="Певец меча",["en"]="Sword-Singer"},[284]={["zh"]="迪亚格纳教团",["es"]="La Orden de Diagna",["fr"]="Ordre de Diagna",["de"]="Orden von Diagna",["ru"]="Орден Диагны",["en"]="Order of Diagna"},[285]={["zh"]="吸血鬼领主",["es"]="Señor de los vampiros",["fr"]="Seigneur vampire",["de"]="Vampirfürst",["ru"]="Вампир-лорд",["en"]="Vampire Lord"},[286]={["zh"]="树精尖刺",["es"]="Espinas de spriggan",["fr"]="Ronces du spriggan",["de"]="Dornen des Zweiglings",["ru"]="Шипы сприггана",["en"]="Spriggan's Thorns"},[287]={["zh"]="翠林公约",["es"]="Pacto verde",["fr"]="Le Pacte Vert",["de"]="Der Grüne Pakt",["ru"]="Зеленый пакт",["en"]="Green Pact"},[288]={["zh"]="养蜂人装束",["es"]="Equipo de apicultor",["fr"]="Harnachement de l'apiculteur",["de"]="Werkzeug des Bienenhüters",["ru"]="Снаряжение пасечника",["en"]="Beekeeper's Gear"},[289]={["zh"]="织歌者之服",["es"]="Vestiduras de hilandero",["fr"]="Tenue du trameur",["de"]="Gewänder des Webers",["ru"]="Одеяние сказителя",["en"]="Spinner's Garments"},[290]={["zh"]="斯库玛走私者",["es"]="Traficante de skooma",["fr"]="Traficant de skouma",["de"]="Skoomaschmuggler",["ru"]="Контрабандист скумы",["en"]="Skooma Smuggler"},[291]={["zh"]="沙克虫外骨骼",["es"]="Exoesqueleto de shalk",["fr"]="Exosquelette de shalk",["de"]="Schröterpanzer",["ru"]="Экзоскелет шалка",["en"]="Shalk Exoskeleton"},[292]={["zh"]="夜母之殇",["es"]="Pesar de madre",["fr"]="Chagrin maternel",["de"]="Muttertränen",["ru"]="Материнская скорбь",["en"]="Mother's Sorrow"},[293]={["zh"]="瘟疫医生",["es"]="Doctor de la plaga",["fr"]="Médecin de la peste",["de"]="Seuchendoktor",["ru"]="Чумной доктор",["en"]="Plague Doctor"},[294]={["zh"]="斯格拉默的天赋权利",["es"]="Patrimonio de Ysgramor",["fr"]="Héritage d'Ysgramor",["de"]="Ysgramors Geburtsrecht",["ru"]="Право Исграмора",["en"]="Ysgramor's Birthright"},[295]={["zh"]="越狱者",["es"]="Prófugo",["fr"]="L’Évasion",["de"]="Ausbrecher",["ru"]="Беглец",["en"]="Jailbreaker"},[296]={["zh"]="洞穴勘探者",["es"]="Espeleología",["fr"]="Spéléologue",["de"]="Höhlenforscher",["ru"]="Исследователь пещер",["en"]="Spelunker"},[297]={["zh"]="蜘蛛教徒斗篷",["es"]="Capucha de sectario de la araña",["fr"]="Capuchon de l'adepte de l'Araignée",["de"]="Spinnenkultistenkutte",["ru"]="Одеяние культа Паучихи",["en"]="Spider Cultist Cowl"},[298]={["zh"]="光语者",["es"]="Portavoz de la luz",["fr"]="Orateur lumineux",["de"]="Lichtsprecher",["ru"]="Глашатай света",["en"]="Light Speaker"},[299]={["zh"]="蛮牙",["es"]="Hiladientes",["fr"]="La Rangée de dents",["de"]="Zahnreihe",["ru"]="Оскал",["en"]="Toothrow"},[300]={["zh"]="气母之触",["es"]="Toque de netch",["fr"]="Toucher du netch",["de"]="Berührung des Netch",["ru"]="Касание нетча",["en"]="Netch's Touch"},[301]={["zh"]="机械体之力",["es"]="Fuerza del autómata",["fr"]="Force de l'automate",["de"]="Stärke des Automaten",["ru"]="Сила автоматона",["en"]="Strength of the Automaton"},[302]={["zh"]="利维坦",["es"]="Leviatán",["fr"]="Le Léviathan",["de"]="Leviathan",["ru"]="Исполин",["en"]="Leviathan"},[303]={["zh"]="蛇人之歌",["es"]="Canto de lamia",["fr"]="Chant de la Lamie",["de"]="Lied der Lamie",["ru"]="Песнь ламии",["en"]="Lamia's Song"},[304]={["zh"]="美杜莎",["es"]="Medusa",["fr"]="La Méduse",["de"]="Versteinernder Blick",["ru"]="Медуза",["en"]="Medusa"},[305]={["zh"]="宝藏猎手",["es"]="Cazador de tesoros",["fr"]="Le chasseur de trésors",["de"]="Schatzjäger",["ru"]="Охотник за сокровищами",["en"]="Treasure Hunter"},[307]={["zh"]="尸鬼复生",["es"]="Draugr colosal",["fr"]="Le Draugr colossal",["de"]="Schwerfälliger Draugr",["ru"]="Могучий драугр",["en"]="Draugr Hulk"},[308]={["zh"]="白骨海盗的褴褛衣衫",["es"]="Harapos del pirata de hueso",["fr"]="Haillons du pirate squelettique",["de"]="Lumpen des Knochenpiraten",["ru"]="Лохмотья костяного пирата",["en"]="Bone Pirate's Tatters"},[309]={["zh"]="游侠骑士铠甲",["es"]="Cota de caballero errante",["fr"]="Maille du Chevalier errant",["de"]="Platten des Wanderritters",["ru"]="Доспех странствующего рыцаря",["en"]="Knight-errant's Mail"},[310]={["zh"]="剑舞者",["es"]="Danzador de espada",["fr"]="La Danse des épées",["de"]="Schwerttänzer",["ru"]="Танцующий с клинками",["en"]="Sword Dancer"},[311]={["zh"]="响笼",["es"]="Desafiador",["fr"]="Le Provocateur",["de"]="Klapperkäfig",["ru"]="Исступление",["en"]="Rattlecage"},[313]={["zh"]="巨神劈砍",["es"]="Cuchillada titánica",["fr"]="Le Fendoir tinanesque",["de"]="Titanisches Trennen",["ru"]="Титаническое рассечение",["en"]="Titanic Cleave"},[314]={["zh"]="穿刺疗愈",["es"]="Remedio perforante",["fr"]="Le Remède perforant",["de"]="Durchschlagende Genesung",["ru"]="Исцеление уколом",["en"]="Puncturing Remedy"},[315]={["zh"]="刺斩",["es"]="Cuchilladas punzantes",["fr"]="Entailles cuisantes",["de"]="Stechende Schnitte",["ru"]="Кровоточащие порезы",["en"]="Stinging Slashes"},[316]={["zh"]="腐蚀箭矢",["es"]="Flecha cáustica",["fr"]="Flèche caustique",["de"]="Beißender Pfeil",["ru"]="Разъедающая стрела",["en"]="Caustic Arrow"},[317]={["zh"]="毁灭冲撞",["es"]="Impacto destructivo",["fr"]="Impact destructeur",["de"]="Zerstörerischer Einschlag",["ru"]="Разрушительное столкновение",["en"]="Destructive Impact"},[318]={["zh"]="盛大复苏",["es"]="Gran rejuvenecimiento",["fr"]="Grand Rajeunissement",["de"]="Große Verjüngung",["ru"]="Высшее восстановление",["en"]="Grand Rejuvenation"},[320]={["zh"]="巾帼英雄",["es"]="Doncella guerrera",["fr"]="La Vierge guerrière",["de"]="Kriegsjungfer",["ru"]="Воительница",["en"]="War Maiden"},[321]={["zh"]="亵渎者",["es"]="Profanador",["fr"]="Profanateur",["de"]="Schänder",["ru"]="Осквернитель",["en"]="Defiler"},[322]={["zh"]="武勇诗人",["es"]="Poeta guerrero",["fr"]="Le Guerrier-Poète",["de"]="Kriegerpoet",["ru"]="Воин-поэт",["en"]="Warrior-Poet"},[323]={["zh"]="刺客的狡诈",["es"]="Astucia de asesino",["fr"]="La Duplicité de l'assassin",["de"]="Assassinenlist",["ru"]="Вероломство убийцы",["en"]="Assassin's Guile"},[324]={["zh"]="魔族诡计",["es"]="Engaño daédrico",["fr"]="La Tromperie daedrique",["de"]="Daedrische Gaunerei",["ru"]="Даэдрическая хитрость",["en"]="Daedric Trickery"},[325]={["zh"]="枷锁挣断者",["es"]="Rompecadenas",["fr"]="Le Brise-entraves",["de"]="Kettensprenger",["ru"]="Крушитель оков",["en"]="Shacklebreaker"},[326]={["zh"]="先锋挑战",["es"]="Desafío de la vanguardia",["fr"]="Défi de l'avant-garde",["de"]="Vorhutdisput",["ru"]="Испытание авангарда",["en"]="Vanguard's Challenge"},[327]={["zh"]="懦夫的装备",["es"]="Petate del cobarde",["fr"]="Barda du couard",["de"]="Feiglingstracht",["ru"]="Снаряжение труса",["en"]="Coward's Gear"},[328]={["zh"]="骑士杀戮者",["es"]="Exterminador de caballeros",["fr"]="Tueur de chevalier",["de"]="Ritterschlächter",["ru"]="Рыцарь-убийца",["en"]="Knight Slayer"},[329]={["zh"]="巫师回击",["es"]="Contragolpe del hechicero",["fr"]="Riposte du sorcier",["de"]="Zaubererreplik",["ru"]="Ответ волшебника",["en"]="Wizard's Riposte"},[330]={["zh"]="自动防御",["es"]="Defensa automática",["fr"]="La Défense automatique",["de"]="Automatisierte Verteidigung",["ru"]="Автоматическая защита",["en"]="Automated Defense"},[331]={["zh"]="战争机器",["es"]="Máquina de guerra",["fr"]="La machine de guerre",["de"]="Kriegsmaschine",["ru"]="Военная машина",["en"]="War Machine"},[332]={["zh"]="建筑大师",["es"]="Maestro arquitecto",["fr"]="Le maître architecte",["de"]="Meisterarchitekt",["ru"]="Главный архитектор",["en"]="Master Architect"},[333]={["zh"]="发明家的守卫",["es"]="Guarda del inventor",["fr"]="La Garde de l'inventeur",["de"]="Erfindergarde",["ru"]="Защита изобретателя",["en"]="Inventor's Guard"},[334]={["zh"]="至坚护甲",["es"]="Armadura inexpugnable",["fr"]="Armure imprenable",["de"]="Unüberwindliche Rüstung",["ru"]="Непробиваемая броня",["en"]="Impregnable Armor"},[335]={["zh"]="尸鬼之栖",["es"]="Remanso del draugr",["fr"]="Le Repos du Draugr",["de"]="Draugrruhe",["ru"]="Покой драугра",["en"]="Draugr's Rest"},[336]={["zh"]="奈恩之柱",["es"]="Pilar de Nirn",["fr"]="Pilier de Nirn",["de"]="Säulen von Nirn",["ru"]="Столп Нирна",["en"]="Pillar of Nirn"},[337]={["zh"]="铁血",["es"]="Sangre de hierro",["fr"]="Sang de fer",["de"]="Eisenblut",["ru"]="Железная кровь",["en"]="Ironblood"},[338]={["zh"]="烈焰绽放",["es"]="Flor de fuego",["fr"]="Fleur de feu",["de"]="Flammenblüte",["ru"]="Пламенный цветок",["en"]="Flame Blossom"},[339]={["zh"]="饮血者",["es"]="Chupasangre",["fr"]="La Buveur de sang",["de"]="Bluttrinker",["ru"]="Кровопийца",["en"]="Blooddrinker"},[340]={["zh"]="乌鸦鬼婆花园",["es"]="Jardín de la bruja cuervo",["fr"]="Jardin de la harfreuse",["de"]="Vettelgarten",["ru"]="Сад ворожеи",["en"]="Hagraven's Garden"},[341]={["zh"]="地血",["es"]="Tierra Sangrienta",["fr"]="Sangreterre",["de"]="Erdbluter",["ru"]="Кровь земли",["en"]="Earthgore"},[342]={["zh"]="铎米豪斯",["es"]="Domihaus",["fr"]="Domihaus",["de"]="Domihaus",["ru"]="Домихаус",["en"]="Domihaus"},[343]={["zh"]="卡鲁里昂的遗产",["es"]="Legado de Caluurion",["fr"]="L’Héritage de Caluurion",["de"]="Caluurions Erbe",["ru"]="Наследие Калуриона",["en"]="Caluurion's Legacy"},[344]={["zh"]="鼓舞礼服",["es"]="Enseres de vivificación",["fr"]="Apparence de vivification",["de"]="Stärkungsprunk",["ru"]="Второе дыхание",["en"]="Trappings of Invigoration"},[345]={["zh"]="乌尔夫诺尔的青睐",["es"]="Favor de Ulfnor",["fr"]="La Faveur d'Ulfnor",["de"]="Ulfnors Gunst",["ru"]="Благосклонность Ульфнора",["en"]="Ulfnor's Favor"},[346]={["zh"]="约尔沃德的指引",["es"]="Consejo de Jorvuld",["fr"]="Le Conseil de Jorvuld",["de"]="Jorvulds Führung",["ru"]="Напутствие Йорвульда",["en"]="Jorvuld's Guidance"},[347]={["zh"]="瘟疫投掷",["es"]="Azote de la plaga",["fr"]="Lance-peste",["de"]="Seuchenschleuder",["ru"]="Чумной метатель",["en"]="Plague Slinger"},[348]={["zh"]="多来米什的诅咒",["es"]="Maldición de Doylemish",["fr"]="La Malédiction de Doylemish",["de"]="Fluch von Doylemish",["ru"]="Проклятие Дойлемиша",["en"]="Curse of Doylemish"},[349]={["zh"]="图尔佛昆",["es"]="Thurvokun",["fr"]="Thurvokun",["de"]="Thurvokun",["ru"]="Турвокун",["en"]="Thurvokun"},[350]={["zh"]="赞恩",["es"]="Zaan",["fr"]="Zaan",["de"]="Zaan",["ru"]="Заан",["en"]="Zaan"},[351]={["zh"]="天定公理",["es"]="Axioma innato",["fr"]="Axiome inné",["de"]="Kernaxiom",["ru"]="Врожденная аксиома",["en"]="Innate Axiom"},[352]={["zh"]="强化黄铜",["es"]="Latón reforzado",["fr"]="Airain fortifié",["de"]="Messingpanzer",["ru"]="Прочная латунь",["en"]="Fortified Brass"},[353]={["zh"]="机械敏锐",["es"]="Agudeza mecánica",["fr"]="Acuité mécanique",["de"]="Mechanikblick",["ru"]="Механическая точность",["en"]="Mechanical Acuity"},[354]={["zh"]="疯狂工匠",["es"]="Inventor demente",["fr"]="Bricoleur fou",["de"]="Wahntüftler",["ru"]="Безумный изобретатель",["en"]="Mad Tinkerer"},[355]={["zh"]="深邃黑暗",["es"]="Oscuridad insondable",["fr"]="Ténèbres insondables",["de"]="Unermessliche Dunkelheit",["ru"]="Непроницаемая тьма",["en"]="Unfathomable Darkness"},[356]={["zh"]="电路",["es"]="Energía viva",["fr"]="Haute tension",["de"]="Stromschlag",["ru"]="Высокое напряжение",["en"]="Livewire"},[357]={["zh"]="完美惩戒斩击",["es"]="Corte disciplinado perfeccionado",["fr"]="Entaille disciplinée perfectionnée",["de"]="Perfektionierter disziplinierter Schnitt",["ru"]="Совершенный выверенный удар",["en"]="Perfected Disciplined Slash"},[358]={["zh"]="完美防御阵地",["es"]="Posición defensiva perfeccionada",["fr"]="Position défensive perfectionnée",["de"]="Perfektionierte defensive Position",["ru"]="Совершенная оборонительная позиция",["en"]="Perfected Defensive Position"},[359]={["zh"]="完美混沌旋风",["es"]="Torbellino caótico perfeccionado",["fr"]="Tourbillon chaotique perfectionné",["de"]="Perfektionierter chaotischer Wirbelwind",["ru"]="Совершенный хаотичный вихрь",["en"]="Perfected Chaotic Whirlwind"},[360]={["zh"]="完美穿刺箭阵",["es"]="Ráfaga perforadora perfeccionada",["fr"]="Jaillissement perforant perfectionné",["de"]="Perfektionierte durchdringende Salve",["ru"]="Совершенный пробивающий веер",["en"]="Perfected Piercing Spray"},[361]={["zh"]="完美专注之力",["es"]="Fuerza concentrada perfeccionada",["fr"]="Force concentrée perfectionnée",["de"]="Perfektionierte konzentrierte Kraft",["ru"]="Совершенная концентрированная сила",["en"]="Perfected Concentrated Force"},[362]={["zh"]="完美无尽祝福",["es"]="Bendición eterna perfeccionada",["fr"]="Bénédiction intemporelle perfectionnée",["de"]="Perfektionierter zeitloser Segen",["ru"]="Совершенное бесконечное благословение",["en"]="Perfected Timeless Blessing"},[363]={["zh"]="惩戒斩击",["es"]="Corte disciplinado",["fr"]="Entaille disciplinée",["de"]="Disziplinierter Schnitt",["ru"]="Выверенный удар",["en"]="Disciplined Slash"},[364]={["zh"]="防御阵地",["es"]="Posición defensiva",["fr"]="Position défensive",["de"]="Defensive Position",["ru"]="Оборонительная позиция",["en"]="Defensive Position"},[365]={["zh"]="混沌旋风",["es"]="Torbellino caótico",["fr"]="Tourbillon chaotique",["de"]="Chaotischer Wirbelwind",["ru"]="Хаотичный вихрь",["en"]="Chaotic Whirlwind"},[366]={["zh"]="穿刺箭阵",["es"]="Ráfaga perforadora",["fr"]="Jaillissement perforant",["de"]="Durchdringende Salve",["ru"]="Пробивающий веер",["en"]="Piercing Spray"},[367]={["zh"]="专注之力",["es"]="Fuerza concentrada",["fr"]="Force concentrée",["de"]="Konzentrierte Kraft",["ru"]="Концентрированная сила",["en"]="Concentrated Force"},[368]={["zh"]="无尽祝福",["es"]="Bendición eterna",["fr"]="Bénédiction intemporelle",["de"]="Zeitloser Segen",["ru"]="Бесконечное благословение",["en"]="Timeless Blessing"},[369]={["zh"]="无情冲锋",["es"]="Carga despiadada",["fr"]="La Charge impitoyable",["de"]="Gnadenloser Ansturm",["ru"]="Безжалостный рывок",["en"]="Merciless Charge"},[370]={["zh"]="狂暴斩击",["es"]="Hendidura inexorable",["fr"]="L’Entaille ravageuse",["de"]="Tobender Schnitt",["ru"]="Свирепый удар",["en"]="Rampaging Slash"},[371]={["zh"]="残暴乱舞",["es"]="Ráfaga cruel",["fr"]="Le Déluge cruel",["de"]="Grausamer Schlaghagel",["ru"]="Жестокий шквал",["en"]="Cruel Flurry"},[372]={["zh"]="雷霆齐射",["es"]="Salva atronadora",["fr"]="La Volée tonitruante",["de"]="Donnernde Salve",["ru"]="Разрушительный залп",["en"]="Thunderous Volley"},[373]={["zh"]="碾压之墙",["es"]="Muro aplastante",["fr"]="Le Mur écrasant",["de"]="Zermalmende Mauer",["ru"]="Сокрушительная стена",["en"]="Crushing Wall"},[374]={["zh"]="精准再生",["es"]="Regeneración precisa",["fr"]="La Régénération précise",["de"]="Präzise Regeneration",["ru"]="Точное восстановление",["en"]="Precise Regeneration"},[380]={["zh"]="先知",["es"]="Profeta",["fr"]="L’Ensemble du Prophète",["de"]="Prophet",["ru"]="Пророк",["en"]="Prophet's"},[381]={["zh"]="破碎灵魂",["es"]="Alma desgarrada",["fr"]="L’Âme brisée",["de"]="Zerbrochene Seele",["ru"]="Расколотая душа",["en"]="Broken Soul"},[382]={["zh"]="幽暗恩惠",["es"]="Gracia de la penumbra",["fr"]="La Gracieuse mélancolie",["de"]="Anmut der Gräue",["ru"]="Милосердие мрака",["en"]="Grace of Gloom"},[383]={["zh"]="狮鹫虐行",["es"]="Ferocidad del grifo",["fr"]="La Férocité du griffon",["de"]="Wildheit des Greifen",["ru"]="Свирепость грифона",["en"]="Gryphon's Ferocity"},[384]={["zh"]="瓦努斯睿智",["es"]="Sabiduría de Vano",["fr"]="La Sagesse de Vanus",["de"]="Weisheit von Vanus",["ru"]="Мудрость Вануса",["en"]="Wisdom of Vanus"},[385]={["zh"]="熟练骑手",["es"]="Jinete adepto",["fr"]="L’Adepte cavalier",["de"]="Versierter Reiter",["ru"]="Умелый наездник",["en"]="Adept Rider"},[386]={["zh"]="蛞蝓人的幻影",["es"]="Semblanza del sload",["fr"]="L’Aspect du Sload",["de"]="Kreckenantlitz",["ru"]="Подобие слоада",["en"]="Sload's Semblance"},[387]={["zh"]="诺克图娜尔的青睐",["es"]="Favor de Nocturnal",["fr"]="Faveur de Nocturne",["de"]="Nocturnals Gunst",["ru"]="Благословение Ноктюрнал",["en"]="Nocturnal's Favor"},[388]={["zh"]="加伦韦之护",["es"]="Égida de Galenwe",["fr"]="L’Égide de Galenwe",["de"]="Ägis von Galenwe",["ru"]="Эгида Галенве",["en"]="Aegis of Galenwe"},[389]={["zh"]="瑞利昆恩的武装",["es"]="Blasón de Relequen",["fr"]="Les Armes de Relequen",["de"]="Waffen von Relequen",["ru"]="Вооружение Релеквена",["en"]="Arms of Relequen"},[390]={["zh"]="希罗利亚衣钵",["es"]="Manto de Siroria",["fr"]="Le Manteau de Siroria",["de"]="Mantel von Siroria",["ru"]="Мантия Сирории",["en"]="Mantle of Siroria"},[391]={["zh"]="奥尔奥瑞姆的法衣",["es"]="Vestimentas de Olorime",["fr"]="La Vêture d'Olorimë",["de"]="Gewandung von Olorime",["ru"]="Одеяние Олоримы",["en"]="Vestment of Olorime"},[392]={["zh"]="完美加伦韦之护",["es"]="Égida de Galenwe perfeccionada",["fr"]="L’Égide de Galenwe perfectionnée",["de"]="Perfektionierte Ägis von Galenwe",["ru"]="Совершенная эгида Галенве",["en"]="Perfected Aegis of Galenwe"},[393]={["zh"]="完美瑞利昆恩的武装",["es"]="Blasón de Relequen perfeccionado",["fr"]="Armes de Relequen perfectionnées",["de"]="Perfektionierte Waffen von Relequen",["ru"]="Совершенное вооружение Релеквена",["en"]="Perfected Arms of Relequen"},[394]={["zh"]="完美希罗利亚衣钵",["es"]="Manto de Siroria perfeccionado",["fr"]="Le Manteau de Siroria perfectionné",["de"]="Perfektionierter Mantel von Siroria",["ru"]="Совершенная мантия Сирории",["en"]="Perfected Mantle of Siroria"},[395]={["zh"]="完美奥尔奥瑞姆的法衣",["es"]="Vestimentas de Olorime perfeccionadas",["fr"]="La vêture d'Olorimë perfectionnée",["de"]="Perfektionierte Gewandung von Olorime",["ru"]="Совершенное одеяние Олоримы",["en"]="Perfected Vestment of Olorime"},[397]={["zh"]="巴洛格",["es"]="Balorgh",["fr"]="Balorgh",["de"]="Balorgh",["ru"]="Балорг",["en"]="Balorgh"},[398]={["zh"]="维科萨",["es"]="Vykosa",["fr"]="Vykosa",["de"]="Vykosa",["ru"]="Вайкоса",["en"]="Vykosa"},[399]={["zh"]="哈努的怜悯",["es"]="Compasión de Hanu",["fr"]="La Compassion de Hanu",["de"]="Hanus Mitgefühl",["ru"]="Сострадание Хану",["en"]="Hanu's Compassion"},[400]={["zh"]="血月",["es"]="Luna de sangre",["fr"]="La Lune de sang",["de"]="Blutmond",["ru"]="Кровавая луна",["en"]="Blood Moon"},[401]={["zh"]="乌瑟丝的避风港",["es"]="Amparo de Ursus",["fr"]="Le Havre d'Ursus",["de"]="Zuflucht von Ursus",["ru"]="Опека Урсус",["en"]="Haven of Ursus"},[402]={["zh"]="月狩",["es"]="Cazador de la luna",["fr"]="Le Chasseur lunaire",["de"]="Mondjäger",["ru"]="Лунный Охотник",["en"]="Moon Hunter"},[403]={["zh"]="野蛮狼人",["es"]="Hombre lobo salvaje",["fr"]="Le Loup-garou sauvage",["de"]="Wilder Werwolf",["ru"]="Кровожадный вервольф",["en"]="Savage Werewolf"},[404]={["zh"]="狱卒的坚韧",["es"]="Tenacidad del carcelero",["fr"]="La Ténacité du Geôlier",["de"]="Tenazität des Kerkermeisters",["ru"]="Упорство тюремщика",["en"]="Jailer's Tenacity"},[405]={["zh"]="亮喉的夸耀",["es"]="Alarde de Garganta Brillante",["fr"]="La Vantardise de Vive-Gorge",["de"]="Hellhalsstolz",["ru"]="Хвастовство ярких-глоток",["en"]="Bright-Throat's Boast"},[406]={["zh"]="冥潭的狡诈",["es"]="Astucia de Agua Muerta",["fr"]="La Duplicité d'Aiguemortes",["de"]="Totwassertücke",["ru"]="Коварство мертвой-воды",["en"]="Dead-Water's Guile"},[407]={["zh"]="希思特勇士",["es"]="Campeón del hist",["fr"]="Le Champion de l'Hist",["de"]="Histchampion",["ru"]="Герой хиста",["en"]="Champion of the Hist"},[408]={["zh"]="坟标收集者",["es"]="Coleccionista de estacas sepulcrales",["fr"]="Collectionneur de marqueurs funéraires",["de"]="Grabpflocksammler",["ru"]="Собиратель могильных столбов",["en"]="Grave-Stake Collector"},[409]={["zh"]="纳迦萨满",["es"]="Chamán naga",["fr"]="Le Chaman Naga",["de"]="Nagaschamane",["ru"]="Нага-шаман",["en"]="Naga Shaman"},[410]={["zh"]="失落军团的力量",["es"]="Poder de la Legión Perdida",["fr"]="La Puissance de la Légion perdue",["de"]="Macht der Verlorenen Legion",["ru"]="Мощь Забытого легиона",["en"]="Might of the Lost Legion"},[411]={["zh"]="勇武冲锋",["es"]="Carga valerosa",["fr"]="Charge vaillante",["de"]="Galanter Ansturm",["ru"]="Натиск кавалера",["en"]="Gallant Charge"},[412]={["zh"]="径向升天击",["es"]="Gancho radial",["fr"]="Uppercut radial",["de"]="Rundum-Aufwärtsschnitt",["ru"]="Круговой апперкот",["en"]="Radial Uppercut"},[413]={["zh"]="幽灵斗篷",["es"]="Manto espectral",["fr"]="Cape spectrale",["de"]="Spektraler Mantel",["ru"]="Призрачная оборона",["en"]="Spectral Cloak"},[414]={["zh"]="剧毒射击",["es"]="Disparo virulento",["fr"]="Tir virulent",["de"]="Virulenter Schuss",["ru"]="Смертельный выстрел",["en"]="Virulent Shot"},[415]={["zh"]="狂野脉冲",["es"]="Impulso salvaje",["fr"]="Impulsion sauvage",["de"]="Wilder Impuls",["ru"]="Дикий импульс",["en"]="Wild Impulse"},[416]={["zh"]="治疗师结界",["es"]="Guardia del sanador",["fr"]="Garde du soigneur",["de"]="Schutz des Pflegers",["ru"]="Оберег лекаря",["en"]="Mender's Ward"},[417]={["zh"]="不屈怒火",["es"]="Furia indomable",["fr"]="Fureur indomptable",["de"]="Unbeugsamer Zorn",["ru"]="Неукротимый гнев",["en"]="Indomitable Fury"},[418]={["zh"]="法术战略家",["es"]="Estratega arcano",["fr"]="Stratège des sorts",["de"]="Zauberstratege",["ru"]="Заклинатель-стратег",["en"]="Spell Strategist"},[419]={["zh"]="战场杂耍家",["es"]="Acróbata del campo de batalla",["fr"]="Acrobate du champ de bataille",["de"]="Schlachtfeldakrobat",["ru"]="Боевая акробатика",["en"]="Battlefield Acrobat"},[420]={["zh"]="苦痛战士",["es"]="Soldado de la angustia",["fr"]="Soldat de l'angoisse",["de"]="Soldat der Pein",["ru"]="Солдат страдания",["en"]="Soldier of Anguish"},[421]={["zh"]="坚定英雄",["es"]="Héroe inquebrantable",["fr"]="Héros inébranlable",["de"]="Standhafter Held",["ru"]="Стойкий герой",["en"]="Steadfast Hero"},[422]={["zh"]="军营守卫者",["es"]="Defensor del batallón",["fr"]="Défense du bataillon",["de"]="Bataillonverteidiger",["ru"]="Защитник батальона",["en"]="Battalion Defender"},[423]={["zh"]="完美勇武冲锋",["es"]="Carga valerosa perfeccionada",["fr"]="Charge galante perfectionnée",["de"]="Perfektionierter galanter Ansturm",["ru"]="Совершенный натиск кавалера",["en"]="Perfected Gallant Charge"},[424]={["zh"]="完美径向升天击",["es"]="Gancho radial perfeccionado",["fr"]="Uppercut radial perfectionné",["de"]="Perfektionierter Rundum-Aufwärtsschnitt",["ru"]="Совершенный круговой апперкот",["en"]="Perfected Radial Uppercut"},[425]={["zh"]="完美幽灵斗篷",["es"]="Manto espectral perfeccionado",["fr"]="Cape spectrale perfectionnée",["de"]="Perfektionierter spektraler Mantel",["ru"]="Совершенная призрачная оборона",["en"]="Perfected Spectral Cloak"},[426]={["zh"]="完美剧毒射击",["es"]="Disparo virulento perfeccionado",["fr"]="Tir virulent perfectionné",["de"]="Perfektionierter virulenter Schuss",["ru"]="Совершенный смертельный выстрел",["en"]="Perfected Virulent Shot"},[427]={["zh"]="完美狂野脉冲",["es"]="Impulso salvaje perfeccionado",["fr"]="Impulsion sauvage perfectionnée",["de"]="Perfektionierter wilder Impuls",["ru"]="Совершенный дикий импульс",["en"]="Perfected Wild Impulse"},[428]={["zh"]="完美治疗师结界",["es"]="Custodia de ensalmador perfeccionado",["fr"]="Garde du guérisseur perfectionnée",["de"]="Perfektionierter Schutz des Pflegers",["ru"]="Совершенный оберег лекаря",["en"]="Perfected Mender's Ward"},[429]={["zh"]="巨大冰川",["es"]="Glaciar poderoso",["fr"]="Le Puissant glacier",["de"]="Mächtiger Gletscher",["ru"]="Великий ледник",["en"]="Mighty Glacier"},[430]={["zh"]="佐格文战团",["es"]="Banda de guerra de Tzogvin",["fr"]="La Bande de guerre de Tzogvin",["de"]="Tzogvins Kriegstrupp",["ru"]="Отряд Тзогвина",["en"]="Tzogvin's Warband"},[431]={["zh"]="冰霜妖术师",["es"]="Conjurador glacial",["fr"]="L’Invocateur glacial",["de"]="Eisiger Anrufer",["ru"]="Ледяной заклинатель",["en"]="Icy Conjurer"},[432]={["zh"]="石之看守",["es"]="Guardián de la tablilla",["fr"]="Le Gardien des pierres",["de"]="Steinwahrer",["ru"]="Хранитель Камня",["en"]="Stonekeeper"},[433]={["zh"]="冰冻守望者",["es"]="Vigilante glacial",["fr"]="L’Observateur glacial",["de"]="Gefrorener Wächter",["ru"]="Морозный страж",["en"]="Frozen Watcher"},[434]={["zh"]="死亡净化",["es"]="Perdición carroñera",["fr"]="Le Trépas des récupérateurs",["de"]="Ausbeutender Niedergang",["ru"]="Беспощадный Пожиратель",["en"]="Scavenging Demise"},[435]={["zh"]="奥罗兰之雷",["es"]="Relámpago aurorano",["fr"]="Le Tonnerre aurorien",["de"]="Donner des Auroraners",["ru"]="Гром аврорианца",["en"]="Auroran's Thunder"},[436]={["zh"]="利刃交响曲",["es"]="Sinfonía de Espadas",["fr"]="La Symphonie des Lames",["de"]="Sinfonie der Klingen",["ru"]="Симфония Клинков",["en"]="Symphony of Blades"},[437]={["zh"]="冷港的最爱",["es"]="Elegido de Puerto Gélido",["fr"]="Élu de Havreglace",["de"]="Kalthafens Günstling",["ru"]="Любимец Хладной Гавани",["en"]="Coldharbour's Favorite"},[438]={["zh"]="森切战猫的勇气",["es"]="Osadía del senche-raht",["fr"]="La Détermination du Senche-raht",["de"]="Mut des Senche-raht",["ru"]="Воля сенч-рата",["en"]="Senche-raht's Grit"},[439]={["zh"]="瓦斯塔蕊的守护",["es"]="Tutela de Vastarie",["fr"]="La Tutelle de Vastarië",["de"]="Vastaries Vormundschaft",["ru"]="Опека Вастари",["en"]="Vastarie's Tutelage"},[440]={["zh"]="狡猾的猫人",["es"]="Alfiq pícaro",["fr"]="L’Alfiq rusé",["de"]="Listiger Alfiq",["ru"]="Хитрый альфик",["en"]="Crafty Alfiq"},[441]={["zh"]="达罗克·布雷的罩衫",["es"]="Vestimentas de Darloc Brae",["fr"]="Le Vêtement de Darloc Brae",["de"]="Gewandung von Darloc Brae",["ru"]="Одеяние Дарлока Брея",["en"]="Vesture of Darloc Brae"},[442]={["zh"]="送葬者的呼唤",["es"]="Llamada del Sepulturero",["fr"]="L’Appel du Croque-mort",["de"]="Ruf des Totengräbers",["ru"]="Зов могильщика",["en"]="Call of the Undertaker"},[443]={["zh"]="纳温塔丝之眼",["es"]="Ojo de Nahviintaas",["fr"]="L’Œil de Nahviintaas",["de"]="Auge von Nahviintaas",["ru"]="Глаз Навинтаса",["en"]="Eye of Nahviintaas"},[444]={["zh"]="伪神的祈祷",["es"]="Devoción del Falso Dios",["fr"]="Le Dévot du Faux dieu",["de"]="Ergebenheit zum falschen Gott",["ru"]="Преданность ложного бога",["en"]="False God's Devotion"},[445]={["zh"]="洛克提兹之牙",["es"]="Diente de Lokkestiiz",["fr"]="La Dent de Lokkestiiz",["de"]="Zahn von Lokkestiiz",["ru"]="Зуб Локкестиза",["en"]="Tooth of Lokkestiiz"},[446]={["zh"]="尤尔纳克林之爪",["es"]="Garra de Yolnahkriin",["fr"]="La griffe d’Yolnahkriin",["de"]="Kralle von Yolnahkriin",["ru"]="Коготь Йолнакрина",["en"]="Claw of Yolnahkriin"},[448]={["zh"]="完美纳温塔丝之眼",["es"]="Ojo de Nahviintaas perfeccionado",["fr"]="L’Œil de Nahviintaas perfectionné",["de"]="Perfektioniertes Auge von Nahviintaas",["ru"]="Совершенный глаз Навинтаса",["en"]="Perfected Eye of Nahviintaas"},[449]={["zh"]="完美伪神的祈祷",["es"]="Devoción del Falso Dios perfeccionada",["fr"]="Le Dévot du faux dieu perfectionné",["de"]="Perfektionierte Ergebenheit zum falschen Gott",["ru"]="Совершенная преданность ложного бога",["en"]="Perfected False God's Devotion"},[450]={["zh"]="完美洛克提兹之牙",["es"]="Diente de Lokkestiiz perfeccionado",["fr"]="Dent de Lokkestiiz perfectionnée",["de"]="Perfektionierter Zahn von Lokkestiiz",["ru"]="Совершенный зуб Локкестиза",["en"]="Perfected Tooth of Lokkestiiz"},[451]={["zh"]="完美尤尔纳克林之爪",["es"]="Garra de Yolnahkriin perfeccionada",["fr"]="La Griffe de Lokkestiiz perfectionnée",["de"]="Perfektionierte Kralle von Yolnahkriin",["ru"]="Совершенный коготь Йолнакрина",["en"]="Perfected Claw of Yolnahkriin"},[452]={["zh"]="空心牙之渴",["es"]="Sed de Colmillo Hueco",["fr"]="La Soif du Croc creux",["de"]="Hohlzahndurst",["ru"]="Жажда Полого Клыка",["en"]="Hollowfang Thirst"},[453]={["zh"]="多罗扎卡之爪",["es"]="Garras de Dro'Zakar",["fr"]="Griffes de Dro'Zakar",["de"]="Krallen von Dro'Zakar",["ru"]="Когти Дро'закара",["en"]="Dro'Zakar's Claws"},[454]={["zh"]="雷纳德的决心",["es"]="Determinación de Renald",["fr"]="Résolution de Ranald",["de"]="Renalds Entschlossenheit",["ru"]="Решимость Ренальда",["en"]="Renald's Resolve"},[455]={["zh"]="祖恩的匡正",["es"]="Rectificación de Z'en",["fr"]="Redressement de Z'en",["de"]="Z'ens Wiedergutmachung",["ru"]="Возмездие З'ена",["en"]="Z'en's Redress"},[456]={["zh"]="蔚蓝枯萎收割者",["es"]="Segador de la peste cerúlea",["fr"]="Le Moissonneur de Pestazur",["de"]="Azurfäuleschnitter",["ru"]="Жнец Лазурной чумы",["en"]="Azureblight Reaper"},[457]={["zh"]="龙之亵渎",["es"]="Profanación del dragón",["fr"]="La Profanation du dragon",["de"]="Drachenschändung",["ru"]="Осквернение дракона",["en"]="Dragon's Defilement"},[458]={["zh"]="格伦德伍尔夫",["es"]="Grundwulf",["fr"]="Grundwulf",["de"]="Grundwulf",["ru"]="Грандвульф",["en"]="Grundwulf"},[459]={["zh"]="马塞洛克",["es"]="Maarselok",["fr"]="Maarselok",["de"]="Maarselok",["ru"]="Марселок",["en"]="Maarselok"},[465]={["zh"]="森查尔守卫者",["es"]="Defensor de Senchal",["fr"]="Défenseur de Senchal",["de"]="Verteidiger von Senchal",["ru"]="Защитник Сенчала",["en"]="Senchal Defender"},[466]={["zh"]="强盗之速",["es"]="Prisa del merodeador",["fr"]="La Hâte du Maraudeur",["de"]="Marodeurshast",["ru"]="Шустрый мародер",["en"]="Marauder's Haste"},[467]={["zh"]="龙卫精英",["es"]="Élite de la Guardia del Dragón",["fr"]="Élite de la Garde du dragon",["de"]="Drachengardeelite",["ru"]="Элита Драконьей стражи",["en"]="Dragonguard Elite"},[468]={["zh"]="英勇海盗",["es"]="Corsario aventurado",["fr"]="Le Corsaire intrépide",["de"]="Kühner Korsar",["ru"]="Удалой корсар",["en"]="Daring Corsair"},[469]={["zh"]="古代龙卫",["es"]="La Guardia del Dragón de antaño",["fr"]="La Garde du dragon antique",["de"]="Alte Drachengarde",["ru"]="Древняя Драконья стража",["en"]="Ancient Dragonguard"},[470]={["zh"]="新月侍祭",["es"]="Acólito de la Luna Nueva",["fr"]="Acolyte de la Nouvelle lune",["de"]="Akolyth des Neuen Mondes",["ru"]="Служитель Новой Луны",["en"]="New Moon Acolyte"},[471]={["zh"]="希蒂之炉",["es"]="El hogar de Hiti",["fr"]="Le Foyer de Hiti",["de"]="Hitis Feuerstelle",["ru"]="Очаг Хити",["en"]="Hiti's Hearth"},[472]={["zh"]="泰坦后裔之力",["es"]="La fuerza de Sangre de Titán",["fr"]="La Force de Titan",["de"]="Titanenkinds Stärke",["ru"]="Сила Титанорожденной",["en"]="Titanborn Strength"},[473]={["zh"]="巴妮的刑罚",["es"]="El tormento de Bani",["fr"]="Le Tourment de Bani",["de"]="Banis Qual",["ru"]="Пытка Бани",["en"]="Bani's Torment"},[474]={["zh"]="盗墓者之握",["es"]="El agarre del draugrkin",["fr"]="La Poigne de Draugrien",["de"]="Griff der Draugrbrut",["ru"]="Хватка Родичей Драугров",["en"]="Draugrkin's Grip"},[475]={["zh"]="盾灵召唤者",["es"]="El invocador de égidas",["fr"]="Le Mande-égide",["de"]="Ägisrufer",["ru"]="Призыватель стража",["en"]="Aegis Caller"},[476]={["zh"]="坟墓守护者",["es"]="El guardián del sepulcro",["fr"]="Gardien du sépulcre",["de"]="Grabwächter",["ru"]="Защитник Могилы",["en"]="Grave Guardian"},[478]={["zh"]="巫母西恩奈特",["es"]="Madre Ciannait",["fr"]="Mère Ciannait",["de"]="Mutter Ciannait",["ru"]="Мать Кьяннайт",["en"]="Mother Ciannait"},[479]={["zh"]="卡尔纳尔的梦魇",["es"]="Pesadilla de Kjalnar",["fr"]="Le Cauchemar de Kjalnar",["de"]="Kjalnars Albtraum",["ru"]="Кошмар Кьялнара",["en"]="Kjalnar's Nightmare"},[480]={["zh"]="暴击反制",["es"]="Estocada crítica",["fr"]="Riposte critique",["de"]="Kritischer Konter",["ru"]="Сокрушительный контрудар",["en"]="Critical Riposte"},[481]={["zh"]="碎链斗士",["es"]="Agresor desencadenado",["fr"]="L’Agresseur déchaîné",["de"]="Entfesselter Aggressor",["ru"]="Освобожденный захватчик",["en"]="Unchained Aggressor"},[482]={["zh"]="无畏战士",["es"]="Combatiente indómito",["fr"]="Le Combattant indompté",["de"]="Furchtloser Kombattant",["ru"]="Бесстрашный боец",["en"]="Dauntless Combatant"},[487]={["zh"]="凛冬之憩",["es"]="Respiro invernal",["fr"]="Répit de l'hiver",["de"]="Winterruhe",["ru"]="Зимний покой",["en"]="Winter's Respite"},[488]={["zh"]="剧毒惩戒",["es"]="Castigo venenoso",["fr"]="Châtiment venimeux",["de"]="Giftschlag",["ru"]="Ядовитая кара",["en"]="Venomous Smite"},[489]={["zh"]="永恒活力",["es"]="Vigor eterno",["fr"]="Vigueur éternelle",["de"]="Ewiger Elan",["ru"]="Неиссякаемые силы",["en"]="Eternal Vigor"},[490]={["zh"]="斯图恩的眷顾",["es"]="Favor de Stuhn",["fr"]="La Faveur de Stuhn",["de"]="Stuhns Gunst",["ru"]="Благословение Стуна",["en"]="Stuhn's Favor"},[491]={["zh"]="龙之馋涎",["es"]="Apetito de dragón",["fr"]="L’Appétit du dragon",["de"]="Appetit des Drachen",["ru"]="Драконий голод",["en"]="Dragon's Appetite"},[492]={["zh"]="凯恩之风",["es"]="Viento de Kyne",["fr"]="Le Vent de Kyne",["de"]="Kynes Wind",["ru"]="Ветер Кин",["en"]="Kyne's Wind"},[493]={["zh"]="完美凯恩之风",["es"]="Viento de Kyne perfeccionado",["fr"]="Le Vent de Kyne perfectionné",["de"]="Kynes perfektionierter Wind",["ru"]="Совершенный ветер Кин",["en"]="Perfected Kyne's Wind"},[494]={["zh"]="威若之令",["es"]="Comandancia de Vrol",["fr"]="Commandement de Vrol",["de"]="Vrols Befehl",["ru"]="Приказ Врола",["en"]="Vrol's Command"},[495]={["zh"]="完美威若之令",["es"]="Comandancia de Vrol perfeccionada",["fr"]="Le Commandement de Vrol perfectionné",["de"]="Vrols perfektionierter Befehl",["ru"]="Совершенный приказ Врола",["en"]="Perfected Vrol's Command"},[496]={["zh"]="决胜时机",["es"]="Rugido oportunista",["fr"]="L’Opportuniste rugissant",["de"]="Tosender Opportunist",["ru"]="Бесстрашный авантюрист",["en"]="Roaring Opportunist"},[497]={["zh"]="完美决胜时机",["es"]="Rugido oportunista perfeccionado",["fr"]="L’Opportuniste rugissant perfectionné",["de"]="Perfektionierter tosender Opportunist",["ru"]="Совершенный бесстрашный авантюрист",["en"]="Perfected Roaring Opportunist"},[498]={["zh"]="扬迪尔之力",["es"]="Robustez de Yandir",["fr"]="La Puissance d’Yandir",["de"]="Yandirs Macht",["ru"]="Мощь Яндира",["en"]="Yandir's Might"},[499]={["zh"]="完美扬迪尔之力",["es"]="Robustez de Yandir perfeccionada",["fr"]="La Puissance d’Yandir perfectionnée",["de"]="Yandirs perfektionierte Macht",["ru"]="Совершенная мощь Яндира",["en"]="Perfected Yandir's Might"},[501]={["zh"]="塔西安扼杀者",["es"]="Estranguladores thrassianos",["fr"]="Les Étrangleurs thrassiens",["de"]="Thrassische Würger",["ru"]="Трассианские душители",["en"]="Thrassian Stranglers"},[503]={["zh"]="狂猎之戒",["es"]="Anillo de la Cacería Salvaje",["fr"]="Anneau de la Chasse sauvage",["de"]="Ring der Wilden Jagd",["ru"]="Кольцо Дикой охоты",["en"]="Ring of the Wild Hunt"},[505]={["zh"]="恒音颈环",["es"]="Torque de constancia tonal",["fr"]="Le Torque de constance tonale",["de"]="Torques der Klangkonstanz",["ru"]="Ожерелье тонального постоянства",["en"]="Torc of Tonal Constancy"},[506]={["zh"]="法术寄生",["es"]="Parásito arcano",["fr"]="Le Parasite magique",["de"]="Magieparasit",["ru"]="Маг-кровопийца",["en"]="Spell Parasite"},[513]={["zh"]="塔尔菲格的背叛",["es"]="Traición de Talfyg",["fr"]="La Traîtrise de Talfyg",["de"]="Talfygs Verrat",["ru"]="Предательство Тальфика",["en"]="Talfyg's Treachery"},[514]={["zh"]="恐惧释放",["es"]="Terror desencadenado",["fr"]="Terreur déchaînée",["de"]="Entfesselter Schrecken",["ru"]="Высвобожденный ужас",["en"]="Unleashed Terror"},[515]={["zh"]="猩红影暮",["es"]="Crepúsculo carmesí",["fr"]="Le Crépuscule écarlate",["de"]="Scharlachrotes Zwielicht",["ru"]="Алый сумрак",["en"]="Crimson Twilight"},[516]={["zh"]="元素催化剂",["es"]="Catalizador elemental",["fr"]="Catalyseur élémentaire",["de"]="Elementarer Katalysator",["ru"]="Катализатор стихий",["en"]="Elemental Catalyst"},[517]={["zh"]="克拉伦格之吼",["es"]="Aullido de Kraglen",["fr"]="Hurlement de Kraglen",["de"]="Kraglens Heulen",["ru"]="Вой Краглена",["en"]="Kraglen's Howl"},[518]={["zh"]="阿卡西斯的天赋",["es"]="Ingenio de Arkasis",["fr"]="Génie d'Arkasis",["de"]="Arkasis' Genie",["ru"]="Гений Аркасиса",["en"]="Arkasis's Genius"},[519]={["zh"]="踏雪靴",["es"]="Calzas de nieve",["fr"]="L’Arpenteurs de neige",["de"]="Schneetreter",["ru"]="Снегоходы",["en"]="Snow Treaders"},[520]={["zh"]="玛拉卡斯残暴之戒",["es"]="Banda de la brutalidad de Malacath",["fr"]="La Bande de brutalité de Malacath",["de"]="Malacaths Band der Brutalität",["ru"]="Кольцо жестокости Малаката",["en"]="Malacath's Band of Brutality"},[521]={["zh"]="血主之拥",["es"]="Abrazo de señor de la sangre",["fr"]="L’Étreinte du Seigneur de sang",["de"]="Umarmung des Blutfürsten",["ru"]="Объятия кровавого лорда",["en"]="Bloodlord's Embrace"},[522]={["zh"]="完美无情冲锋",["es"]="Carga despiadada perfeccionada",["fr"]="Charge impitoyable perfectionnée",["de"]="Perfektionierter gnadenloser Ansturm",["ru"]="Совершенный безжалостный рывок",["en"]="Perfected Merciless Charge"},[523]={["zh"]="完美狂暴斩击",["es"]="Hendidura inexorable perfeccionada",["fr"]="Entaille ravageuse perfectionnée",["de"]="Perfektionierter tobender Schnitt",["ru"]="Совершенный свирепый удар",["en"]="Perfected Rampaging Slash"},[524]={["zh"]="完美残暴乱舞",["es"]="Ráfaga cruel perfeccionada",["fr"]="Le Déluge cruel perfectionné",["de"]="Perfektionierter grausamer Schlaghagel",["ru"]="Совершенный жестокий шквал",["en"]="Perfected Cruel Flurry"},[525]={["zh"]="完美雷霆齐射",["es"]="Salva atronadora perfeccionada",["fr"]="Volée tonitruante perfectionnée",["de"]="Perfektionierte donnernde Salve",["ru"]="Совершенный разрушительный залп",["en"]="Perfected Thunderous Volley"},[526]={["zh"]="完美碾压之墙",["es"]="Muro aplastante perfeccionado",["fr"]="Mur écrasant perfectionné",["de"]="Perfektionierte zermalmende Mauer",["ru"]="Совершенная сокрушительная стена",["en"]="Perfected Crushing Wall"},[527]={["zh"]="完美精准再生",["es"]="Regeneración precisa perfeccionada",["fr"]="Régénération précise perfectionnée",["de"]="Perfektionierte präzise Regeneration",["ru"]="Совершенное точное восстановление",["en"]="Perfected Precise Regeneration"},[528]={["zh"]="完美巨神劈砍",["es"]="Cuchillada titánica perfeccionada",["fr"]="Fendoir titanesque perfectionné",["de"]="Perfektioniertes titanisches Trennen",["ru"]="Совершенное титаническое рассечение",["en"]="Perfected Titanic Cleave"},[529]={["zh"]="完美穿刺疗愈",["es"]="Remedio perforante perfeccionado",["fr"]="Remède perforant perfectionné",["de"]="Perfektionierte durchschlagende Genesung",["ru"]="Совершенное исцеление уколом",["en"]="Perfected Puncturing Remedy"},[530]={["zh"]="完美刺斩",["es"]="Cuchilladas punzantes perfeccionadas",["fr"]="Entailles cuisantes perfectionnées",["de"]="Perfektionierte stechende Schnitte",["ru"]="Совершенные кровоточащие порезы",["en"]="Perfected Stinging Slashes"},[531]={["zh"]="完美腐蚀箭矢",["es"]="Flecha cáustica perfeccionada",["fr"]="Flèche caustique perfectionnée",["de"]="Perfektionierter beißender Pfeil",["ru"]="Совершенная разъедающая стрела",["en"]="Perfected Caustic Arrow"},[532]={["zh"]="完美毁灭冲撞",["es"]="Impacto destructivo perfeccionado",["fr"]="Impact destructeur perfectionné",["de"]="Perfektionierter zerstörerischer Einschlag",["ru"]="Совершенное разрушительное столкновение",["en"]="Perfected Destructive Impact"},[533]={["zh"]="完美盛大复苏",["es"]="Gran rejuvenecimiento perfeccionado",["fr"]="Grand rajeunissement perfectionné",["de"]="Perfektionierte große Verjüngung",["ru"]="Совершенное высшее восстановление",["en"]="Perfected Grand Rejuvenation"},[534]={["zh"]="石僵傀",["es"]="Cascarón pétreo",["fr"]="L’Enveloppe de pierre",["de"]="Steinhülle",["ru"]="Каменная оболочка",["en"]="Stone Husk"},[535]={["zh"]="荆棘夫人",["es"]="Lady Espina",["fr"]="La Dame Ronce",["de"]="Fürstin Dorn",["ru"]="Леди Шипов",["en"]="Lady Thorn"},[536]={["zh"]="辐光堡垒",["es"]="Bastión radiante",["fr"]="Le Bastion radieux",["de"]="Strahlende Bastion",["ru"]="Лучезарный оплот",["en"]="Radiant Bastion"},[537]={["zh"]="虚无召唤者",["es"]="Invocador del vacío",["fr"]="Le Mande-vide",["de"]="Leerenrufer",["ru"]="Призыватель пустоты",["en"]="Voidcaller"},[538]={["zh"]="女巫骑士的藐视",["es"]="Desafío del caballero brujo",["fr"]="La Défiance du Chevalier-Sorcière",["de"]="Trotz des Hexenritters",["ru"]="Дерзость рыцаря-колдуна",["en"]="Witch-Knight's Defiance"},[539]={["zh"]="红鹰之怒",["es"]="Furia del Águila Roja",["fr"]="La Fureur de l'Aigle rouge",["de"]="Zorn des Roten Adlers",["ru"]="Ярость Красного Орла",["en"]="Red Eagle's Fury"},[540]={["zh"]="卡尔斯的遗产",["es"]="Legado de Karth",["fr"]="L’Héritage de Karth",["de"]="Erbe des Karth",["ru"]="Наследие Карта",["en"]="Legacy of Karth"},[541]={["zh"]="以太天穹",["es"]="Ascensión aetérica",["fr"]="L’Ascension Aéthérienne",["de"]="Ätherischer Aufstieg",["ru"]="Этерическое вознесение",["en"]="Aetherial Ascension"},[542]={["zh"]="邪恶虹吸",["es"]="Maleficio de succión",["fr"]="Siphon de la malédiction",["de"]="Fluchsiphon",["ru"]="Поглощение магии",["en"]="Hex Siphon"},[543]={["zh"]="疫病之主",["es"]="Anfitrión pestilente",["fr"]="Hôte pestilentiel",["de"]="Pestilenter Wirt",["ru"]="Смертоносный рой",["en"]="Pestilent Host"},[544]={["zh"]="爆炸反制",["es"]="Reprimenda explosiva",["fr"]="Camouflet explosif",["de"]="Explosiver Tadel",["ru"]="Яростный отпор",["en"]="Explosive Rebuke"},[557]={["zh"]="行刑者之刃",["es"]="Hoja del verdugo",["fr"]="Lame du bourreau",["de"]="Klinge des Henkers",["ru"]="Клинок палача",["en"]="Executioner's Blade"},[558]={["zh"]="虚无猛击",["es"]="Aporreo del vacío",["fr"]="Percussion du vide",["de"]="Leerenwucht",["ru"]="Удар пустоты",["en"]="Void Bash"},[559]={["zh"]="狂暴振奋",["es"]="Ímpetu frenético",["fr"]="L’Élan frénétique",["de"]="Tobendes Momentum",["ru"]="Неистовая сила",["en"]="Frenzied Momentum"},[560]={["zh"]="近程狙击",["es"]="Disparo a quemarropa",["fr"]="Tir à Bout portant",["de"]="Volltreffer",["ru"]="Выстрел в упор",["en"]="Point-Blank Snipe"},[561]={["zh"]="元素之怒",["es"]="Ira elemental",["fr"]="Courroux des éléments",["de"]="Zorn der Elemente",["ru"]="Гнев стихий",["en"]="Wrath of Elements"},[562]={["zh"]="力量溢流",["es"]="Desbordamiento de fuerza",["fr"]="Le Débordement de force",["de"]="Machtüberschuss",["ru"]="Переполняющая сила",["en"]="Force Overflow"},[563]={["zh"]="完美行刑者之刃",["es"]="Hoja del verdugo perfeccionada",["fr"]="La Lame du bourreau perfectionnée",["de"]="Perfektionierte Klinge des Henkers",["ru"]="Совершенный клинок палача",["en"]="Perfected Executioner's Blade"},[564]={["zh"]="完美虚无猛击",["es"]="Aporreo del vacío perfeccionado",["fr"]="La Percussion du Vide perfectionnée",["de"]="Perfektionierte Leerenwucht",["ru"]="Совершенный удар пустоты",["en"]="Perfected Void Bash"},[565]={["zh"]="完美狂暴振奋",["es"]="Ímpetu frenético perfeccionado",["fr"]="L’Élan frénétique perfectionné",["de"]="Perfektioniertes tobendes Momentum",["ru"]="Совершенная неистовая сила",["en"]="Perfected Frenzied Momentum"},[566]={["zh"]="完美近程狙击",["es"]="Disparo a quemarropa perfeccionado",["fr"]="Le Tir de précision à bout portant perfectionné",["de"]="Perfektionierter Volltreffer",["ru"]="Совершенный выстрел в упор",["en"]="Perfected Point-Blank Snipe"},[567]={["zh"]="完美元素之怒",["es"]="Ira elemental perfeccionada",["fr"]="Le Courroux des éléments perfectionné",["de"]="Perfektionierter Zorn der Elemente",["ru"]="Совершенный гнев стихий",["en"]="Perfected Wrath of Elements"},[568]={["zh"]="完美力量溢流",["es"]="Desbordamiento de fuerza perfeccionado",["fr"]="Le Débordement de force perfectionné",["de"]="Perfektionierter Machtüberschuss",["ru"]="Совершенная переполняющая сила",["en"]="Perfected Force Overflow"},[569]={["zh"]="真誓之怒",["es"]="Furia de los Más Fieles",["fr"]="Furie du Fidèle",["de"]="Wahrgeschworenen-Raserei",["ru"]="Ярость Истинной Клятвы",["en"]="True-Sworn Fury"},[570]={["zh"]="金拉斯的盛怒",["es"]="Ira de Kinras",["fr"]="Colère de Kinras",["de"]="Kinras' Zorn",["ru"]="Гнев Кинраса",["en"]="Kinras's Wrath"},[571]={["zh"]="德雷克的冲刺",["es"]="Prisa del dragón",["fr"]="Ruée du dragon",["de"]="Dracheneile",["ru"]="Натиск Змея",["en"]="Drake's Rush"},[572]={["zh"]="脱缰祭仪者",["es"]="Ritualista desatado",["fr"]="Ritualiste déchaîné",["de"]="Entfesselter Ritualist",["ru"]="Освобожденный ритуалист",["en"]="Unleashed Ritualist"},[573]={["zh"]="大衮之统御",["es"]="Dominio de Dagon",["fr"]="Domaine de Dagon",["de"]="Dagons Herrschaft",["ru"]="Сила Дагона",["en"]="Dagon's Dominion"},[574]={["zh"]="弑愚者的加护",["es"]="Custodia de la Matanecios",["fr"]="Garde de Follicide",["de"]="Schutz der Narrentöterin",["ru"]="Оберег Убийцы Глупцов",["en"]="Foolkiller's Ward"},[575]={["zh"]="白地骑士团之戒",["es"]="Anillo de la Orden Pálida",["fr"]="L’anneau de l'Ordre pâle",["de"]="Ring des Fahlen Ordens",["ru"]="Кольцо Бледного ордена",["en"]="Ring of the Pale Order"},[576]={["zh"]="埃尔诺菲的珍珠项链",["es"]="Perlas de los ehlnofey",["fr"]="Les Perles des Ehlnofey",["de"]="Perlen der Ehlnofey",["ru"]="Жемчужины эльнофей",["en"]="Pearls of Ehlnofey"},[577]={["zh"]="恩克拉迪斯的巨兽",["es"]="Behemot de Engracio",["fr"]="Béhémoth d'Encratis",["de"]="Encratis' Behemoth",["ru"]="Чудовище Энкратиса",["en"]="Encratis's Behemoth"},[578]={["zh"]="佐德鲁斯男爵",["es"]="Barón Zaudrus",["fr"]="Le baron Zaudrus",["de"]="Baron Zaudrus",["ru"]="Барон Зодрус",["en"]="Baron Zaudrus"},[579]={["zh"]="霜噬",["es"]="Congelación",["fr"]="Engelure",["de"]="Frostbiss",["ru"]="Обморожение",["en"]="Frostbite"},[580]={["zh"]="死地刺客",["es"]="Asesino de las Tierras Muertas",["fr"]="L’assassin des Terres mortes",["de"]="Totenländer-Assassine",["ru"]="Убийца из Мертвых Земель",["en"]="Deadlands Assassin"},[581]={["zh"]="泥沼掠夺者",["es"]="Saqueador de la ciénaga",["fr"]="Le Pillard du marais",["de"]="Moorräuber",["ru"]="Болотный разбойник",["en"]="Bog Raider"},[582]={["zh"]="希思特低语者",["es"]="Susurrador del hist",["fr"]="Le Complice de l'Hist",["de"]="Histflüsterer",["ru"]="Шептун хиста",["en"]="Hist Whisperer"},[583]={["zh"]="哈特兰征服者",["es"]="Conquistador de las Tierras Centrales",["fr"]="Le Conquérant de la Patrie",["de"]="Herzlanderoberer",["ru"]="Завоеватель Среднеземья",["en"]="Heartland Conqueror"},[584]={["zh"]="钻石的胜利",["es"]="Victoria del diamante",["fr"]="La Victoire du Diamant",["de"]="Sieg des Diamanten",["ru"]="Алмазная победа",["en"]="Diamond's Victory"},[585]={["zh"]="撒西勒勇士",["es"]="Campeón saxhleel",["fr"]="Le Champion des Saxhleel",["de"]="Saxhleel-Champion",["ru"]="Воитель саксхлил",["en"]="Saxhleel Champion"},[586]={["zh"]="苏尔山的折磨",["es"]="Tormento de los Sul-Xan",["fr"]="Le Tourment des Sul-Xan",["de"]="Folter der Sul-Xan",["ru"]="Пытка сул-зан",["en"]="Sul-Xan's Torment"},[587]={["zh"]="巴塞的癫狂",["es"]="Manía de Bahsei",["fr"]="La Folie de Bahsei",["de"]="Bahseis Manie",["ru"]="Безумие Басей",["en"]="Bahsei's Mania"},[588]={["zh"]="石语者的誓言",["es"]="Juramento del portavoz de la piedra",["fr"]="Le Serment du Parlepierre",["de"]="Eid des Steinsprechers",["ru"]="Клятва говорящего с камнями",["en"]="Stone-Talker's Oath"},[589]={["zh"]="完美撒西勒勇士",["es"]="Campeón saxhleel perfeccionado",["fr"]="Le Champion des Saxhleel perfectionné",["de"]="Perfektionierter Saxhleel-Champion",["ru"]="Совершенный воитель саксхлил",["en"]="Perfected Saxhleel Champion"},[590]={["zh"]="完美苏尔山的折磨",["es"]="Tormento de los Sul-Xan perfeccionado",["fr"]="Le Tourment des Sul-Xan perfectionné",["de"]="Perfektionierte Folter der Sul-Xan",["ru"]="Совершенная пытка сул-зан",["en"]="Perfected Sul-Xan's Torment"},[591]={["zh"]="完美巴塞的癫狂",["es"]="Manía de Bahsei perfeccionada",["fr"]="La Folie de Bahsei perfectionnée",["de"]="Bahseis perfektionierte Manie",["ru"]="Совершенное безумие Басей",["en"]="Perfected Bahsei's Mania"},[592]={["zh"]="完美石语者的誓言",["es"]="Juramento del portavoz de la piedra perfeccionado",["fr"]="Le Serment du Parlepierre perfectionné",["de"]="Perfektionierter Eid des Steinsprechers",["ru"]="Совершенная клятва говорящего с камнями",["en"]="Perfected Stone-Talker's Oath"},[593]={["zh"]="西帝斯的凝视",["es"]="Mirada de Sithis",["fr"]="Le Regard de Sithis",["de"]="Sithis' Blick",["ru"]="Взгляд Ситиса",["en"]="Gaze of Sithis"},[594]={["zh"]="捕鲸者的涉水裙",["es"]="Kilt de vadeo de arponero",["fr"]="Kilt de plongée de harponneur",["de"]="Watkilt des Harpunierers",["ru"]="Болотный килт гарпунера",["en"]="Harpooner's Wading Kilt"},[596]={["zh"]="死亡交易者的祭仪戒",["es"]="Regocijo del aniquilador",["fr"]="La Fête du porteur de mort",["de"]="Fete des Todbringers",["ru"]="Торжество несущего смерть",["en"]="Death Dealer's Fete"},[597]={["zh"]="换形者的项圈",["es"]="Cadena del Cambiaformas",["fr"]="Chaîne du changeforme",["de"]="Kette des Gestaltwandlers",["ru"]="Цепочка перевертыша",["en"]="Shapeshifter's Chain"},[598]={["zh"]="永恒警戒者佐奥",["es"]="Zoal el Insomne",["fr"]="Zoal l'Éveillé",["de"]="Zoal der Immerwache",["ru"]="Зоал Неусыпный",["en"]="Zoal the Ever-Wakeful"},[599]={["zh"]="献祭者查尔",["es"]="Charr el Inmolador",["fr"]="L’immolateur Charr",["de"]="Entflammer Charr",["ru"]="Поджигатель Чарр",["en"]="Immolator Charr"},[600]={["zh"]="毁灭者格洛戈洛克",["es"]="Glorgoloch el Destructor",["fr"]="Glorgoloch le destructeur",["de"]="Glorgoloch der Zerstörer",["ru"]="Глорголок Разрушитель",["en"]="Glorgoloch the Destroyer"},[602]={["zh"]="猩红誓言碎裂",["es"]="Juramento Carmesí",["fr"]="Déchirure du Serment écarlate",["de"]="Spalt des Karmesinschwurs",["ru"]="Прозрение Алой Клятвы",["en"]="Crimson Oath's Rive"},[603]={["zh"]="斯克里昂盛宴",["es"]="Festín de escorión",["fr"]="Festin de Scorion",["de"]="Festmahl des Skorions",["ru"]="Пир скориона",["en"]="Scorion's Feast"},[604]={["zh"]="急速苦痛",["es"]="Descarga de agonía",["fr"]="Élan d'agonie",["de"]="Auftreibende Agonie",["ru"]="Стремительная агония",["en"]="Rush of Agony"},[605]={["zh"]="银玫瑰未眠",["es"]="Vigilia de la Rosa Plateada",["fr"]="Veille de la Rose d'argent",["de"]="Silberrosen-Wacht",["ru"]="Дозор Серебряной розы",["en"]="Silver Rose Vigil"},[606]={["zh"]="雷电召唤者",["es"]="Llamada al trueno",["fr"]="Mande-tonnerre",["de"]="Donnerrufer",["ru"]="Громовержец",["en"]="Thunder Caller"},[607]={["zh"]="恐怖美食家",["es"]="Delicia siniestra",["fr"]="Gourmet sinistre",["de"]="Grausiger Gourmet",["ru"]="Жуткий гурман",["en"]="Grisly Gourmet"},[608]={["zh"]="骑士长谢里克",["es"]="Prior Thierric",["fr"]="Prieur Thierric",["de"]="Prior Thierric",["ru"]="Приор Тьеррик",["en"]="Prior Thierric"},[609]={["zh"]="岩浆化身",["es"]="Encarnado de magma",["fr"]="Le Magma Incarné",["de"]="Magmaverkörperung",["ru"]="Живое Пламя",["en"]="Magma Incarnate"},[610]={["zh"]="恶毒活力",["es"]="Vitalidad precaria",["fr"]="Vitalité perdue",["de"]="Elendsvitalität",["ru"]="Проклятая жизненная сила",["en"]="Wretched Vitality"},[611]={["zh"]="死地毁灭者",["es"]="Demoledor de las Tierras Muertas",["fr"]="Démolisseur des Terres mortes",["de"]="Totenländer-Demolierer",["ru"]="Разрушитель из Мертвых Земель",["en"]="Deadlands Demolisher"},[612]={["zh"]="钢铁坚壶",["es"]="Frasco de hierro",["fr"]="Flasque de fer",["de"]="Eisenfläschchen",["ru"]="Питье неуязвимости",["en"]="Iron Flask"},[613]={["zh"]="掌控之眼",["es"]="Ojo del Brazo",["fr"]="Œil de l'Étreinte",["de"]="Griffblick",["ru"]="Око Хватки",["en"]="Eye of the Grasp"},[614]={["zh"]="赫克索斯结界",["es"]="Custodia de Hexos",["fr"]="La Garde d'Hexos",["de"]="Hexos-Abschirmung",["ru"]="Оберег дома Гексос",["en"]="Hexos' Ward"},[615]={["zh"]="金马赤的残酷",["es"]="Crueldad de kynmarcher",["fr"]="Cruauté de la Kynmarcheuse",["de"]="Grausamkeit der Kynandantin",["ru"]="Жестокость кинмаршера",["en"]="Kynmarcher's Cruelty"},[616]={["zh"]="黑暗汇聚",["es"]="Convergencia oscura",["fr"]="Convergence noire",["de"]="Dunkle Konvergenz",["ru"]="Темное слияние",["en"]="Dark Convergence"},[617]={["zh"]="瘟疫突至",["es"]="Brote de plaga",["fr"]="Brise-peste",["de"]="Seuchenbruch",["ru"]="Вспышка чумы",["en"]="Plaguebreak"},[618]={["zh"]="霍斯加之苦寒",["es"]="Escalofrío de Hrothgar",["fr"]="Frisson de Hrothgar",["de"]="Hrothgars Kühle",["ru"]="Холод Хротгара",["en"]="Hrothgar's Chill"},[619]={["zh"]="马里伽利格的漩涡",["es"]="Maelstrom de Maligalig",["fr"]="Le Maelstrom de Marigalig",["de"]="Maligaligs Mahlstrom",["ru"]="Вихрь Малигалига",["en"]="Maligalig's Maelstrom"},[620]={["zh"]="狮鹫劫掠",["es"]="Represalia del grifo",["fr"]="Représailles de griffon",["de"]="Vergeltung des Greifen",["ru"]="Возмездие грифона",["en"]="Gryphon's Reprisal"},[621]={["zh"]="冰川守护者",["es"]="Guardián glacial",["fr"]="Gardien glacial",["de"]="Gletscherwächter",["ru"]="Ледяной страж",["en"]="Glacial Guardian"},[622]={["zh"]="逆转潮汐",["es"]="Marea cambiante",["fr"]="Marée renversée",["de"]="Gezeitenwechsel",["ru"]="Приливная волна",["en"]="Turning Tide"},[623]={["zh"]="风暴诅咒的复仇",["es"]="Venganza del maldito de la tormenta",["fr"]="Vengeance maudite par l'orage",["de"]="Rache des Sturmverfluchten",["ru"]="Месть проклятого бурей",["en"]="Storm-Cursed's Revenge"},[624]={["zh"]="树精的活力",["es"]="Vigor de spriggan",["fr"]="Vigueur du spriggan",["de"]="Elan des Zweiglings",["ru"]="Бодрость сприггана",["en"]="Spriggan's Vigor"},[625]={["zh"]="马尔肯恩权力之戒",["es"]="Anillo de majestad markyn",["fr"]="L’Anneau de majesté de Markyn",["de"]="Markyn-Ring der Erhabenheit",["ru"]="Кольцо величия Маркина",["en"]="Markyn Ring of Majesty"},[626]={["zh"]="贝哈尔扎的指环",["es"]="Banda de Belharza",["fr"]="La Bande de Belharza",["de"]="Belharzas Band",["ru"]="Перстень Белхарзы",["en"]="Belharza's Band"},[627]={["zh"]="毁灭肩甲",["es"]="Bufa de la ruina",["fr"]="La Spallière de ruine",["de"]="Schulterbänder des Untergangs",["ru"]="Наплечник разрушения",["en"]="Spaulder of Ruin"},[629]={["zh"]="振奋战吼",["es"]="Grito arengador",["fr"]="Le Cri de ralliement",["de"]="Schlachtruf",["ru"]="Боевой клич",["en"]="Rallying Cry"},[630]={["zh"]="劈砍割裂",["es"]="Sesgar y escindir",["fr"]="Tranche et Taille",["de"]="Kappspalter",["ru"]="Безжалостная рубка",["en"]="Hew and Sunder"},[631]={["zh"]="衰弱光环",["es"]="El aura enervante",["fr"]="Aura affaiblissante",["de"]="Enervierende Aura",["ru"]="Изнуряющая аура",["en"]="Enervating Aura"},[632]={["zh"]="卡尔盖达",["es"]="Kargaeda",["fr"]="Kargaeda",["de"]="Kargaeda",["ru"]="Каргеда",["en"]="Kargaeda"},[633]={["zh"]="娜扎莱",["es"]="Nazaray",["fr"]="Nazaray",["de"]="Nazaray",["ru"]="Назарей",["en"]="Nazaray"},[634]={["zh"]="努那塔克",["es"]="Nunatak",["fr"]="Nunatak",["de"]="Nunatak",["ru"]="Нунатак",["en"]="Nunatak"},[635]={["zh"]="玛丽达夫人",["es"]="Lady Malygda",["fr"]="La dame Malygda",["de"]="Fürstin Malygda",["ru"]="Леди Малигда",["en"]="Lady Malygda"},[636]={["zh"]="瑟斯克男爵",["es"]="Barón Thirsk",["fr"]="Le baron Thirsk",["de"]="Baron Thirsk",["ru"]="Барон Тирск",["en"]="Baron Thirsk"},[640]={["zh"]="秩序之怒",["es"]="Cólera de la Orden",["fr"]="Fureur de l'Ordre",["de"]="Zorn des Ordens",["ru"]="Гнев ордена",["en"]="Order's Wrath"},[641]={["zh"]="蝮蛇蔑视",["es"]="Desdén de la serpiente",["fr"]="Dédain du serpent",["de"]="Verachtung der Schlange",["ru"]="Презрение змея",["en"]="Serpent's Disdain"},[642]={["zh"]="德鲁伊辫子",["es"]="Trenza del druida",["fr"]="La Tresse druidique",["de"]="Druidenflechte",["ru"]="Плетение друидов",["en"]="Druid's Braid"},[643]={["zh"]="高岛的祝福",["es"]="Bendición de Isla Alta",["fr"]="Bénédiction de l'Île-Haute",["de"]="Segen von Hochinsel",["ru"]="Благословение Высокого острова",["en"]="Blessing of High Isle"},[644]={["zh"]="忠坚勇气",["es"]="Valía de los Inquebrantables",["fr"]="Endurance des Inébranlables",["de"]="Mut der Standhaften",["ru"]="Храбрость миротворца",["en"]="Steadfast's Mettle"},[645]={["zh"]="西斯忒斯之怒",["es"]="Ceño de las Systres",["fr"]="La Grimace des Systres",["de"]="Blick der Systren",["ru"]="Оскал Систреса",["en"]="Systres' Scowl"},[646]={["zh"]="深渊漩涡",["es"]="Espiral de las profundidades",["fr"]="Le Tourbillon des profondeurs",["de"]="Wirbel der Tiefen",["ru"]="Глубинный смерч",["en"]="Whorl of the Depths"},[647]={["zh"]="珊瑚激流",["es"]="Marea de coral",["fr"]="Lame de corail",["de"]="Korallenbrandung",["ru"]="Коралловое течение",["en"]="Coral Riptide"},[648]={["zh"]="珠光结界",["es"]="Custodia nacarada",["fr"]="Garde perlescente",["de"]="Perlenglanz-Schutz",["ru"]="Перламутровый оберег",["en"]="Pearlescent Ward"},[649]={["zh"]="劫掠者的收获",["es"]="Ganancias del saqueador",["fr"]="Le Profit du pillard",["de"]="Profit des Brandschatzers",["ru"]="Удача мародера",["en"]="Pillager's Profit"},[650]={["zh"]="完美劫掠者的收获",["es"]="Ganancias del saqueador perfeccionado",["fr"]="Le Profit du pillard perfectionné",["de"]="Perfektionierter Profit des Brandschatzers",["ru"]="Совершенная удача мародера",["en"]="Perfected Pillager's Profit"},[651]={["zh"]="完美珠光结界",["es"]="Custodia nacarada perfeccionado",["fr"]="La Garde perlescente perfectionnée",["de"]="Perfektionierter Perlenglanz-Schutz",["ru"]="Совершенный перламутровый оберег",["en"]="Perfected Pearlescent Ward"},[652]={["zh"]="完美珊瑚激流",["es"]="Marea de coral perfeccionado",["fr"]="La Lame de corail perfectionnée",["de"]="Perfektionierte Korallenbrandung",["ru"]="Совершенное коралловое течение",["en"]="Perfected Coral Riptide"},[653]={["zh"]="完美深渊漩涡",["es"]="Espiral de las profundidades perfeccionado",["fr"]="Tourbillon des profondeurs perfectionné",["de"]="Perfektionierter Wirbel der Tiefen",["ru"]="Совершенный глубинный смерч",["en"]="Perfected Whorl of the Depths"},[654]={["zh"]="莫拉低语",["es"]="Susurros de Mora",["fr"]="Murmures de Mora",["de"]="Moras Flüstern",["ru"]="Шепот Моры",["en"]="Mora's Whispers"},[655]={["zh"]="都瓦·哈足铠",["es"]="Escarpes de dov-rha",["fr"]="Solerets de Dov-rha",["de"]="Dov-rha-Panzerschuhe",["ru"]="Сабатоны дов-ра",["en"]="Dov-rha Sabatons"},[656]={["zh"]="左撇子的盾带",["es"]="Cinturón de égida de los Mano Izquierda",["fr"]="Ceinture de l'égide du gaucher",["de"]="Ägisriemen der Linkshänder",["ru"]="Защитный пояс леворуких эльфов",["en"]="Lefthander's Aegis Belt"},[657]={["zh"]="海蛇盘虬",["es"]="Anillo de serpiente marina",["fr"]="Anneau de serpent de mer",["de"]="Wicklung der Seeschlange",["ru"]="Колье морского змея",["en"]="Sea-Serpent's Coil"},[658]={["zh"]="橡木之环",["es"]="Anillo de alma de roble",["fr"]="Anneau d'âme du chêne",["de"]="Eichenseelenring",["ru"]="Кольцо души дуба",["en"]="Oakensoul Ring"},[660]={["zh"]="深根热忱",["es"]="Fervor de Raíz Profunda",["fr"]="Le Zèle de Longuesouche",["de"]="Tiefwurz-Eifer",["ru"]="Пыл Глубокого Корня",["en"]="Deeproot Zeal"},[661]={["zh"]="石头盟约",["es"]="Acuerdo de piedra",["fr"]="L’Accord de pierre",["de"]="Steinabkommen",["ru"]="Гармония камня",["en"]="Stone's Accord"},[662]={["zh"]="乌萨克之怒",["es"]="Rabia del ursauk",["fr"]="La Rage de l'Ursauk",["de"]="Zorns des Ursauk",["ru"]="Ярость урсака",["en"]="Rage of the Ursauk"},[663]={["zh"]="潘格瑞巢母",["es"]="Madre de la guarida pangrit",["fr"]="La Couveuse pangrit",["de"]="Pangrit-Baumutter",["ru"]="Мамаша пангритов",["en"]="Pangrit Denmother"},[664]={["zh"]="必死",["es"]="Inevitabilidad de la tumba",["fr"]="Inévitabilité funèbre",["de"]="Unvermeidlichkeit des Grabs",["ru"]="Неотвратимость смерти",["en"]="Grave Inevitability"},[665]={["zh"]="命匣鬼爪",["es"]="Presa de la filacteria",["fr"]="Emprise du phylactère",["de"]="Phylakteriumsgriff",["ru"]="Хватка филактерии",["en"]="Phylactery's Grasp"},[666]={["zh"]="大德鲁伊德维里克",["es"]="El archidruida Devyric",["fr"]="L’archidruide Devyric",["de"]="Erzdruide Devyric",["ru"]="Архидруид Девирик",["en"]="Archdruid Devyric"},[667]={["zh"]="透光守门者",["es"]="El Centinela Eufótico",["fr"]="Le Portier euphotique",["de"]="Euphotischer Torhüter",["ru"]="Прибрежный Страж",["en"]="Euphotic Gatekeeper"},[668]={["zh"]="佩伊特之倦",["es"]="Languidez de Peryite",["fr"]="La Langueur de Péryite",["de"]="Peryites Entkräftung",["ru"]="Слабость Периайта",["en"]="Languor of Peryite"},[669]={["zh"]="诺克图娜尔的阴谋",["es"]="Ardid de Nocturnal",["fr"]="Le Stratagème de Nocturne",["de"]="Nocturnals List",["ru"]="Коварство Ноктюрнал",["en"]="Nocturnal's Ploy"},[670]={["zh"]="玛拉香油",["es"]="Bálsamo de Mara",["fr"]="Baume de Mara",["de"]="Maras Trost",["ru"]="Утешение Мары",["en"]="Mara's Balm"},[671]={["zh"]="后巷美食家",["es"]="Gastrónomo del callejón",["fr"]="Le Gourmet des ruelles",["de"]="Hintergassen-Gourmet",["ru"]="Тайный гурман",["en"]="Back-Alley Gourmand"},[672]={["zh"]="凤凰蛾秘术",["es"]="Teúrgo de polilla fénix",["fr"]="Papillon phénix théurge",["de"]="Phönixmotten-Theurg",["ru"]="Магия мотылька-феникса",["en"]="Phoenix Moth Theurge"},[673]={["zh"]="德拉奥菲的棱堡",["es"]="Baluarte del Draoife",["fr"]="Le bastion du Draoife",["de"]="Bastion des Draoife",["ru"]="Оплот Драойфе",["en"]="Bastion of the Draoife"},[674]={["zh"]="鹿人欢乐装束",["es"]="Revestimiento de Jolgorio del Fauno",["fr"]="Barde de farce de faune",["de"]="Faunstreich-Bekleidung",["ru"]="Одеяние шалости фавна",["en"]="Faun's Lark Cladding"},[675]={["zh"]="风暴编织者的腾跃",["es"]="Retozón de tejedora de tormentas",["fr"]="Gambade de la Tisseuse de foudre",["de"]="Sturmwebertollen",["ru"]="Пляска призывателя бури",["en"]="Stormweaver's Cavort"},[676]={["zh"]="塞拉班的结界",["es"]="Custodia de Syrabane",["fr"]="La Garde de Syrabane",["de"]="Syrabanns Schutz",["ru"]="Оберег Сирабейна",["en"]="Syrabane's Ward"},[677]={["zh"]="奇美拉的叱责",["es"]="Reprensión de quimera",["fr"]="La Rebuffade de la chimère",["de"]="Zurechtweisung der Schimäre",["ru"]="Отпор химеры",["en"]="Chimera's Rebuke"},[678]={["zh"]="古老生长酿造师",["es"]="Artesano de bosque maduro",["fr"]="Le Brasseur de la Vieille croissance",["de"]="Altwuchs-Brauer",["ru"]="Зельевар из Старого леса",["en"]="Old Growth Brewer"},[679]={["zh"]="森林怨灵利爪",["es"]="Garra de espectro del bosque",["fr"]="La Griffe du spectre forestier",["de"]="Kralle des Waldgespensts",["ru"]="Коготь лесного призрака",["en"]="Claw of the Forest Wraith"},[680]={["zh"]="仪式长的羁绊",["es"]="Vínculo del maestro de ritos",["fr"]="Lien du maître des rites",["de"]="Bindung des Ritenmeisters",["ru"]="Узы мастера обрядов",["en"]="Ritemaster's Bond"},[681]={["zh"]="尼克斯犬虫嚎叫",["es"]="Aullido de sabueso nix",["fr"]="Hurlement du chien nix",["de"]="Nixhund-Heulen",["ru"]="Вой никс-гончей",["en"]="Nix-Hound's Howl"},[682]={["zh"]="泰尔瓦尼执法者",["es"]="Sicario Telvanni",["fr"]="Le Défenseur telvanni",["de"]="Telvanni-Vollstrecker",["ru"]="Защитник дома Телванни",["en"]="Telvanni Enforcer"},[683]={["zh"]="扭曲者洛科萨",["es"]="Roksa el Deformado",["fr"]="Roksa le Déformé",["de"]="Roksa die Verkrümmte",["ru"]="Рокса Искалеченный",["en"]="Roksa the Warped"},[684]={["zh"]="符文雕刻师烈焰",["es"]="Deflagración de tallador de runas",["fr"]="Brasier du tailleur de rune",["de"]="Lodern des Runenschnitzers",["ru"]="Пыл резчика рун",["en"]="Runecarver's Blaze"},[685]={["zh"]="异典启示",["es"]="Inspiración de Apocrypha",["fr"]="Inspiration apocryphale",["de"]="Apocryphische Inspiration",["ru"]="Вдохновение Апокрифа",["en"]="Apocryphal Inspiration"},[686]={["zh"]="深渊支撑",["es"]="Abrazo abisal",["fr"]="Renfort abyssal",["de"]="Abgründiges Wappnen",["ru"]="Глубинная защита",["en"]="Abyssal Brace"},[687]={["zh"]="炼狱奥泽赞",["es"]="Ozezan la Infernal",["fr"]="Ozezan l'Infernal",["de"]="Ozezan das Inferno",["ru"]="Озезан Бушующее Пламя",["en"]="Ozezan the Inferno"},[688]={["zh"]="星宿之蛇",["es"]="Serpiente en las Estrellas",["fr"]="Le Serpent dans les étoiles",["de"]="Schlange inmitten der Sterne",["ru"]="Звездный Змей",["en"]="Snake in the Stars"},[689]={["zh"]="裂甲刃",["es"]="Quebrador de caparazones",["fr"]="Brise-carapace",["de"]="Panzerspalten",["ru"]="Крушитель панцирей",["en"]="Shell Splitter"},[690]={["zh"]="阿卡托什的审判",["es"]="Juicio de Akatosh",["fr"]="Le Jugement d'Akatosh",["de"]="Urteil von Akatosh",["ru"]="Правосудие Акатоша",["en"]="Judgment of Akatosh"},[691]={["zh"]="地穴教士法衣",["es"]="Vestimentas de canónigo de la cripta",["fr"]="Vêtements du Chanoine de la crypte",["de"]="Gewandung des Kryptenvogts",["ru"]="Одеяние могильного каноника",["en"]="Cryptcanon Vestments"},[692]={["zh"]="神秘环境腿铠",["es"]="Grebas de ambiente esotérico",["fr"]="Grèves d'environnement ésotérique",["de"]="Beinschienen der esoterischen Umgebung",["ru"]="Эзотерические поножи",["en"]="Esoteric Environment Greaves"},[693]={["zh"]="末代野精灵王之颈环",["es"]="Torque del último rey ayleid",["fr"]="Le torque du Dernier roi ayléide",["de"]="Torques des letzten Ayleïdenkönigs",["ru"]="Ожерелье последнего короля айлейдов",["en"]="Torc of the Last Ayleid King"},[694]={["zh"]="北地精灵乌尔法师护符",["es"]="Amuleto de mago ur velothi",["fr"]="L’Amulette d'ur-mage Vélothi",["de"]="Amulett des velothischen Ur-Magiers",["ru"]="Амулет велотийского ур-мага",["en"]="Velothi Ur-Mage's Amulet"},[695]={["zh"]="破碎命运",["es"]="Destino fragmentado",["fr"]="Le Destin brisé",["de"]="Zerschlagenes Schicksal",["ru"]="Расколотая судьба",["en"]="Shattered Fate"},[696]={["zh"]="泰尔瓦尼效率",["es"]="Eficacia de los Telvanni",["fr"]="L’Efficacité Telvanni",["de"]="Telvanni-Effizienz",["ru"]="Рациональность Телванни",["en"]="Telvanni Efficiency"},[697]={["zh"]="探求者的结合",["es"]="Síntesis de buscador",["fr"]="La Synthèse du chercheur",["de"]="Suchersynthese",["ru"]="Синтез искателя",["en"]="Seeker Synthesis"},[698]={["zh"]="维威克的二元性",["es"]="Dualidad de Vivec",["fr"]="La Dualité de Vivec",["de"]="Vivecs Dualität",["ru"]="Двойственность Вивека",["en"]="Vivec's Duality"},[699]={["zh"]="卡蒙帮",["es"]="Camonna Tong",["fr"]="La Camonna Tong",["de"]="Camonna Tong",["ru"]="Камонна Тонг",["en"]="Camonna Tong"},[700]={["zh"]="坚石潜伏者",["es"]="Acechador inexorable",["fr"]="Le Rôdeur adamant",["de"]="Unerschütterlicher Lauerer",["ru"]="Адамантовый луркер",["en"]="Adamant Lurker"},[701]={["zh"]="祥和静谧",["es"]="Paz y serenidad",["fr"]="La Paix et Sérénité",["de"]="Frieden und Gelassenheit",["ru"]="Безграничный покой",["en"]="Peace and Serenity"},[702]={["zh"]="安苏尔的折磨",["es"]="Tormento de Ansuul",["fr"]="Le Tourment d'Ansuul",["de"]="Ansuuls Qual",["ru"]="Истязание Ансул",["en"]="Ansuul's Torment"},[703]={["zh"]="信念考验",["es"]="Prueba de determinación",["fr"]="L’Épreuve de détermination",["de"]="Prüfung der Entschlossenheit",["ru"]="Испытание решимости",["en"]="Test of Resolve"},[704]={["zh"]="变化的希望",["es"]="Esperanza transformativa",["fr"]="L’Espoir transformatif",["de"]="Transformative Hoffnung",["ru"]="Преображающая надежда",["en"]="Transformative Hope"},[705]={["zh"]="完美变化的希望",["es"]="Conjunto de esperanza transformativa perfeccionado",["fr"]="L’Espoir transformatif perfectionné",["de"]="Perfektionierte transformative Hoffnung",["ru"]="Совершенная преображающая надежда",["en"]="Perfected Transformative Hope"},[706]={["zh"]="完美信念考验",["es"]="Conjunto de la prueba de determinación perfeccionado",["fr"]="L’Épreuve de détermination perfectionnée",["de"]="Perfektionierte Prüfung der Entschlossenheit",["ru"]="Совершенное испытание решимости",["en"]="Perfected Test of Resolve"},[707]={["zh"]="完美安苏尔的折磨",["es"]="Tormento de Ansuul perfeccionado",["fr"]="Le Tourment d'Ansuul perfectionné",["de"]="Ansuuls perfektionierte Qual",["ru"]="Совершенное истязание Ансул",["en"]="Perfected Ansuul's Torment"},[708]={["zh"]="完美祥和静谧",["es"]="Conjunto de paz y serenidad perfeccionado",["fr"]="La Paix et Sérénité perfectionnées perfectionnée",["de"]="Perfektionierter Frieden und Gelassenheit",["ru"]="Совершенный безграничный покой",["en"]="Perfected Peace and Serenity"},[711]={["zh"]="克罗维亚高地将军",["es"]="General de las tierras altas colovianas",["fr"]="Le général des hautes terres coloviennes",["de"]="Colovianischer Hochlandgeneral",["ru"]="Генерал Коловианского нагорья",["en"]="Colovian Highlands General"},[712]={["zh"]="杰拉尔山脉酋长",["es"]="Jefe de guerra de las montañas Jerall",["fr"]="Le chef de guerre des monts Jerall",["de"]="Kriegshäuptling der Jerallberge",["ru"]="Вождь Джерольских гор",["en"]="Jerall Mountains Warchief"},[713]={["zh"]="尼贝内湾作战参谋长",["es"]="Alto oficial de la bahía de Nibenay",["fr"]="Le haut commissaire de la baie Nibenaise",["de"]="Nibenaybucht-Schlachtenvogt",["ru"]="Рив-командир Нибенейской бухты",["en"]="Nibenay Bay Battlereeve"},[722]={["zh"]="复苏圣师",["es"]="Hierofante Resucitado",["fr"]="Le Hiérophante réveillé",["de"]="Wiedererwachter Hierophant",["ru"]="Пробудившийся жрец",["en"]="Reawakened Hierophant"},[723]={["zh"]="玄武血勇士",["es"]="Guerrero de sangre de basalto",["fr"]="Le Guerrier aux veines de basalte",["de"]="Basaltblut-Krieger",["ru"]="Воин базальтовой крови",["en"]="Basalt-Blooded Warrior"},[724]={["zh"]="腐朽贵族",["es"]="Nobleza en decadencia",["fr"]="Corruption noble",["de"]="Adel im Verfall",["ru"]="Упадок аристократии",["en"]="Nobility in Decay"},[726]={["zh"]="割魂者",["es"]="Siega de alma",["fr"]="Tranche-âme",["de"]="Seelenspalter",["ru"]="Рассекатель душ",["en"]="Soulcleaver"},[727]={["zh"]="风暴巨石",["es"]="Monolito de las tormentas",["fr"]="Le Monolithe des tempêtes",["de"]="Monolith der Stürme",["ru"]="Грозовой монолит",["en"]="Monolith of Storms"},[728]={["zh"]="怒日",["es"]="Sol iracundo",["fr"]="Colère du soleil",["de"]="Wutsonne",["ru"]="Гнев солнца",["en"]="Wrathsun"},[729]={["zh"]="四季园丁",["es"]="Jardinero de las estaciones",["fr"]="Jardinier des saisons",["de"]="Gärtner der Jahreszeiten",["ru"]="Садовник времен года",["en"]="Gardener of Seasons"},[730]={["zh"]="安塞尔莫灰烬",["es"]="Cenizas de Anthelmir",["fr"]="Les Cendres d'Anthelmir",["de"]="Zunder von Anthelmir",["ru"]="Пепел Антелмир",["en"]="Cinders of Anthelmir"},[731]={["zh"]="斯拉史洛格的渴望",["es"]="Hambre de Sluthrug",["fr"]="La Faim de Sluthrug",["de"]="Sluthrugs Hunger",["ru"]="Голод Слатруга",["en"]="Sluthrug's Hunger"},[732]={["zh"]="黑手套训练",["es"]="Arraigo de Guante Negro",["fr"]="L’Ancrage du Gant noir",["de"]="Schwarzhandschuh-Erdung",["ru"]="Стойкость Черной Перчатки",["en"]="Black-Glove Grounding"},[734]={["zh"]="安塞尔莫的构造体",["es"]="Autómata de Anthelmir",["fr"]="L’Assemblage d'Anthelmir",["de"]="Anthelmirs Konstrukt",["ru"]="Творение Антелмир",["en"]="Anthelmir's Construct"},[735]={["zh"]="盲径入会",["es"]="Iniciación de la Senda Invidente",["fr"]="L’Induction du Sentier aveugle",["de"]="Einberufung des blinden Pfades",["ru"]="Посвящение Слепого Пути",["en"]="Blind Path Induction"},[736]={["zh"]="污秽梦魇",["es"]="Pesadilla mancillada",["fr"]="Le Cauchemar terni",["de"]="Befleckter Albtraum",["ru"]="Тусклый кошмар",["en"]="Tarnished Nightmare"},[737]={["zh"]="映射怒火",["es"]="Furia reflejada",["fr"]="La Fureur réfléchie",["de"]="Spiegelzorn",["ru"]="Отраженная ярость",["en"]="Reflected Fury"},[738]={["zh"]="盲眼邪神",["es"]="La Invidente",["fr"]="L’Aveugle",["de"]="Die Blinde",["ru"]="Слепая",["en"]="The Blind"},[754]={["zh"]="橡父惩戒",["es"]="Castigo del Padre Roble",["fr"]="La Revanche du Père-Chêne",["de"]="Eichvater-Vergeltung",["ru"]="Возмездие отца-дуба",["en"]="Oakfather's Retribution"},[755]={["zh"]="钝刃",["es"]="Hojas embotadas",["fr"]="Les Lames émoussées",["de"]="Stumpfklingen",["ru"]="Затупившиеся клинки",["en"]="Blunted Blades"},[756]={["zh"]="班达祝福",["es"]="Bendición de Baan Dar",["fr"]="La Bénédiction de Baan Dar",["de"]="Baan Dars Segen",["ru"]="Благословение Баан-Дара",["en"]="Baan Dar's Blessing"}},
    --setNames

------------------------------------------------------------------------------------------------------------------------
--The equipTypes of the sets
--Coming from the debug function DebugScanAllSetData -> SV table "setsEquipTypes" -> Minified
[LIBSETS_TABLEKEY_SETS_EQUIP_TYPES] ={[1]={[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[122]=1,[123]=1,[124]=1,[127]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[162]=1,[163]=1,[164]=1,[165]=1,[166]=1,[167]=1,[168]=1,[169]=1,[170]=1,[171]=1,[172]=1,[173]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[183]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[190]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[256]=1,[257]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[264]=1,[265]=1,[266]=1,[267]=1,[268]=1,[269]=1,[270]=1,[271]=1,[272]=1,[273]=1,[274]=1,[275]=1,[276]=1,[277]=1,[278]=1,[279]=1,[280]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[341]=1,[342]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[349]=1,[350]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[397]=1,[398]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[429]=1,[430]=1,[431]=1,[432]=1,[433]=1,[434]=1,[435]=1,[436]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[458]=1,[459]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[478]=1,[479]=1,[480]=1,[481]=1,[482]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[506]=1,[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[534]=1,[535]=1,[536]=1,[537]=1,[538]=1,[539]=1,[540]=1,[541]=1,[542]=1,[543]=1,[544]=1,[569]=1,[570]=1,[571]=1,[572]=1,[573]=1,[574]=1,[577]=1,[578]=1,[579]=1,[580]=1,[581]=1,[582]=1,[583]=1,[584]=1,[585]=1,[586]=1,[587]=1,[588]=1,[589]=1,[590]=1,[591]=1,[592]=1,[593]=1,[598]=1,[599]=1,[600]=1,[602]=1,[603]=1,[604]=1,[605]=1,[606]=1,[607]=1,[608]=1,[609]=1,[610]=1,[611]=1,[612]=1,[613]=1,[614]=1,[615]=1,[616]=1,[617]=1,[618]=1,[619]=1,[620]=1,[621]=1,[622]=1,[623]=1,[624]=1,[629]=1,[630]=1,[631]=1,[632]=1,[633]=1,[634]=1,[635]=1,[636]=1,[640]=1,[641]=1,[642]=1,[643]=1,[644]=1,[645]=1,[646]=1,[647]=1,[648]=1,[649]=1,[650]=1,[651]=1,[652]=1,[653]=1,[660]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[666]=1,[667]=1,[668]=1,[669]=1,[670]=1,[671]=1,[672]=1,[673]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[683]=1,[684]=1,[685]=1,[686]=1,[687]=1,[688]=1,[689]=1,[690]=1,[695]=1,[696]=1,[697]=1,[698]=1,[699]=1,[700]=1,[701]=1,[702]=1,[703]=1,[704]=1,[705]=1,[706]=1,[707]=1,[708]=1,[711]=1,[712]=1,[713]=1,[722]=1,[723]=1,[724]=1,[726]=1,[727]=1,[728]=1,[729]=1,[730]=1,[731]=1,[732]=1,[734]=1,[735]=1,[736]=1,[737]=1,[738]=1,[754]=1,[755]=1,[756]=1},[2]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[119]=1,[120]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[694]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[209]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[756]=1,[755]=1,[754]=1,[737]=1,[253]=1,[736]=1,[731]=1,[730]=1,[729]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[728]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[693]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[690]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[669]=1,[668]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[660]=1,[657]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[642]=1,[641]=1,[640]=1,[631]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[614]=1,[613]=1,[612]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[611]=1,[610]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[603]=1,[602]=1,[597]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[576]=1,[574]=1,[480]=1,[481]=1,[482]=1,[573]=1,[572]=1,[571]=1,[570]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[569]=1,[544]=1,[543]=1,[542]=1,[541]=1,[505]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[3]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[521]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[127]=1,[640]=1,[641]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[674]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[756]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[755]=1,[754]=1,[737]=1,[736]=1,[731]=1,[730]=1,[729]=1,[728]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[702]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[691]=1,[690]=1,[689]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[688]=1,[685]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[684]=1,[673]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[380]=1,[618]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[617]=1,[616]=1,[615]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[614]=1,[613]=1,[612]=1,[611]=1,[610]=1,[607]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[606]=1,[605]=1,[604]=1,[603]=1,[602]=1,[592]=1,[429]=1,[430]=1,[431]=1,[591]=1,[433]=1,[434]=1,[435]=1,[590]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[589]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[588]=1,[587]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[581]=1,[580]=1,[579]=1,[480]=1,[481]=1,[482]=1,[574]=1,[573]=1,[572]=1,[571]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[570]=1,[569]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[4]={[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[122]=1,[123]=1,[124]=1,[127]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[162]=1,[163]=1,[164]=1,[165]=1,[166]=1,[167]=1,[168]=1,[169]=1,[170]=1,[171]=1,[172]=1,[173]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[183]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[190]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[256]=1,[257]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[265]=1,[266]=1,[267]=1,[268]=1,[269]=1,[270]=1,[271]=1,[272]=1,[273]=1,[274]=1,[275]=1,[276]=1,[277]=1,[278]=1,[279]=1,[280]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[341]=1,[342]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[349]=1,[350]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[397]=1,[398]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[429]=1,[430]=1,[431]=1,[432]=1,[433]=1,[434]=1,[435]=1,[436]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[458]=1,[459]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[478]=1,[479]=1,[480]=1,[481]=1,[482]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[506]=1,[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[534]=1,[535]=1,[536]=1,[537]=1,[538]=1,[539]=1,[540]=1,[541]=1,[542]=1,[543]=1,[544]=1,[569]=1,[570]=1,[571]=1,[572]=1,[573]=1,[574]=1,[577]=1,[578]=1,[579]=1,[580]=1,[581]=1,[582]=1,[583]=1,[584]=1,[585]=1,[586]=1,[587]=1,[588]=1,[589]=1,[590]=1,[591]=1,[592]=1,[598]=1,[599]=1,[600]=1,[602]=1,[603]=1,[604]=1,[605]=1,[606]=1,[607]=1,[608]=1,[609]=1,[610]=1,[611]=1,[612]=1,[613]=1,[614]=1,[615]=1,[616]=1,[617]=1,[618]=1,[619]=1,[620]=1,[621]=1,[622]=1,[623]=1,[624]=1,[627]=1,[629]=1,[630]=1,[631]=1,[632]=1,[633]=1,[634]=1,[635]=1,[636]=1,[640]=1,[641]=1,[642]=1,[643]=1,[644]=1,[645]=1,[646]=1,[647]=1,[648]=1,[649]=1,[650]=1,[651]=1,[652]=1,[653]=1,[654]=1,[660]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[666]=1,[667]=1,[668]=1,[669]=1,[670]=1,[671]=1,[672]=1,[673]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[683]=1,[684]=1,[685]=1,[686]=1,[687]=1,[688]=1,[689]=1,[690]=1,[695]=1,[696]=1,[697]=1,[698]=1,[699]=1,[700]=1,[701]=1,[702]=1,[703]=1,[704]=1,[705]=1,[706]=1,[707]=1,[708]=1,[711]=1,[712]=1,[713]=1,[722]=1,[723]=1,[724]=1,[726]=1,[727]=1,[728]=1,[729]=1,[730]=1,[731]=1,[732]=1,[734]=1,[735]=1,[736]=1,[737]=1,[738]=1,[754]=1,[755]=1,[756]=1},[5]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[524]=1,[529]=1,[530]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[557]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[119]=1,[120]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[541]=1,[215]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[728]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[217]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[218]=1,[542]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[736]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[727]=1,[216]=1,[729]=1,[730]=1,[731]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[726]=1,[724]=1,[723]=1,[722]=1,[253]=1,[708]=1,[707]=1,[706]=1,[705]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[660]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[653]=1,[652]=1,[314]=1,[315]=1,[651]=1,[650]=1,[649]=1,[648]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[647]=1,[646]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[645]=1,[644]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[643]=1,[358]=1,[359]=1,[641]=1,[640]=1,[631]=1,[624]=1,[364]=1,[365]=1,[623]=1,[622]=1,[621]=1,[620]=1,[370]=1,[371]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[380]=1,[611]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[610]=1,[607]=1,[606]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[605]=1,[413]=1,[604]=1,[603]=1,[602]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[592]=1,[425]=1,[591]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[564]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[563]=1,[558]=1,[544]=1,[225]=1,[543]=1,[219]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[6]={[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[117]=1,[118]=1,[121]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[171]=1,[172]=1,[173]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[190]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[313]=1,[316]=1,[317]=1,[318]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[357]=1,[360]=1,[361]=1,[362]=1,[363]=1,[366]=1,[367]=1,[368]=1,[369]=1,[372]=1,[373]=1,[374]=1,[380]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[412]=1,[414]=1,[415]=1,[416]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[424]=1,[426]=1,[427]=1,[428]=1,[429]=1,[430]=1,[431]=1,[433]=1,[434]=1,[435]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[480]=1,[481]=1,[482]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[506]=1,[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[522]=1,[525]=1,[526]=1,[527]=1,[528]=1,[531]=1,[532]=1,[533]=1,[536]=1,[537]=1,[538]=1,[539]=1,[540]=1,[541]=1,[542]=1,[543]=1,[544]=1,[559]=1,[560]=1,[561]=1,[562]=1,[565]=1,[566]=1,[567]=1,[568]=1,[569]=1,[570]=1,[571]=1,[572]=1,[573]=1,[574]=1,[579]=1,[580]=1,[581]=1,[582]=1,[583]=1,[584]=1,[585]=1,[586]=1,[587]=1,[588]=1,[589]=1,[590]=1,[591]=1,[592]=1,[602]=1,[603]=1,[604]=1,[605]=1,[606]=1,[607]=1,[610]=1,[611]=1,[612]=1,[613]=1,[614]=1,[615]=1,[616]=1,[617]=1,[618]=1,[619]=1,[620]=1,[621]=1,[622]=1,[623]=1,[624]=1,[629]=1,[630]=1,[631]=1,[640]=1,[641]=1,[642]=1,[643]=1,[644]=1,[645]=1,[646]=1,[647]=1,[648]=1,[649]=1,[650]=1,[651]=1,[652]=1,[653]=1,[660]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[668]=1,[669]=1,[670]=1,[671]=1,[672]=1,[673]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[684]=1,[685]=1,[686]=1,[688]=1,[689]=1,[690]=1,[695]=1,[696]=1,[697]=1,[698]=1,[699]=1,[700]=1,[701]=1,[702]=1,[703]=1,[704]=1,[705]=1,[706]=1,[707]=1,[708]=1,[722]=1,[723]=1,[724]=1,[726]=1,[727]=1,[728]=1,[729]=1,[730]=1,[731]=1,[732]=1,[735]=1,[736]=1,[737]=1,[754]=1,[755]=1,[756]=1},[7]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[529]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[119]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[737]=1,[736]=1,[731]=1,[730]=1,[253]=1,[729]=1,[728]=1,[727]=1,[726]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[684]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[673]=1,[672]=1,[314]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[653]=1,[652]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[651]=1,[650]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[649]=1,[358]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[364]=1,[643]=1,[641]=1,[640]=1,[631]=1,[624]=1,[370]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[380]=1,[614]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[613]=1,[612]=1,[611]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[610]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[603]=1,[602]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[564]=1,[558]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[8]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[127]=1,[640]=1,[641]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[676]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[756]=1,[755]=1,[754]=1,[737]=1,[736]=1,[731]=1,[730]=1,[729]=1,[728]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[705]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[688]=1,[685]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[684]=1,[673]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[656]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[617]=1,[616]=1,[615]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[614]=1,[613]=1,[612]=1,[611]=1,[610]=1,[607]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[606]=1,[605]=1,[604]=1,[603]=1,[602]=1,[592]=1,[429]=1,[430]=1,[431]=1,[591]=1,[433]=1,[434]=1,[435]=1,[590]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[589]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[588]=1,[587]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[581]=1,[580]=1,[579]=1,[480]=1,[481]=1,[482]=1,[574]=1,[573]=1,[572]=1,[571]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[570]=1,[569]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[9]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[127]=1,[640]=1,[641]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[675]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[756]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[755]=1,[754]=1,[737]=1,[736]=1,[731]=1,[730]=1,[729]=1,[728]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[702]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[692]=1,[690]=1,[689]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[688]=1,[685]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[684]=1,[673]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[616]=1,[615]=1,[614]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[613]=1,[612]=1,[611]=1,[610]=1,[607]=1,[606]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[605]=1,[604]=1,[603]=1,[602]=1,[594]=1,[592]=1,[429]=1,[430]=1,[431]=1,[591]=1,[433]=1,[434]=1,[435]=1,[590]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[589]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[588]=1,[587]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[581]=1,[580]=1,[579]=1,[480]=1,[481]=1,[482]=1,[574]=1,[573]=1,[572]=1,[571]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[570]=1,[569]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[10]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[519]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[127]=1,[640]=1,[641]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[756]=1,[755]=1,[754]=1,[737]=1,[736]=1,[731]=1,[730]=1,[729]=1,[728]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[705]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[688]=1,[685]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[684]=1,[673]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[655]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[617]=1,[616]=1,[615]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[614]=1,[613]=1,[612]=1,[611]=1,[610]=1,[607]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[606]=1,[605]=1,[604]=1,[603]=1,[602]=1,[592]=1,[429]=1,[430]=1,[431]=1,[591]=1,[433]=1,[434]=1,[435]=1,[590]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[589]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[588]=1,[587]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[581]=1,[580]=1,[579]=1,[480]=1,[481]=1,[482]=1,[574]=1,[573]=1,[572]=1,[571]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[570]=1,[569]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[12]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[520]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[117]=1,[118]=1,[631]=1,[121]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[225]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[209]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[736]=1,[731]=1,[730]=1,[729]=1,[253]=1,[728]=1,[727]=1,[726]=1,[724]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[673]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[658]=1,[653]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[652]=1,[651]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[650]=1,[649]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[642]=1,[641]=1,[640]=1,[630]=1,[629]=1,[626]=1,[625]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[380]=1,[381]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[614]=1,[613]=1,[612]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[611]=1,[610]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[603]=1,[602]=1,[596]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[575]=1,[574]=1,[480]=1,[481]=1,[482]=1,[573]=1,[572]=1,[571]=1,[570]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[569]=1,[544]=1,[543]=1,[503]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[13]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[127]=1,[640]=1,[641]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[643]=1,[644]=1,[645]=1,[646]=1,[756]=1,[755]=1,[754]=1,[671]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[615]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[614]=1,[613]=1,[612]=1,[611]=1,[610]=1,[737]=1,[736]=1,[607]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[606]=1,[605]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[604]=1,[603]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[602]=1,[729]=1,[728]=1,[727]=1,[726]=1,[702]=1,[724]=1,[723]=1,[722]=1,[707]=1,[592]=1,[591]=1,[590]=1,[589]=1,[588]=1,[587]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[706]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[705]=1,[574]=1,[573]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[572]=1,[571]=1,[570]=1,[569]=1,[696]=1,[708]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[700]=1,[699]=1,[690]=1,[689]=1,[688]=1,[698]=1,[429]=1,[430]=1,[431]=1,[685]=1,[433]=1,[434]=1,[435]=1,[684]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[730]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[731]=1,[673]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[539]=1,[538]=1,[537]=1,[480]=1,[481]=1,[482]=1,[536]=1,[660]=1,[697]=1,[672]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[670]=1,[501]=1,[669]=1,[668]=1,[653]=1,[652]=1,[506]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1}},
    --LIBSETS_TABLEKEY_SETS_EQUIP_TYPES

------------------------------------------------------------------------------------------------------------------------
--The armorTypes of the sets
--Coming from the debug function DebugScanAllSetData -> SV table "setsArmorTypes" -> Minified
[LIBSETS_TABLEKEY_SETS_ARMOR_TYPES] = {[1]={[513]=1,[516]=1,[19]=1,[534]=1,[535]=1,[25]=1,[26]=1,[539]=1,[540]=1,[541]=1,[542]=1,[31]=1,[32]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[51]=1,[54]=1,[55]=1,[56]=1,[569]=1,[572]=1,[64]=1,[65]=1,[578]=1,[579]=1,[582]=1,[583]=1,[584]=1,[73]=1,[74]=1,[587]=1,[76]=1,[78]=1,[591]=1,[592]=1,[81]=1,[82]=1,[84]=1,[85]=1,[598]=1,[87]=1,[600]=1,[603]=1,[92]=1,[606]=1,[95]=1,[608]=1,[609]=1,[610]=1,[611]=1,[612]=1,[613]=1,[103]=1,[104]=1,[619]=1,[109]=1,[110]=1,[623]=1,[627]=1,[629]=1,[632]=1,[633]=1,[634]=1,[635]=1,[124]=1,[640]=1,[641]=1,[642]=1,[643]=1,[133]=1,[134]=1,[649]=1,[650]=1,[139]=1,[140]=1,[141]=1,[654]=1,[147]=1,[148]=1,[661]=1,[664]=1,[666]=1,[155]=1,[668]=1,[157]=1,[671]=1,[160]=1,[161]=1,[162]=1,[163]=1,[164]=1,[165]=1,[166]=1,[167]=1,[168]=1,[169]=1,[170]=1,[683]=1,[172]=1,[687]=1,[176]=1,[177]=1,[178]=1,[691]=1,[181]=1,[183]=1,[696]=1,[185]=1,[698]=1,[701]=1,[190]=1,[704]=1,[193]=1,[194]=1,[708]=1,[711]=1,[200]=1,[713]=1,[207]=1,[208]=1,[722]=1,[723]=1,[724]=1,[726]=1,[215]=1,[728]=1,[217]=1,[218]=1,[219]=1,[734]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[235]=1,[236]=1,[240]=1,[241]=1,[242]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[256]=1,[257]=1,[258]=1,[261]=1,[264]=1,[265]=1,[266]=1,[267]=1,[268]=1,[269]=1,[270]=1,[271]=1,[272]=1,[273]=1,[274]=1,[275]=1,[276]=1,[277]=1,[278]=1,[279]=1,[280]=1,[281]=1,[285]=1,[289]=1,[290]=1,[292]=1,[294]=1,[297]=1,[298]=1,[300]=1,[303]=1,[305]=1,[320]=1,[323]=1,[324]=1,[325]=1,[328]=1,[329]=1,[332]=1,[333]=1,[334]=1,[335]=1,[338]=1,[341]=1,[342]=1,[343]=1,[346]=1,[349]=1,[350]=1,[351]=1,[352]=1,[353]=1,[354]=1,[380]=1,[384]=1,[385]=1,[386]=1,[387]=1,[390]=1,[391]=1,[394]=1,[395]=1,[397]=1,[398]=1,[399]=1,[402]=1,[405]=1,[408]=1,[409]=1,[410]=1,[417]=1,[418]=1,[431]=1,[432]=1,[435]=1,[436]=1,[437]=1,[438]=1,[439]=1,[440]=1,[443]=1,[444]=1,[448]=1,[449]=1,[452]=1,[98]=1,[455]=1,[97]=1,[616]=1,[458]=1,[459]=1,[88]=1,[86]=1,[712]=1,[80]=1,[79]=1,[75]=1,[466]=1,[66]=1,[468]=1,[469]=1,[470]=1,[471]=1,[684]=1,[588]=1,[474]=1,[599]=1,[577]=1,[677]=1,[478]=1,[479]=1,[480]=1,[481]=1,[482]=1,[675]=1,[57]=1,[537]=1,[646]=1,[487]=1,[22]=1,[636]=1,[490]=1,[491]=1,[492]=1,[493]=1,[653]=1,[667]=1,[496]=1,[497]=1,[678]=1,[679]=1,[680]=1,[501]=1,[688]=1,[695]=1,[697]=1,[705]=1,[506]=1,[727]=1,[729]=1,[730]=1,[738]=1,[754]=1},[2]={[514]=1,[517]=1,[519]=1,[20]=1,[534]=1,[23]=1,[538]=1,[539]=1,[540]=1,[541]=1,[543]=1,[33]=1,[34]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[48]=1,[49]=1,[50]=1,[51]=1,[54]=1,[58]=1,[59]=1,[60]=1,[573]=1,[577]=1,[578]=1,[67]=1,[580]=1,[69]=1,[582]=1,[583]=1,[584]=1,[73]=1,[74]=1,[75]=1,[77]=1,[590]=1,[79]=1,[80]=1,[81]=1,[594]=1,[84]=1,[598]=1,[599]=1,[600]=1,[89]=1,[90]=1,[91]=1,[604]=1,[607]=1,[608]=1,[609]=1,[610]=1,[611]=1,[612]=1,[614]=1,[617]=1,[106]=1,[620]=1,[111]=1,[624]=1,[630]=1,[632]=1,[633]=1,[634]=1,[123]=1,[636]=1,[127]=1,[640]=1,[641]=1,[642]=1,[132]=1,[647]=1,[652]=1,[142]=1,[143]=1,[144]=1,[146]=1,[148]=1,[662]=1,[665]=1,[666]=1,[155]=1,[156]=1,[669]=1,[159]=1,[672]=1,[161]=1,[162]=1,[163]=1,[164]=1,[165]=1,[166]=1,[167]=1,[168]=1,[169]=1,[170]=1,[683]=1,[173]=1,[687]=1,[176]=1,[177]=1,[178]=1,[180]=1,[183]=1,[696]=1,[697]=1,[187]=1,[188]=1,[702]=1,[195]=1,[198]=1,[199]=1,[712]=1,[713]=1,[207]=1,[208]=1,[722]=1,[723]=1,[212]=1,[214]=1,[727]=1,[216]=1,[729]=1,[219]=1,[734]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[232]=1,[234]=1,[237]=1,[240]=1,[241]=1,[242]=1,[755]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[256]=1,[257]=1,[259]=1,[262]=1,[264]=1,[265]=1,[266]=1,[267]=1,[268]=1,[269]=1,[270]=1,[271]=1,[272]=1,[273]=1,[274]=1,[275]=1,[276]=1,[277]=1,[278]=1,[279]=1,[280]=1,[281]=1,[283]=1,[286]=1,[295]=1,[296]=1,[299]=1,[301]=1,[302]=1,[307]=1,[308]=1,[310]=1,[321]=1,[323]=1,[324]=1,[325]=1,[327]=1,[331]=1,[334]=1,[336]=1,[339]=1,[341]=1,[342]=1,[344]=1,[347]=1,[349]=1,[350]=1,[351]=1,[352]=1,[353]=1,[355]=1,[380]=1,[383]=1,[385]=1,[386]=1,[387]=1,[389]=1,[393]=1,[397]=1,[398]=1,[400]=1,[403]=1,[406]=1,[408]=1,[409]=1,[410]=1,[419]=1,[420]=1,[430]=1,[432]=1,[434]=1,[436]=1,[437]=1,[438]=1,[439]=1,[441]=1,[445]=1,[450]=1,[453]=1,[456]=1,[535]=1,[458]=1,[459]=1,[112]=1,[105]=1,[635]=1,[100]=1,[724]=1,[99]=1,[95]=1,[467]=1,[468]=1,[469]=1,[470]=1,[92]=1,[472]=1,[87]=1,[82]=1,[475]=1,[78]=1,[681]=1,[478]=1,[479]=1,[480]=1,[481]=1,[482]=1,[679]=1,[68]=1,[644]=1,[586]=1,[570]=1,[488]=1,[656]=1,[490]=1,[491]=1,[667]=1,[674]=1,[28]=1,[27]=1,[677]=1,[678]=1,[498]=1,[499]=1,[685]=1,[689]=1,[695]=1,[699]=1,[707]=1,[711]=1,[506]=1,[726]=1,[728]=1,[731]=1,[736]=1,[738]=1},[3]={[515]=1,[518]=1,[521]=1,[21]=1,[534]=1,[535]=1,[536]=1,[539]=1,[540]=1,[541]=1,[30]=1,[544]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[48]=1,[51]=1,[52]=1,[53]=1,[54]=1,[571]=1,[61]=1,[62]=1,[63]=1,[577]=1,[578]=1,[581]=1,[582]=1,[71]=1,[584]=1,[585]=1,[74]=1,[75]=1,[589]=1,[78]=1,[79]=1,[80]=1,[593]=1,[82]=1,[83]=1,[84]=1,[598]=1,[599]=1,[600]=1,[602]=1,[92]=1,[605]=1,[94]=1,[95]=1,[608]=1,[609]=1,[610]=1,[611]=1,[612]=1,[101]=1,[102]=1,[615]=1,[618]=1,[107]=1,[108]=1,[621]=1,[622]=1,[113]=1,[114]=1,[631]=1,[632]=1,[633]=1,[122]=1,[635]=1,[636]=1,[640]=1,[641]=1,[642]=1,[131]=1,[644]=1,[645]=1,[135]=1,[136]=1,[137]=1,[138]=1,[651]=1,[655]=1,[145]=1,[148]=1,[663]=1,[666]=1,[155]=1,[158]=1,[161]=1,[162]=1,[163]=1,[164]=1,[165]=1,[166]=1,[167]=1,[168]=1,[169]=1,[170]=1,[171]=1,[686]=1,[687]=1,[176]=1,[177]=1,[178]=1,[179]=1,[692]=1,[183]=1,[184]=1,[697]=1,[186]=1,[700]=1,[703]=1,[706]=1,[196]=1,[197]=1,[711]=1,[712]=1,[201]=1,[207]=1,[208]=1,[210]=1,[211]=1,[724]=1,[213]=1,[726]=1,[727]=1,[728]=1,[729]=1,[219]=1,[732]=1,[734]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[231]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[756]=1,[245]=1,[246]=1,[247]=1,[248]=1,[253]=1,[256]=1,[257]=1,[260]=1,[263]=1,[264]=1,[265]=1,[266]=1,[267]=1,[268]=1,[269]=1,[270]=1,[271]=1,[272]=1,[273]=1,[274]=1,[275]=1,[276]=1,[277]=1,[278]=1,[279]=1,[280]=1,[281]=1,[282]=1,[284]=1,[287]=1,[288]=1,[291]=1,[293]=1,[304]=1,[309]=1,[311]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[330]=1,[334]=1,[337]=1,[340]=1,[341]=1,[342]=1,[345]=1,[348]=1,[349]=1,[350]=1,[351]=1,[352]=1,[353]=1,[356]=1,[380]=1,[382]=1,[385]=1,[386]=1,[387]=1,[388]=1,[392]=1,[397]=1,[398]=1,[401]=1,[404]=1,[407]=1,[408]=1,[409]=1,[410]=1,[421]=1,[422]=1,[429]=1,[432]=1,[433]=1,[436]=1,[437]=1,[438]=1,[439]=1,[442]=1,[446]=1,[451]=1,[454]=1,[457]=1,[458]=1,[459]=1,[634]=1,[96]=1,[465]=1,[93]=1,[87]=1,[468]=1,[469]=1,[470]=1,[81]=1,[696]=1,[473]=1,[73]=1,[72]=1,[476]=1,[70]=1,[478]=1,[479]=1,[480]=1,[481]=1,[482]=1,[583]=1,[648]=1,[574]=1,[643]=1,[660]=1,[667]=1,[489]=1,[490]=1,[491]=1,[670]=1,[29]=1,[494]=1,[495]=1,[24]=1,[673]=1,[676]=1,[677]=1,[678]=1,[679]=1,[682]=1,[683]=1,[690]=1,[695]=1,[506]=1,[713]=1,[722]=1,[723]=1,[737]=1,[738]=1}}, --LIBSETS_TABLEKEY_SETS_ARMOR_TYPES

------------------------------------------------------------------------------------------------------------------------
--The weaponTypes of the sets
--Coming from the debug function DebugScanAllSetData -> SV table "setsWeaponTypes" -> Minified
[LIBSETS_TABLEKEY_SETS_WEAPONS_TYPES] = {[1]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[524]=1,[529]=1,[530]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[557]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[728]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[729]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[218]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[219]=1,[736]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[541]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[730]=1,[731]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[727]=1,[726]=1,[724]=1,[723]=1,[253]=1,[722]=1,[708]=1,[707]=1,[706]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[668]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[660]=1,[653]=1,[314]=1,[315]=1,[652]=1,[651]=1,[650]=1,[649]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[648]=1,[647]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[646]=1,[645]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[644]=1,[358]=1,[359]=1,[643]=1,[641]=1,[640]=1,[624]=1,[364]=1,[365]=1,[623]=1,[622]=1,[621]=1,[620]=1,[370]=1,[371]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[380]=1,[611]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[610]=1,[607]=1,[606]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[605]=1,[413]=1,[604]=1,[603]=1,[602]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[592]=1,[425]=1,[591]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[564]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[563]=1,[558]=1,[544]=1,[543]=1,[225]=1,[542]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[2]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[524]=1,[529]=1,[530]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[557]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[119]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[728]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[729]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[218]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[219]=1,[736]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[541]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[730]=1,[731]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[727]=1,[726]=1,[724]=1,[723]=1,[253]=1,[722]=1,[708]=1,[707]=1,[706]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[668]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[660]=1,[653]=1,[314]=1,[315]=1,[652]=1,[651]=1,[650]=1,[649]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[648]=1,[647]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[646]=1,[645]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[644]=1,[358]=1,[359]=1,[643]=1,[641]=1,[640]=1,[631]=1,[364]=1,[365]=1,[624]=1,[623]=1,[622]=1,[621]=1,[370]=1,[371]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[611]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[610]=1,[607]=1,[606]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[605]=1,[413]=1,[604]=1,[603]=1,[602]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[592]=1,[425]=1,[591]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[564]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[563]=1,[558]=1,[544]=1,[543]=1,[225]=1,[542]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[3]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[524]=1,[529]=1,[530]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[557]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[120]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[217]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[218]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[219]=1,[541]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[736]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[729]=1,[730]=1,[731]=1,[732]=1,[542]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[754]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[728]=1,[727]=1,[726]=1,[724]=1,[253]=1,[723]=1,[722]=1,[708]=1,[707]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[669]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[668]=1,[660]=1,[314]=1,[315]=1,[653]=1,[652]=1,[651]=1,[650]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[649]=1,[648]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[647]=1,[646]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[645]=1,[358]=1,[359]=1,[644]=1,[643]=1,[641]=1,[640]=1,[364]=1,[365]=1,[624]=1,[623]=1,[622]=1,[621]=1,[370]=1,[371]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[611]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[610]=1,[607]=1,[606]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[605]=1,[413]=1,[604]=1,[603]=1,[602]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[592]=1,[425]=1,[591]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[564]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[563]=1,[558]=1,[544]=1,[543]=1,[242]=1,[225]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[4]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[522]=1,[528]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[755]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[754]=1,[737]=1,[736]=1,[731]=1,[253]=1,[730]=1,[729]=1,[728]=1,[727]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[685]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[684]=1,[313]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[660]=1,[653]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[652]=1,[651]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[357]=1,[650]=1,[649]=1,[648]=1,[647]=1,[646]=1,[363]=1,[645]=1,[644]=1,[643]=1,[641]=1,[640]=1,[369]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[380]=1,[614]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[613]=1,[612]=1,[611]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[610]=1,[412]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[603]=1,[424]=1,[602]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[565]=1,[559]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[5]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[522]=1,[528]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[755]=1,[754]=1,[737]=1,[736]=1,[253]=1,[731]=1,[730]=1,[729]=1,[728]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[727]=1,[726]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[688]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[685]=1,[313]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[668]=1,[660]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[653]=1,[652]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[357]=1,[651]=1,[650]=1,[649]=1,[648]=1,[647]=1,[363]=1,[646]=1,[645]=1,[644]=1,[643]=1,[641]=1,[369]=1,[640]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[613]=1,[612]=1,[611]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[610]=1,[412]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[603]=1,[424]=1,[602]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[565]=1,[559]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[6]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[522]=1,[528]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[118]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[737]=1,[736]=1,[731]=1,[730]=1,[253]=1,[729]=1,[728]=1,[727]=1,[726]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[684]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[673]=1,[313]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[653]=1,[652]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[651]=1,[650]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[357]=1,[649]=1,[648]=1,[647]=1,[646]=1,[645]=1,[363]=1,[644]=1,[643]=1,[641]=1,[640]=1,[630]=1,[369]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[380]=1,[614]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[613]=1,[612]=1,[611]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[610]=1,[412]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[603]=1,[424]=1,[602]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[565]=1,[559]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[8]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[525]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[121]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[225]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[736]=1,[731]=1,[730]=1,[729]=1,[253]=1,[728]=1,[727]=1,[726]=1,[724]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[673]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[672]=1,[671]=1,[670]=1,[669]=1,[316]=1,[668]=1,[660]=1,[653]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[652]=1,[651]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[650]=1,[649]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[648]=1,[647]=1,[646]=1,[360]=1,[645]=1,[644]=1,[643]=1,[642]=1,[641]=1,[366]=1,[640]=1,[624]=1,[623]=1,[622]=1,[621]=1,[372]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[380]=1,[613]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[612]=1,[611]=1,[610]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[607]=1,[606]=1,[605]=1,[414]=1,[604]=1,[603]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[602]=1,[592]=1,[591]=1,[426]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[566]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[560]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[506]=1,[539]=1,[538]=1,[537]=1,[536]=1,[531]=1},[9]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[527]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[117]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[224]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[737]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[736]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[731]=1,[730]=1,[729]=1,[728]=1,[253]=1,[727]=1,[726]=1,[724]=1,[723]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[672]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[653]=1,[318]=1,[652]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[651]=1,[650]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[649]=1,[648]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[647]=1,[646]=1,[645]=1,[644]=1,[643]=1,[362]=1,[642]=1,[641]=1,[640]=1,[629]=1,[624]=1,[368]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[374]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[380]=1,[613]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[612]=1,[611]=1,[610]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[607]=1,[606]=1,[605]=1,[604]=1,[603]=1,[416]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[602]=1,[592]=1,[591]=1,[590]=1,[589]=1,[428]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[568]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[562]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[506]=1,[539]=1,[538]=1,[537]=1,[536]=1,[533]=1},[11]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[524]=1,[529]=1,[530]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[557]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[728]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[729]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[218]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[219]=1,[736]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[541]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[730]=1,[731]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[727]=1,[726]=1,[724]=1,[723]=1,[253]=1,[722]=1,[708]=1,[707]=1,[706]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[668]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[660]=1,[653]=1,[314]=1,[315]=1,[652]=1,[651]=1,[650]=1,[649]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[648]=1,[647]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[646]=1,[645]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[644]=1,[358]=1,[359]=1,[643]=1,[641]=1,[640]=1,[624]=1,[364]=1,[365]=1,[623]=1,[622]=1,[621]=1,[620]=1,[370]=1,[371]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[380]=1,[611]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[610]=1,[607]=1,[606]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[605]=1,[413]=1,[604]=1,[603]=1,[602]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[592]=1,[425]=1,[591]=1,[590]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[564]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[563]=1,[558]=1,[544]=1,[543]=1,[225]=1,[542]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[12]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[526]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[225]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[736]=1,[731]=1,[730]=1,[729]=1,[253]=1,[728]=1,[727]=1,[726]=1,[724]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[673]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[317]=1,[660]=1,[653]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[652]=1,[651]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[650]=1,[649]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[648]=1,[647]=1,[646]=1,[645]=1,[361]=1,[644]=1,[643]=1,[642]=1,[641]=1,[640]=1,[367]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[373]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[380]=1,[613]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[612]=1,[611]=1,[610]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[607]=1,[606]=1,[605]=1,[604]=1,[415]=1,[603]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[602]=1,[592]=1,[591]=1,[590]=1,[427]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[567]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[561]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[506]=1,[539]=1,[538]=1,[537]=1,[536]=1,[532]=1},[13]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[526]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[737]=1,[736]=1,[731]=1,[730]=1,[253]=1,[729]=1,[728]=1,[727]=1,[726]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[684]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[317]=1,[668]=1,[660]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[653]=1,[652]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[651]=1,[650]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[649]=1,[648]=1,[647]=1,[646]=1,[361]=1,[645]=1,[644]=1,[643]=1,[642]=1,[641]=1,[367]=1,[640]=1,[624]=1,[623]=1,[622]=1,[621]=1,[373]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[612]=1,[611]=1,[610]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[607]=1,[606]=1,[605]=1,[604]=1,[415]=1,[603]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[602]=1,[592]=1,[591]=1,[590]=1,[427]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[567]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[561]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[506]=1,[539]=1,[538]=1,[537]=1,[536]=1,[532]=1},[14]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[523]=1,[529]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[629]=1,[630]=1,[119]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[642]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[225]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[737]=1,[736]=1,[731]=1,[730]=1,[253]=1,[729]=1,[728]=1,[727]=1,[726]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[724]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[684]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[673]=1,[672]=1,[314]=1,[671]=1,[670]=1,[669]=1,[668]=1,[660]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[653]=1,[652]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[651]=1,[650]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[649]=1,[358]=1,[648]=1,[647]=1,[646]=1,[645]=1,[644]=1,[364]=1,[643]=1,[641]=1,[640]=1,[631]=1,[624]=1,[370]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[380]=1,[614]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[613]=1,[612]=1,[611]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[411]=1,[610]=1,[607]=1,[606]=1,[605]=1,[604]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[423]=1,[603]=1,[602]=1,[592]=1,[591]=1,[590]=1,[429]=1,[430]=1,[431]=1,[589]=1,[433]=1,[434]=1,[435]=1,[588]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[587]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[586]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[579]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[564]=1,[558]=1,[544]=1,[543]=1,[542]=1,[541]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},[15]={[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[526]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[629]=1,[630]=1,[631]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[225]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[217]=1,[218]=1,[219]=1,[732]=1,[754]=1,[755]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[242]=1,[243]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[736]=1,[731]=1,[730]=1,[729]=1,[253]=1,[728]=1,[727]=1,[726]=1,[724]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[723]=1,[722]=1,[708]=1,[707]=1,[706]=1,[705]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[673]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[672]=1,[671]=1,[670]=1,[669]=1,[668]=1,[317]=1,[660]=1,[653]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[652]=1,[651]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[650]=1,[649]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[648]=1,[647]=1,[646]=1,[645]=1,[361]=1,[644]=1,[643]=1,[642]=1,[641]=1,[640]=1,[367]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[373]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[380]=1,[613]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[612]=1,[611]=1,[610]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[607]=1,[606]=1,[605]=1,[604]=1,[415]=1,[603]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[602]=1,[592]=1,[591]=1,[590]=1,[427]=1,[589]=1,[429]=1,[430]=1,[431]=1,[588]=1,[433]=1,[434]=1,[435]=1,[587]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[586]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[585]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[574]=1,[573]=1,[572]=1,[480]=1,[481]=1,[482]=1,[571]=1,[570]=1,[569]=1,[567]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[561]=1,[544]=1,[543]=1,[542]=1,[541]=1,[540]=1,[506]=1,[539]=1,[538]=1,[537]=1,[536]=1,[532]=1}},
    --LIBSETS_TABLEKEY_SETS_WEAPON_TYPES

------------------------------------------------------------------------------------------------------------------------
--The sets containing jewelry
--Coming from the debug function DebugScanAllSetData -> SV table "setsWithJewelry" -> Minified
[LIBSETS_TABLEKEY_SETS_JEWELRY] = {[513]=1,[514]=1,[515]=1,[516]=1,[517]=1,[518]=1,[520]=1,[19]=1,[20]=1,[21]=1,[22]=1,[23]=1,[24]=1,[25]=1,[26]=1,[27]=1,[28]=1,[29]=1,[30]=1,[31]=1,[32]=1,[33]=1,[34]=1,[35]=1,[36]=1,[37]=1,[38]=1,[39]=1,[40]=1,[41]=1,[43]=1,[44]=1,[46]=1,[47]=1,[48]=1,[49]=1,[50]=1,[51]=1,[52]=1,[53]=1,[54]=1,[55]=1,[56]=1,[57]=1,[58]=1,[59]=1,[60]=1,[61]=1,[62]=1,[63]=1,[64]=1,[65]=1,[66]=1,[67]=1,[68]=1,[69]=1,[70]=1,[71]=1,[72]=1,[73]=1,[74]=1,[75]=1,[76]=1,[77]=1,[78]=1,[79]=1,[80]=1,[81]=1,[82]=1,[83]=1,[84]=1,[85]=1,[86]=1,[87]=1,[88]=1,[89]=1,[90]=1,[91]=1,[92]=1,[93]=1,[94]=1,[95]=1,[96]=1,[97]=1,[98]=1,[99]=1,[100]=1,[101]=1,[102]=1,[103]=1,[104]=1,[105]=1,[106]=1,[107]=1,[108]=1,[109]=1,[110]=1,[111]=1,[112]=1,[113]=1,[114]=1,[116]=1,[117]=1,[118]=1,[119]=1,[120]=1,[121]=1,[122]=1,[123]=1,[124]=1,[125]=1,[126]=1,[127]=1,[128]=1,[129]=1,[130]=1,[131]=1,[132]=1,[133]=1,[134]=1,[135]=1,[136]=1,[137]=1,[138]=1,[139]=1,[140]=1,[141]=1,[142]=1,[143]=1,[144]=1,[145]=1,[146]=1,[147]=1,[148]=1,[661]=1,[662]=1,[663]=1,[664]=1,[665]=1,[541]=1,[155]=1,[156]=1,[157]=1,[158]=1,[159]=1,[160]=1,[161]=1,[727]=1,[728]=1,[217]=1,[677]=1,[678]=1,[679]=1,[680]=1,[681]=1,[682]=1,[171]=1,[172]=1,[173]=1,[686]=1,[218]=1,[176]=1,[177]=1,[178]=1,[179]=1,[180]=1,[181]=1,[694]=1,[695]=1,[184]=1,[185]=1,[186]=1,[187]=1,[188]=1,[701]=1,[190]=1,[703]=1,[704]=1,[193]=1,[194]=1,[195]=1,[196]=1,[197]=1,[198]=1,[199]=1,[200]=1,[201]=1,[542]=1,[219]=1,[204]=1,[205]=1,[206]=1,[207]=1,[208]=1,[209]=1,[210]=1,[211]=1,[212]=1,[213]=1,[214]=1,[215]=1,[216]=1,[729]=1,[730]=1,[731]=1,[732]=1,[736]=1,[242]=1,[735]=1,[224]=1,[737]=1,[226]=1,[227]=1,[228]=1,[229]=1,[230]=1,[231]=1,[232]=1,[756]=1,[234]=1,[235]=1,[236]=1,[237]=1,[238]=1,[239]=1,[240]=1,[241]=1,[754]=1,[755]=1,[244]=1,[245]=1,[246]=1,[247]=1,[248]=1,[726]=1,[724]=1,[723]=1,[722]=1,[253]=1,[708]=1,[707]=1,[706]=1,[705]=1,[258]=1,[259]=1,[260]=1,[261]=1,[262]=1,[263]=1,[702]=1,[700]=1,[699]=1,[698]=1,[697]=1,[696]=1,[693]=1,[690]=1,[689]=1,[688]=1,[685]=1,[684]=1,[673]=1,[672]=1,[671]=1,[670]=1,[669]=1,[281]=1,[282]=1,[283]=1,[284]=1,[285]=1,[286]=1,[287]=1,[288]=1,[289]=1,[290]=1,[291]=1,[292]=1,[293]=1,[294]=1,[295]=1,[296]=1,[297]=1,[298]=1,[299]=1,[300]=1,[301]=1,[302]=1,[303]=1,[304]=1,[305]=1,[668]=1,[307]=1,[308]=1,[309]=1,[310]=1,[311]=1,[660]=1,[658]=1,[657]=1,[653]=1,[652]=1,[651]=1,[650]=1,[649]=1,[320]=1,[321]=1,[322]=1,[323]=1,[324]=1,[325]=1,[326]=1,[327]=1,[328]=1,[329]=1,[330]=1,[331]=1,[332]=1,[333]=1,[334]=1,[335]=1,[336]=1,[337]=1,[338]=1,[339]=1,[340]=1,[648]=1,[647]=1,[343]=1,[344]=1,[345]=1,[346]=1,[347]=1,[348]=1,[646]=1,[645]=1,[351]=1,[352]=1,[353]=1,[354]=1,[355]=1,[356]=1,[644]=1,[643]=1,[642]=1,[641]=1,[640]=1,[631]=1,[630]=1,[629]=1,[626]=1,[625]=1,[624]=1,[623]=1,[622]=1,[621]=1,[620]=1,[619]=1,[618]=1,[617]=1,[616]=1,[615]=1,[614]=1,[613]=1,[612]=1,[380]=1,[381]=1,[382]=1,[383]=1,[384]=1,[385]=1,[386]=1,[387]=1,[388]=1,[389]=1,[390]=1,[391]=1,[392]=1,[393]=1,[394]=1,[395]=1,[611]=1,[610]=1,[607]=1,[399]=1,[400]=1,[401]=1,[402]=1,[403]=1,[404]=1,[405]=1,[406]=1,[407]=1,[408]=1,[409]=1,[410]=1,[606]=1,[605]=1,[604]=1,[603]=1,[602]=1,[597]=1,[417]=1,[418]=1,[419]=1,[420]=1,[421]=1,[422]=1,[596]=1,[592]=1,[591]=1,[590]=1,[589]=1,[588]=1,[429]=1,[430]=1,[431]=1,[587]=1,[433]=1,[434]=1,[435]=1,[586]=1,[437]=1,[438]=1,[439]=1,[440]=1,[441]=1,[442]=1,[443]=1,[444]=1,[445]=1,[446]=1,[585]=1,[448]=1,[449]=1,[450]=1,[451]=1,[452]=1,[453]=1,[454]=1,[455]=1,[456]=1,[457]=1,[584]=1,[583]=1,[582]=1,[581]=1,[580]=1,[579]=1,[576]=1,[465]=1,[466]=1,[467]=1,[468]=1,[469]=1,[470]=1,[471]=1,[472]=1,[473]=1,[474]=1,[475]=1,[476]=1,[575]=1,[574]=1,[573]=1,[480]=1,[481]=1,[482]=1,[572]=1,[571]=1,[570]=1,[569]=1,[487]=1,[488]=1,[489]=1,[490]=1,[491]=1,[492]=1,[493]=1,[494]=1,[495]=1,[496]=1,[497]=1,[498]=1,[499]=1,[544]=1,[543]=1,[243]=1,[503]=1,[225]=1,[505]=1,[506]=1,[540]=1,[539]=1,[538]=1,[537]=1,[536]=1},
    --LIBSETS_TABLEKEY_SETS_JEWELRY

------------------------------------------------------------------------------------------------------------------------
--The wayshrines's node index to their related zoneId
--Coming from Excel file LibSets_SetData.xlsx, map "ESO dumped wayshrine info", colum J (generated lua code)
[LIBSETS_TABLEKEY_WAYSHRINENODEID2ZONEID] = {[1]=3,[2]=3,[3]=3,[4]=3,[5]=3,[6]=3,[7]=3,[8]=3,[9]=20,[10]=20,[11]=20,[12]=20,[13]=20,[14]=19,[15]=19,[16]=19,[17]=19,[18]=19,[19]=19,[20]=3,[21]=383,[22]=19,[23]=19,[24]=57,[25]=57,[26]=57,[27]=57,[28]=57,[29]=57,[30]=57,[31]=19,[32]=57,[33]=92,[34]=92,[35]=92,[36]=92,[37]=92,[38]=92,[39]=92,[40]=92,[41]=41,[42]=104,[43]=104,[44]=104,[45]=104,[46]=104,[47]=117,[48]=117,[49]=117,[50]=117,[51]=117,[52]=117,[53]=117,[55]=20,[56]=19,[57]=104,[58]=104,[59]=104,[60]=104,[61]=104,[62]=3,[63]=92,[64]=3,[65]=41,[66]=41,[67]=41,[68]=41,[69]=41,[71]=41,[72]=41,[73]=41,[74]=41,[75]=41,[76]=41,[77]=41,[78]=117,[79]=57,[80]=57,[81]=57,[82]=20,[83]=20,[84]=20,[85]=117,[86]=20,[87]=101,[88]=101,[89]=101,[90]=101,[91]=101,[92]=101,[93]=101,[94]=101,[95]=101,[96]=101,[97]=101,[98]=41,[99]=58,[100]=58,[101]=58,[102]=58,[103]=58,[104]=58,[105]=58,[106]=58,[107]=58,[108]=41,[109]=103,[110]=103,[111]=103,[112]=103,[113]=103,[114]=103,[115]=103,[116]=103,[117]=103,[118]=103,[119]=103,[120]=103,[121]=381,[122]=381,[123]=381,[124]=381,[125]=281,[126]=281,[127]=381,[128]=347,[129]=347,[130]=347,[131]=347,[132]=347,[133]=347,[134]=347,[135]=347,[136]=347,[137]=104,[138]=534,[139]=347,[140]=347,[141]=537,[142]=537,[143]=108,[144]=382,[145]=347,[146]=347,[147]=108,[148]=108,[149]=108,[150]=108,[151]=108,[152]=108,[153]=108,[154]=108,[155]=104,[156]=382,[157]=382,[158]=382,[159]=382,[160]=382,[161]=382,[162]=382,[163]=382,[164]=383,[165]=383,[166]=383,[167]=383,[168]=383,[169]=383,[170]=181,[171]=117,[172]=280,[173]=281,[174]=381,[175]=381,[176]=381,[177]=381,[178]=381,[179]=534,[180]=534,[181]=535,[182]=535,[183]=535,[184]=347,[185]=382,[186]=92,[187]=103,[188]=58,[189]=19,[190]=20,[191]=383,[192]=117,[193]=3,[194]=381,[195]=101,[196]=104,[197]=108,[198]=57,[199]=181,[200]=181,[201]=181,[202]=181,[203]=181,[204]=92,[205]=57,[206]=92,[207]=383,[208]=20,[210]=3,[211]=381,[212]=41,[213]=383,[214]=383,[216]=3,[217]=888,[218]=888,[219]=888,[220]=888,[225]=888,[226]=888,[227]=888,[229]=888,[230]=888,[231]=888,[232]=888,[233]=888,[234]=888,[235]=888,[236]=181,[237]=684,[238]=684,[239]=684,[240]=684,[241]=684,[242]=684,[243]=684,[244]=684,[245]=684,[246]=684,[247]=181,[249]=684,[250]=684,[251]=823,[252]=823,[253]=823,[254]=823,[255]=816,[256]=816,[257]=816,[258]=382,[260]=117,[261]=117,[262]=381,[263]=19,[264]=57,[265]=383,[266]=41,[267]=3,[268]=108,[269]=20,[270]=888,[272]=849,[273]=849,[274]=849,[275]=849,[276]=849,[277]=849,[278]=849,[279]=849,[280]=849,[281]=849,[282]=849,[284]=849,[285]=381,[286]=104,[287]=57,[288]=381,[289]=3,[290]=41,[291]=382,[292]=3,[293]=41,[294]=58,[295]=281,[296]=383,[297]=3,[298]=41,[299]=58,[300]=537,[301]=103,[302]=19,[303]=92,[304]=108,[305]=117,[306]=108,[307]=20,[309]=57,[310]=888,[311]=382,[312]=101,[313]=92,[314]=104,[315]=381,[316]=117,[317]=383,[318]=19,[319]=57,[320]=382,[321]=382,[322]=103,[323]=92,[324]=534,[325]=383,[326]=888,[327]=888,[328]=849,[329]=849,[330]=849,[331]=849,[332]=888,[333]=849,[334]=849,[335]=849,[336]=849,[337]=980,[338]=980,[339]=980,[340]=980,[341]=92,[342]=3,[343]=823,[344]=347,[345]=888,[346]=980,[347]=980,[348]=684,[349]=1011,[350]=1011,[351]=1011,[352]=1011,[353]=1011,[354]=1011,[355]=1011,[356]=1011,[357]=1011,[358]=1011,[359]=1011,[360]=1027,[361]=816,[362]=823,[363]=19,[364]=1011,[365]=1011,[366]=1011,[367]=1011,[368]=1011,[369]=1011,[370]=108,[371]=382,[372]=103,[373]=1027,[374]=726,[375]=726,[376]=726,[377]=726,[378]=726,[379]=726,[380]=101,[381]=1086,[382]=1086,[383]=1086,[384]=1086,[386]=1086,[387]=1086,[388]=726,[389]=101,[390]=823,[391]=1086,[392]=101,[395]=888,[396]=1086,[397]=1086,[398]=383,[399]=1086,[400]=1086,[401]=1086,[402]=1133,[403]=1133,[404]=1133,[405]=1133,[406]=1133,[407]=1133,[409]=20,[410]=1086,[411]=0,[412]=0,[413]=0,[414]=0,[415]=1160,[416]=1160,[417]=1160,[418]=1160,[419]=1160,[420]=1160,[421]=1160,[422]=1133,[423]=1133,[424]=684,[425]=92,[426]=1160,[427]=92,[428]=684,[429]=1160,[430]=0,[431]=0,[432]=0,[433]=0,[434]=1160,[435]=0,[436]=1160,[437]=823,[438]=1160,[439]=1160,[440]=0,[441]=1207,[442]=1207,[443]=1207,[444]=1207,[445]=1207,[446]=0,[447]=1207,[448]=0,[449]=1207,[450]=1160,[451]=1160,[452]=1160,[453]=1160,[454]=57,[455]=1160,[456]=1207,[457]=1207,[458]=1261,[459]=1261,[460]=1261,[461]=1261,[462]=1261,[463]=1261,[464]=1261,[465]=849,[466]=823,[467]=1261,[468]=1261,[469]=1261,[470]=3,[471]=1261,[472]=1261,[473]=1261,[476]=1286,[477]=1286,[478]=1286,[479]=1286,[480]=1286,[481]=1261,[482]=1261,[483]=1261,[484]=1261,[485]=58,[486]=1261,[487]=1283,[488]=1318,[489]=1283,[490]=1286,[491]=1286,[493]=1283,[494]=1286,[495]=1283,[496]=1286,[497]=1011,[498]=20,[499]=535,[501]=1318,[502]=1318,[503]=1318,[504]=1318,[505]=1318,[506]=1318,[507]=1318,[508]=1318,[509]=1318,[510]=1318,[511]=1318,[512]=1318,[513]=1318,[517]=1318,[518]=1318,[519]=1318,[520]=1318,[521]=1318,[522]=1318,[523]=1318,[524]=1383,[525]=1383,[526]=1383,[527]=1383,[528]=1383,[529]=1383,[530]=1383,[531]=41,[532]=103,[533]=1318,[534]=1414,[535]=1414,[536]=1414,[537]=1414,[538]=1414,[539]=1414,[540]=1413,[541]=1413,[542]=1413,[543]=1413,[544]=1413,[545]=1413,[546]=1413,[547]=1413,[548]=1413,[549]=1414,[550]=1413,[551]=1383,[552]=1414,[553]=1414,[554]=1414,[555]=1414,[556]=1207,[557]=980,[565]=684,[566]=1383,[567]=1413},
    --LIBSETS_TABLEKEY_WAYSHRINENODEID2ZONEID



------------------------------------------------------------------------------------------------------------------------
    --The preloaded zoneIds of dungeons, and their parent zoneIds and isTrial booleans
    [LIBSETS_TABLEKEY_DUNGEON_ZONE_MAPPING] = {
        [638]={parentZoneId=888,isTrial=true},   --Aetherian Archive
        [148]={parentZoneId=117,isTrial=false},   --Arx Corinium
        [1000]={parentZoneId=980,isTrial=true},   --Asylum Sanctorium
        [1389]={parentZoneId=41,isTrial=true},   --Bal Sunnar
        [1471]={parentZoneId=684,isTrial=false},   --Bedlam Veil
        [1228]={parentZoneId=823,isTrial=false},   --Black Drake Villa
        [38]={parentZoneId=92,isTrial=false},   --Blackheart Haven
        [64]={parentZoneId=103,isTrial=false},   --Blessed Crucible
        [973]={parentZoneId=888,isTrial=false},   --Bloodroot Forge
        [1201]={parentZoneId=1160,isTrial=false},   --Castle Thorn
        [176]={parentZoneId=108,isTrial=false},   --City of Ash I
        [681]={parentZoneId=108,isTrial=false},   --City of Ash II
        [1051]={parentZoneId=1011,isTrial=true},   --Cloudrest
        [1301]={parentZoneId=1011,isTrial=false},   --Coral Aerie
        [848]={parentZoneId=117,isTrial=false},   --Cradle of Shadows
        [130]={parentZoneId=20,isTrial=false},   --Crypt of Hearts I
        [932]={parentZoneId=20,isTrial=false},   --Crypt of Hearts II
        [63]={parentZoneId=57,isTrial=false},   --Darkshade Caverns I
        [930]={parentZoneId=57,isTrial=false},   --Darkshade Caverns II
        [1081]={parentZoneId=823,isTrial=false},   --Depths of Malatar
        [449]={parentZoneId=101,isTrial=false},   --Direfrost Keep
        [1268]={parentZoneId=1261,isTrial=false},   --The Dread Cellar
        [1344]={parentZoneId=1318,isTrial=true},   --Dreadsail Reef
        [1360]={parentZoneId=1318,isTrial=false},   --Earthen Root Enclave
        [126]={parentZoneId=383,isTrial=false},   --Elden Hollow I
        [931]={parentZoneId=383,isTrial=false},   --Elden Hollow II
        [1436]={parentZoneId=1413,isTrial=false},   --Infinite Archive
        [974]={parentZoneId=888,isTrial=false},   --Falkreath Hold
        [1009]={parentZoneId=92,isTrial=false},   --Fang Lair
        [1080]={parentZoneId=101,isTrial=false},   --Frostvault
        [283]={parentZoneId=41,isTrial=false},   --Fungal Grotto I
        [934]={parentZoneId=41,isTrial=false},   --Fungal Grotto II
        [1361]={parentZoneId=1318,isTrial=false},   --Graven Deep
        [975]={parentZoneId=849,isTrial=true},   --Halls of Fabrication
        [636]={parentZoneId=888,isTrial=true},   --Hel Ra Citadel
        [1152]={parentZoneId=684,isTrial=false},   --Icereach
        [678]={parentZoneId=584,isTrial=false},   --Imperial City Prison
        [1196]={parentZoneId=1160,isTrial=false},   --Kyne's Aegis
        [1123]={parentZoneId=383,isTrial=false},   --Lair of Maarselok
        [1055]={parentZoneId=108,isTrial=false},   --March of Sacrifices
        [725]={parentZoneId=816,isTrial=true},   --Maw of Lorkhaj
        [1052]={parentZoneId=382,isTrial=false},   --Moon Hunter Keep
        [1122]={parentZoneId=1086,isTrial=false},   --Moongrave Fane
        [1470]={parentZoneId=1207,isTrial=false},   --Oathsworn Pit
        [1267]={parentZoneId=3,isTrial=false},   --Red Petal Bastion
        [843]={parentZoneId=117,isTrial=false},   --Ruins of Mazzatun
        [639]={parentZoneId=888,isTrial=true},   --Sanctum Ophidia
        [1427]={parentZoneId=1414,isTrial=true},   --Sanity's Edge
        [1010]={parentZoneId=19,isTrial=false},   --Scalecaller Peak
        [1390]={parentZoneId=103,isTrial=false},   --Scrivener's Hall
        [31]={parentZoneId=382,isTrial=false},   --Selene's Web
        [1302]={parentZoneId=20,isTrial=false},   --Shipwright's Regret
        [144]={parentZoneId=3,isTrial=false},   --Spindleclutch I
        [936]={parentZoneId=3,isTrial=false},   --Spindleclutch II
        [1197]={parentZoneId=1161,isTrial=false},   --Stone Garden
        [1121]={parentZoneId=1086,isTrial=true},   --Sunspire
        [131]={parentZoneId=58,isTrial=false},   --Tempest Island
        [380]={parentZoneId=381,isTrial=false},   --The Banished Cells I
        [935]={parentZoneId=381,isTrial=false},   --The Banished Cells II
        [1229]={parentZoneId=57,isTrial=false},   --The Cauldron
        [1153]={parentZoneId=92,isTrial=false},   --Unhallowed Grave
        [11]={parentZoneId=347,isTrial=false},   --Vaults of Madness
        [22]={parentZoneId=104,isTrial=false},   --Volenfell
        [146]={parentZoneId=19,isTrial=false},   --Wayrest Sewers I
        [933]={parentZoneId=19,isTrial=false},   --Wayrest Sewers II
        [688]={parentZoneId=584,isTrial=false},   --White-Gold Tower
    },

------------------------------------------------------------------------------------------------------------------------
    --The preloaded zoneIds of public dungeons, and their parent zoneIds
    [LIBSETS_TABLEKEY_PUBLICDUNGEON_ZONE_MAPPING] = {
        [308]={parentZoneId=104, DLCID=DLC_BASE_GAME},   --Lost City of the Na-Totambu
        [486]={parentZoneId=381, DLCID=DLC_BASE_GAME},   --Toothmaul Gully
        [169]={parentZoneId=92, DLCID=DLC_BASE_GAME},   --Razak's Wheel
        [557]={parentZoneId=347, DLCID=DLC_BASE_GAME},   --Village of the Lost
        [306]={parentZoneId=57, DLCID=DLC_BASE_GAME},   --Forgotten Crypts
        [339]={parentZoneId=101, DLCID=DLC_BASE_GAME},   --Hall of the Dead
        [284]={parentZoneId=3, DLCID=DLC_BASE_GAME},   --Bad Man's Hallows
        [124]={parentZoneId=383, DLCID=DLC_BASE_GAME},   --Root Sunder Ruins
        [137]={parentZoneId=108, DLCID=DLC_BASE_GAME},   --Rulanyil's Fall
        [138]={parentZoneId=58, DLCID=DLC_BASE_GAME},   --Crimson Cove
        [487]={parentZoneId=382, DLCID=DLC_BASE_GAME},   --The Vile Manse
        [162]={parentZoneId=20, DLCID=DLC_BASE_GAME},   --Obsidian Scar
        [134]={parentZoneId=117, DLCID=DLC_BASE_GAME},   --Sanguine's Demesne
        [216]={parentZoneId=41, DLCID=DLC_BASE_GAME},   --Crow's Wood
        [142]={parentZoneId=19, DLCID=DLC_BASE_GAME},   --Bonesnap Ruins
        [341]={parentZoneId=103, DLCID=DLC_BASE_GAME},   --The Lion's Den
        [706]={parentZoneId=684, DLCID=DLC_ORSINIUM},   --Old Orsinium
        [705]={parentZoneId=684, DLCID=DLC_ORSINIUM},   --Rkindaleft
        [919]={parentZoneId=849, DLCID=DLC_MORROWIND},   --Forgotten Wastes
        [918]={parentZoneId=849, DLCID=DLC_MORROWIND},   --Nchuleftingth
        [1090]={parentZoneId=1086, DLCID=DLC_ELSWEYR},   --Orcrest
        [1089]={parentZoneId=1086, DLCID=DLC_ELSWEYR},   --Rimmen Necropolis
        [1020]={parentZoneId=1011, DLCID=DLC_SUMMERSET},   --Karnwasten
        [1021]={parentZoneId=1011, DLCID=DLC_SUMMERSET},   --Sunhold
        [1186]={parentZoneId=1160, DLCID=DLC_GREYMOOR},   --Labyrinthian
        [1187]={parentZoneId=1161, DLCID=DLC_GREYMOOR},   --Nchuthnkarst
        [1260]={parentZoneId=1261, DLCID=DLC_BLACKWOOD},   --The Silent Halls
        [1259]={parentZoneId=1261, DLCID=DLC_BLACKWOOD},   --Zenithar's Abbey
        [1338]={parentZoneId=1318, DLCID=DLC_HIGH_ISLE},   --Ghost Haven Bay
        [1337]={parentZoneId=1318, DLCID=DLC_HIGH_ISLE},   --Spire of the Crimson Coin
        [1415]={parentZoneId=1414, DLCID=DLC_NECROM},   --Gorne
        [1416]={parentZoneId=1413, DLCID=DLC_NECROM},   --The Underweave
    },

------------------------------------------------------------------------------------------------------------------------
    --The preloaded mapping between the ESO ingame set item collection tree-view (parent zone, zone, sets in zone) and
    --the zoneIds of the mappable zones
    --The table contains the setItemCollection parentCategoryId -> their subCategories as table, flags if those are a
    --dungeon, trial, arena
    --If zoneIds can be mapped to them there will be a zoneId sub-table entry as well! Some of the entries cannot be
    --mapped to zoneIds so they will miss that sub-table!
    [LIBSETS_TABLEKEY_SET_ITEM_COLLECTIONS_ZONE_MAPPING] =
    {
        --Aldmeri-Dominion
        { parentCategory=1, category=11, zoneIds={381}},--Auridon
        { parentCategory=1, category=12, zoneIds={383}},--Grahtwood
        { parentCategory=1, category=13, zoneIds={108}},--Greenshade
        { parentCategory=1, category=14, zoneIds={58}},--Malabal Tor
        { parentCategory=1, category=15, zoneIds={382}},--Reaper's March
        --Daggerfall-Convenion
        { parentCategory=2, category=16, zoneIds={104}},--Alik'r Desert
        { parentCategory=2, category=17, zoneIds={92}},--Bangkorai
        { parentCategory=2, category=18, zoneIds={3}},--Glenumbra
        { parentCategory=2, category=19, zoneIds={20}},--Rivenspire
        { parentCategory=2, category=20, zoneIds={19}},--Stormhaven
        --Ebonheart Pact
        { parentCategory=3, category=21, zoneIds={57}},--Deshaan
        { parentCategory=3, category=22, zoneIds={101}},--Eastmarch
        { parentCategory=3, category=23, zoneIds={103}},--The Rift
        { parentCategory=3, category=24, zoneIds={117}},--Shadowfen
        { parentCategory=3, category=25, zoneIds={41}},--Stonefalls
        --DLC-Regions
        { parentCategory=4, category=29, zoneIds={684}},--Wrothgar
        { parentCategory=4, category=30, zoneIds={816}},--Hew's Bane
        { parentCategory=4, category=31, zoneIds={823}},--Gold Coast
        { parentCategory=4, category=32, zoneIds={849}},--Vvardenfell
        { parentCategory=4, category=33, zoneIds={980,981}},--Clockwork City/Brass Fortress
        { parentCategory=4, category=34, zoneIds={1011,1027}},--Summerset/Artaeum
        { parentCategory=4, category=35, zoneIds={726}},--Murkmire
        { parentCategory=4, category=36, zoneIds={1086}},--Northern Elsweyr
        { parentCategory=4, category=37, zoneIds={1133}},--Southern Elsweyr
        { parentCategory=4, category=38, zoneIds={1160,1161}},--Western Skyrim/Blackreach: Greymoor Caverns
        { parentCategory=4, category=89, zoneIds={1207,1208}},--The Reach/Blackreach: Arkthzand Cavern
        { parentCategory=4, category=93, zoneIds={1261}},--Blackwood
        { parentCategory=4, category=97, zoneIds={1286,1282,1283}},--The Deadlands/Fargrave/The Shambles
        { parentCategory=4, category=100, zoneIds={1318}},--High isle
        { parentCategory=4, category=104, zoneIds={1383}},--Galen
        { parentCategory=124, category=107, zoneIds={1413,1414}},--Apocrypha/Telvanni Peninsula TODO: Attention: This DLC uses it's own category...
        --Dungeons
        { parentCategory=5, category=39, zoneIds={148}, isDungeon=true},--Arx Corinium
        { parentCategory=5, category=40, zoneIds={380,935}, isDungeon=true},--The Banished Cells I
        { parentCategory=5, category=41, zoneIds={38}, isDungeon=true},--Blackheart Haven
        { parentCategory=5, category=42, zoneIds={64}, isDungeon=true},--Blessed Crucible
        { parentCategory=5, category=43, zoneIds={176,681}, isDungeon=true},--City of Ash I
        { parentCategory=5, category=44, zoneIds={130,932}, isDungeon=true},--Crypt of Hearts I
        { parentCategory=5, category=45, zoneIds={63,930}, isDungeon=true},--Darkshade Caverns I
        { parentCategory=5, category=46, zoneIds={449}, isDungeon=true},--Direfrost Keep
        { parentCategory=5, category=47, zoneIds={126,931}, isDungeon=true},--Elden Hollow I
        { parentCategory=5, category=48, zoneIds={283,934}, isDungeon=true},--Fungal Grotto I
        { parentCategory=5, category=49, zoneIds={31}, isDungeon=true},--Selene's Web
        { parentCategory=5, category=50, zoneIds={144,936}, isDungeon=true},--Spindleclutch I
        { parentCategory=5, category=51, zoneIds={131}, isDungeon=true},--Tempest Island
        { parentCategory=5, category=52, zoneIds={11}, isDungeon=true},--Vaults of Madness
        { parentCategory=5, category=53, zoneIds={22}, isDungeon=true},--Volenfell
        { parentCategory=5, category=54, zoneIds={146,933}, isDungeon=true},--Wayrest Sewers I
        --DLC-Dungeons
        { parentCategory=6, category=55, zoneIds={678}, isDungeon=true},--Imperial City Prison
        { parentCategory=6, category=56, zoneIds={688}, isDungeon=true},--White-Gold Tower
        { parentCategory=6, category=57, zoneIds={848}, isDungeon=true},--Cradle of Shadows
        { parentCategory=6, category=58, zoneIds={843}, isDungeon=true},--Ruins of Mazzatun
        { parentCategory=6, category=59, zoneIds={973}, isDungeon=true},--Bloodroot Forge
        { parentCategory=6, category=60, zoneIds={974}, isDungeon=true},--Falkreath Hold
        { parentCategory=6, category=61, zoneIds={1009}, isDungeon=true},--Fang Lair
        { parentCategory=6, category=62, zoneIds={1010}, isDungeon=true},--Scalecaller Peak
        { parentCategory=6, category=63, zoneIds={1055}, isDungeon=true},--March of Sacrifices
        { parentCategory=6, category=64, zoneIds={1052}, isDungeon=true},--Moon Hunter Keep
        { parentCategory=6, category=65, zoneIds={1081}, isDungeon=true},--Depths of Malatar
        { parentCategory=6, category=66, zoneIds={1080}, isDungeon=true},--Frostvault
        { parentCategory=6, category=67, zoneIds={1123}, isDungeon=true},--Lair of Maarselok
        { parentCategory=6, category=68, zoneIds={1122}, isDungeon=true},--Moongrave Fane
        { parentCategory=6, category=69, zoneIds={1152}, isDungeon=true},--Icereach
        { parentCategory=6, category=70, zoneIds={1153}, isDungeon=true},--Unhallowed Grave
        { parentCategory=6, category=87, zoneIds={1201}, isDungeon=true},--Castle Thorn
        { parentCategory=6, category=88, zoneIds={1197}, isDungeon=true},--Stone Garden
        { parentCategory=6, category=91, zoneIds={1228}, isDungeon=true},--Black Drake Villa
        { parentCategory=6, category=92, zoneIds={1229}, isDungeon=true},--The Cauldron
        { parentCategory=6, category=95, zoneIds={1268}, isDungeon=true},--Dread Cellar
        { parentCategory=6, category=96, zoneIds={1267}, isDungeon=true},--Red Petal Bastion
        { parentCategory=6, category=98, zoneIds={1301}, isDungeon=true},--Coral Aerie
        { parentCategory=6, category=99, zoneIds={1302}, isDungeon=true},--Shipwright's Regret
        { parentCategory=6, category=102, zoneIds={1360}, isDungeon=true},--Earthen Root Enclave
        { parentCategory=6, category=103, zoneIds={1361}, isDungeon=true},--Graven Deep
        { parentCategory=6, category=105, zoneIds={1389}, isDungeon=true},--Bal Sunnar
        { parentCategory=6, category=106, zoneIds={1390}, isDungeon=true},--Scrivener's Hall
        { parentCategory=6, category=110, zoneIds={1470}, isDungeon=true},--Oathsworn Pit
        { parentCategory=6, category=111, zoneIds={1471}, isDungeon=true},--Bedlam Veil
        --Trials
        { parentCategory=7, category=71, zoneIds={638}, isTrial=true},--Aetherian Archive
        { parentCategory=7, category=72, zoneIds={636}, isTrial=true},--Hel Ra Citadel
        { parentCategory=7, category=73, zoneIds={639}, isTrial=true},--Sanctum Ophidia
        { parentCategory=7, category=74, zoneIds={725}, isTrial=true},--Maw of Lorkhaj
        { parentCategory=7, category=75, zoneIds={975}, isTrial=true},--Halls of Fabrication
        { parentCategory=7, category=76, zoneIds={1000}, isTrial=true},--Asylum Sanctorium
        { parentCategory=7, category=77, zoneIds={1051}, isTrial=true},--Cloudrest
        { parentCategory=7, category=78, zoneIds={1121}, isTrial=true},--Sunspire
        { parentCategory=7, category=79, zoneIds={1196}, isTrial=true},--Kyne's Aegis
        { parentCategory=7, category=101, zoneIds={1344}, isTrial=true},--Dreadsail Reef
        { parentCategory=7, category=108, zoneIds={1427}, isTrial=true},--Sanity's Edge
        --Arenas
        { parentCategory=8, category=80, zoneIds={635}, isArena=true},--Dragonstar Arena
        { parentCategory=8, category=81, zoneIds={677}, isArena=true},--Maelstrom Arena
        { parentCategory=8, category=82, zoneIds={1082}, isArena=true},--Blackrose Prison
        { parentCategory=8, category=90, zoneIds={1227}, isArena=true},--Vateshran Hollows
        --PVP
        { parentCategory=9, category=83, zoneIds={181}},--Cyrodiil
        { parentCategory=9, category=84, zoneIds={584}},--Imperial City
        { parentCategory=9, category=85},--Battlegrounds
        --Miscellaneous
        { parentCategory=10, category=26, zoneIds={280,281,534,535,537}},   --Beginner zones
        { parentCategory=10, category=27, zoneIds={347}},--Coldharbour
        { parentCategory=10, category=28, zoneIds={888}},--Craglorn
        { parentCategory=10, category=86},--Antiquities
    }, --LIBSETS_TABLEKEY_SET_ITEM_COLLECTIONS_ZONE_MAPPING

------------------------------------------------------------------------------------------------------------------------
--  IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT
------------------------------------------------------------------------------------------------------------------------
-- [SET PROCS LAST UPDATE: 14.03.2021, API10034, Baertram] -> CURRENTLY NOT FURTHER SUPPORTED! NOT UPDATED!!!
-- Data provided by ExoY (and the internet + furthe ringaem tests).
-- See addon ExoYs ProcSet timer: https://www.esoui.com/downloads/info2783-ExoYsProcSetTimer.html (it's not using LibSets!)

    --The set procs
    --Generated lua code coming from file LibSets_SetData.xlsx -> tab SetProcs, column g "Lua table coce of set procs".
    -->Copy the code and as it's multiline Excel will add "" :-( Search and replace the double "" first with ". Then search for "[ and replace with [.
    -->Afterwards search for regex "\n and replace with \n
    -->Then lua minify the tables!
    --
    --The base format of each table entry will be:
    --[[
    [number setId] = {
        [number LIBSETS_SETPROC_CHECKTYPE_ constant from LibSets_ConstantsLibraryInternal.lua] = {
            [number index1toN] = {
                ["abilityIds"] = {number abilityId1, number abilityId2, ...},
                    --Only for LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_EFFECT_CHANGED
                    ["unitTag"] = String unitTag e.g. "player", "playerpet", "group", "boss", etc.,

                    --Only for LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT
                    ["source"] = number combatUnitType e.g. COMBAT_UNIT_TYPE_PLAYER
                    ["target"] = number combatUnitType e.g. COMBAT_UNIT_TYPE_PLAYER

                    --Only for LIBSETS_SETPROC_CHECKTYPE_EVENT_POWER_UPDATE
                    ["powerType"] = number powerType e.g. POWERTYPE_STAMINA

                    --Only for LIBSETS_SETPROC_CHECKTYPE_EVENT_BOSSES_CHANGED
                    ["unitTag"] = String unitTagOfBoss e.g. boss1, boss2, ...

                    --Only for LIBSETS_SETPROC_CHECKTYPE_SPECIAL
                    [number index1toN] = boolean specialFunctionIsGiven e.g. true/false (if true: the abilityId1's callback function should run a special                                             function as well, which will be registered for the

                ["cooldown"] = {number cooldownForAbilityId1 e.g. 12000, number cooldownForAbilityId2, ...},
                ["icon"] = String iconPathOfTheBuffIconToUse e.g. "/esoui/art/icons/ability_buff_minor_vitality.dds"
            },
        },
        [number LIBSETS_SETPROC_CHECKTYPE_ constant from LibSets_ConstantsLibraryInternal.lua] = {
        ...
        },
        ...
        --String comment name of the set -> description of the proc EN / description of the proc DE
    },
    ]]
    --[[
    2022-04-20, Disabled
    [LIBSETS_TABLEKEY_SET_PROCS] = {
------------------------------------------------------------------------------------------------------------------------
        [147] = {[LIBSETS_SETPROC_CHECKTYPE_EVENT_POWER_UPDATE] = {
            [1] = {
              ["unitTag"] = "player",
              ["powerType"] = POWERTYPE_STAMINA,
                  },
        },
        [LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {127070},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
              ["cooldown"] = {8000},
                  },
          },
        [LIBSETS_SETPROC_CHECKTYPE_SPECIAL] = {
            [1] = true,
        },},     --Way of Martial Knowledge ->  / "
        [160] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {61459},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
              ["cooldown"] = {12000},
                  },
          },},     --Burning Spellweave ->  / "
        [167] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {59590},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        },},     --Nightflame ->  / "
        [180] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {61771},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Powerful Assault ->  / "
        [181] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {65706},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Meritorious Service ->  / "
        [185] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {66902},
              ["cooldown"] = {5000},
            --Attention: No source COMBAT_UNIT_TYPE_PLAYER here!
                  },
          },},     --Spell Power Cure ->  / "
        [211] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {71107},
              ["cooldown"] = {15000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Permafrost ->  / "
        [225] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {75746},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Clever Alchemist ->  / "
        [230] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {75801, 75804},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Moondancer -> 1: Lunar blessing (magReg), 2: Shadow blessing (spellPower) / "
        [268] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {81036},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
              ["cooldown"] = {15000},
            },
          },},     --Sentinel of Rkugamz ->  / "
        [271] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {80545},
              ["cooldown"] = {6000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Sellistrix ->  / "
        [276] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {80865},
              ["cooldown"] = {7000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Tremorscale ->  / "
        [280] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {84504},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Grothdarr -> Grothdarr proc 1 / "
        [341] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {97855},
              ["cooldown"] = {20000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Earthgore ->  / "
        [353] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {99204},
              ["cooldown"] = {18000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Mechanical Acuity ->  / "
        [391] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {107141},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Vestment of Olorime ->  / "
        [395] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {109084},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Perfect Vestment of Olorime ->  / "
        [446] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {121878},
              ["cooldown"] = {8000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Claw of Yolnakhriin ->  / "
        [451] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {121878},
              ["cooldown"] = {8000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Perfected Claw of Yolnakhriin ->  / "
        [452] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {126924},
              ["cooldown"] = {9000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
                  },
          },},     --Hollowfang Thirst ->  / "
        [455] = {[LIBSETS_SETPROC_CHECKTYPE_EVENT_BOSSES_CHANGED] = {
            [1] = {
              ["abilityIds"] = {126597},
              ["cooldown"] = {22000}
                  },
          },
        [LIBSETS_SETPROC_CHECKTYPE_SPECIAL] = {
            [1] = true,
        },},     --Z'en's Redress ->  / "
        [471] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {133210},
              ["cooldown"] = {12000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
        },
          },},     --Hiti\'s Hearth ->  / "
        [487] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {135659},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Winter's Respite ->  / "
        [488] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {135690},
              ["cooldown"] = {15000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Venomous Smite ->  / "
        [492] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {136098},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Kyne's Wind ->  / "
        [493] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {137995},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Perfected Kyne's Wind ->  / "
        [496] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {135923},
              ["debuffIds"] = {135924},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
              ["cooldown"] = {22000},
                  },
        },
        [LIBSETS_SETPROC_CHECKTYPE_SPECIAL] = {
            [1] = true,
        },},     --Roaring Opportunist ->  / "
        [497] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {137986},
              ["debuffIds"] = {137985},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
              ["cooldown"] = 22000
                  },
        },
        [LIBSETS_SETPROC_CHECKTYPE_SPECIAL] = {
            [1] = true,
           },},     --Perfected Roaring Opportunist ->  / "
        [535] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
        [1] = {
              ["abilityIds"] = {141905},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
        }},     --Lady Thorn ->  / "
        [558] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147747},
              ["cooldown"] = {13000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Void Bash ->  / "
        [561] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147858},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Wrath of Elements ->  / "
        [562] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147875},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Force Overflow ->  / "
        [564] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147747},
              ["cooldown"] = {13000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Perfected Void Bash ->  / "
        [567] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147858},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Perfected Wrath of Elements ->  / "
        [568] = {[LIBSETS_SETPROC_CHECKTYPE_ABILITY_EVENT_COMBAT_EVENT] = {
            [1] = {
              ["abilityIds"] = {147875},
              ["cooldown"] = {10000},
              ["source"] = COMBAT_UNIT_TYPE_PLAYER,
            },
          },},     --Perfected Force Overflow ->  / "
    }, --LIBSETS_TABLEKEY_SET_PROCS
    ]]
------------------------------------------------------------------------------------------------------------------------
}--lib.setDataPreloaded
--**********************************************************************************************************************
local setDataPreloaded = lib.setDataPreloaded
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------
-- Special set bonus
------------------------------------------------------------------------------------------------------------------------
--Special setIds where the bonus is kind of different from other sets, e.g. 3 lines of bonus but line 1 and line 2 both reflect to setBonus 1,
--or line1 is 5 set items, line2 is 10 set items and line3 is 12 set items
lib.specialBonusSets = {
    [695] = { [5]=1, [10]=2, [12]=3} --Shattered Fate: Bonus line 1 relates to 5 items bonus, line 2 to 10 and line 3 to 12
}


------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
--Entries to the setItemCollection mappings which are APIversion dependend
local mappingForSetItemCollectionsNotYetLive = {
    --[[
        --EXAMPLE
        { parentCategory=6, category=91, zoneIds={1228}, isDungeon=true},--Black Drake Villa
        { parentCategory=6, category=92, zoneIds={1229}, isDungeon=true},--The Cauldron
    ]]
}
if isPTSAPIVersionLive == true then
    for _, mappingData in ipairs(mappingForSetItemCollectionsNotYetLive) do
        tins(lib.setDataPreloaded[LIBSETS_TABLEKEY_SET_ITEM_COLLECTIONS_ZONE_MAPPING], mappingData)
    end
end


------------------------------------------------------------------------------------------------------------------------
-- Remove set data from the preloaded tables which is not given at the current API version (live server)
------------------------------------------------------------------------------------------------------------------------
--The setIds which do not exist at the current API versin (determined at LoadSets() -> CheckSetExists(setId))
local nonExistingSetIdsAtCurrentApiVersion = lib.nonExistingSetIdsAtCurrentApiVersion

local function removeFutureSetData()
    if not isPTSAPIVersionLive then
        if setsOfNewerAPIVersion ~= nil and not ZO_IsTableEmpty(setsOfNewerAPIVersion) then
            ------------------------------------------------------------------------------------------------------------------------
            --Check all setIds of the new APIversion
            local nonSetIds = lib.noSetIdSets
            local setInfo = lib.setInfo
            local setIdsToSetItemIds =          setDataPreloaded[LIBSETS_TABLEKEY_SETITEMIDS]
            local setIdsNoSetToSetItemIds =     setDataPreloaded[LIBSETS_TABLEKEY_SETITEMIDS_NO_SETID]
            local setNamesNoSetToSetItemIds =   setDataPreloaded[LIBSETS_TABLEKEY_SETNAMES_NO_SETID]
            local preloadedEquipTypeData =      setDataPreloaded[LIBSETS_TABLEKEY_SETS_EQUIP_TYPES]
            local preloadedArmorTypeData =      setDataPreloaded[LIBSETS_TABLEKEY_SETS_ARMOR_TYPES]
            local preloadedWeaponTypeData =     setDataPreloaded[LIBSETS_TABLEKEY_SETS_WEAPONS_TYPES]
            local preloadedIsJewelryData =      setDataPreloaded[LIBSETS_TABLEKEY_SETS_JEWELRY]
            local setIdsToSetNames =            setDataPreloaded[LIBSETS_TABLEKEY_SETNAMES]
            for _, setIdOfNewAPIVersion in ipairs(setsOfNewerAPIVersion) do
                if setIdOfNewAPIVersion ~= nil then
                    --Remove the setIds of non-sets from the table
                    if nonSetIds[setIdOfNewAPIVersion] ~= nil then
                        lib.noSetIdSets[setIdOfNewAPIVersion] = nil
                        --Add to currently not available setIds
                        nonExistingSetIdsAtCurrentApiVersion[setIdOfNewAPIVersion] = true
                    end
                    --Remove setIds from the setInfo
                    if setInfo[setIdOfNewAPIVersion] ~= nil then
                        lib.setInfo[setIdOfNewAPIVersion] = nil
                        --Add to currently not available setIds
                        nonExistingSetIdsAtCurrentApiVersion[setIdOfNewAPIVersion] = true
                    end
                    --Remove itemIds from the list where setIds are only available in newer APIversions
                    if setIdsToSetItemIds[setIdOfNewAPIVersion] ~= nil then
                        setDataPreloaded[LIBSETS_TABLEKEY_SETITEMIDS][setIdOfNewAPIVersion] = nil
                    end
                    --Remove itemIds of non-sets from the list where setIds are only available in newer APIversions
                    if setIdsNoSetToSetItemIds[setIdOfNewAPIVersion] ~= nil then
                        setDataPreloaded[LIBSETS_TABLEKEY_SETITEMIDS_NO_SETID][setIdOfNewAPIVersion] = nil
                    end
                    --Remove names from the list where setIds are only available in newer APIversions
                    if setIdsToSetNames[setIdOfNewAPIVersion] ~= nil then
                        setDataPreloaded[LIBSETS_TABLEKEY_SETNAMES][setIdOfNewAPIVersion] = nil
                    end
                    --Remove names of non-sets from the list where setIds are only available in newer APIversions
                    if setNamesNoSetToSetItemIds[setIdOfNewAPIVersion] ~= nil then
                        setDataPreloaded[LIBSETS_TABLEKEY_SETNAMES_NO_SETID][setIdOfNewAPIVersion] = nil
                    end
                    --Remove the equipTypes
                    for _, equipTypeData in pairs(preloadedEquipTypeData) do
                        if equipTypeData[setIdOfNewAPIVersion] ~= nil then
                            setDataPreloaded[LIBSETS_TABLEKEY_SETS_EQUIP_TYPES][setIdOfNewAPIVersion] = nil
                        end
                    end
                    --Remove the armorTypes
                    for _, armorTypeData in pairs(preloadedArmorTypeData) do
                        if armorTypeData[setIdOfNewAPIVersion] ~= nil then
                            setDataPreloaded[LIBSETS_TABLEKEY_SETS_ARMOR_TYPES][setIdOfNewAPIVersion] = nil
                        end
                    end
                    --Remove the weaponTypes
                    for _, weaponTypeData in pairs(preloadedWeaponTypeData) do
                        if weaponTypeData[setIdOfNewAPIVersion] ~= nil then
                            setDataPreloaded[LIBSETS_TABLEKEY_SETS_WEAPONS_TYPES][setIdOfNewAPIVersion] = nil
                        end
                    end
                    --Remove the isJewelrySet
                    if preloadedIsJewelryData[setIdOfNewAPIVersion] ~= nil then
                        lib.setDataPreloaded[LIBSETS_TABLEKEY_SETS_JEWELRY][setIdOfNewAPIVersion] = nil
                    end
                end
            end
        end --if setsOfNewerAPIVersion ~= nil then
        ------------------------------------------------------------------------------------------------------------------------
        --Remove the wayshrines that are new!
        if not zoneIdsOfNewAPIVersionOnly == nil then
            local wayshrines2ZoneIds = lib.setDataPreloaded[LIBSETS_TABLEKEY_WAYSHRINENODEID2ZONEID]
            for _, zoneIdOfNewAPIVersion in pairs(zoneIdsOfNewAPIVersionOnly) do
                if zoneIdOfNewAPIVersion ~= nil then
                    for wayshrineNodeIndex, zoneId in pairs(wayshrines2ZoneIds) do
                        if zoneId == zoneIdOfNewAPIVersion then
                            lib.setDataPreloaded[LIBSETS_TABLEKEY_WAYSHRINENODEID2ZONEID][wayshrineNodeIndex] = nil
                        end
                    end
                end
            end
        end
        ------------------------------------------------------------------------------------------------------------------------
    end
end
lib.removeFutureSetData = removeFutureSetData
------------------------------------------------------------------------------------------------------------------------